/**
 * @constructor
 */
function ModelElement(_objResourceData){
	var me = this;
    //the object was created in in the player rather than in the editor; false by default
    me.flagUserCreated = false;
    me.objData = {
        depth: -32767,
        flagColorModifiable: false
    };
    //properties not stored in model
    me.objDataConst = {};
    me.objDataConst.highlightWidth = me.DEFAULT_HIGHLIGHT_WIDTH;
    me.objDataConst.showCorrectWidth = me.SHOW_CORRECT_HIGHLIGH_WIDTH;
    me.objDataConst.showIncorrectWidth = me.SHOW_INCORRECT_HIGHLIGH_WIDTH;
    me.objDataConst.selectWidth = me.DEFAULT_SELECT_WIDTH;
    me.objDataConst.flagScalable = false;
	me.fFillProperties(_objResourceData);
	me.depth = me.objData.depth;
    me.flagHighlight = false;
    //если flagSelect==true, а flagHighlight==false, то рамка подсветки мигает
    me.flagSelect = false;
    //when correct elements are highlighted
    me.flagBlink = false;
    //different blinking for correctness status indication
    me.flagShowCorrect = false;
    me.flagShowIncorrect = false;
    //for hit-wise complex objects
    me.hitDetails = null;
    //
    me.hitPoint = new Point(0,0);
    //if the element is being created it is incomplete and might be ignored in hitTest
    me.flagComplete = true;
    //for complex elements
    me.flagStructureEditable = false;
    //
    me.flagFillColored = false;
    me.flagLineColored = false;
    //
    me.flagBlinkingFrameWhenSelected = true;
    //some of the verifiable elements, like texts, may be shown, even though differently (initially without text)
    me.flagShowIfVerified = false;
    //element should be compared to itself in the correct state, e.g. text field
    me.flagVerifyOwnState = false;
    //if true this element's visualization may result in scrollers for MapSceneVisualization provided the former
    //is large enough
    me.flagScrollerProvoking = false;
    //
    me.flagVisibleInPlayer = me.isInitiallyTurnedOn();
    //
    me.flagValid = true;
}
extend(ModelElement, ObjectWithMethodizedProperties);

//factor for increasing the sensitive area for touch-screens
ModelElement.prototype.TOUCH_SIZE_FACTOR = 3;
ModelElement.prototype.ZOOM_RATIO = 1;

ModelElement.prototype.DEFAULT_HIGHLIGHT_WIDTH = 2;
ModelElement.prototype.SHOW_CORRECT_HIGHLIGH_WIDTH = 4;
ModelElement.prototype.SHOW_INCORRECT_HIGHLIGH_WIDTH = 4;
ModelElement.prototype.DEFAULT_SELECT_WIDTH = 1;
ModelElement.prototype.UNSELECTABLE_CLASS = "unselectable";
ModelElement.prototype.NO_CANVAS = "";

ModelElement.prototype.typePrefix = null;
ModelElement.prototype.familyPrefix = null;
ModelElement.prototype.className = null;
ModelElement.prototype.localizedNamePrefix = "Элемент ";

ModelElement.prototype.POINT_WITHIN_METHOD = "isPointWithin";

//name, property, type and optionally default value in case INTEGER or NUMBER property=="undefined"
ModelElement.prototype.aPropretyNameMap = [
    ["Depth", "depth",  TYPE_INTEGER],
    ["Id", "id",  TYPE_INTEGER],
    ["groupId", "groupId", TYPE_INTEGER],
    ["CreationId", "creationId",  TYPE_INTEGER],
    ["IsVisible", "flagVisible",  TYPE_BOOLEAN],
    ["Is_Ver", "flagVerifiable",  TYPE_BOOLEAN],
    ["NoEdit", "flagNoEdit",  TYPE_BOOLEAN],
    ["type_id", "typeId",  TYPE_INTEGER],
    ["Name", "name",  TYPE_STRING],
    ["CHXY", "flagCheckCoordinates",  TYPE_BOOLEAN],
    ["Pl_ECSt", "flagColorModifiable",  TYPE_BOOLEAN],
    ["Pl_ESSt", "flagShapeModifiable",  TYPE_BOOLEAN],
    ["Pl_MSt", "flagMovable",  TYPE_BOOLEAN],
    ["Pl_HSw", "flagCanBeHighlighted", TYPE_BOOLEAN],
    ["Pl_VSw", "flagCanBeTurnedOn",  TYPE_BOOLEAN],
    ["Pl_VSwOn", "flagInitiallyTurnedOn",  TYPE_BOOLEAN],
    ["FF", "flagEmptyFill",  TYPE_BOOLEAN],
    ["errorMode", "errorMode",  TYPE_STRING],
    ["RZ", "errorZoneRadius",  TYPE_NUMBER],
    ["fromZoneId", "fromZoneId",  TYPE_INTEGER],
    ["fromZoneType", "fromZoneType",  TYPE_STRING],
    ["OW", "objectWeight",  TYPE_NUMBER]
];

//auxiliary methods for tools: e.g. setting "fillColor" property should call "setFillColor" for rectangle element
//but "setFontColor" for text element

ModelElement.prototype.aMethodNameForProperty = [];

//
ModelElement.prototype.setMethodNameForProperty(
    ModelElement.prototype.aMethodNameForProperty, "isVisible", false, "getVisible"
);
ModelElement.prototype.setMethodNameForProperty(
    ModelElement.prototype.aMethodNameForProperty, "isVisible", true, "setVisible"
);
ModelElement.prototype.setMethodNameForProperty(
    ModelElement.prototype.aMethodNameForProperty, "canBeTurnedOn", false, "isCanBeTurnedOn"
);
ModelElement.prototype.setMethodNameForProperty(
    ModelElement.prototype.aMethodNameForProperty, "name", false, "getName"
);
ModelElement.prototype.setMethodNameForProperty(
    ModelElement.prototype.aMethodNameForProperty, "id", false, "getId"
);
ModelElement.prototype.setMethodNameForProperty(
    ModelElement.prototype.aMethodNameForProperty, "colorModifiable", false, "isColorModifiable"
);
//

ModelElement.prototype.getLocalizedNamePrefix = function(){
    return this.localizedNamePrefix;
};

ModelElement.prototype.isUserCreated = function(){
    return this.flagUserCreated;
};

ModelElement.prototype.setUserCreated = function(flag){
    this.flagUserCreated = flag;
};


ModelElement.prototype.isComplete = function(){
    return this.flagComplete;
};

ModelElement.prototype.setComplete = function(flag){
    this.flagComplete = flag;
};

ModelElement.prototype.isStructureEditable = function(){
    return this.flagStructureEditable;
};

ModelElement.prototype.setStructureEditable = function(flag){
    this.flagStructureEditable = flag;
};

ModelElement.prototype.getType = function(){
    return this.typePrefix;
};

ModelElement.prototype.getClassName = function(){
    return this.className;
};

//
ModelElement.prototype.isHighlighted = function(){
    return this.flagHighlight;
};

ModelElement.prototype.setHighlighted = function(flag){
    this.flagHighlight = flag;
};

//
ModelElement.prototype.isBlinking = function(){
    return this.flagBlink;
};

//
ModelElement.prototype.setBlinking = function(flag){
    var me = this;
    me.flagBlink = flag;
    me.flagShowCorrect = false;
    me.flagShowIncorrect = false;
};

//
ModelElement.prototype.isShowCorrect = function(){
    return this.flagShowCorrect;
};

//
ModelElement.prototype.setShowCorrect = function(flag){
    var me = this;
    if(flag == me.flagShowCorrect)
        return;
    me.flagShowCorrect = flag;
    me.flagBlink = flag;
    if(flag)
        me.flagShowIncorrect = false;
};

//
ModelElement.prototype.isShowIncorrect = function(){
    return this.flagShowIncorrect;
};

//
ModelElement.prototype.setShowIncorrect = function(flag){
    var me = this;
    if(flag == me.flagShowIncorrect)
        return;
    me.flagShowIncorrect = flag;
    me.flagBlink = flag;
    if(flag)
        me.flagShowCorrect = false;
};


//
ModelElement.prototype.getHitDetails = function(){
    return this.hitDetails;
};

ModelElement.prototype.setHitDetails = function(details){
    this.hitDetails = details;
};

//
ModelElement.prototype.canBeSelected = function(){
    var objData = this.objData;
    return (objData.flagCanBeHighlighted || objData.flagColorModifiable ||
        objData.flagShapeModifiable || objData.flagMovable) && (!objData.flagVerifiable || this.isShowIfVerifiable());
};

//
ModelElement.prototype.isSelected = function(){
    return this.flagSelect;
};

ModelElement.prototype.setSelected = function(flag){
    this.flagSelect = flag;
    if(flag) {
        this.setBlinking(false);
    }
};

//
ModelElement.prototype.isBlinkingFrameWhenSelected = function () {
    return this.flagBlinkingFrameWhenSelected;
};

//
ModelElement.prototype.isShowIfVerifiable = function(){
    return this.flagShowIfVerified;
};

//
ModelElement.prototype.isVerifyOwnState = function(){
    return this.flagVerifyOwnState;
};

//
ModelElement.prototype.isScrollerProvoking = function(){
    return this.flagScrollerProvoking;
};

//
ModelElement.prototype.canBeScaled = function(){
    return this.objDataConst.flagScalable;
};

//
ModelElement.prototype.getBounds = function(){
    return null;
};

ModelElement.prototype.fFillProperties = function(_objResourceData){
	var i, a = this.aPropretyNameMap, n = a.length, sProp, sType, sName;
	var me = this;
	var objData = me.objData;
	for(i=0; i<n; i++){
		sName = a[i][0];
		sProp = a[i][1];
		sType = a[i][2];
		if(typeof _objResourceData[sName] != TYPE_UNDEFINED){
			switch(sType){
				case TYPE_INTEGER:
					objData[sProp] = !isNaN(_objResourceData[sName]) ? parseInt(_objResourceData[sName]) : undefined;
					break;
				case TYPE_NUMBER:
					objData[sProp] = !isNaN(_objResourceData[sName]) ? parseFloat(_objResourceData[sName]) : undefined;
					break;
				case TYPE_BOOLEAN:
					objData[sProp] = (""+_objResourceData[sName]).toLowerCase() == "true";
					break;
				case TYPE_STRING:
					objData[sProp] = (""+_objResourceData[sName]).slice(0);
					break;
                case TYPE_OBJECT:
                    objData[sProp] = _objResourceData[sName];
				default:
					continue;
			}
		}
	}
};

//
ModelElement.prototype.getData = function(){
    return this.objData;
};

//
ModelElement.prototype.getDepth = function(){
	return this.objData.depth;
};

//
ModelElement.prototype.getId = function(){
    return this.objData.id;
};

//
ModelElement.prototype.getGroupId = function(){
    return this.objData.groupId;
};

//
ModelElement.prototype.getCreationId = function(){
    return this.objData.creationId;
};

//
ModelElement.prototype.getTypeId = function(){
    return this.objData.typeId;
};

//
ModelElement.prototype.isVisible = function(){
    //return true;
    return (ModelMainInstance.prototype.flagPlayerMode) ? this.flagVisibleInPlayer :this.objData.flagVisible;
};

ModelElement.prototype.setVisible = function(flag){
    if(ModelMainInstance.prototype.flagPlayerMode)
        this.flagVisibleInPlayer = flag;
    else
        this.objData.flagVisible = flag;
};

//
ModelElement.prototype.isVerifiable = function(){
    return this.objData.flagVerifiable;
};

//
ModelElement.prototype.isNoEdit = function(){
    return this.objData.flagNoEdit;
};

//
ModelElement.prototype.getName = function(){
    return this.objData.name;
};

//
ModelElement.prototype.isCheckCoordinates = function(){
    return this.objData.flagCheckCoordinates;
};

//
ModelElement.prototype.isColorModifiable = function(){
    return this.objData.flagColorModifiable;
};

//
ModelElement.prototype.isShapeModifiable = function(){
    return this.objData.flagShapeModifiable;
};

//
ModelElement.prototype.isMovable = function(){
    return this.objData.flagMovable;
};

//
ModelElement.prototype.isMovableByMouse = function(){
    return this.isMovable();
};

//
ModelElement.prototype.isMovableByKeyboard = function(){
    return this.isMovable();
};

//
ModelElement.prototype.canBeHighlighted = function(){
    return this.objData.flagCanBeHighlighted;
};

//
ModelElement.prototype.canBeRemoved = function(){
    return this.flagUserCreated || !ModelMainInstance.prototype.flagPlayerMode;
};

//
ModelElement.prototype.canBeFillColored = function(){
    return this.flagFillColored;
};

//
ModelElement.prototype.canBeLineColored =function(){
    return this.flagLineColored;
};

//
ModelElement.prototype.isOtherwiseModifiable = function(){
    //for class-specific properties of the object
    return false;
};

//
ModelElement.prototype.isCanBeTurnedOn = function(){
    return this.objData.flagCanBeTurnedOn;
};

//
ModelElement.prototype.isInitiallyTurnedOn = function(){
    return this.objData.flagInitiallyTurnedOn;
};

//
ModelElement.prototype.isEmptyFill = function(){
    return this.objData.flagEmptyFill;
};

//
ModelElement.prototype.getErrorMode = function(){
    return this.objData.errorMode;
};

//
ModelElement.prototype.getErrorZoneRadius = function(){
    return this.objData.errorZoneRadius;
};

//
ModelElement.prototype.getFromZoneId = function(){
    return this.objData.fromZoneId;
};

//
ModelElement.prototype.getFromZoneType = function(){
    return this.objData.fromZoneType;
};

//
ModelElement.prototype.getObjectWeight = function(){
    return this.objData.objectWeight;
};

//
ModelElement.prototype.getHighlightWidth = function(){
    var me = this;
    return me.flagShowCorrect ? me.objDataConst.showCorrectWidth :
        (me.flagShowIncorrect ? me.objDataConst.showIncorrectWidth : me.objDataConst.highlightWidth);
};

//
ModelElement.prototype.getSelectWidth = function(){
    return this.objDataConst.selectWidth;
};

//
ModelElement.prototype.getHitTestInfoForPoint = function(p0){
    return new HitTestInfo(this, false, false);
};

//
ModelElement.prototype.getHitPoint = function(){
    return this.hitPoint;
};

//
ModelElement.prototype.setHitPoint = function(p){
    this.hitPoint.x = p.x;
    this.hitPoint.y = p.y;
};

//
ModelElement.prototype.reset = function(){
};

//
ModelElement.prototype.isValid = function(){
    return this.flagValid;
};

//
ModelElement.prototype.invalidate = function(){
    if(this.flagValid)
        this.flagValid = false;
};

//
ModelElement.prototype.validate = function(){
    if(!this.flagValid)
        this.flagValid = true;
};


//
ModelElement.prototype.deactivate = function(){
};

//a static method; used to check if all the elements in aModelElement have specified metod
ModelElement.prototype.checkIfAllHaveMethod = function(aModelElement, sMethodName){
    var a = aModelElement, i, n = a.length;
    for(i=0; i<n; i++){
        if(typeof a[i][sMethodName] != TYPE_FUNCTION)
            break;
    }
    return i==n;
};

//a static method; used to check if all the elements in aModelElement have specified permission
//flag invert should be true if all the elements should have no specified permission
ModelElement.prototype.checkIfAllAre = function(aModelElement, sFunctionName, invert){
    var a = aModelElement, i, n = a.length;
    var s = sFunctionName;
    var flagNot = (typeof invert == TYPE_BOOLEAN) && invert;
    var element, v;
    for(i=0; i<n; i++){
        element = a[i];
        if(typeof element[s] == TYPE_FUNCTION){
            try {
                v = flagNot ? (!element[s].call(element)) : element[s].call(element);
                if(typeof v != TYPE_BOOLEAN || !v)
                    break;
            }catch(e){
                break;
            }
        }else
            break;
    }
    return i==n;
};

//checks if the element is "equal" to refElement by criteria (position, color, text etc.) specified in options
ModelElement.prototype.isMatchForElement = function(refElement, options){
    return false;
};

//a "crutch" method to tell between closed and unclosed polyline color verification
ModelElement.prototype.isForbiddenPropertyCheck = function(checkPropertyObj){
    return false;
};


/**
 * @constructor
 */
function ElementPart(parentElement, _objResourceData){
    ElementPart.superclass.constructor.call(this, _objResourceData);
    var me = this;
    me.parentElement = parentElement;
}
extend(ElementPart, ModelElement);

ElementPart.prototype.getParentElement = function(){
    return this.parentElement;
};
/**
 * @constructor
 */
function ModelElementWithFill(_objResourceData){
    ModelElementWithFill.superclass.constructor.call(this, _objResourceData);
    var me = this;
    me.flagFillColored = true;
}
extend(ModelElementWithFill, ModelElement);

ModelElementWithFill.prototype.aMethodNameForProperty =
    miscFunctions.cloneObject(ModelElementWithFill.superclass.aMethodNameForProperty);
ModelElementWithFill.prototype.setMethodNameForProperty(
    ModelElementWithFill.prototype.aMethodNameForProperty, "fillColor", false, "getFillColor"
);
ModelElementWithFill.prototype.setMethodNameForProperty(
    ModelElementWithFill.prototype.aMethodNameForProperty, "fillColor", true, "setFillColor"
);
ModelElementWithFill.prototype.setMethodNameForProperty(
    ModelElementWithFill.prototype.aMethodNameForProperty, "fillAlpha", false, "getFillAlpha"
);
ModelElementWithFill.prototype.setMethodNameForProperty(
    ModelElementWithFill.prototype.aMethodNameForProperty, "fillAlpha", true, "setFillAlpha"
);
ModelElementWithFill.prototype.setMethodNameForProperty(
    ModelElementWithFill.prototype.aMethodNameForProperty, "checkColor", false, "isCheckColor"
);

ModelElementWithFill.prototype.aPropretyNameMap = ModelElementWithFill.superclass.aPropretyNameMap.slice(0);
(function(){
    var a = ModelElementWithFill.prototype.aPropretyNameMap;
    a.splice(a.length, 0,
        ["CHCL", "flagCheckColor",  TYPE_BOOLEAN],
        ["CHCF", "flagCheckFillColor",  TYPE_BOOLEAN],
        ["FA", "fillAlpha",  TYPE_NUMBER],
        ["FC", "fillColor",  TYPE_INTEGER]
    );
})();

//
ModelElementWithFill.prototype.isCheckColor = function(){
    return this.objData.flagCheckColor;
};

//
ModelElementWithFill.prototype.isCheckFillColor = function(){
    return this.objData.flagCheckFillColor;
};

//
ModelElementWithFill.prototype.getFillAlpha = function(){
    return this.objData.fillAlpha;
};

//
ModelElementWithFill.prototype.setFillAlpha = function(v){
    this.objData.fillAlpha = v;
};

//
ModelElementWithFill.prototype.getFillColor = function(){
    return this.objData.fillColor;
};

//
ModelElementWithFill.prototype.setFillColor = function(v){
    this.objData.fillColor = v;
};

//
ModelElementWithFill.prototype.isEmptyFill = function(){
    var me = this;
    var objData = me.objData;
    return objData.fillColor == undefined || objData.fillAlpha == undefined;
};
/**
 * @constructor
 */
function ModelElementWithBorder(_objResourceData){
    ModelElementWithBorder.superclass.constructor.call(this, _objResourceData);
    var me = this;
    me.objDataConst.borderWidth = me.DEFAULT_BORDER_WIDTH;
    //
    me.flagLineColored = true;
}
extend(ModelElementWithBorder, ModelElementWithFill);

ModelElementWithBorder.prototype.DEFAULT_BORDER_WIDTH = 4;

ModelElementWithBorder.prototype.aMethodNameForProperty =
    miscFunctions.cloneObject(ModelElementWithBorder.superclass.aMethodNameForProperty);
ModelElementWithBorder.prototype.setMethodNameForProperty(
    ModelElementWithBorder.prototype.aMethodNameForProperty, "lineColor", false, "getLineColor"
);
ModelElementWithBorder.prototype.setMethodNameForProperty(
    ModelElementWithBorder.prototype.aMethodNameForProperty, "lineColor", true, "setLineColor"
);
ModelElementWithBorder.prototype.setMethodNameForProperty(
    ModelElementWithBorder.prototype.aMethodNameForProperty, "lineAlpha", false, "getLineAlpha"
);
ModelElementWithBorder.prototype.setMethodNameForProperty(
    ModelElementWithBorder.prototype.aMethodNameForProperty, "lineAlpha", true, "setLineAlpha"
);
ModelElementWithBorder.prototype.setMethodNameForProperty(
    ModelElementWithBorder.prototype.aMethodNameForProperty, "checkLineColor", false, "isCheckBorderColor"
);

ModelElementWithBorder.prototype.aPropretyNameMap = ModelElementWithBorder.superclass.aPropretyNameMap.slice(0);
(function(){
    var a = ModelElementWithBorder.prototype.aPropretyNameMap;
    a.splice(a.length, 0,
        ["CHCB", "flagCheckBorderColor",  TYPE_BOOLEAN],
        ["LA", "lineAlpha",  TYPE_NUMBER],
        ["LC", "lineColor",  TYPE_INTEGER]
    );
})();

//
ModelElementWithBorder.prototype.isCheckBorderColor = function(){
    return this.objData.flagCheckBorderColor;
};

//
ModelElementWithBorder.prototype.getLineAlpha = function(){
    return this.objData.lineAlpha;
};

//
ModelElementWithBorder.prototype.setLineAlpha = function(v){
    this.objData.lineAlpha = v;
};

//
ModelElementWithBorder.prototype.getLineColor = function(){
    return this.objData.lineColor;
};

//
ModelElementWithBorder.prototype.setLineColor = function(v){
    this.objData.lineColor = v;
};

//
ModelElementWithBorder.prototype.getBorderWidth = function(){
    return this.objDataConst.borderWidth;
};

//
ModelElementWithBorder.prototype.isEmptyBorder = function(){
    var me = this;
    var objData = me.objData;
    return objData.lineColor == undefined || objData.lineAlpha == undefined;
};
/**
 * @constructor
 */
function ModelElementVisualization(_element, _sceneVisualization){
    ModelElementVisualization.superclass.constructor.call(this);
    var me = this;
    me.element = _element;
    me.sceneVisualization = _sceneVisualization;
    me.ctx = me.sceneVisualization.getCanvasContext();
    me.highlightColor = me.HIGHLIGHT_COLOR;
    me.showCorrectColor = me.SHOW_CORRECT_COLOR;
    me.showIncorrectColor = me.SHOW_INCORRECT_COLOR;
    me.highlightAlpha = me.HIGHLIGHT_ALPHA;
    me.showCorrectAlpha = me.SHOW_CORRECT_ALPHA;
    me.showIncorrectAlpha = me.SHOW_INCORRECT_ALPHA;
}
extend(ModelElementVisualization, NonDOMEventDispatcher);

ModelElementVisualization.prototype.HIGHLIGHT_COLOR = 0xFF00FF;
ModelElementVisualization.prototype.SHOW_CORRECT_COLOR = 0x009900;
ModelElementVisualization.prototype.SHOW_INCORRECT_COLOR = 0xF0F00000;
ModelElementVisualization.prototype.HIGHLIGHT_ALPHA = 70;
ModelElementVisualization.prototype.SHOW_CORRECT_ALPHA = 70;
ModelElementVisualization.prototype.SHOW_INCORRECT_ALPHA = 70;
ModelElementVisualization.prototype.SELECT_COLOR = 0x2000A2;

ModelElementVisualization.prototype.getElement = function(){
    return this.element;
};

//
ModelElementVisualization.prototype.setCanvasContext = function(_ctx){
    var me = this;
    if(_ctx != me.ctx){
        me.ctx = _ctx;
        if(me.sceneVisualization.getCanvasToSceneSizeRatioZoom() != 0)
            me.update();
    }
};

//
ModelElementVisualization.prototype.getHighlightColor = function(){
    var me = this;
    var element = me.getElement();
    return element.isShowCorrect() ? me.showCorrectColor :
        (element.isShowIncorrect() ? me.showIncorrectColor : me.highlightColor);
};

//
ModelElementVisualization.prototype.getHighlightAlpha = function(){
    var me = this;
    var element = me.getElement();
    return element.isShowCorrect() ? me.showCorrectAlpha :
        (element.isShowIncorrect() ? me.showIncorrectAlpha : me.highlightAlpha);
};

//
ModelElementVisualization.prototype.update = function(){
};

ModelElementVisualization.prototype.deactivate = function(){
    ModelElementVisualization.superclass.deactivate.call(this);
};
/**
 * @constructor
 */
function ModelElementController(element, visualization, mapSceneController){
    ModelElementController.superclass.constructor.call(this);
    var me = this;
    me.ownToolId = null;
    me.element = element;
    me.visualization = visualization;
    me.mapSceneController = mapSceneController;
    //
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_DOWN, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_UP, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_MOVE, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_DRAG, {listener:me.mouseOnElementListener, target:me}, false);
    //
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.SET_TOOL, {listener:me.fSetToolListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.START_TOOL, {listener:me.fStartToolListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.TRACK_TOOL, {listener:me.fTrackToolListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.USE_TOOL, {listener:me.fUseToolListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.END_TOOL, {listener:me.fEndToolListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.CANCEL_TOOL, {listener:me.fCancelToolListener, target:me}, false);
}
extend(ModelElementController, NonDOMEventDispatcher);

ModelElementController.prototype.flagToolStarted = false;

//
ModelElementController.prototype.getElement = function(){
    return this.element;
};

//
ModelElementController.prototype.isOwnToolOn = function(){
    return this.mapSceneController.getCurrentToolId() == this.ownToolId;
};

//
ModelElementController.prototype.isSelectToolOn = function(){
    return this.mapSceneController.getCurrentToolId() == ModelTool.prototype.TOOL_SELECT;
};

//
ModelElementController.prototype.mouseOnElementListener = function(e) {
};

//
ModelElementController.prototype.fSetToolListener = function(e){
    var me = this;
    me.flagToolStarted = false;
    me.element.setStructureEditable(e.data.id == me.ownToolId && me.element.isUserCreated());
};

//
ModelElementController.prototype.fStartToolListener = function(e){
    this.flagToolStarted = true;
};

//
ModelElementController.prototype.fTrackToolListener = function(e){
    var me = this;
    if(e.data.id == me.ownToolId){
        me.mapSceneController.updateVisualization();
    }
};

//
ModelElementController.prototype.fUseToolListener = function(e){
    //
};

//
ModelElementController.prototype.fEndToolListener = function(e){
    var me = this;
    me.flagToolStarted = false;
    if(e.data.id == me.ownToolId){
        me.mapSceneController.updateVisualization();
    }
};

//
ModelElementController.prototype.fCancelToolListener = function(e){
    var me = this;
    me.flagToolStarted = false;
    if(e.data.id == me.ownToolId){
        me.mapSceneController.updateVisualization();
    }
};

//
ModelElementController.prototype.isToolStarted = function(){
    return this.flagToolStarted;
};

//
ModelElementController.prototype.deactivate = function(){
    ModelElementController.superclass.deactivate.call(this);
    var me = this;
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.SET_TOOL, {listener:me.fSetToolListener, target:me}, false);
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.START_TOOL, {listener:me.fStartToolListener, target:me}, false);
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.TRACK_TOOL, {listener:me.fTrackToolListener, target:me}, false);
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.USE_TOOL, {listener:me.fUseToolListener, target:me}, false);
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.END_TOOL, {listener:me.fEndToolListener, target:me}, false);
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.CANCEL_TOOL, {listener:me.fCancelToolListener, target:me}, false);
    //
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MOUSE_DOWN, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MOUSE_UP, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MOUSE_DRAG, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MOUSE_MOVE, {listener:me.mouseOnElementListener, target:me}, false);
};
/**
 * @constructor
 */
function ModelTool(modelToolsController){
    //abstract class for tools intended to work with model elements
    var me = this;
    //a command tool only acts on the selected elements or determines default setting for some parameter (e.g. fontSize)
    me.flagCommand = false;
    me.flagStarted = false;
    me.flagEnabled = true;
    me.flagMapSceneModelDependent = false;
    //if true, color palette-related parameters are set even if the tool is inactive
    me.flagConfigureByPaletteAlways = false;
    me.modelToolsController = modelToolsController;
    me.objConfiguration = {};
    //
    me.modelElement = null;
    //
    me.modelToolsController.addEventListener(
        ModelEvent.prototype.PALETTE_COLOR_SET, {target:me, listener:me.fSetPaletteColorListener}, false
    );
    //
    me.modelToolsController.addEventListener(
        ModelEvent.prototype.SYMBOL_SELECTED, {target:me, listener:me.fSymbolSelected}, false
    );
}
extend(ModelTool, Object);

ModelTool.prototype.TOOL_SELECT         = "model_tool_select";
ModelTool.prototype.TOOL_POINT          = "model_tool_point";
ModelTool.prototype.TOOL_ARROW          = "model_tool_arrow";
ModelTool.prototype.TOOL_POLYLINE       = "model_tool_polyline";
ModelTool.prototype.TOOL_RECTANGLE      = "model_tool_rectangle";
ModelTool.prototype.TOOL_ELLIPSE        = "model_tool_ellipse";
ModelTool.prototype.TOOL_DIAGRAM        = "model_tool_diagram";
ModelTool.prototype.TOOL_CHECKBOX_SET   = "model_tool_checkbox_set";
ModelTool.prototype.TOOL_TEXT           = "model_tool_text";
ModelTool.prototype.TOOL_BOUND_TEXT     = "model_tool_bound_text";

ModelTool.prototype.TOOL_LINE_COLOR     = "model_tool_line_color";
ModelTool.prototype.TOOL_FILL_COLOR     = "model_tool_fill_color";

ModelTool.prototype.TOOL_REMOVE          = "model_tool_remove";
ModelTool.prototype.TOOL_PAN             = "model_tool_pan";
ModelTool.prototype.TOOL_HELP            = "model_tool_help";

ModelTool.prototype.TOOL_FONT_FACE       = "model_tool_font_face";
ModelTool.prototype.TOOL_FONT_SIZE       = "model_tool_font_size";
ModelTool.prototype.TOOL_TEXT_ALIGN      = "model_tool_text_align";
ModelTool.prototype.TOOL_ZOOM            = "model_tool_zoom";

ModelTool.prototype.TOOL_RESET           = "model_tool_reset";
ModelTool.prototype.TOOL_SHOW_HIDDEN     = "model_tool_show_hidden";
ModelTool.prototype.TOOL_CONFIRM         = "model_tool_confirm";
ModelTool.prototype.TOOL_SHOW_HINT       = "model_tool_show_hint";
ModelTool.prototype.TOOL_SHOW_CORRECT    = "model_tool_show_correct";

ModelTool.prototype.id = null;

//reuse: same functions as for ModelElement, different arrays
ModelTool.prototype.aMethodNameForProperty = [];
ModelTool.prototype.setMethodNameForProperty = ModelElement.prototype.setMethodNameForProperty;
ModelTool.prototype.getMethodNameForProperty = ModelElement.prototype.getMethodNameForProperty;

//
ModelTool.prototype.getId = function(){
    return this.id;
};

//
ModelTool.prototype.getModelToolsController = function(){
    return this.modelToolsController;
};

//
ModelTool.prototype.getConfiguration = function(){
    return this.objConfiguration;
};

//
ModelTool.prototype.reset = function(){
};

//
ModelTool.prototype.configure = function(objToolData){
    //for configuring the tool according to the data stored in the model
};

//
ModelTool.prototype.configureByModelEventData = function(eventData){
    //to configure tool by the interface-generated data, e.g. switch a droplist-based tool
    //or set  color
};

//
//e.g. FontFaceTool "picks" the font style from the selected text object
ModelTool.prototype.configureByMapSceneModel = function(mapSceneModel, options){
};

//
//e.g. remove tool depends on selection
ModelTool.prototype.isMapSceneModelDependent = function(){
    return this.flagMapSceneModelDependent;
};

//
ModelTool.prototype.isCommand = function(){
    return this.flagCommand;
};

//
ModelTool.prototype.isEnabled = function(){
    return this.flagEnabled;
};

//
ModelTool.prototype.setEnabled = function(flag){
    this.flagEnabled = flag;
};

//
ModelTool.prototype.getModelElement = function(){
    return this.modelElement;
};

//
ModelTool.prototype.start = function(p, aElement, options){
    this.flagStarted = true;
};

//for commands
ModelTool.prototype.applyCommand = function(mapSceneModel, options){
};

//
ModelTool.prototype.use = function(p, aElement, options){
};

//
ModelTool.prototype.useDoubleClick = function(p, aElement, options){
};

//
ModelTool.prototype.useHoldDown = function(p, aElement, options){
};

//
ModelTool.prototype.useMouseUp = function(p, aElement, options){
    //by default, the current tool ends if there was dragging
    var me = this;
    if(me.modelToolsController.isDragged())
        me.getModelToolsController().endCurrentTool(p, aElement, options);
};

//
ModelTool.prototype.track = function(p, aElement, options){
};

//
ModelTool.prototype.end = function(p, aElement, options){
    this.flagStarted = false;
    this.modelElement = null;
};

//
ModelTool.prototype.cancel = function(p, aElement, options){
    this.flagStarted = false;
};

//
ModelTool.prototype.setDetails = function(objDetails){
};

//
ModelTool.prototype.isStarted = function(){
    return this.flagStarted;
};


//e.g. pan tool cannot be set if the scene scale is 100%
//command tools (delete) cannot be set if there is no selected elements
ModelTool.prototype.canBeSet = function(mapSceneModel, options){
    return true;
};

//
ModelTool.prototype.canBeStarted = function(p, aElement, options){
    return true;
};

//
ModelTool.prototype.canBeUsed = function(p, aElement, options){
    return true;
};

//
ModelTool.prototype.canHighlight = function(p, aElement, options){
    return false;
};

//
ModelTool.prototype.canSelect = function(p, aElement, options){
    return false;
};

//
ModelTool.prototype.fSetPaletteColorListener = function(e){
    var me = this;
    if(me.flagConfigureByPaletteAlways || me.getModelToolsController().getCurrentToolId() == me.getId()){
        var data = e.data;
        var sMethodAlpha = me.getMethodNameForProperty(data.propertyAlpha, true);
        var sMethodColor = me.getMethodNameForProperty(data.propertyColor, true);
        if(sMethodColor != null && sMethodAlpha != null){
            me[sMethodColor](data.color);
            me[sMethodAlpha](data.alpha);
            me.modelToolsController.notifyOfToolConfiguration(me.getId());
        }

    }
};

//
ModelTool.prototype.fSymbolSelected = function(e){
    var me = this;
    var data = e.data;
    var sMethod = me.getMethodNameForProperty(data.property, true);
    if(sMethod != null){
        me[sMethod](data.value);
        me.modelToolsController.notifyOfToolConfiguration(me.getId());
    }
};

//
ModelTool.prototype.deactivate = function(){
    var me = this;
    me.modelToolsController.removeEventListener(
        ModelEvent.prototype.PALETTE_COLOR_SET, {target:me, listener:me.fSetPaletteColorListener}, false
    );
    me.modelToolsController.removeEventListener(
        ModelEvent.prototype.SYMBOL_SELECTED, {target:me, listener:me.fSymbolSelected}, false
    )
};
/**
 * @constructor
 */
function SymbolSetModelTool(modelToolsController){
    SymbolSetModelTool.superclass.constructor.call(this, modelToolsController);
}
extend(SymbolSetModelTool, ModelTool);

//override
SymbolSetModelTool.prototype.configure = function(objToolData) {
    var me = this;
    var objConfiguration = me.objConfiguration;
    objConfiguration.iconAlpha = parseFloat(objToolData.iconAlpha);
    objConfiguration.iconColor = parseInt(objToolData.iconColor);
    objConfiguration.symbolId = objToolData.iconObjId;
    objConfiguration.flagSeparateButtons = (""+objToolData.separateButtons).toLowerCase() == "true";
    objConfiguration.aSymbol = (objToolData.symbolSet.length>0) ? objToolData.symbolSet.split(",") : [];
};

//override
SymbolSetModelTool.prototype.configureByModelEventData = function(eventData){
    var me = this;
    var objConfiguration = me.objConfiguration;
    if(objConfiguration == null || typeof eventData.subType == TYPE_UNDEFINED)
        return;
    switch(eventData.subType){
        case ModelEvent.prototype.SYMBOL_SELECTED:
            objConfiguration.symbolId = eventData.symbolId;
            break;
        default:
    }
};

//override
SymbolSetModelTool.prototype.setDetails = function(objDetails){
    var me = this;
    var objConfiguration = me.objConfiguration;
    if(objConfiguration == null)
        return;

    if(typeof objDetails.buttonNumber == TYPE_NUMBER)
        objConfiguration.symbolId = objConfiguration.aSymbol[objDetails.buttonNumber];
};

/**
 * @constructor
 */
function ColorModelTool(modelToolsController){
    ColorModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.flagCommand = true;
    me.flagStarted = true;
    me.flagMapSceneModelDependent = true;
    //
    me.sCanBeColoredMethod = null;
    me.sColorProperty = null;
    me.sAlphaProperty = null;
    //
    me.sColorPropertyCtrl = null;
    me.sAlphaPropertyCtrl = null;
    //
    me.modelToolsController.addEventListener(
        ModelEvent.prototype.SET_TOOL, {target:me, listener:me.fSetToolListener}, false
    );
    me.modelToolsController.addEventListener(
        ModelEvent.prototype.TOGGLE_DROPLIST, {target:me, listener:me.fToggleDroplistListener}, false
    );
}
extend(ColorModelTool, ModelTool);

ColorModelTool.prototype.aPaletteColor =  [
    {color:0x000000, name:"color-name.black"},
    {color:0x666666, name:"color-name.dark-gray"},
    {color:0x999999, name:"color-name.gray"},
    {color:0xCCCCCC, name:"color-name.light-gray"},
    {color:0xFFFFFF, name:"color-name.white"},
    {color:0xFF0000, name:"color-name.red"},
    {color:0xFF9933, name:"color-name.orange"},
    {color:0x009900, name:"color-name.green"},
    {color:0x0000CC, name:"color-name.blue"},
    {color:0xCC3399, name:"color-name.purple"},
    {color:0xFFCCCC, name:"color-name.pink"},
    {color:0xFFFF99, name:"color-name.yellow"},
    {color:0x99FF99, name:"color-name.light-green"},
    {color:0x99CCFF, name:"color-name.light-blue"},
    {color:0xCC99FF, name:"color-name.light-violet"}
];

/*
 aColorNames = ["Чёрный", "Тёмно-серый", "Серый", "Светло-серый", "Белый", "Красный", "Оранжевый", "Зелёный", "Синий", "Фиолетовый", "Розовый", "Жёлтый", "Салатовый", "Голубой", "Сиреневый"];
*/

ColorModelTool.prototype.configure = function(objToolData) {
    var me = this;
    var objConfiguration = me.objConfiguration;
    var aPaletteColor = me.aPaletteColor;
    objConfiguration.aRGB = [];
    objConfiguration.aSymbol = [];
    var aRGB = objConfiguration.aRGB;
    var aSymbol = objConfiguration.aSymbol;
    var symbolSet = (objToolData.symbolSet.length>0) ? objToolData.symbolSet.split(",") : [];
    var i, j, n = symbolSet.length, s;
    var lPrefix = me.colorSymbolPrefix.length;
    for(i=0; i<n; i++){
        s = symbolSet[i];
        j = s.indexOf(me.colorSymbolPrefix);
        if(j==0){
            aRGB.push(aPaletteColor[parseInt(s.slice(lPrefix))].color);
            aSymbol.push(me.symbolId);
        }
    }
};

//
ColorModelTool.prototype.getColor = function(){
    return this.objConfiguration.color;
};

//
ColorModelTool.prototype.setColor = function(v){
    this.objConfiguration.color = v;
};

//
ColorModelTool.prototype.getAlpha = function(){
    return this.objConfiguration.alpha;
};

//
ColorModelTool.prototype.setAlpha = function(v){
    this.objConfiguration.alpha = v;
};

//override
ColorModelTool.prototype.canBeSet = function(mapSceneModel, options){
    var me = this;
    var i, n = mapSceneModel.getElementsCount();
    var modelElement, fMethod;
    var flagCanBeColored, flag = false;
    for(i=0; i<n; i++) {
        modelElement = mapSceneModel.getDepthSortedElementNo(i);
        if(modelElement.isSelected() && modelElement.isComplete()){
            if(modelElement.isColorModifiable() && typeof modelElement[me.sCanBeColoredMethod] == TYPE_FUNCTION)
                fMethod = modelElement[me.sCanBeColoredMethod];
            else
                return false;
            flagCanBeColored = fMethod.call(modelElement);
            if (!flag && flagCanBeColored)
                flag = true;
            else if(!flagCanBeColored)
                return false;
        }
    }
    return flag;
};

//override
ColorModelTool.prototype.applyCommand = function(mapSceneModel, options){
    var me = this;
    var objConfiguration = me.objConfiguration;
    var i, n = mapSceneModel.getElementsCount(), cnt = 0;
    var modelElement, sMethodColor, sMethodAlpha;
    var color, alpha;
    var sColorProperty, sAlphaProperty;
    if((typeof options != TYPE_UNDEFINED) && (options != null) && (typeof options.ctrlKey == TYPE_BOOLEAN) && options.ctrlKey){
        sColorProperty = me.sColorPropertyCtrl;
        sAlphaProperty = me.sAlphaPropertyCtrl;
    } else{
        sColorProperty = me.sColorProperty;
        sAlphaProperty = me.sAlphaProperty;
    }
    for(i=0; i<n; i++) {
        modelElement = mapSceneModel.getDepthSortedElementNo(i);
        if(
            modelElement.isSelected() && modelElement.isColorModifiable() &&
            typeof modelElement[me.sCanBeColoredMethod] == TYPE_FUNCTION && modelElement[me.sCanBeColoredMethod]()
        ){
            sMethodColor = modelElement.getMethodNameForProperty(sColorProperty, true);
            sMethodAlpha = modelElement.getMethodNameForProperty(sAlphaProperty, true);
            color = objConfiguration.flagSeparateButtons ? objConfiguration.aRGB[options.buttonNumber] : objConfiguration.color;
            alpha = objConfiguration.alpha;
            if(sMethodColor != null && sMethodAlpha != null) {
                modelElement[sMethodColor](color);
                modelElement[sMethodAlpha](alpha);
                cnt++;
            }
        }
    }
    return cnt>0;
};

//
ColorModelTool.prototype.fSetToolListener = function(e){
    var me = this;
    var data = e.data;
    var theTool = me.modelToolsController.getToolById(data.id);
    if(theTool == me)
        return;
    //the color tool parametres might be determined by those of the just-chosen tool, e.g., by PointModelTool iconAlpha and iconColor
    var sMethodAlpha = theTool.getMethodNameForProperty(me.sAlphaProperty, false);
    var sMethodColor = theTool.getMethodNameForProperty(me.sColorProperty, false);
    //in case of separate buttons color does not change
    if(!me.objConfiguration.flagSeparateButtons && sMethodColor != null && sMethodAlpha != null){
        me.setColor(theTool[sMethodColor]());
        me.setAlpha(theTool[sMethodAlpha]());
        me.modelToolsController.notifyOfToolConfiguration(me.id);
    }
};

//
ColorModelTool.prototype.fToggleDroplistListener = function(e){
    var me = this;
    if(!(e.data.id == me.id && e.data.open))
        return;
    var mapSceneModel = me.modelToolsController.getMapSceneModel();
    var commonColor = -1;
    var commonAlpha = -1;
    var c, a;
    var i, n = mapSceneModel.getElementsCount();
    var modelElement, sMethodColor, sMethodAlpha;
    //check if all the selected elements with modifiable color have the same color and alpha
    for(i=0; i<n; i++) {
        modelElement = mapSceneModel.getDepthSortedElementNo(i);
        if(
            modelElement.isSelected() && modelElement.isColorModifiable() &&
            typeof modelElement[me.sCanBeColoredMethod] == TYPE_FUNCTION && modelElement[me.sCanBeColoredMethod]()
        ){
            sMethodColor = modelElement.getMethodNameForProperty(me.sColorProperty, false);
            sMethodAlpha = modelElement.getMethodNameForProperty(me.sAlphaProperty, false);
            if(sMethodColor != null && sMethodAlpha != null) {
                c = modelElement[sMethodColor]();
                if(commonColor == -1)
                    commonColor = c;
                else if(commonColor != c){
                    commonColor = -1;
                    break;
                }
                a = modelElement[sMethodAlpha]();
                if(commonAlpha == -1)
                    commonAlpha = a;
                else if(commonAlpha != a){
                    commonAlpha = a;
                    break;
                }
            }
        }
    }
    if(commonColor != -1 && commonAlpha != -1){
        //all have the same color and alpha
        me.setColor(commonColor);
        me.setAlpha(commonAlpha);
        me.modelToolsController.notifyOfToolConfiguration(me.id);
    }
};

//override
ColorModelTool.prototype.configureByModelEventData = function(eventData){
    var me = this;
    var objConfiguration = me.objConfiguration;
    if(objConfiguration == null || typeof eventData.subType == TYPE_UNDEFINED)
        return;

    switch(eventData.subType){
        case ModelEvent.prototype.PALETTE_COLOR_SET:
            if(eventData.propertyColor == me.sColorProperty && eventData.propertyAlpha == me.sAlphaProperty){
                me.setColor(eventData.color);
                me.setAlpha(eventData.alpha);
                me.modelToolsController.notifyOfToolConfiguration(me.id);
            }
            break;
        default:
    }
};

//override
ColorModelTool.prototype.deactivate = function(){
    ColorModelTool.superclass.deactivate.call(this);
    var me = this;
    me.modelToolsController.removeEventListener(
        ModelEvent.prototype.SET_TOOL, {target:me, listener:me.fSetToolListener}, false
    );
    me.modelToolsController.removeEventListener(
        ModelEvent.prototype.TOGGLE_DROPLIST, {target:me, listener:me.fToggleDroplistListener}, false
    );
};
/**
 * @constructor
 */
function ListedCommandModelTool(modelToolsController){
    ListedCommandModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.flagCommand = true;
    me.flagStarted = true;
    me.flagMapSceneModelDependent = true;
    //
    me.sProperty = null;
    //e.g. "fontSize" is stored as "textSize"
    me.aPropertyAlias = [];
    me.sIsModifiableMethod = null;
    //if symbolId is not in aElement
    me.defaultElement = null;
    me.fontForSelected = null;
}
extend(ListedCommandModelTool, ModelTool);

ListedCommandModelTool.prototype.DEFAULT_PROPERTY_VALUE = null;

ListedCommandModelTool.prototype.aElement = [];

ListedCommandModelTool.prototype.getListedPropertyName = function(){
    return this.sProperty;
};

ListedCommandModelTool.prototype.getElementsCount = function(){
    return this.aElement.length;
};

//override
ListedCommandModelTool.prototype.configure = function(objToolData){
    //for configuring the tool according to the data stored in the model
    var me = this;
    var sProperty = me.sProperty, sValue;
    var objConfiguration = me.objConfiguration;
    var j;
    var aElement = me.aElement, n = aElement.length;

    if(objToolData == null){
        sValue = me.DEFAULT_PROPERTY_VALUE
    }else if(typeof objToolData[sProperty] != TYPE_STRING){
        var aAlias = me.aPropertyAlias, l = aAlias.length;
        for(j=0; j<l; j++)
            if(typeof objToolData[aAlias[j]] == TYPE_STRING)
                break;
        sValue = (j<l) ? objToolData[aAlias[j]] : me.DEFAULT_PROPERTY_VALUE;
    }else
        sValue = objToolData[sProperty];

    objConfiguration.aElement = aElement;
    objConfiguration.defaultElement = me.defaultElement;
    objConfiguration.aSymbol = [];
    objConfiguration.symbolId = ""+me.getElementNumberByPropertyValue(sValue);
    for(j=0; j<n; j++)
        objConfiguration.aSymbol.push(""+j);
    objConfiguration.fontForSelected = me.fontForSelected;
};

//override
ListedCommandModelTool.prototype.configureByModelEventData = function(eventData){
    var me = this;
    var objConfiguration = me.objConfiguration;
    if(objConfiguration == null || typeof eventData.subType == TYPE_UNDEFINED)
        return;

    switch(eventData.subType){
        case ModelEvent.prototype.SYMBOL_SELECTED:
            if(objConfiguration.symbolId != eventData.symbolId){
                objConfiguration.symbolId = eventData.symbolId;
                me.modelToolsController.notifyOfToolConfiguration(me.id);
            }
            break;
        default:
    }
};

//
ListedCommandModelTool.prototype.getElementNumberByPropertyValue = function(sValue){
    var me = this;
    var sProperty = me.sProperty;
    var q = parseInt(sValue);
    var i, aElement = me.aElement, n = aElement.length, element;
    for(i=0; i<n; i++){
        element = aElement[i];
        if(element[sProperty] == q)
            return i;
    }
    return -1;
};

//
ListedCommandModelTool.prototype.getPropertyValueByElementNumber = function(q){
    var me = this;
    var sProperty = me.sProperty;
    var aElement = me.aElement;
    if(q>=0 && q<aElement.length)
        return ""+aElement[q][sProperty];
    return null;
};

//override
ListedCommandModelTool.prototype.configureByMapSceneModel = function(mapSceneModel, options){
    var me = this;
    var aSelected = mapSceneModel.getScaleFrame().getSelected(), modelElement;
    var sMethod;
    if(aSelected.length>0){
        modelElement = aSelected[aSelected.length-1];
        sMethod = modelElement.getMethodNameForProperty(me.sProperty, false);
        if(sMethod != null) {
            me.objConfiguration.symbolId = "" + me.getElementNumberByPropertyValue(modelElement[sMethod]());
            me.modelToolsController.notifyOfToolConfiguration(me.id);
        }
    }
};

//override
ListedCommandModelTool.prototype.applyCommand = function(mapSceneModel, options){
    var me = this;
    var objConfiguration = me.objConfiguration;
    var i, n = mapSceneModel.getElementsCount(), cnt = 0;
    var modelElement, sMethod;

    for(i=0; i<n; i++) {
        modelElement = mapSceneModel.getDepthSortedElementNo(i);
        if(modelElement.isSelected()) {
            sMethod = modelElement.getMethodNameForProperty(me.sProperty, true);
            if(
                sMethod != null &&
                typeof modelElement[me.sIsModifiableMethod] == TYPE_FUNCTION && modelElement[me.sIsModifiableMethod]()
            ){
                modelElement[sMethod](me.getPropertyValueByElementNumber(parseInt(objConfiguration.symbolId)));
                cnt++;
            }
        }
    }
    return cnt>0;
};


/**
 * @constructor
 */
function ModelToolsController(mapSceneModel, mapSceneController, modelResource){
    //tools selector class
    ModelToolsController.superclass.constructor.call(this);
    var me = this;
    me.mapSceneModel = mapSceneModel;
    me.mapSceneController = mapSceneController;
    me.currentToolId = null;
    me.currentTool = null;
    me.lastStartPoint = null;
    me.flagDragged = false;
    me.flagMouseUpBeforeDrag = false;
    me.aId = [];
    me.aClassName = [];
    me.aTool = [];
    //create all the available tools
    var aToolId = ModelToolsController.prototype.aToolId;
    var i, n = aToolId.length;
    var cls, tool, objToolId;
    for(i=0; i<n; i++){
        objToolId = aToolId[i];
        if(
            !(typeof objToolId.always == TYPE_BOOLEAN && objToolId.always) &&
            (
                typeof objToolId.name != TYPE_STRING ||
                typeof modelResource.customizePlayerAttributes[objToolId.name] == TYPE_UNDEFINED ||
                modelResource.customizePlayerAttributes[objToolId.name].toLowerCase() != "true"
            )
        )
            continue;
        cls = window[objToolId.className];
        if(typeof cls != TYPE_UNDEFINED){
            tool = new cls(me);
            me.aId.push(objToolId.id);
            me.aClassName.push(objToolId.className);
            me.aTool.push(tool);
            if(tool.isMapSceneModelDependent())
                me.setToolEnabled(objToolId.id, tool.canBeSet(mapSceneModel, null));
            else
                me.setToolEnabled(objToolId.id, true);
        }
    }
    //by default, selection tool is chosen
    me.setCurrentToolId(ModelTool.prototype.TOOL_SELECT);
    //
    me.mapSceneModel.addEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:me, listener:me.fMapSceneListener}, false
    );
    //
    me.mapSceneModel.addEventListener(
        ModelEvent.prototype.SCROLL_TOGGLE, {target:me, listener:me.fMapSceneListener}, false
    );
    //
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MODEL_ELEMENT_SELECTED, {target:me, listener:me.fMapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MODEL_ELEMENT_DESELECTED, {target:me, listener:me.fMapSceneControllerListener}, false
    );
    //
    me.aToolEventSource = [];
}
extend(ModelToolsController, NonDOMEventDispatcher);

ModelToolsController.prototype.minimalDragRadius = 10;
ModelToolsController.prototype.minimalDragRadiusSquared =
    ModelToolsController.prototype.minimalDragRadius * ModelToolsController.prototype.minimalDragRadius;

//"name" is the tag name in the model xml, "always" is the tool unconditional presence flag
ModelToolsController.prototype.aToolId = [
    //top toolbar
    {id:ModelTool.prototype.TOOL_SELECT,        className:"SelectModelTool",        name:"Select",      always:true},
    {id:ModelTool.prototype.TOOL_PAN,           className:"PanModelTool",           name:"Pan"},
    {id:ModelTool.prototype.TOOL_REMOVE,        className:"RemoveModelTool",        name:"Delete",      always:true},
    {id:ModelTool.prototype.TOOL_LINE_COLOR,    className:"LineColorModelTool",     name:"LineColBtn"},
    {id:ModelTool.prototype.TOOL_FILL_COLOR,    className:"FillColorModelTool",     name:"FillCol"},
    {id:ModelTool.prototype.TOOL_POINT,         className:"PointModelTool",         name:"CreatePoint"},
    {id:ModelTool.prototype.TOOL_ARROW,         className:"ArrowModelTool",         name:"CreateArrow"},
    {id:ModelTool.prototype.TOOL_POLYLINE,      className:"PolylineModelTool",      name:"CreateLine"},
    {id:ModelTool.prototype.TOOL_RECTANGLE,     className:"RectangleModelTool",     name:"CreateRectangle"},
    {id:ModelTool.prototype.TOOL_CHECKBOX_SET,  className:"CheckboxSetModelTool",   name:null},
    {id:ModelTool.prototype.TOOL_TEXT,          className:"TextModelTool",          name:"CreateText"},
    {id:ModelTool.prototype.TOOL_BOUND_TEXT,    className:"BtextModelTool",         name:"CreateBoundText"},
    {id:ModelTool.prototype.TOOL_DIAGRAM,       className:"DiagramModelTool",       name:"CreateDiagram"},
    {id:ModelTool.prototype.TOOL_FONT_SIZE,     className:"FontSizeModelTool",      name:"TextSizePanel"},
    {id:ModelTool.prototype.TOOL_FONT_FACE,     className:"FontFaceModelTool",      name:"FontFacePanel"},
    {id:ModelTool.prototype.TOOL_TEXT_ALIGN,    className:"TextAlignModelTool",     name:"TextAlign"},
    {id:ModelTool.prototype.TOOL_ZOOM,          className:"ZoomModelTool",          name:"ZoomList"},
    //right toolbar
    {id:ModelTool.prototype.TOOL_RESET,         className:"ResetModelTool",         name:null,          always:true},
    {id:ModelTool.prototype.TOOL_SHOW_HIDDEN,   className:"ShowHiddenModelTool",    name:null,          always:true},
    {id:ModelTool.prototype.TOOL_CONFIRM,       className:"ConfirmModelTool",       name:null,          always:true},
    {id:ModelTool.prototype.TOOL_SHOW_CORRECT,  className:"ShowCorrectModelTool",   name:null,          always:true},
    {id:ModelTool.prototype.TOOL_SHOW_HINT,     className:"ShowHintModelTool",      name:null,          always:true}
];

//
ModelToolsController.prototype.getMapSceneModel = function(){
    return this.mapSceneModel;
};

//
ModelToolsController.prototype.getMapSceneController = function(){
    return this.mapSceneController;
};

//
ModelToolsController.prototype.addElementToModel = function(element){
    this.mapSceneController.addElementToModel(element);
    this.mapSceneModel.invalidate();
};

//returns true if the element was successfully removed, false otherwise
ModelToolsController.prototype.removeElementFromModel = function(element){
    if(this.mapSceneController.removeElementFromModel(element)) {
        this.mapSceneModel.invalidate();
        return true;
    }else
        return false;
};

//
ModelToolsController.prototype.getCount = function(){
    return this.aClassName.length;
};

//
ModelToolsController.prototype.getClassNameNo = function(q){
    return ((q>=0) && (q<this.aClassName.length)) ? this.aClassName[q] : null;
};

//
ModelToolsController.prototype.getIdByClassName = function(sClassName){
    var me = this;
    var a = me.aToolId, i, n = a.length;
    for(i=0; i<n; i++)
        if(a[i].className == sClassName)
            return a[i].id;
    return null;
};

//
ModelToolsController.prototype.configureTool = function(sId, objToolData){
    var me = this;
    var i = me.aId.indexOf(sId);
    if(i != -1) {
        var modelTool = me.aTool[i];
        modelTool.configure(objToolData);
        me.notifyOfToolConfiguration(sId);
    }
};

//
ModelToolsController.prototype.resetTool = function(sId){
    var me = this;
    var i = me.aId.indexOf(sId);
    if(i != -1)
        me.aTool[i].reset();
};

//
ModelToolsController.prototype.notifyOfToolConfiguration = function(sId){
    var me = this;
    var i = me.aId.indexOf(sId);
    if(i != -1) {
        var modelTool = me.aTool[i];
        var evt = new ModelEvent(ModelEvent.prototype.CONFIGURE_TOOL, true, true);
        if (evt != null) {
            evt.data = {id: me.aId[i], configuration:modelTool.getConfiguration()};
            me.dispatchEvent(evt);
        }
    }
};

//
ModelToolsController.prototype.getToolById = function(sId){
    var me = this;
    var i = me.aId.indexOf(sId);
    return (i!=-1) ? me.aTool[i] : null;
};

//
ModelToolsController.prototype.isToolEnabled = function(sId){
    var modelTool = this.getToolById(sId);
    return (modelTool != null) ? modelTool.isEnabled() : false;
};

//
ModelToolsController.prototype.setToolEnabled = function(sId, flag){
    var modelTool = this.getToolById(sId);
    if(modelTool != null && modelTool.isEnabled() != flag) {
        modelTool.setEnabled(flag);
        var evt  = new ModelEvent(
            flag ? ModelEvent.prototype.ENABLE_TOOL : ModelEvent.prototype.DISABLE_TOOL,
            true, true
        );
        if(evt != null){
            evt.data = {id:sId};
            this.dispatchEvent(evt);
        }
    }
};


//
ModelToolsController.prototype.getCurrentToolId = function(){
    return this.currentToolId;
};

//
ModelToolsController.prototype.setCurrentToolId = function(sId, objDetails){
    var me = this;
    var details = (typeof objDetails == TYPE_OBJECT) ? objDetails : null;
    var modelTool = me.getToolById(sId);
    if(!(modelTool instanceof ModelTool))
        return;
    var sEvent;
    if(modelTool.canBeSet(me.mapSceneModel, null)) {
        if (modelTool.isCommand()) {
            modelTool.applyCommand(me.mapSceneModel, details);
            me.updateToolStates();
            sEvent = ModelEvent.prototype.APPLY_TOOL;
        } else {
            if (me.currentTool instanceof ModelTool && me.currentTool.isStarted())
                me.currentTool.cancel();
            me.currentToolId = sId;
            me.currentTool = modelTool;
            modelTool.setDetails(details);
            sEvent = ModelEvent.prototype.SET_TOOL;
        }
        var evt = new ModelEvent(sEvent, true, true);
        if (evt != null) {
            evt.data = {id: sId, details: details};
            me.dispatchEvent(evt);
        }
    }
};

//
ModelToolsController.prototype.canSetTool = function(sId){
    var me = this;
    var modelTool = me.getToolById(sId);
    return (modelTool instanceof ModelTool) ? modelTool.canBeSet(me.mapSceneModel, null) : false;
};

//
ModelToolsController.prototype.getCurrentTool = function(){
    return this.currentTool;
};

//
ModelToolsController.prototype.isDragged = function(){
    return this.flagDragged;
};

//
ModelToolsController.prototype.isCurrentToolStarted = function(){
    return (this.currentTool instanceof ModelTool) ? this.currentTool.isStarted() : false;
};

//
ModelToolsController.prototype.startCurrentTool = function(p, aElement, options){
    var me = this;
    var currentTool = me.currentTool;
    if(currentTool != null && !currentTool.isStarted()) {
        currentTool.start(p, aElement, options);
        me.resetDragged(p);
        var evt  = new ModelEvent(ModelEvent.prototype.START_TOOL, true, true);
        if(evt != null){
            evt.data = {id:me.currentToolId};
            me.dispatchEvent(evt);
        }
    }

};

//
ModelToolsController.prototype.resetDragged = function(p){
    var me = this;
    me.lastStartPoint = (p != null) ? new Point(p.x, p.y) : null;
    me.flagDragged = false;
    me.flagMouseUpBeforeDrag = false;
};

//
ModelToolsController.prototype.useCurrentTool = function(p, aElement, options){
    var me = this;
    var currentTool = me.currentTool;
    if(currentTool != null && currentTool.isStarted()) {
        currentTool.use(p, aElement, options);
        var evt  = new ModelEvent(ModelEvent.prototype.USE_TOOL, true, true);
        if(evt != null){
            evt.data = {id:me.currentToolId};
            me.dispatchEvent(evt);
        }
    }
};

//
ModelToolsController.prototype.useCurrentToolDoubleClick = function(p, aElement, options){
    var me = this;
    var currentTool = me.currentTool;
    if(currentTool != null && currentTool.isStarted()) {
        currentTool.useDoubleClick(p, aElement, options);
        var evt  = new ModelEvent(ModelEvent.prototype.USE_TOOL, true, true);
        if(evt != null){
            evt.data = {id:me.currentToolId};
            me.dispatchEvent(evt);
        }
    }
};

//
ModelToolsController.prototype.useCurrentToolMouseUp = function(p, aElement, options){
    var me = this;
    var currentTool = me.currentTool;
    if(currentTool != null && currentTool.isStarted()) {
        if(!me.flagDragged && me.lastStartPoint!=null &&
            me.lastStartPoint.distanceSquared(p)<me.minimalDragRadiusSquared
        )
            me.flagMouseUpBeforeDrag = true;
        currentTool.useMouseUp(p, aElement, options);
        var evt  = new ModelEvent(ModelEvent.prototype.USE_TOOL, true, true);
        if(evt != null){
            evt.data = {id:me.currentToolId};
            me.dispatchEvent(evt);
        }
    }
};

//
ModelToolsController.prototype.useCurrentToolHoldDown =function(p, aElement, options){
    var me = this;
    var currentTool = me.currentTool;
    if(currentTool != null) {
        currentTool.useHoldDown(p, aElement, options);
        var evt  = new ModelEvent(ModelEvent.prototype.USE_TOOL, true, true);
        if(evt != null){
            evt.data = {id:me.currentToolId};
            me.dispatchEvent(evt);
        }
    }
};

//
ModelToolsController.prototype.trackCurrentTool = function(p, aElement, options){
    var me = this;
    var currentTool = me.currentTool;
    if(currentTool != null && currentTool.isStarted()) {
        if(!me.flagMouseUpBeforeDrag && me.lastStartPoint!=null &&
            me.lastStartPoint.distanceSquared(p)>=me.minimalDragRadiusSquared
        )
            me.flagDragged = true;
        currentTool.track(p, aElement, options);
        var evt  = new ModelEvent(ModelEvent.prototype.TRACK_TOOL, true, true);
        if(evt != null){
            evt.data = {id:me.currentToolId};
            me.dispatchEvent(evt);
        }
    }
};

//
ModelToolsController.prototype.endCurrentTool = function(p, aElement, options){
    var me = this;
    var currentTool = me.currentTool;
    if(currentTool != null && currentTool.isStarted()) {
        var modelElement = currentTool.getModelElement();
        currentTool.end(p, aElement, options);
        if(modelElement != null && modelElement.isSelected()){
            me.getMapSceneModel().getScaleFrame().addToSelected(modelElement);
        }
        var evt  = new ModelEvent(ModelEvent.prototype.END_TOOL, true, true);
        if(evt != null){
            evt.data = {id:me.currentToolId};
            me.dispatchEvent(evt);
        }
    }
    me.flagDragged = false;
    me.flagMouseUpBeforeDrag = false;
};

//
ModelToolsController.prototype.cancelCurrentTool = function(p, aElement, options){
    var me = this;
    var currentTool = me.currentTool;
    if(currentTool != null) {
        if(currentTool.isStarted()) {
            currentTool.cancel(p, aElement, options);
            me.mapSceneModel.invalidate();
            var evt = new ModelEvent(ModelEvent.prototype.CANCEL_TOOL, true, true);
            if (evt != null) {
                evt.data = {id: me.currentToolId};
                me.dispatchEvent(evt);
            }
        }
    }
    me.flagDragged = false;
    me.flagMouseUpBeforeDrag = false;
    me.setCurrentToolId(ModelTool.prototype.TOOL_SELECT);
};

//
ModelToolsController.prototype.currentToolCanBeStarted = function(p, aElement, options){
    return (this.currentTool != null) ? this.currentTool.canBeStarted(p, aElement, options) : false;
};

//
ModelToolsController.prototype.currentToolCanBeUsed = function(p, aElement, options){
    return (this.currentTool != null) ? this.currentTool.canBeUsed(p, aElement, options) : false;
};


//
ModelToolsController.prototype.updateToolStates = function(){
    var me = this;
    var mapSceneModel = me.mapSceneModel;
    var a = me.aTool, aId = me.aId, i, n = a.length;
    var modelTool;
    for(i=0; i<n; i++){
        modelTool = a[i];
        if(modelTool.isMapSceneModelDependent()) {
            me.setToolEnabled(aId[i], modelTool.canBeSet(mapSceneModel, null));
            modelTool.configureByMapSceneModel(mapSceneModel, null);
        }
    }
};

//
ModelToolsController.prototype.fMapSceneListener = function(e){
    var me = this;
    switch(e.type){
        case ModelEvent.prototype.RESET_SCENE:
            var evt = new ModelEvent(ModelEvent.prototype.RESET_SCENE, false, true);
            if(evt == null)
                return;
            evt.data = {time:new Date().getTime()};
            me.dispatchEvent(evt);
            break;
        case ModelEvent.prototype.SCROLL_TOGGLE:
            me.updateToolStates();
            break;
        default:
    }
};

//
ModelToolsController.prototype.fMapSceneControllerListener = function(e){
    var me = this;
    var mapSceneModel = me.mapSceneModel;
    var a = me.aTool, aId = me.aId, i, n = a.length;
    var modelTool;
    switch(e.type){
        case ModelEvent.prototype.MODEL_ELEMENT_SELECTED:
        case ModelEvent.prototype.MODEL_ELEMENT_DESELECTED:
            me.updateToolStates();
            break;
        default:
    }
};

//
ModelToolsController.prototype.fToolEventsListener = function(e){
    var me = this;
    var evt, data = e.data;
    var toolId;
    var modelTool;
    switch(e.type){
        case ModelEvent.prototype.TOOLBAR_BUTTON_EVENT:
            toolId = ToolbarButtonLibrary.prototype.getToolIdByName(data.buttonId);
            if(toolId != null && typeof data.subType == TYPE_STRING){
                modelTool = me.getToolById(toolId);
                switch(data.subType){
                    case ModelEvent.prototype.SYMBOL_SELECTED:
                        modelTool.configureByModelEventData(data);
                        if(toolId == me.currentToolId && typeof data.symbolId == TYPE_STRING){
                            //we will have to update mouse icon in case we switch sub-tools like "rectangle"/"ellipse"
                            me.setCurrentToolId(toolId, {symbolId:data.symbolId, buttonNumber: data.buttonNumber});
                        }
                        if(modelTool.isCommand())
                            modelTool.applyCommand(me.mapSceneModel,
                                {ctrlKey:(typeof e.data.ctrlKey == TYPE_BOOLEAN) ? e.data.ctrlKey : false, data:data}
                            );
                        if(modelTool instanceof ListedCommandModelTool){
                            evt = new ModelEvent(ModelEvent.prototype.SYMBOL_SELECTED, true, true);
                            if (evt != null) {
                                evt.data = {
                                    id: toolId,
                                    property: modelTool.getListedPropertyName(),
                                    value: modelTool.getPropertyValueByElementNumber(data.buttonNumber)
                                };
                            }
                            me.dispatchEvent(evt);
                        }
                        break;
                    case ModelEvent.prototype.TOGGLE_DROPLIST:
                        evt = new ModelEvent(ModelEvent.prototype.TOGGLE_DROPLIST, true, true);
                        if (evt != null) {
                            evt.data = {id: toolId, open: data.open};
                            me.dispatchEvent(evt);
                        }
                        break;
                    case ModelEvent.prototype.PALETTE_COLOR_SET:
                        modelTool.configureByModelEventData(data);
                        if(modelTool.isCommand())
                            modelTool.applyCommand(me.mapSceneModel,
                                {ctrlKey:(typeof e.data.ctrlKey == TYPE_BOOLEAN) ? e.data.ctrlKey : false}
                            );
                        evt = new ModelEvent(ModelEvent.prototype.PALETTE_COLOR_SET, true, true);
                        if (evt != null) {
                            evt.data = {
                                id: toolId,
                                propertyColor:data.propertyColor, propertyAlpha:data.propertyAlpha,
                                color: data.color, alpha:data.alpha
                            };
                            me.dispatchEvent(evt);
                        }
                        break;

                    case ModelEvent.prototype.USE_TOOL:
                        if(typeof data.toolId == TYPE_STRING) {
                            var modelToolToUse = me.getToolById(data.toolId);
                            var details = (typeof data.details == TYPE_OBJECT) ? data.details : null;
                            if(modelToolToUse != null){
                                if (modelToolToUse.isCommand()) {
                                    modelToolToUse.applyCommand(me.mapSceneModel, details);
                                    me.updateToolStates();
                                    evt = new ModelEvent(ModelEvent.prototype.APPLY_TOOL, true, true);
                                    if (evt != null) {
                                        evt.data = {id: data.toolId, details: details};
                                        me.dispatchEvent(evt);
                                    }
                                }
                            }
                        }
                        break;

                    default:
                }

            }
            break;
        default:
    }
};

//
ModelToolsController.prototype.addToolEventsSource = function(objDispatcher){
    if(!(objDispatcher instanceof NonDOMEventDispatcher))
        return;
    var me = this;
    var a = me.aToolEventSource;
    if(a.indexOf(objDispatcher) != -1)
        return;
    a.push(objDispatcher);
    objDispatcher.addEventListener(
        ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, {target:me, listener:me.fToolEventsListener}, false
    );
};

//
ModelToolsController.prototype.removeToolEventsSource = function(objDispatcher){
    if(!(objDispatcher instanceof NonDOMEventDispatcher))
        return;
    var me = this;
    var a = me.aToolEventSource, i = a.indexOf(objDispatcher);
    if(i == -1)
        return;
    objDispatcher.removeEventListener(
        ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, {target:me, listener:me.fToolEventsListener}, false
    );
    a.splice(i, 1);
};

//
ModelToolsController.prototype.deactivate = function(){
    ModelToolsController.superclass.deactivate.call(this);
    var me = this;
    me.mapSceneModel.removeEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:me, listener:me.fMapSceneListener}, false
    );
    me.mapSceneModel.removeEventListener(
        ModelEvent.prototype.SCROLL_TOGGLE, {target:me, listener:me.fMapSceneListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MODEL_ELEMENT_SELECTED, {target:me, listener:me.fMapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MODEL_ELEMENT_DESELECTED, {target:me, listener:me.fMapSceneControllerListener}, false
    );
    while(me.aToolEventSource.length>0)
        me.removeToolEventsSource(me.aToolEventSource[0]);
    var aTool = me.aTool;
    var i, n = aTool.length;
    for(i=0; i<n; i++)
        aTool[i].deactivate();
};
/**
 * @constructor
 */
function SelectModelTool(modelToolsController){
    SelectModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.flagStarted = true;
}
extend(SelectModelTool, ModelTool);

SelectModelTool.prototype.id = ModelTool.prototype.TOOL_SELECT;

//override
SelectModelTool.prototype.start = function(p, aElement, options){
};

//override
SelectModelTool.prototype.track = function(p, aElement, options){
};

//override
SelectModelTool.prototype.end = function(p, aElement, options){
};

//override
SelectModelTool.prototype.cancel = function(p, aElement, options){
};

//override
SelectModelTool.prototype.canHighlight = function(p, aElement, options){
    return true;
};

SelectModelTool.prototype.useHoldDown = function(p, aElement, options){
    var me = this;
    var element, sType, hitDetails;
    if(aElement.length>0){
        element = aElement[0];
        sType = element.getType();
        hitDetails = element.getHitDetails();
        if(
            (
                sType == ModelResource.prototype.TYPE_TEXT ||
                (sType == ModelResource.prototype.TYPE_BTEXT && hitDetails != null && hitDetails.flagTextField)
            )
            &&
            !element.isEditingMode() && element.isTextModifiable()
        ){
            element.setHitPoint(p);
            element.setEditingMode(true);
            element.invalidate();
        }
    }
};

//
SelectModelTool.prototype.canSelect = function(p, aElement, options){
    if(aElement!=null && aElement.length>0) {
        var element = aElement[0];
        return ((element instanceof ModelElement) && element.canBeSelected());
    }
    return false;
};
/**
 * @constructor
 */
function PanModelTool(modelToolsController){
    PanModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.flagStarted = true;
    me.flagMapSceneModelDependent = true;
}
extend(PanModelTool, ModelTool);

PanModelTool.prototype.id = ModelTool.prototype.TOOL_PAN;

//override
PanModelTool.prototype.canBeSet = function(mapSceneModel, options){
    return mapSceneModel.isNeedHorizontalScrollBar() || mapSceneModel.isNeedVerticalScrollBar();
};
/**
 * @constructor
 */
function RemoveModelTool(modelToolsController){
    RemoveModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.flagCommand = true;
    me.flagStarted = true;
    me.flagMapSceneModelDependent = true;
}
extend(RemoveModelTool, ModelTool);

RemoveModelTool.prototype.id = ModelTool.prototype.TOOL_REMOVE;

//override
RemoveModelTool.prototype.canBeSet = function(mapSceneModel, options){
    var i, n = mapSceneModel.getElementsCount();
    var modelElement;
    var flagRemovable, flag = false;
    for(i=0; i<n; i++) {
        modelElement = mapSceneModel.getDepthSortedElementNo(i);
        if(modelElement.isSelected() && modelElement.isComplete()){
            flagRemovable = modelElement.canBeRemoved();
            if (!flag && flagRemovable)
                flag = true;
            else if(!flagRemovable)
                return false;
        }
    }
    return flag;
};

//override
RemoveModelTool.prototype.applyCommand = function(mapSceneModel, options){
    var me = this;
    var modelToolsController = me.modelToolsController;
    var i, n = mapSceneModel.getElementsCount(), cnt = 0;
    var modelElement, flag;
    for(i=0; i<n; i++) {
        modelElement = mapSceneModel.getDepthSortedElementNo(i);
        if(modelElement.isSelected() && modelElement.canBeRemoved()){
            flag = modelToolsController.removeElementFromModel(modelElement);
            if(flag) {
                i--;
                n--;
                cnt++;
            }
        }
    }
    return cnt>0;
};
/**
 * @constructor
 */
function LineColorModelTool(modelToolsController) {
    FillColorModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.sCanBeColoredMethod = "canBeLineColored";
    me.sColorProperty = "lineColor";
    me.sAlphaProperty = "lineAlpha";
}
extend(LineColorModelTool, ColorModelTool);

LineColorModelTool.prototype.id = ModelTool.prototype.TOOL_LINE_COLOR;
LineColorModelTool.prototype.colorSymbolPrefix = "mcLineColor";
LineColorModelTool.prototype.symbolId = "mcLineColor";

//override
LineColorModelTool.prototype.configure = function(objToolData) {
    var me = this;
    var objConfiguration = me.objConfiguration;
    objConfiguration.flagSeparateButtons = (""+objToolData.separateButtons).toLowerCase() == "true";
    objConfiguration.lineAlpha = parseFloat(objToolData.lineAlpha);
    objConfiguration.alpha = parseFloat(objToolData.currentAlpha);
    if(isNaN(objConfiguration.alpha))
        objConfiguration.alpha = objConfiguration.lineAlpha
    objConfiguration.color = parseInt(objToolData.currentColor);
    LineColorModelTool.superclass.configure.call(this, objToolData);
};

/**
 * @constructor
 */
function FillColorModelTool(modelToolsController) {
    FillColorModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.sCanBeColoredMethod = "canBeFillColored";
    me.sColorProperty = "fillColor";
    me.sAlphaProperty = "fillAlpha";
    //
    me.sColorPropertyCtrl = "fillColorCtrl";
    me.sAlphaPropertyCtrl = "fillAlphaCtrl";
}
extend(FillColorModelTool, ColorModelTool);

FillColorModelTool.prototype.id = ModelTool.prototype.TOOL_FILL_COLOR;

FillColorModelTool.prototype.colorSymbolPrefix = "mcFillColor";
FillColorModelTool.prototype.symbolId = "mcFillColor";

//override
FillColorModelTool.prototype.configure = function(objToolData) {
    var me = this;
    var objConfiguration = me.objConfiguration;
    objConfiguration.flagSeparateButtons = (""+objToolData.separateButtons).toLowerCase() == "true";
    objConfiguration.fillAlpha = parseFloat(objToolData.fillAlpha);
    objConfiguration.alpha = objConfiguration.flagSeparateButtons ? objConfiguration.fillAlpha : parseFloat(objToolData.currentAlpha);
    if(isNaN(objConfiguration.alpha))
        objConfiguration.alpha = objConfiguration.fillAlpha;
    objConfiguration.color = parseInt(objToolData.currentColor);
    FillColorModelTool.superclass.configure.call(this, objToolData);
};
/**
 * @constructor
 */
function PolylineModelTool(modelToolsController){
    //creation of polylines
    PolylineModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    //
    me.nAddedPoints = 0;
    me.objConfiguration = {
        lineColor: 16711680,
        lineAlpha: 100,
        fillColor: 16750899,
        fillAlpha: 70
    };
}
extend(PolylineModelTool, ModelTool);

PolylineModelTool.prototype.id = ModelTool.prototype.TOOL_POLYLINE;

PolylineModelTool.prototype.aMethodNameForProperty =
    miscFunctions.cloneObject(PolylineModelTool.superclass.aMethodNameForProperty);
PolylineModelTool.prototype.setMethodNameForProperty(
    PolylineModelTool.prototype.aMethodNameForProperty, "fillColor", false, "getFillColor"
);
PolylineModelTool.prototype.setMethodNameForProperty(
    PolylineModelTool.prototype.aMethodNameForProperty, "fillColor", true, "setFillColor"
);
PolylineModelTool.prototype.setMethodNameForProperty(
    PolylineModelTool.prototype.aMethodNameForProperty, "fillAlpha", false, "getFillAlpha"
);
PolylineModelTool.prototype.setMethodNameForProperty(
    PolylineModelTool.prototype.aMethodNameForProperty, "fillAlpha", true, "setFillAlpha"
);
PolylineModelTool.prototype.setMethodNameForProperty(
    PolylineModelTool.prototype.aMethodNameForProperty, "lineColor", false, "getLineColor"
);
PolylineModelTool.prototype.setMethodNameForProperty(
    PolylineModelTool.prototype.aMethodNameForProperty, "lineColor", true, "setLineColor"
);
PolylineModelTool.prototype.setMethodNameForProperty(
    PolylineModelTool.prototype.aMethodNameForProperty, "lineAlpha", false, "getLineAlpha"
);
PolylineModelTool.prototype.setMethodNameForProperty(
    PolylineModelTool.prototype.aMethodNameForProperty, "lineAlpha", true, "setLineAlpha"
);


PolylineModelTool.prototype.DEFAULT_MINIMAL_DISTANCE_BETWEEN_POINTS = 5;

//override
PolylineModelTool.prototype.canBeStarted = function(p, aElement, options){
    var l = aElement.length, element, hitDetails, c;
    if(l>0){
        element = aElement[0];
        if(element instanceof PolylineModelElement && element.isVisible() && !(element.isVerifiable() && ModelMainInstance.prototype.flagPlayerMode)) {
            hitDetails = element.getHitDetails();
            c = element.getPointCount();
            var flagPointsProhibit =
                (element.isCloseLine() && hitDetails.handlerNumber != -1) ||
                (hitDetails.handlerNumber>0 && hitDetails.handlerNumber != (c-1));
            //click with ctrl should remove the point
            var flagModifiersProhibit = (
                typeof options.event != TYPE_UNDEFINED &&
                typeof options.event.ctrlKey == TYPE_BOOLEAN &&
                options.event.ctrlKey
            );
            if(hitDetails != null && (flagPointsProhibit || flagModifiersProhibit || hitDetails.segmentNumber != -1))
                return false;
        }
    }
    return true;
};

//override
PolylineModelTool.prototype.canBeUsed = function(p, aElement, options){
    return this.isStarted();
};

//override
PolylineModelTool.prototype.configure = function(objToolData) {
    if(typeof objToolData == TYPE_UNDEFINED || objToolData == null)
        return;
    var me = this;
    var objConfiguration = me.objConfiguration;
    if(typeof objToolData.lineColor != TYPE_UNDEFINED)
        objConfiguration.lineColor = parseInt(objToolData.lineColor);
    if(typeof objToolData.lineAlpha != TYPE_UNDEFINED)
        objConfiguration.lineAlpha = parseInt(objToolData.lineAlpha);
    if(typeof objToolData.fillColor != TYPE_UNDEFINED)
        objConfiguration.fillColor = parseInt(objToolData.fillColor);
    if(typeof objToolData.fillAlpha != TYPE_UNDEFINED)
        objConfiguration.fillAlpha = parseInt(objToolData.fillAlpha);
};

//
PolylineModelTool.prototype.getFillAlpha = function(){
    return this.objConfiguration.fillAlpha;
};

//
PolylineModelTool.prototype.setFillAlpha = function(v){
    this.objConfiguration.fillAlpha = v;
};

//
PolylineModelTool.prototype.getFillColor = function(){
    return this.objConfiguration.fillColor;
};

//
PolylineModelTool.prototype.setFillColor = function(v){
    this.objConfiguration.fillColor = v;
};

//override
PolylineModelTool.prototype.start = function(p, aElement, options){
    var me = this;
    me.nAddedPoints = 0;
    var mapSceneModel = me.modelToolsController.getMapSceneModel();
    var modelElement;
    var hitDetails;
    var flagNew;
    if(aElement.length > 0 && (aElement[0] instanceof PolylineModelElement && aElement[0].isVisible() && !(aElement[0].isVerifiable() && ModelMainInstance.prototype.flagPlayerMode))){
        modelElement = aElement[0];
        if(modelElement.isCloseLine())
            flagNew = true;
        else {
            hitDetails = modelElement.getHitDetails();
            flagNew = (
                typeof hitDetails.handlerNumber != TYPE_NUMBER ||
                (hitDetails.handlerNumber != 0 && hitDetails.handlerNumber != modelElement.getPointCount()-1)
            );
        }
    }else
        flagNew = true;

    if(flagNew) {
        //a new polyline is created
        var id = mapSceneModel.getUniqueId();
        var objConfiguration = me.objConfiguration;
        //default parameters
        var objNewResourceData = {
            Id: id,
            Depth: mapSceneModel.getMaximalDepth() + 1,
            CHCB: "false",
            CHCF: "false",
            CHCL: "false",
            CHXY: "false",
            CLF: "false",
            FA: objConfiguration.fillAlpha,
            FC: objConfiguration.fillColor,
            FF: "false",
            IsVisible: "true",
            Is_Ver: "false",
            LA: objConfiguration.lineAlpha,
            LC: objConfiguration.lineColor,
            Name: PolylineModelElement.prototype.getLocalizedNamePrefix() + (id + 1),
            NoEdit: "false",
            OW: "1",
            Pl_ECSt: "true",
            Pl_ESSt: "true",
            Pl_HSw: "true",
            Pl_MSt: "true",
            Pl_VSw: "false",
            Pl_VSwOn: "true",
            errorMode: "radius",
            RZ: "15",
            X: "" + p.x,
            Y: "" + p.y,
            aNode: [],
            type_id: "0"
        };
        me.modelElement = new PolylineModelElement(objNewResourceData);
        me.modelElement.setUserCreated(true);
        me.modelElement.setComplete(false);
        //
        modelElement = me.modelElement;
        modelElement.addPoint(new Point(0, 0));
        modelElement.addPoint(new Point(0, 0));
        me.nAddedPoints = 2;
        //
        me.modelToolsController.addElementToModel(modelElement);
        //
        me.flagStarted = true;
        //
        modelElement.invalidate();
    }else{
        //add new segments to an existing polyline
        modelElement = aElement[0];
        if(modelElement.isCloseLine())
            return;
        hitDetails = modelElement.getHitDetails();
        if(typeof hitDetails.handlerNumber != TYPE_NUMBER || hitDetails.handlerNumber == -1)
            return;
        var x = modelElement.getX();
        var y = modelElement.getY();
        var aPoint = modelElement.getPoints(), p2;
        var i, n = modelElement.getPointCount();
        if(hitDetails.handlerNumber == 0){
            //reverse the points so that new points will be added at the end of the list
            var q = Math.floor(n/2);
            var shift = aPoint[0].subtract(aPoint[n-1]);
            for(i = 0; i<q; i++){
                p2 = aPoint[n-1-i];
                aPoint[n-1-i] = aPoint[i];
                aPoint[i] = p2;
            }
            for(i=0; i<n; i++)
                aPoint[i].offset(shift.x, shift.y);

            x -= shift.x;
            y -= shift.y;
            modelElement.setX(x);
            modelElement.setY(y);
            modelElement.setHitDetails(null);
        }
        p2 = aPoint[n-1];
        modelElement.addPoint(new Point(p2.x, p2.y));
        me.nAddedPoints++;
        //
        me.modelElement = modelElement;
        //
        me.modelElement.setComplete(false);
        me.flagStarted = true;
    }
};

//override
PolylineModelTool.prototype.use  = function(p, aElement, options){
    var me = this;
    if(!me.isStarted())
        return;
    var modelElement = me.modelElement;
    if(modelElement.isCloseEnds()){
        modelElement.closeLine(true);
        //do not call the tool's other manipulative methods directly,
        //do it via controller so that events fire properly
        me.getModelToolsController().endCurrentTool(p, aElement, options);
    }else {
        var dMin = me.DEFAULT_MINIMAL_DISTANCE_BETWEEN_POINTS, dMin2 = dMin * dMin;
        var aPoint = modelElement.getPoints();
        var pPrev = aPoint[aPoint.length - 2];
        var pLast = aPoint[aPoint.length - 1];
        var x = modelElement.getX();
        var y = modelElement.getY();
        pLast.x = p.x - x;
        pLast.y = p.y - y;
        if (pPrev.distanceSquared(pLast) < dMin2)
            return;
        modelElement.addPoint(new Point(pLast.x, pLast.y));
        if(modelElement.isCloseEnds()){
            modelElement.closeLine(true);
            me.getModelToolsController().endCurrentTool(p, aElement, options);
        }
        modelElement.invalidate();
        me.nAddedPoints++;
    }
};

//override
PolylineModelTool.prototype.useDoubleClick = function(p, aElement, options){
    this.getModelToolsController().endCurrentTool(p, aElement, options);
};

//override
PolylineModelTool.prototype.useHoldDown = function(p, aElement, options){
    //holdDown on an existing point deletes it; during PLO creation finishes current PLO
    var me = this;
    var element, hitDetails;
    if(me.getModelToolsController().isCurrentToolStarted())
        this.getModelToolsController().endCurrentTool(p, aElement, options);


    if(aElement.length>0){
        element = aElement[0];
        if(aElement[0].getType() == ModelResource.prototype.TYPE_PLO && aElement[0].isComplete()){
            hitDetails = element.getHitTestInfoForPoint(p).getDetails();
            if(hitDetails != null) {
                if (hitDetails.handlerNumber != -1) {
                    element.removePoint(hitDetails.handlerNumber, true);
                    element.setPressedHandlerNo(-1);
                    //after point is removed, a segment might need to be removed
                    hitDetails = element.getHitTestInfoForPoint(p).getDetails();
                }
                if (hitDetails != null && hitDetails.segmentNumber != -1)
                    element.removeSegment(hitDetails.segmentNumber);
                element.setHitDetails(null);
                element.invalidate();
            }
        }
    }
};

//override
PolylineModelTool.prototype.track = function(p, aElement, options){
    var me = this;
    if(!me.isStarted())
        return;
    if(options.event.type == "mousemove" || options.event.type == "touchmove") {
        var modelElement = me.modelElement;
        var aPoint = modelElement.getPoints();
        var pLast = aPoint[aPoint.length - 1];
        var x = modelElement.getX();
        var y = modelElement.getY();
        pLast.x = p.x - x;
        pLast.y = p.y - y;
    }
};

//override
PolylineModelTool.prototype.useMouseUp = function(p, aElement, options){
    //add a point to the current polyline
    var me = this;
    if(me.modelToolsController.isDragged())
        me.getModelToolsController().useCurrentTool(p, aElement, options);
};

//override
PolylineModelTool.prototype.end = function(p, aElement, options){
    var me = this;
    if(me.isStarted()){
        var dMin = me.DEFAULT_MINIMAL_DISTANCE_BETWEEN_POINTS, dMin2 = dMin*dMin;
        var flagKeep = true;
        var modelElement = me.modelElement;
        var aPoint = modelElement.getPoints();
        var pPrev = aPoint[aPoint.length-2];
        var pLast = aPoint[aPoint.length - 1];
        if(pPrev.distanceSquared(pLast)<dMin2) {
            if(aPoint.length <= modelElement.getMinimalPointCount())
                flagKeep = false;
            else {
                modelElement.removePoint();
                me.nAddedPoints--;
            }
        }
        if(flagKeep) {
            modelElement.setComplete(true);
            modelElement.setStructureEditable(true);
            modelElement.setVerticesMovable(true);
            modelElement.setSelected(true);
            me.modelToolsController.updateToolStates();
        }else
            me.modelToolsController.removeElementFromModel(modelElement);
        me.nAddedPoints = 0;
        me.modelElement = null;
        PolylineModelTool.superclass.end.call(me);
    }
};

//override
PolylineModelTool.prototype.cancel = function(p, aElement, options){
    var me = this;
    if(me.isStarted()){
        var modelElement = me.modelElement;
        if(me.nAddedPoints == modelElement.getPointCount())
            me.modelToolsController.removeElementFromModel(modelElement);
        else{
            var i;
            for(i=0; i<me.nAddedPoints; i++)
                modelElement.removePoint(modelElement.getPointCount()-1, false);
            me.nAddedPoints = 0;
            modelElement.setComplete(true);
        }
        PolylineModelTool.superclass.cancel.call(me);
    }
};

/**
 * @constructor
 */
function ArrowModelTool(modelToolsController) {
    //creation of arrows
    ArrowModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.modelElement = null;
    me.objConfiguration = {
        iconAlpha: 100,
        iconColor: 204,
        symbolId: "mcSymbolAR_Size2",
        flagSeparateButtons: false,
        aSymbol: ["mcSymbolAR_Size2"]
    };
}
extend(ArrowModelTool, SymbolSetModelTool);

ArrowModelTool.prototype.id = ModelTool.prototype.TOOL_ARROW;

//override
ArrowModelTool.prototype.canBeStarted = function(p, aElement, options){
    return true;
};

//override
ArrowModelTool.prototype.canBeUsed = function(p, aElement, options){
    return this.isStarted();
};

//override
ArrowModelTool.prototype.start = function(p, aElement, options) {
    var me = this;
    var objConfiguration = me.objConfiguration;
    var mapSceneModel = me.modelToolsController.getMapSceneModel();
    var modelElement;
    var id = mapSceneModel.getUniqueId();
    //default parameters
    var objNewResourceData = {
        Id: id,
        Symbol: objConfiguration.symbolId,
        Depth: mapSceneModel.getMaximalDepth() + 1,
        CHCB: "false",
        CHCF: "false",
        CHCL: "false",
        CHXY: "false",
        FA: ""+objConfiguration.iconAlpha,
        FC: ""+objConfiguration.iconColor,
        IsVisible: "true",
        Is_Ver: "false",
        Name: ArrowModelElement.prototype.getLocalizedNamePrefix() + (id + 1),
        NoEdit: "false",
        OW: "1",
        Pl_ECSt: "true",
        Pl_ESSt: "true",
        Pl_HSw: "true",
        Pl_MSt: "true",
        Pl_VSw: "false",
        Pl_VSwOn: "true",
        errorMode: "radius",
        RZ: "15",
        XS: "" + p.x,
        YS: "" + p.y,
        XF: "" + p.x,
        YF: "" + p.y,
        type_id: "0"
    };
    me.modelElement = new ArrowModelElement(objNewResourceData);
    me.modelElement.setUserCreated(true);
    me.modelElement.setComplete(false);
    //
    modelElement = me.modelElement;
    me.modelToolsController.addElementToModel(modelElement);
    //
    me.flagStarted = true;
};

//override
ArrowModelTool.prototype.use  = function(p, aElement, options) {
    var me = this;
    if (!me.isStarted())
        return;
    if(options.event.type == "mousedown" || options.event.type == "touchstart"){
        var modelElement = me.modelElement;
        modelElement.setComplete(true);
        modelElement.setStructureEditable(true);
        modelElement.setSelected(true);
        me.modelToolsController.updateToolStates();
        //do not call the tool's other manipulative methods directly,
        //do it via controller so that events fire properly
        me.getModelToolsController().endCurrentTool(p, aElement, options);
    }
};

//override
ArrowModelTool.prototype.useDoubleClick = function(p, aElement, options){
    this.getModelToolsController().endCurrentTool(p, aElement, options);
    this.getModelToolsController().startCurrentTool(p, aElement, options);
};

//override
ArrowModelTool.prototype.track = function(p, aElement, options) {
    var me = this;
    if (!me.isStarted())
        return;
    if(options.event.type == "mousemove" || options.event.type == "touchmove") {
        var modelElement = me.modelElement;
        modelElement.setXTo(p.x);
        modelElement.setYTo(p.y);
    }
};

//override
ArrowModelTool.prototype.end = function(p, aElement, options) {
    var me = this;
    if(me.isStarted()){
        var modelElement = me.modelElement;
        modelElement.setComplete(true);
        modelElement.setStructureEditable(true);
        modelElement.setSelected(true);
        me.modelToolsController.updateToolStates();
        me.modelElement = null;
        ArrowModelTool.superclass.end.call(me);
    }
};

//override
ArrowModelTool.prototype.cancel = function(p, aElement, options) {
    var me = this;
    var modelElement = me.modelElement;
    me.modelToolsController.removeElementFromModel(modelElement);
    ArrowModelTool.superclass.cancel.call(me);
};

/**
 * @constructor
 */
function RectangleModelTool(modelToolsController) {
    //creation of rectangles
    RectangleModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.sSubType = "rectangle";
    me.sPrefix = RectangleModelElement.prototype.localizedNamePrefixRectangle;
    me.objConfiguration = {
        lineAlpha: 100,
        lineColor: 0xFF0000,
        fillAlpha: 70,
        fillColor: 0xFF9933
    };
}
extend(RectangleModelTool, ModelTool);

RectangleModelTool.prototype.id = ModelTool.prototype.TOOL_RECTANGLE;

RectangleModelTool.prototype.aMethodNameForProperty =
    miscFunctions.cloneObject(RectangleModelTool.superclass.aMethodNameForProperty);
RectangleModelTool.prototype.setMethodNameForProperty(
    RectangleModelTool.prototype.aMethodNameForProperty, "fillColor", false, "getFillColor"
);
RectangleModelTool.prototype.setMethodNameForProperty(
    RectangleModelTool.prototype.aMethodNameForProperty, "fillColor", true, "setFillColor"
);
RectangleModelTool.prototype.setMethodNameForProperty(
    RectangleModelTool.prototype.aMethodNameForProperty, "fillAlpha", false, "getFillAlpha"
);
RectangleModelTool.prototype.setMethodNameForProperty(
    RectangleModelTool.prototype.aMethodNameForProperty, "fillAlpha", true, "setFillAlpha"
);
RectangleModelTool.prototype.setMethodNameForProperty(
    RectangleModelTool.prototype.aMethodNameForProperty, "lineColor", false, "getLineColor"
);
RectangleModelTool.prototype.setMethodNameForProperty(
    RectangleModelTool.prototype.aMethodNameForProperty, "lineColor", true, "setLineColor"
);
RectangleModelTool.prototype.setMethodNameForProperty(
    RectangleModelTool.prototype.aMethodNameForProperty, "lineAlpha", false, "getLineAlpha"
);
RectangleModelTool.prototype.setMethodNameForProperty(
    RectangleModelTool.prototype.aMethodNameForProperty, "lineAlpha", true, "setLineAlpha"
);

//override
RectangleModelTool.prototype.canBeStarted = function(p, aElement, options){
    return true;
};

//override
RectangleModelTool.prototype.canBeUsed = function(p, aElement, options){
    return this.isStarted();
};

//override
RectangleModelTool.prototype.reset = function(){
    //subtype property is not stored in xml hence the need for reset method
    this.sSubType = "rectangle";
};

//override
RectangleModelTool.prototype.configure = function(objToolData) {
    var me = this;
    var objConfiguration = me.objConfiguration;
    objConfiguration.lineAlpha = parseFloat(objToolData.lineAlpha);
    objConfiguration.lineColor = parseInt(objToolData.lineColor);
    objConfiguration.fillAlpha = parseFloat(objToolData.fillAlpha);
    objConfiguration.fillColor = parseInt(objToolData.fillColor);
};

//
RectangleModelTool.prototype.getFillAlpha = function(){
    return this.objConfiguration.fillAlpha;
};

//
RectangleModelTool.prototype.setFillAlpha = function(v){
    this.objConfiguration.fillAlpha = v;
};

//
RectangleModelTool.prototype.getFillColor = function(){
    return this.objConfiguration.fillColor;
};

//
RectangleModelTool.prototype.setFillColor = function(v){
    this.objConfiguration.fillColor = v;
};

//
RectangleModelTool.prototype.getLineAlpha = function(){
    return this.objConfiguration.lineAlpha;
};

//
RectangleModelTool.prototype.setLineAlpha = function(v){
    this.objConfiguration.lineAlpha = v;
};

//
RectangleModelTool.prototype.getLineColor = function(){
    return this.objConfiguration.lineColor;
};

//
RectangleModelTool.prototype.setLineColor = function(v){
    this.objConfiguration.lineColor = v;
};

//override
RectangleModelTool.prototype.configureByModelEventData = function(eventData){
    var me = this;
    if(typeof eventData.subType == TYPE_UNDEFINED)
        return;
    switch(eventData.subType){
        case ModelEvent.prototype.SYMBOL_SELECTED:
            switch(eventData.symbolId){
                case "mcSymbolRN_Size0":
                    me.sSubType = "rectangle";
                    me.sPrefix = RectangleModelElement.prototype.localizedNamePrefixRectangle;
                    break;
                case "mcSymbolEN_Size0":
                    me.sSubType = "ellipse";
                    me.sPrefix = RectangleModelElement.prototype.localizedNamePrefixEllipse;
                    break;
            }
            break;
        default:
    }
};

//override
RectangleModelTool.prototype.start = function(p, aElement, options) {
    var me = this;
    var mapSceneModel = me.modelToolsController.getMapSceneModel();
    var modelElement;
    var id = mapSceneModel.getUniqueId();
    var objConfiguration = me.objConfiguration;
    //default parameters
    var objNewResourceData = {
        Id: id,
        subType: me.sSubType,
        Depth: mapSceneModel.getMaximalDepth() + 1,
        CHCB: "false",
        CHCF: "false",
        CHCL: "false",
        CHXY: "false",
        FA: objConfiguration.fillAlpha,
        FC: objConfiguration.fillColor,
        LA: objConfiguration.lineAlpha,
        LC: objConfiguration.lineColor,
        IsVisible: "true",
        Is_Ver: "false",
        Name: me.sPrefix + (id + 1),
        NoEdit: "false",
        OW: "1",
        Pl_ECSt: "true",
        Pl_ESSt: "true",
        Pl_HSw: "true",
        Pl_MSt: "true",
        Pl_VSw: "false",
        Pl_VSwOn: "true",
        errorMode: "radius",
        RZ: "15",
        XS: "" + p.x,
        YS: "" + p.y,
        XF: "" + p.x,
        YF: "" + p.y,
        type_id: "0"
    };
    me.modelElement = new RectangleModelElement(objNewResourceData);
    me.modelElement.setUserCreated(true);
    me.modelElement.setComplete(false);
    //
    modelElement = me.modelElement;
    me.modelToolsController.addElementToModel(modelElement);
    //
    me.flagStarted = true;
};

//override
RectangleModelTool.prototype.use  = function(p, aElement, options) {
    var me = this;
    if (!me.isStarted())
        return;
    if(options.event.type == "mousedown" || options.event.type == "touchstart"){
        var modelElement = me.modelElement;
        modelElement.setComplete(true);
        modelElement.setStructureEditable(true);
        //do not call the tool's other manipulative methods directly,
        //do it via controller so that events fire properly
        me.getModelToolsController().endCurrentTool(p, aElement, options);
    }
};

//override
RectangleModelTool.prototype.useDoubleClick = function(p, aElement, options){
    this.getModelToolsController().endCurrentTool(p, aElement, options);
    this.getModelToolsController().startCurrentTool(p, aElement, options);
};


//override
RectangleModelTool.prototype.track = function(p, aElement, options) {
    var me = this;
    if (!me.isStarted())
        return;
    if(options.event.type == "mousemove" || options.event.type == "touchmove") {
        var modelElement = me.modelElement;
        modelElement.setXBR(p.x);
        modelElement.setYBR(p.y);
    }
};

//override
RectangleModelTool.prototype.end = function(p, aElement, options) {
    var me = this;
    if(me.isStarted()){
        var modelElement = me.modelElement;
        modelElement.setXBR(p.x);
        modelElement.setYBR(p.y);
        //TL should be, well, the top left corner
        var xTL = modelElement.getXTL();
        var yTL = modelElement.getYTL();
        var bW = modelElement.getBorderWidth();
        if(xTL> p.x){
            modelElement.setXTL(p.x-bW);
            modelElement.setXBR(xTL+bW);
        }
        if(yTL> p.y){
            modelElement.setYTL(p.y-bW);
            modelElement.setYBR(yTL+bW);
        }
        modelElement.setComplete(true);
        modelElement.setStructureEditable(true);
        modelElement.setSelected(true);
        me.modelToolsController.updateToolStates();
        me.modelElement = null;
        RectangleModelTool.superclass.end.call(me);
    }
};

//override
RectangleModelTool.prototype.cancel = function(p, aElement, options) {
    var me = this;
    var modelElement = me.modelElement;
    me.modelToolsController.removeElementFromModel(modelElement);
    RectangleModelTool.superclass.cancel.call(me);
};

/**
 * @constructor
 */
function TextModelTool(modelToolsController) {
    //creation of texts
    TextModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    //these values are for fontSize = 20; corresponding "static" get-methods need to be added to TextModelElements
    me.minWidth = 9;
    me.defaultWidth = 87.3;
    me.objConfiguration = {
        hAlign: "left",
        fontColor: 0,
        fontAlpha: 100,
        fontSize: 20,
        fontFace: "Arial",
        borderColor:255,
        borderAlpha:0
    };
    me.flagConfigureByPaletteAlways = true;
}
extend(TextModelTool, ModelTool);

TextModelTool.prototype.id = ModelTool.prototype.TOOL_TEXT;

TextModelTool.prototype.aMethodNameForProperty =
    miscFunctions.cloneObject(TextModelTool.superclass.aMethodNameForProperty);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "fontSize", false, "getFontSize"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "fontSize", true, "setFontSize"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "fontFace", false, "getFontFace"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "fontFace", true, "setFontFace"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "horizontalAlign", false, "getHorizontalAlign"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "horizontalAlign", true, "setHorizontalAlign"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "fillColor", false, "getFontColor"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "fillColor", true, "setFontColor"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "fillAlpha", false, "getFontAlpha"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "fillAlpha", true, "setFontAlpha"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "lineColor", false, "getBorderColor"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "lineColor", true, "setBorderColor"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "lineAlpha", false, "getBorderAlpha"
);
TextModelTool.prototype.setMethodNameForProperty(
    TextModelTool.prototype.aMethodNameForProperty, "lineAlpha", true, "setBorderAlpha"
);

//override
TextModelTool.prototype.canBeStarted = function(p, aElement, options){
    return true;
};

//override
TextModelTool.prototype.configure = function(objToolData) {
    if(typeof objToolData == TYPE_UNDEFINED || objToolData == null)
        return;
    var me = this;
    var objConfiguration = me.objConfiguration;
    if(typeof objToolData.hAlign == TYPE_STRING)
        objConfiguration.hAlign = objToolData.hAlign;
    if(typeof objToolData.fontColor != TYPE_UNDEFINED)
        objConfiguration.fontColor = parseInt(objToolData.fontColor);
    if(typeof objToolData.fontAlpha != TYPE_UNDEFINED)
        objConfiguration.fontAlpha = parseInt(objToolData.fontAlpha);
    if(typeof objToolData.fontSize == TYPE_NUMBER)
        objConfiguration.fontSize = parseFloat(objToolData.fontSize);
    if(typeof objToolData.fontFace == TYPE_STRING)
        objConfiguration.fontFace = objToolData.fontFace;
    if(typeof objToolData.borderColor != TYPE_UNDEFINED)
        objConfiguration.borderColor = parseInt(objToolData.borderColor);
    if(typeof objToolData.borderAlpha != TYPE_UNDEFINED)
        objConfiguration.borderAlpha = parseInt(objToolData.borderAlpha);
};

//override
TextModelTool.prototype.start  = function(p, aElement, options) {
    var me = this;
    var element = null;
    if(aElement!=null && aElement.length>0){
        if(me.fIsOverEditableTextModelElement(p, aElement, options)){
            element = aElement[0];
            if (!element.isEditingMode()) {
                element.setSelected(true);
                element.setEditingMode(true);
            }
        }
        if(me.isStarted())
            this.getModelToolsController().endCurrentTool(p, aElement, options);
    }
    if(element==null){
        var mapSceneModel = me.modelToolsController.getMapSceneModel();
        var modelElement;
        var id = mapSceneModel.getUniqueId();
        //default parameters
        var fontSize = 20;
        var height =
            TextFunctions.prototype.getLineHeightByFontSize(fontSize)+
            2*TextFunctions.prototype.getVerticalPaddingByFontSize(fontSize);
        var objConfiguration = me.objConfiguration;
        var objNewResourceData = {
            Id: id,
            Depth: mapSceneModel.getMaximalDepth() + 1,
            CHCL: "false",
            CHCZ: "false",
            CHFC: "false",
            CHCS: "false",
            CHCT: "true",
            IsVisible: "true",
            Is_Ver: "false",
            Name: TextModelElement.prototype.getLocalizedNamePrefix() + (id + 1),
            NoEdit: "false",
            OW: "1",
            Pl_ECSt: "true",
            Pl_ESSt: "true",
            Pl_HSw: "true",
            Pl_MSt: "true",
            Pl_VSw: "false",
            Pl_VSwOn: "true",
            vAlign: "top",
            hAlign: objConfiguration.hAlign,
            wordWrap: "true",
            BorderAlpha: objConfiguration.borderAlpha,
            BorderColor: objConfiguration.borderColor,
            BackgroundAlpha: 0,
            BackgroundColor: 16777215,
            fontAlpha: objConfiguration.fontAlpha,
            fontColor: objConfiguration.fontColor,
            fontSize: objConfiguration.fontSize,
            fontFace: objConfiguration.fontFace,
            text: "",
            RZ: "15",
            X: "" + (p.x-me.minWidth),
            Y: "" + (p.y-height),
            Width: "" + me.minWidth,
            Height: "" + height,
            type_id: "0"
        };
        me.modelElement = new TextModelElement(objNewResourceData);
        me.modelElement.setUserCreated(true);
        me.modelElement.setComplete(false);
        //
        modelElement = me.modelElement;
        me.modelToolsController.addElementToModel(modelElement);
        //
        me.flagStarted = true;
    }
};

//override
TextModelTool.prototype.use  = function(p, aElement, options) {
};

//override
TextModelTool.prototype.useMouseUp = function(p, aElement, options){
    var me = this;
    if (!me.isStarted())
        return;
    var modelElement = me.modelElement;
    if(modelElement != null) {
        if (me.modelToolsController.isDragged()) {
            var w = p.x - modelElement.getX();
            modelElement.setWidth((w < me.minWidth) ? me.minWidth : w);
        } else
            modelElement.setWidth(me.defaultWidth);
    }
    me.getModelToolsController().endCurrentTool(p, aElement, options);
};

//override
TextModelTool.prototype.track = function(p, aElement, options){
    var me = this;
    if (!me.isStarted())
        return;
    if(options.event.type == "mousemove" || options.event.type == "touchmove") {
        var modelElement = me.modelElement;
        var w = p.x-modelElement.getX();
        modelElement.setWidth((w<me.minWidth) ? me.minWidth : w);
    }
};

//override
TextModelTool.prototype.end = function(p, aElement, options) {
    var me = this;
    if(me.isStarted()){
        var modelElement = me.modelElement;
        if(modelElement != null) {
            modelElement.setComplete(true);
            modelElement.setStructureEditable(true);
            if (!modelElement.isEditingMode())
                modelElement.setEditingMode(true);
            modelElement.setSelected(true);
            me.modelToolsController.updateToolStates();
            me.modelElement = null;
        }
        TextModelTool.superclass.end.call(me);
    }
};

//
TextModelTool.prototype.fIsOverEditableTextModelElement = function(p, aElement, options){
    if(aElement!=null && aElement.length>0){
        var element = aElement[0];
        if(element instanceof TextModelElement)
            return element.isTextModifiable();
        else if(element instanceof BtextModelElement){
            var elementTF = element.getTextField();
            if(elementTF.isTextModifiable()) {
                var hitDetails = elementTF.getHitDetails();
                return (hitDetails != null) && hitDetails.flagHit;
            }
        }
    }
    return false;
};

//override
TextModelTool.prototype.canHighlight = function(p, aElement, options){
    return (
        (typeof options.mouseIsDown != TYPE_BOOLEAN || !options.mouseIsDown) &&
        this.fIsOverEditableTextModelElement(p, aElement,options)
    );
};

//
TextModelTool.prototype.getFontSize = function(){
    return this.objConfiguration.fontSize;
};

//
TextModelTool.prototype.setFontSize = function(v){
    this.objConfiguration.fontSize = parseFloat(v);
};

//
TextModelTool.prototype.getFontColor = function(){
    return this.objConfiguration.fontColor;
};

//
TextModelTool.prototype.setFontColor = function(v){
    this.objConfiguration.fontColor = parseInt(v);
};

//
TextModelTool.prototype.getFontAlpha = function(){
    return this.objConfiguration.fontAlpha;
};

//
TextModelTool.prototype.setFontAlpha = function(v){
    this.objConfiguration.fontAlpha = parseFloat(v);
};

//
TextModelTool.prototype.getBorderColor = function(){
    return this.objConfiguration.borderColor;
};

//
TextModelTool.prototype.setBorderColor = function(v){
    this.objConfiguration.borderColor = parseInt(v);
};

//
TextModelTool.prototype.getBorderAlpha = function(){
    return this.objConfiguration.borderAlpha;
};

//
TextModelTool.prototype.setBorderAlpha = function(v){
    this.objConfiguration.borderAlpha = parseFloat(v);
};

//
TextModelTool.prototype.getFontFace = function(){
    return this.objConfiguration.fontFace;
};

//
TextModelTool.prototype.setFontFace = function(s){
    this.objConfiguration.fontFace = s;
};

//
TextModelTool.prototype.getHorizontalAlign = function(){
    return this.objConfiguration.hAlign;
};

//
TextModelTool.prototype.setHorizontalAlign = function(s){
    this.objConfiguration.hAlign = s;
};

//

/**
 * @constructor
 */
function BtextModelTool(modelToolsController) {
    //creation of texts
    BtextModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    //0 - creating leader line, 1 - creating text field
    me.stepNo = -1;
    me.textFieldHeight = -1;
    me.objConfiguration = {
        hAlign: "left",
        fontColor: 0,
        fontAlpha: 100,
        fontSize: 20,
        fontFace: "Arial",
        borderColor:255,
        borderAlpha:0,
        backgroundColor:16777215,
        backgroundAlpha:50
    };
}
extend(BtextModelTool, TextModelTool);

BtextModelTool.prototype.id = ModelTool.prototype.TOOL_BOUND_TEXT;

//override
BtextModelTool.prototype.canBeStarted = function(p, aElement, options){
    //unlike TextModelElement method, p should be NOT over editable text model element
    if(aElement.length>0)
        return !this.fIsOverEditableTextModelElement(p, aElement, options);
    return true;
};

BtextModelTool.prototype.start  = function(p, aElement, options) {
    var me = this;
    var mapSceneModel = me.modelToolsController.getMapSceneModel();
    var modelElement;
    var id = mapSceneModel.getUniqueId();
    //default parameters
    var fontSize = 20;
    me.textFieldHeight =
        TextFunctions.prototype.getLineHeightByFontSize(fontSize)+
        2*TextFunctions.prototype.getVerticalPaddingByFontSize(fontSize);
    var objConfiguration = me.objConfiguration;
    var objNewResourceData = {
        Id: id,
        Depth: mapSceneModel.getMaximalDepth() + 1,
        CHCL: "false",
        CHCZ: "false",
        CHFC: "false",
        CHCS: "false",
        CHCT: "true",
        IsVisible: "true",
        Is_Ver: "false",
        Name: BtextModelElement.prototype.getLocalizedNamePrefix() + (id + 1),
        NoEdit: "false",
        OW: "1",
        Pl_ECSt: "true",
        Pl_ESSt: "true",
        Pl_HSw: "true",
        Pl_MSt: "true",
        Pl_VSw: "false",
        Pl_VSwOn: "true",
        vAlign: "top",
        hAlign: objConfiguration.hAlign,
        wordWrap: "true",
        BorderAlpha: objConfiguration.borderAlpha,
        BorderColor: objConfiguration.borderColor,
        BackgroundAlpha: objConfiguration.backgroundAlpha,
        BackgroundColor: objConfiguration.backgroundColor,
        fontAlpha: objConfiguration.fontAlpha,
        fontColor: objConfiguration.fontColor,
        fontSize: objConfiguration.fontSize,
        fontFace: objConfiguration.fontFace,
        text: "",
        errorMode: "radius",
        RZ: "15",
        XHandler: "" + p.x,
        YHandler: "" + p.y,
        X: "" + (p.x-me.minWidth),
        Y: "" + (p.y-me.textFieldHeight),
        Width: "" + me.minWidth,
        Height:"" + me.textFieldHeight,
        type_id: "0"
    };
    me.modelElement = new BtextModelElement(objNewResourceData);
    me.modelElement.setUserCreated(true);
    me.modelElement.setComplete(false);
    //
    modelElement = me.modelElement;
    me.modelToolsController.addElementToModel(modelElement);
    //
    me.stepNo = 0;
    me.flagStarted = true;
};

BtextModelTool.prototype.use = function(p, aElement, options) {
    var me = this;
    if (!me.isStarted())
        return;
    var modelElement = me.modelElement;
    switch(me.stepNo){
        case 0:
            modelElement.setXTextField(p.x-me.minWidth);
            modelElement.setYTextField(p.y-me.textFieldHeight);
            me.modelToolsController.resetDragged(p);
            me.stepNo = 1;
            break;
        case 1:
            break;
        default:
    }
};

BtextModelTool.prototype.useMouseUp = function(p, aElement, options) {
    var me = this;
    if (!me.isStarted())
        return;
    var modelElement = me.modelElement;
    switch(me.stepNo){
        case 0:
            if (me.modelToolsController.isDragged()) {
                modelElement.setXTextField(p.x-me.minWidth);
                modelElement.setYTextField(p.y-me.textFieldHeight);
                me.modelToolsController.resetDragged(p);
                me.stepNo = 1;
            }
            break;
        case 1:
            if (me.modelToolsController.isDragged()) {
                var w = p.x - modelElement.getXTextField();
                modelElement.setWidthTextField((w < me.minWidth) ? me.minWidth : w);
            }else
                modelElement.setWidthTextField(me.defaultWidth);
            me.getModelToolsController().endCurrentTool(p, aElement, options);
            break;
        default:
    }
};

//override
BtextModelTool.prototype.track = function(p, aElement, options) {
    var me = this;
    if (!me.isStarted())
        return;
    if(options.event.type == "mousemove" || options.event.type == "touchmove") {
        var modelElement = me.modelElement;
        switch(me.stepNo){
            case 0:
                modelElement.setXTextField(p.x-me.minWidth);
                modelElement.setYTextField(p.y-me.textFieldHeight);
                break;
            case 1:
                var w = p.x-modelElement.getXTextField();
                modelElement.setWidthTextField((w<me.minWidth) ? me.minWidth : w);
                break;
            default:
        }
    }
};

//override
BtextModelTool.prototype.end = function(p, aElement, options) {
    var me = this;
    if (me.isStarted()) {
        me.modelToolsController.resetDragged(null);
        me.stepNo = -1;
    }
    BtextModelTool.superclass.end.call(me, p, aElement, options);
};

//override
BtextModelTool.prototype.canHighlight = function(p, aElement, options){
    return false;
};

//override
BtextModelTool.prototype.cancel = function(p, aElement, options) {
    var me = this;
    var modelElement = me.modelElement;
    me.modelToolsController.removeElementFromModel(modelElement);
    me.modelToolsController.resetDragged(null);
    me.stepNo = -1;
    BtextModelTool.superclass.cancel.call(me);
};

//in the player, no checkboxes can be created
/**
 * @constructor
 */
function CheckboxSetModelTool(modelToolsController){
    CheckboxSetModelTool.superclass.constructor.call(this, modelToolsController);
}
extend(CheckboxSetModelTool, ModelTool);

CheckboxSetModelTool.prototype.id = ModelTool.prototype.TOOL_CHECKBOX_SET;

//override
CheckboxSetModelTool.prototype.canBeStarted = function(p, aElement, options){
    return false;
};

//override
CheckboxSetModelTool.prototype.canBeUsed = function(p, aElement, options){
    return false;
};
/**
 * @constructor
 */
function DiagramModelTool(modelToolsController) {
    //creation of diagrams
    DiagramModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.objConfiguration = {
        iconAlpha: 100,
        iconColor: 204,
        symbolId: "mcSymbolD1_Size2",
        flagSeparateButtons: false,
        aSymbol: ["mcSymbolD1_Size2"]
    };
}
extend(DiagramModelTool, SymbolSetModelTool);

DiagramModelTool.prototype.id = ModelTool.prototype.TOOL_DIAGRAM;

//override
DiagramModelTool.prototype.canBeStarted = function(p, aElement, options){
    var l = aElement.length, element, hitDetails;
    if(l>0){
        element = aElement[0];
        if(element instanceof DiagramModelElement) {
            hitDetails = element.getHitDetails();
            if(hitDetails != null && (hitDetails.handlerNumber != -1))
                return false;
        }
    }
    return true;
};
//override
DiagramModelTool.prototype.canBeUsed = function(p, aElement, options){
    //a diagram is created at the "start"-click, it is complete right after creation
    return false;
};

//override
DiagramModelTool.prototype.start = function(p, aElement, options) {
    var me = this;
    var mapSceneModel = me.modelToolsController.getMapSceneModel();
    var modelElement;
    var id = mapSceneModel.getUniqueId();
    var objConfiguration = me.objConfiguration;
    //default parameters
    var objNewResourceData = {
        Id: id,
        Symbol: objConfiguration.symbolId,
        Depth: mapSceneModel.getMaximalDepth() + 1,
        CHCL: "false",
        CHSZ: "false",
        CHTY: "false",
        CHXY: "false",
        FA: "undefined",
        FC: "undefined",
        Pl_Angles: "true",
        Pl_ECSt: "true",
        Pl_ESSt: "true",
        Pl_MSt: "true",
        Pl_VSwOn: "true",
        Pl_VSw: "false",
        Pl_HSw: "true",
        IsVisible: "true",
        Is_Ver: "false",
        angleError: "18",
        angleStep: "3.6",
        diagramColor: "16750899,16777113,39168,204,13382553,10092441",
        diagramAlpha: "100,100,100,100,100,100",
        sectorWeight: "1,1,1,1,1,1",
        Name: DiagramModelElement.prototype.getLocalizedNamePrefix() + (id + 1),
        NoEdit: "false",
        errorMode: "radius",
        RZ: "15",
        OW: "1",
        X: ""+ p.x,
        Y: ""+ p.y,
        type_id: "0"
    };
    modelElement = new DiagramModelElement(objNewResourceData);
    modelElement.setUserCreated(true);
    modelElement.setComplete(true);
    modelElement.setSelected(true);
    modelElement.setStructureEditable(true);
    modelElement.setHandlersMovable(true);
    //
    me.modelToolsController.addElementToModel(modelElement);
    me.modelToolsController.updateToolStates();
};
/**
 * @constructor
 */
function PointModelTool(modelToolsController) {
    //creation of symbols
    PointModelTool.superclass.constructor.call(this, modelToolsController);
    //default configuration
    var objConfiguration = this.objConfiguration;
    objConfiguration.iconAlpha = parseFloat("100");
    objConfiguration.iconColor = parseInt("204");
    objConfiguration.symbolId = "mcSymbol00_Size1";
    objConfiguration.flagSeparateButtons = false;
    objConfiguration.aSymbol = [objConfiguration.symbolId];
}
extend(PointModelTool, SymbolSetModelTool);

PointModelTool.prototype.id = ModelTool.prototype.TOOL_POINT;

PointModelTool.prototype.aMethodNameForProperty =
    miscFunctions.cloneObject(PointModelTool.superclass.aMethodNameForProperty);
PointModelTool.prototype.setMethodNameForProperty(
    PointModelTool.prototype.aMethodNameForProperty, "fillColor", false, "getIconColor"
);
PointModelTool.prototype.setMethodNameForProperty(
    PointModelTool.prototype.aMethodNameForProperty, "fillColor", true, "setIconColor"
);
PointModelTool.prototype.setMethodNameForProperty(
    PointModelTool.prototype.aMethodNameForProperty, "fillAlpha", false, "getIconAlpha"
);
PointModelTool.prototype.setMethodNameForProperty(
    PointModelTool.prototype.aMethodNameForProperty, "fillAlpha", true, "setIconAlpha"
);

//override
PointModelTool.prototype.canBeUsed = function(p, aElement, options){
    //a symbol is created at the "start"-click, it is complete right after creation
    return false;
};

//
PointModelTool.prototype.getIconColor = function(){
    return this.objConfiguration.iconColor;
};

//
PointModelTool.prototype.setIconColor = function(v){
    this.objConfiguration.iconColor = v;
};

//
PointModelTool.prototype.getIconAlpha = function(){
    return this.objConfiguration.iconAlpha;
};

//
PointModelTool.prototype.setIconAlpha = function(v){
    this.objConfiguration.iconAlpha = v;
};

//override
PointModelTool.prototype.start = function(p, aElement, options) {
    var me = this;
    var mapSceneModel = me.modelToolsController.getMapSceneModel();
    var modelElement;
    var objConfiguration = me.objConfiguration;
    var id = mapSceneModel.getUniqueId();
    //default parameters
    var objNewResourceData = {
        Id: id,
        Symbol: objConfiguration.symbolId,
        Depth: mapSceneModel.getMaximalDepth() + 1,
        CHCL: "false",
        CHSZ: "false",
        CHTY: "false",
        CHXY: "false",
        FA: objConfiguration.iconAlpha,
        FC: objConfiguration.iconColor,
        Pl_Angles: "undefined",
        Pl_ECSt: "true",
        Pl_ESSt: "true",
        Pl_MSt: "true",
        Pl_VSwOn: "true",
        Pl_VSw: "false",
        Pl_HSw: "true",
        IsVisible: "true",
        Is_Ver: "false",
        Name: PointModelElement.prototype.getLocalizedNamePrefix() + (id + 1),
        NoEdit: "false",
        fromZoneId: "undefined",
        fromZoneType: "undefined",
        errorMode: "radius",
        RZ: "15",
        OW: "1",
        X: ""+ p.x,
        Y: ""+ p.y,
        type_id: "0"
    };
    modelElement = new PointModelElement(objNewResourceData);
    modelElement.setUserCreated(true);
    modelElement.setComplete(true);
    modelElement.setSelected(true);
    //
    me.modelElement = modelElement;
    //
    me.modelToolsController.addElementToModel(modelElement);
    me.modelToolsController.updateToolStates();
};

/**
 * @constructor
 */
function FontFaceModelTool(modelToolsController){
    FontFaceModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.sProperty = "fontFace";
    me.sIsModifiableMethod = "isFontFaceModifiable";
    //
    if(FontFaceModelTool.prototype.aElement == null)
        FontFaceModelTool.prototype.aElement = [
            {
                italic:false, bold:false, size:18, face:"Arial", sample:"A",
                panelSample:document.localization.getString("interface.dialogs.font-face.normal", "Нормальный")
            },
            {
                italic:false, bold:true,  size:18, face:"Arial", sample:"A",
                panelSample:document.localization.getString("interface.dialogs.font-face.bold", "Жирный")
            },
            {
                italic:true,  bold:false, size:18, face:"Arial", sample:"A",
                panelSample:document.localization.getString("interface.dialogs.font-face.italic", "Курсив")
            },
            {
                italic:true,  bold:true,  size:18, face:"Arial", sample:"A",
                panelSample:document.localization.getString("interface.dialogs.font-face.bold-italic", "Ж.курсив")
            }
        ];
}
extend(FontFaceModelTool, ListedCommandModelTool);

FontFaceModelTool.prototype.id = ModelTool.prototype.TOOL_FONT_FACE;

FontFaceModelTool.prototype.DEFAULT_PROPERTY_VALUE = "Arial";

FontFaceModelTool.prototype.aElement = null;

//
FontFaceModelTool.prototype.getElementNumberByPropertyValue = function(sValue){
    var me = this;
    var a = sValue.split("_");
    var j = a.indexOf("bold");
    var bold = (j!=-1);
    if(bold)
        a.splice(j, 1);
    j = a.indexOf("italic");
    var italic = (j!=-1);
    if(italic)
        a.splice(j, 1);
    var face = a[0];
    var i, aElement = me.aElement, n = aElement.length, element;
    for(i=0; i<n; i++){
        element = aElement[i];
        if(element.bold == bold && element.italic == italic && element.face == face)
            return i;
    }
    return -1;
};

//
FontFaceModelTool.prototype.getPropertyValueByElementNumber = function(q){
    var me = this;
    var aElement = me.aElement, n = aElement.length, element, s;
    if(q>=0 && q<n){
        element = aElement[q];
        s = element.face;
        if(element.bold)
            s = s.concat("_bold");
        if(element.italic)
            s = s.concat("_italic");
        return s;
    }
    return null;
};
/**
 * @constructor
 */
function FontSizeModelTool(modelToolsController){
    FontSizeModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.sProperty = "fontSize";
    me.aPropertyAlias = ["textSize"];
    me.sIsModifiableMethod = "isFontSizeModifiable";
}
extend(FontSizeModelTool, ListedCommandModelTool);

FontSizeModelTool.prototype.id = ModelTool.prototype.TOOL_FONT_SIZE;

FontSizeModelTool.prototype.DEFAULT_PROPERTY_VALUE = "20";

FontSizeModelTool.prototype.aElement = [
    {fontSize:10}, {fontSize:11}, {fontSize:12}, {fontSize:14},
    {fontSize:16}, {fontSize:18}, {fontSize:20}, {fontSize:22},
    {fontSize:24}, {fontSize:26}, {fontSize:28}, {fontSize:36},
    {fontSize:48}, {fontSize:72}, {fontSize:90}, {fontSize:120}
];
(function(){
    var a = FontSizeModelTool.prototype.aElement;
    var i, l = a.length, obj;
    for(i=0; i<l; i++){
        obj= a[i];
        obj.sample = obj.panelSample = ""+obj.fontSize;
        obj.italic = false;
        obj.bold = true;
        obj.size = 18;
        obj.face = "Arial";
        obj.color = "#000000";
    }
})();

/**
 * @constructor
 */
function TextAlignModelTool(modelToolsController){
    TextAlignModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.sProperty = "horizontalAlign";
    me.sIsModifiableMethod = "isHorizontalAlignModifiable";
}
extend(TextAlignModelTool, ListedCommandModelTool);

TextAlignModelTool.prototype.id = ModelTool.prototype.TOOL_TEXT_ALIGN;

TextAlignModelTool.prototype.DEFAULT_PROPERTY_VALUE = "left";

TextAlignModelTool.prototype.aElement = [
    {align:"left",      symbolId:"0",   iconId:"gr.text.align.left"},
    {align:"center",    symbolId:"1",   iconId:"gr.text.align.center"},
    {align:"right",     symbolId:"2",   iconId:"gr.text.align.right"}
];

//override
TextAlignModelTool.prototype.configure = function(objToolData){
    //for configuring the tool according to the data stored in the model
    var me = this;
    var i;
    var sValue;
    var objConfiguration = me.objConfiguration;
    sValue = me.DEFAULT_PROPERTY_VALUE;
    objConfiguration.aSymbol = [];
    objConfiguration.aIcon = [];
    for(i=0; i< me.aElement.length; i++) {
        objConfiguration.aSymbol.push(me.aElement[i].symbolId);
        objConfiguration.aIcon.push(me.aElement[i].iconId);
    }
    objConfiguration.symbolId = ""+me.getElementNumberByPropertyValue(sValue);
};

//override
//TextAlign droplist should not pick its state from the selected text field
TextAlignModelTool.prototype.configureByMapSceneModel = function(mapSceneModel, options){
};

//override
TextAlignModelTool.prototype.getElementNumberByPropertyValue = function(sValue) {
    var me = this;
    var a = me.aElement;
    var i, n = a.length;
    for(i=0; i<n; i++)
        if(a[i].align == sValue)
            return i;
    return -1;
};

//override
TextAlignModelTool.prototype.getPropertyValueByElementNumber = function(q){
    var me = this;
    var aElement = me.aElement, n = aElement.length, element, s;
    if(q>=0 && q<n){
        return aElement[q].align;
    }
    return null;
};


/**
 * @constructor
 */
function ResetModelTool(modelToolsController){
    ResetModelTool.superclass.constructor.call(this, modelToolsController);
    //resets the whole model to its initial state
    var me = this;
    me.flagStarted = true;
    me.flagCommand = true;
}
extend(ResetModelTool, ModelTool);

ResetModelTool.prototype.id = ModelTool.prototype.TOOL_RESET;

//override
ResetModelTool.prototype.applyCommand = function(mapSceneModel, options){
    var me = this;
    var modelToolsController = me.getModelToolsController();
    modelToolsController.getMapSceneController().resetMapScene();
    mapSceneModel.fillWithInitialElements();
    modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_SELECT);
};
/**
 * @constructor
 */
function ShowHiddenModelTool(modelToolsController){
    ShowHiddenModelTool.superclass.constructor.call(this, modelToolsController);
    //
    if(ShowHiddenModelTool.prototype.PANEL_SAMPLE==null) {
        ShowHiddenModelTool.prototype.PANEL_SAMPLE =
            document.localization.getString("interface.dialogs.show-hidden.panel-sample", "Показать скрытые");
        ShowHiddenModelTool.prototype.elementTemplate = {
            italic:false, bold:false, size:14, face:"Arial",
            sample:ShowHiddenModelTool.prototype.PANEL_SAMPLE, panelSample:null
        };
    }
    //resets the whole model to its initial state
    var me = this;
    me.sProperty = "isVisible";
    me.sIsModifiableMethod = "isCanBeTurnedOn";
    //
    me.defaultElement = miscFunctions.cloneObject(me.elementTemplate);
    me.fontForSelected = {italic:false, bold:true, size:14, face:"Arial", color:"#000000"};
}
extend(ShowHiddenModelTool, ListedCommandModelTool);

ShowHiddenModelTool.prototype.id = ModelTool.prototype.TOOL_SHOW_HIDDEN;

ShowHiddenModelTool.prototype.DEFAULT_PROPERTY_VALUE = "0";
ShowHiddenModelTool.prototype.PANEL_SAMPLE = null;

ShowHiddenModelTool.prototype.elementTemplate = null;

ShowHiddenModelTool.prototype.aElement = [];

//override
ShowHiddenModelTool.prototype.getElementNumberByPropertyValue = function(sValue){
    return -1;
};

//override
ShowHiddenModelTool.prototype.getPropertyValueByElementNumber = function(q){
    return null;
};

//override
ShowHiddenModelTool.prototype.configure = function(objToolData){
    //update aElement array of the class
    var me = this;
    var aElement = me.aElement, template = me.elementTemplate, element;
    aElement.splice(0);
    if(typeof objToolData.items != TYPE_UNDEFINED && Array.isArray(objToolData.items)) {
        var aItem = objToolData.items, item, i, n = aItem.length;
        for (i = 0; i < n; i++) {
            item = aItem[i];
            element = miscFunctions.cloneObject(template);
            element.panelSample = item.name;
            if(typeof item.groupId == TYPE_NUMBER)
                element.groupId = item.groupId;
            else
                element.id = item.id;
            element.visible = item.visible;
            aElement.push(element);
        }
    }
    //call the method of the superclass just in case
    ShowHiddenModelTool.superclass.configure.call(me, {});
};

//override
ShowHiddenModelTool.prototype.applyCommand = function(mapSceneModel, options){
    var me = this;
    if(options == null || typeof options.data != TYPE_OBJECT || options.data == null)
        return;
    var data = options.data;
    var q = parseInt(data.symbolId);
    var aElement = me.aElement, element;
    var modelElement, elementsGroup, aModelElement;
    var i, l, flag;
    var sMethod;
    if(q>=0 && q<aElement.length){
        element = aElement[q];
        if(typeof element.groupId == TYPE_NUMBER){
            elementsGroup = mapSceneModel.getGroupById(element.groupId);
            aModelElement = elementsGroup.getElements();
        }else {
            modelElement = mapSceneModel.getElementById(element.id);
            aModelElement = [modelElement];
        }
        l = aModelElement.length;
        flag = false;
        for(i=0; i<l; i++){
            modelElement = aModelElement[i];
            sMethod = modelElement.getMethodNameForProperty(me.sProperty, true);
            if(
                sMethod != null &&
                typeof modelElement[me.sIsModifiableMethod] == TYPE_FUNCTION && modelElement[me.sIsModifiableMethod]()
            ){
                if(!flag){
                    element.visible = !element.visible;
                    flag = true;
                }
                modelElement[sMethod](element.visible);
            }
        }
        if(flag)
            me.modelToolsController.getMapSceneModel().invalidate();
    }
};

/**
 * @constructor
 */
function ZoomModelTool(modelToolsController){
    ZoomModelTool.superclass.constructor.call(this, modelToolsController);
    var me = this;
    me.sProperty = "zoomRatio";
    me.sIsModifiableMethod = "isZoomRatioModifiable";
}
extend(ZoomModelTool, ListedCommandModelTool);

ZoomModelTool.prototype.id = ModelTool.prototype.TOOL_ZOOM;

ZoomModelTool.prototype.DEFAULT_PROPERTY_VALUE = "1";

ZoomModelTool.prototype.aElement = [
    {zoomRatio:0.25}, {zoomRatio:0.50},
    {zoomRatio:1.00},
    {zoomRatio:2.00}, {zoomRatio:4.00}
];
(function(){
    var a = ZoomModelTool.prototype.aElement;
    var i, l = a.length, obj;
    for(i=0; i<l; i++){
        obj= a[i];
        obj.sample = obj.panelSample = ""+Math.ceil(100*obj.zoomRatio)+"%";
        obj.italic = false;
        obj.bold = true;
        obj.size = 18;
        obj.face = "Arial";
        obj.color = "#000000";
    }
})();

//override
ZoomModelTool.prototype.configure = function(objToolData){
    var me = this;
    ZoomModelTool.superclass.configure.call(me, objToolData);
};

//override
ZoomModelTool.prototype.getPropertyValueByElementNumber = function(q){
    var me = this;
    var aElement = me.aElement;
    if(q>=0 && q<aElement.length)
        return aElement[q].zoomRatio;
    return 1.0;
};

//override
ZoomModelTool.prototype.applyCommand = function(mapSceneModel, options) {
    var me = this;
    var v = me.getPropertyValueByElementNumber(options.data.buttonNumber);
    if(v != mapSceneModel.getZoomRatio()){
        mapSceneModel.setZoomRatio(v);
        mapSceneModel.invalidate();
    }

};
/**
 * @constructor
 */
function ConfirmModelTool(modelToolsController){
    ConfirmModelTool.superclass.constructor.call(this, modelToolsController);
    //resets the whole model to its initial state
    var me = this;
    me.flagStarted = true;
    me.flagCommand = true;
}
extend(ConfirmModelTool, ModelTool);

ConfirmModelTool.prototype.id = ModelTool.prototype.TOOL_CONFIRM;

//override
ConfirmModelTool.prototype.applyCommand = function(mapSceneModel, options){
    mapSceneModel.increaseAttemptsCount();
    var objVerification = mapSceneModel.verifyElements();
    var correctPercentage = objVerification.correctPercentage; //0

    var sMessage = null;
    switch(objVerification.evaluationType){
        case "hard":
            if(objVerification.extra>0) {
                sMessage = miscFunctions.unescapeEntities(
                    document.localization.getString(
                        "interface.dialogs.messages.remove-extra-objects",
                        "Ответ не принят –<br/>создано избыточное количество объектов.<br/>Удалите лишние объекты или начните заново."
                    )
                );
            }
            break;
        default:
    }
    mapSceneModel.showResponse(100*correctPercentage, {message:sMessage});
    //all the verifiable elements are checked hence cmi.completion_status = complete
    SCORM.setDataValue("cmi.success_status", (objVerification.successStatus == "failed") ? "failed" : "passed");
    SCORM.setDataValue("cmi.completion_status", "completed");
    SCORM.setDataValue("cmi.progress_measure", 1.0);
    SCORM.setDataValue("cmi.score.scaled", (objVerification.successStatus == "failed") ? "0.0000000" : correctPercentage.toFixed(7));
    SCORM.commitData();
    //
    mapSceneModel.notifyOfElementsVerification(objVerification);
};

/**
 * @constructor
 */
function ShowCorrectModelTool(modelToolsController){
    ShowCorrectModelTool.superclass.constructor.call(this, modelToolsController);
    //highlights correctly set verifiable elements
    var me = this;
    me.flagStarted = true;
    me.flagCommand = true;
    //
    me.prevAttemptsCount = -1;
    me.aBlinkId = [];
    me.aBlinkGreen = [];
    me.flagSelfTest = false;
}
extend(ShowCorrectModelTool, ModelTool);

ShowCorrectModelTool.prototype.id = ModelTool.prototype.TOOL_SHOW_CORRECT;

ShowCorrectModelTool.prototype.configure = function(objToolData){
    //for configuring the tool according to the data stored in the model
    this.flagSelfTest = (typeof objToolData["selfTest"] != TYPE_UNDEFINED) ? (objToolData["selfTest"] == "true") : false;
};

//override
ShowCorrectModelTool.prototype.applyCommand = function(mapSceneModel, options){
    var me = this;
    mapSceneModel.unblinkAll();
    var attemptsCount = mapSceneModel.getAttemptsCount();
    var i, n;
    if(!me.flagSelfTest && me.prevAttemptsCount == attemptsCount){
        //to prevent cheating by repeatedly using "show correct" button, only first use results in verification
        //whereas subsequent ones will yield the same cached result until another attempt is made
        n = me.aBlinkId.length;
        for(i=0; i<n; i++){
            if(me.aBlinkId[i] != -1) {
                element = mapSceneModel.getElementById(me.aBlinkId[i]);
                if (element != null) {
                    if(me.aBlinkGreen[i])
                        element.setShowCorrect(true);
                    else
                        element.setShowIncorrect(true);
                } else
                    me.aBlinkId[i] = -1;
            }
        }
    }else {
        var objVerification = mapSceneModel.verifyElements();
        me.aBlinkId.splice(0);
        me.prevAttemptsCount = attemptsCount;
        if (objVerification.correct > 0 || objVerification.aReferenceMismatch.length>0) {
            var aa = [objVerification.aReferenceMatch, objVerification.aReferenceMismatch];
            var ii, a, flag;
            var element;
            for(ii=0; ii<2; ii++){
                a = aa[ii];
                n = a.length;
                flag = (ii==0);
                for(i=0; i<n; i++){
                    if(a[i] == null)
                        continue;
                    element = mapSceneModel.getElementById(a[i]);
                    if(ii==0)
                        element.setShowCorrect(true);
                    else
                        element.setShowIncorrect(true);
                    me.aBlinkId.push(a[i]);
                    me.aBlinkGreen.push(flag);
                }
            }
        }
    }
};
/**
 * @constructor
 */
function ShowHintModelTool(modelToolsController){
    ShowHintModelTool.superclass.constructor.call(this, modelToolsController);
    //highlights correctly set verifiable elements
    var me = this;
    me.flagStarted = true;
    me.flagCommand = true;
}
extend(ShowHintModelTool, ModelTool);

ShowHintModelTool.prototype.id = ModelTool.prototype.TOOL_SHOW_HINT;

//override
ShowHintModelTool.prototype.applyCommand = function(mapSceneModel, options){
    var me = this;
    var modelToolsController = me.getModelToolsController();

    mapSceneModel.showHint();
};
/**
 * @constructor
 */
function ArrowModelLibrary(){
}
extend(ArrowModelLibrary, Object);

//
ArrowModelLibrary.prototype.aSizeParameters =[
    {
        symbolId:"mcSymbolAN_Size1", tailWidth:3, headWidth:12, headLength:12,
        //the arrow being modified scales down to zero linearly with its length if the latter is below the scaleLength
        scaleLength:24
    },
    {
        symbolId:"mcSymbolAN_Size0", tailWidth:10, headWidth:20, headLength:20,
        scaleLength:40
    },
    {
        symbolId:"mcSymbolAN_Size2", tailWidth:28, headWidth:48, headLength:48,
        scaleLength:100
    },

    {
        symbolId:"mcSymbolAP_Size1", tailWidth:3, headWidth:12, headLength:12,
        scaleLength:24,
        headDepression:3
    },
    {
        symbolId:"mcSymbolAP_Size0", tailWidth:10, headWidth:20, headLength:20,
        scaleLength:40,
        headDepression:3
    },
    {
        symbolId:"mcSymbolAP_Size2", tailWidth:28, headWidth:48, headLength:48,
        scaleLength:100,
        headDepression:3
    },

    {
        symbolId:"mcSymbolAD_Size1", tailWidth:3, headWidth:12, headLength:12,
        scaleLength:24,
        dashPeriod:20, dashLength:13
    },
    {
        symbolId:"mcSymbolAD_Size0", tailWidth:10, headWidth:20, headLength:20,
        scaleLength:40,
        dashPeriod:20, dashLength:13
    },
    {
        symbolId:"mcSymbolAD_Size2", tailWidth:28, headWidth:48, headLength:48,
        scaleLength:100,
        dashPeriod:20, dashLength:13
    },

    {
        symbolId:"mcSymbolAQ_Size1", tailWidth:3, headWidth:12, headLength:12,
        scaleLength:24,
        dashPeriod:20, dashLength:13,
        headDepression:3
    },
    {
        symbolId:"mcSymbolAQ_Size0", tailWidth:10, headWidth:20, headLength:20,
        scaleLength:40,
        dashPeriod:20, dashLength:13,
        headDepression:3
    },
    {
        symbolId:"mcSymbolAQ_Size2", tailWidth:28, headWidth:48, headLength:48,
        scaleLength:100,
        dashPeriod:20, dashLength:13,
        headDepression:3
    },

    {
        symbolId:"mcSymbolAR_Size1", headWidth:12, headLength:12,
        headWedge:4, tailWedge:8,
        scaleLength:24,
        tailCleavage:5,
        headDepression:3
    },
    {
        symbolId:"mcSymbolAR_Size0", headWidth:20, headLength:20,
        headWedge:10, tailWedge: 20,
        scaleLength:40,
        tailCleavage:10,
        headDepression:3
    },
    {
        symbolId:"mcSymbolAR_Size2", headWidth:48, headLength:48,
        headWedge:28, tailWedge:56,
        scaleLength:100,
        tailCleavage:20,
        headDepression:3
    }
];

//
ArrowModelLibrary.prototype.getSizeById = function(sId){
    var a = ArrowModelLibrary.prototype.aSizeParameters, i;
    var n = a.length;
    for(i=0; i<n; i++)
        if(a[i].symbolId == sId)
            break;
    if(i==n)
        return null;
    var obj = a[i];
    var objSize = {};
    var attr;
    for(attr in obj)
        objSize[attr] = obj[attr];
    return objSize;
};

//
ArrowModelLibrary.prototype.getArrowOutlineArray = function(element){
    var xFrom = element.getXFrom();
    var yFrom = element.getYFrom();
    var xTo = element.getXTo();
    var yTo = element.getYTo();

    var dX = xTo-xFrom;
    var dY = yTo-yFrom;

    var lArrow = Math.sqrt(dX*dX+dY*dY);
    var cArrow;
    var sArrow;
    if(lArrow<1E-10){
        cArrow = 1;
        sArrow = 0;
    }else{
        cArrow = dY/lArrow;
        sArrow = dX/lArrow;
    }

    var objSize = ArrowModelLibrary.prototype.getSizeById(element.getSymbolId());
    var scaleLength = (typeof objSize.scaleLength == TYPE_NUMBER) ? objSize.scaleLength : 0;
    var scaleFactor = (lArrow>scaleLength) ? 1 : (lArrow/scaleLength);

    var headWidth = objSize.headWidth*scaleFactor;
    var headLength = objSize.headLength*scaleFactor;
    var headDepression = ((typeof objSize.headDepression == TYPE_NUMBER) ?  objSize.headDepression : 0)*scaleFactor;

    var flagWedgelike = typeof objSize.headWedge == TYPE_NUMBER;
    var headWedge;
    var tailWedge;
    if(flagWedgelike){
        headWedge = objSize.headWedge*scaleFactor;
        tailWedge = objSize.tailWedge*scaleFactor;
    }else{
        headWedge = objSize.tailWidth*scaleFactor;
        tailWedge = objSize.tailWidth*scaleFactor;
    }

    var flagCleaved = typeof objSize.tailCleavage == TYPE_NUMBER;
    var tailCleavage = flagCleaved ? (objSize.tailCleavage*scaleFactor) : 0;

    //now calculate points to be joined for the arrow outline
    //in unrotated and unscaled coords
    var aPoint = [new Point(0, 0)];

    aPoint.push(new Point(-tailWedge/2, -tailCleavage));
    aPoint.push(new Point(-headWedge/2, lArrow-headLength+headDepression));
    aPoint.push(new Point(-headWidth/2, lArrow-headLength));
    aPoint.push(new Point(0, lArrow));
    aPoint.push(new Point(headWidth/2, lArrow-headLength));
    aPoint.push(new Point(headWedge/2, lArrow-headLength+headDepression));
    aPoint.push(new Point(tailWedge/2, -tailCleavage));

    var i, n = aPoint.length, p;

    for(i=0; i<n; i++){
        p = aPoint[i];
        aPoint[i] = new Point(p.x*cArrow+p.y*sArrow, -p.x*sArrow+ p.y*cArrow);
    }

    return aPoint;
};
/**
 * @constructor
 */
function CheckboxSetModelLibrary(){
    CheckboxSetModelLibrary.superclass.constructor.call(this, CheckboxSetModelLibrary.prototype.iconStorage);
}
extend(CheckboxSetModelLibrary, AbstractSVGLibrary);

CheckboxSetModelLibrary.prototype.iconStorage = [
    {
        id:"checkbox.unselected",
        xShift:-48, yShift:-48,
        data:'<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"	 width="96px" height="96px" viewBox="0 0 96 96" enable-background="new 0 0 96 96" xml:space="preserve"><g>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.6314" d="M40.35,56l0.55-0.7c0.033-0.066,0.05-0.133,0.05-0.2V40.95		H55.4c0.033,0,0.083-0.017,0.149-0.05c0.033-0.033,0.066-0.05,0.101-0.05l0.35-0.4v0.45l-0.15,0.15		c-0.066,0.034-0.133,0.083-0.199,0.15c-0.067,0.033-0.15,0.05-0.25,0.05H41.25V55.1c0,0.134-0.033,0.25-0.1,0.351L40.75,56H40.35z"		/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.3137" d="M56,41.7c-0.033,0-0.066,0.017-0.1,0.05		c-0.167,0.067-0.334,0.1-0.5,0.1H41.85V55.1c0,0.267-0.083,0.517-0.25,0.75L41.5,56h-0.4l0.3-0.35c0.1-0.167,0.15-0.351,0.15-0.551		V41.55H55.4c0.133,0,0.267-0.033,0.399-0.1c0.066-0.034,0.134-0.067,0.2-0.1V41.7z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.4745" d="M40.75,56l0.4-0.55c0.067-0.101,0.1-0.217,0.1-0.351V41.25		H55.4c0.1,0,0.183-0.017,0.25-0.05c0.066-0.067,0.133-0.117,0.199-0.15L56,40.9v0.45c-0.066,0.033-0.134,0.066-0.2,0.1		c-0.133,0.066-0.267,0.1-0.399,0.1H41.55V55.1c0,0.2-0.05,0.384-0.15,0.551L41.1,56H40.75z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.7961" d="M56,40.45l-0.35,0.4c-0.034,0-0.067,0.017-0.101,0.05		c-0.066,0.033-0.116,0.05-0.149,0.05H40.95V55.1c0,0.067-0.017,0.134-0.05,0.2L40.35,56H40V40h16V40.45z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.1569" d="M56,42.05c-0.2,0.067-0.4,0.1-0.6,0.1H42.15V55.1		c0,0.334-0.1,0.634-0.3,0.9H41.5l0.1-0.15c0.167-0.233,0.25-0.483,0.25-0.75V41.85H55.4c0.166,0,0.333-0.033,0.5-0.1		c0.033-0.033,0.066-0.05,0.1-0.05V42.05z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill="#FFFFFF" d="M41.85,56c0.2-0.267,0.3-0.566,0.3-0.9V42.15H55.4		c0.199,0,0.399-0.033,0.6-0.1V56H41.85z"/>			<path fill-rule="evenodd" clip-rule="evenodd" fill="none" stroke="#99CCFF" stroke-linecap="round" stroke-linejoin="round" stroke-miterlimit="3" d="		M56,40v0.45 M56,41.35v0.35 M56,42.05V56H41.85"/></g></svg>'
    },
    {
        id:"checkbox.selected",
        xShift:-48, yShift:-48,
        data:'<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"	 width="96px" height="96px" viewBox="0 0 96 96" enable-background="new 0 0 96 96" xml:space="preserve"><g>	<path fill-rule="evenodd" clip-rule="evenodd" fill="#FFFFFF" d="M41.85,56c0.2-0.267,0.3-0.566,0.3-0.9V42.15H55.4		c0.199,0,0.399-0.033,0.6-0.1V56H41.85z M52.45,44.7l-5.4,6l-2.6-2.9v3.25l2.65,2.75l5.35-5.75V44.7z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.6314" d="M40.35,56l0.55-0.7c0.033-0.066,0.05-0.133,0.05-0.2V40.95		H55.4c0.033,0,0.083-0.017,0.149-0.05c0.033-0.033,0.066-0.05,0.101-0.05l0.35-0.4v0.45l-0.15,0.15		c-0.066,0.034-0.133,0.083-0.199,0.15c-0.067,0.033-0.15,0.05-0.25,0.05H41.25V55.1c0,0.134-0.033,0.25-0.1,0.351L40.75,56H40.35z"		/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.1569" d="M56,42.05c-0.2,0.067-0.4,0.1-0.6,0.1H42.15V55.1		c0,0.334-0.1,0.634-0.3,0.9H41.5l0.1-0.15c0.167-0.233,0.25-0.483,0.25-0.75V41.85H55.4c0.166,0,0.333-0.033,0.5-0.1		c0.033-0.033,0.066-0.05,0.1-0.05V42.05z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.3137" d="M41.1,56l0.3-0.35c0.1-0.167,0.15-0.351,0.15-0.551V41.55		H55.4c0.133,0,0.267-0.033,0.399-0.1c0.066-0.034,0.134-0.067,0.2-0.1v0.35c-0.033,0-0.066,0.017-0.1,0.05		c-0.167,0.067-0.334,0.1-0.5,0.1H41.85V55.1c0,0.267-0.083,0.517-0.25,0.75L41.5,56H41.1z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.4745" d="M40.75,56l0.4-0.55c0.067-0.101,0.1-0.217,0.1-0.351V41.25		H55.4c0.1,0,0.183-0.017,0.25-0.05c0.066-0.067,0.133-0.117,0.199-0.15L56,40.9v0.45c-0.066,0.033-0.134,0.066-0.2,0.1		c-0.133,0.066-0.267,0.1-0.399,0.1H41.55V55.1c0,0.2-0.05,0.384-0.15,0.551L41.1,56H40.75z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.7961" d="M56,40.45l-0.35,0.4c-0.034,0-0.067,0.017-0.101,0.05		c-0.066,0.033-0.116,0.05-0.149,0.05H40.95V55.1c0,0.067-0.017,0.134-0.05,0.2L40.35,56H40V40h16V40.45z"/>	<polygon fill-rule="evenodd" clip-rule="evenodd" points="52.45,44.7 52.45,48.05 47.1,53.8 44.45,51.05 44.45,47.8 47.05,50.7 			"/>			<path fill-rule="evenodd" clip-rule="evenodd" fill="none" stroke="#99CCFF" stroke-linecap="round" stroke-linejoin="round" stroke-miterlimit="3" d="		M56,40v0.45 M56,41.35v0.35 M56,42.05V56H41.85"/></g></svg>'
    },
    {
        id:"radiobutton.unselected",
        xShift:-48, yShift:-48,
        data:'<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"	 width="96px" height="96px" viewBox="0 0 96 96" enable-background="new 0 0 96 96" xml:space="preserve"><g>	<path fill-rule="evenodd" clip-rule="evenodd" fill="#FFFFFF" d="M55.15,47.45c0,0.233,0,0.483,0,0.75c0,0.267,0,0.517,0,0.75		c-0.034,0.267-0.084,0.517-0.15,0.75l-0.1,0.649c-0.334,1.134-0.934,2.15-1.801,3.051c-1.366,1.366-3,2.083-4.899,2.149		c-1.9-0.066-3.517-0.783-4.85-2.149c-0.9-0.9-1.517-1.917-1.85-3.051l-0.3-1.95c0-0.066,0-0.133,0-0.2c0-0.1,0-0.184,0-0.25		l0.15-1.3c0.3-1.367,0.967-2.583,2-3.65c1.333-1.367,2.95-2.083,4.85-2.15C50.1,40.917,51.733,41.633,53.1,43		c0.867,0.9,1.467,1.917,1.801,3.05L55,46.7C55.066,46.933,55.116,47.183,55.15,47.45z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill="#666666" d="M48.2,39.2c2.467,0,4.583,0.883,6.35,2.65s2.65,3.883,2.65,6.35		s-0.884,4.583-2.65,6.35s-3.883,2.65-6.35,2.65s-4.583-0.884-6.35-2.65s-2.65-3.883-2.65-6.35s0.883-4.583,2.65-6.35		S45.733,39.2,48.2,39.2z M55.15,48.95l0.05-0.55c0-0.066,0-0.133,0-0.2c0-0.067,0-0.117,0-0.15l-0.05-0.6		c-0.034-0.267-0.084-0.517-0.15-0.75l-0.1-0.65C54.566,44.917,53.967,43.9,53.1,43c-1.366-1.367-3-2.083-4.899-2.15		c-1.9,0.066-3.517,0.783-4.85,2.15c-1.033,1.067-1.7,2.283-2,3.65l-0.15,1.3c0,0.066,0,0.15,0,0.25c0,0.066,0,0.133,0,0.2l0.3,1.95		c0.333,1.134,0.95,2.15,1.85,3.051c1.333,1.366,2.95,2.083,4.85,2.149c1.899-0.066,3.533-0.783,4.899-2.149		c0.867-0.9,1.467-1.917,1.801-3.051L55,49.7C55.066,49.467,55.116,49.217,55.15,48.95z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.6627" d="M55.15,47.45l0.05,0.6c0,0.034,0,0.083,0,0.15		c0,0.066,0,0.133,0,0.2l-0.05,0.55c0-0.233,0-0.483,0-0.75C55.15,47.933,55.15,47.683,55.15,47.45z"/></g></svg>'
    },
    {
        id:"radiobutton.selected",
        xShift:-48, yShift:-48,
        data:'<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"	 width="96px" height="96px" viewBox="0 0 96 96" enable-background="new 0 0 96 96" xml:space="preserve"><g>	<path fill-rule="evenodd" clip-rule="evenodd" fill="#FFFFFF" d="M55.15,47.45c0,0.233,0,0.483,0,0.75c0,0.267,0,0.517,0,0.75		c-0.034,0.267-0.084,0.517-0.15,0.75l-0.1,0.649c-0.334,1.134-0.934,2.15-1.801,3.051c-1.366,1.366-3,2.083-4.899,2.149		c-1.9-0.066-3.517-0.783-4.85-2.149c-0.9-0.9-1.517-1.917-1.85-3.051l-0.3-1.95c0-0.066,0-0.133,0-0.2c0-0.1,0-0.184,0-0.25		l0.15-1.3c0.3-1.367,0.967-2.583,2-3.65c1.333-1.367,2.95-2.083,4.85-2.15C50.1,40.917,51.733,41.633,53.1,43		c0.867,0.9,1.467,1.917,1.801,3.05L55,46.7C55.066,46.933,55.116,47.183,55.15,47.45z M45,45c-0.9,0.9-1.333,1.967-1.3,3.2		c-0.034,1.267,0.4,2.316,1.3,3.149c0.9,0.9,1.967,1.351,3.2,1.351s2.283-0.45,3.149-1.351c0.9-0.833,1.351-1.883,1.351-3.149		c0-1.233-0.45-2.3-1.351-3.2c-0.866-0.867-1.916-1.3-3.149-1.3S45.9,44.133,45,45z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill="#666666" d="M55.15,48.95l0.05-0.55c0-0.066,0-0.133,0-0.2		c0-0.067,0-0.117,0-0.15l-0.05-0.6c-0.034-0.267-0.084-0.517-0.15-0.75l-0.1-0.65C54.566,44.917,53.967,43.9,53.1,43		c-1.366-1.367-3-2.083-4.899-2.15c-1.9,0.066-3.517,0.783-4.85,2.15c-1.033,1.067-1.7,2.283-2,3.65l-0.15,1.3		c0,0.066,0,0.15,0,0.25c0,0.066,0,0.133,0,0.2l0.3,1.95c0.333,1.134,0.95,2.15,1.85,3.051c1.333,1.366,2.95,2.083,4.85,2.149		c1.899-0.066,3.533-0.783,4.899-2.149c0.867-0.9,1.467-1.917,1.801-3.051L55,49.7C55.066,49.467,55.116,49.217,55.15,48.95z		 M48.2,39.2c2.467,0,4.583,0.883,6.35,2.65s2.65,3.883,2.65,6.35s-0.884,4.583-2.65,6.35s-3.883,2.65-6.35,2.65		s-4.583-0.884-6.35-2.65s-2.65-3.883-2.65-6.35s0.883-4.583,2.65-6.35S45.733,39.2,48.2,39.2z"/>	<path fill-rule="evenodd" clip-rule="evenodd" fill-opacity="0.6627" d="M55.15,47.45l0.05,0.6c0,0.034,0,0.083,0,0.15		c0,0.066,0,0.133,0,0.2l-0.05,0.55c0-0.233,0-0.483,0-0.75C55.15,47.933,55.15,47.683,55.15,47.45z"/>	<radialGradient id="SVGID_1_" cx="46.5" cy="46.2002" r="7.65" gradientUnits="userSpaceOnUse">		<stop  offset="0" style="stop-color:#B4CBFE"/>		<stop  offset="1" style="stop-color:#003399"/>	</radialGradient>	<path fill-rule="evenodd" clip-rule="evenodd" fill="url(#SVGID_1_)" d="M45,45c0.9-0.867,1.967-1.3,3.2-1.3s2.283,0.433,3.149,1.3		c0.9,0.9,1.351,1.967,1.351,3.2c0,1.267-0.45,2.316-1.351,3.149c-0.866,0.9-1.916,1.351-3.149,1.351s-2.3-0.45-3.2-1.351		c-0.9-0.833-1.333-1.883-1.3-3.149C43.667,46.967,44.1,45.9,45,45z M50.55,45.8c-0.633-0.633-1.416-0.95-2.35-0.95		s-1.733,0.316-2.4,0.95c-0.633,0.667-0.95,1.467-0.95,2.4s0.316,1.717,0.95,2.35c0.667,0.7,1.467,1.05,2.4,1.05		s1.717-0.35,2.35-1.05c0.7-0.633,1.05-1.416,1.05-2.35S51.25,46.467,50.55,45.8z"/>	<radialGradient id="SVGID_2_" cx="50.0996" cy="49.9502" r="6.025" gradientUnits="userSpaceOnUse">		<stop  offset="0" style="stop-color:#B4CBFE"/>		<stop  offset="1" style="stop-color:#003399"/>	</radialGradient>	<path fill-rule="evenodd" clip-rule="evenodd" fill="url(#SVGID_2_)" d="M50.55,45.8c0.7,0.667,1.05,1.467,1.05,2.4		s-0.35,1.717-1.05,2.35c-0.633,0.7-1.416,1.05-2.35,1.05s-1.733-0.35-2.4-1.05c-0.633-0.633-0.95-1.416-0.95-2.35		s0.316-1.733,0.95-2.4c0.667-0.633,1.467-0.95,2.4-0.95S49.917,45.167,50.55,45.8z"/></g></svg>'
    }
];
