/**
 * @constructor
 */
function MapSceneModel(_modelResource){
    MapSceneModel.superclass.constructor.call(this);
	var me = this;
    me.modelResource = miscFunctions.cloneObject(_modelResource);
    me.zoomRatio = parseFloat(me.modelResource[ModelResource.prototype.CUSTOMIZE_PLAYER_PROPERTY]["ZoomList"]["zoomRatio"]);
    //
    //массив ссылок на элементы, упорядоченные по глубине
    me.aElementByDepth = null;
    me.aGroup = [];
    me.fFillElements(me.modelResource[ModelResource.prototype.OBJECTS_PROPERTY]);
    me.sceneWidth = document.localization.getParameter("model-width", 800);
    me.sceneHeight = document.localization.getParameter("model-height", 600);
    me.flagValid = false;
    //рамка масштабирования
    me.scaleFrameModel = new ScaleFrameModel(me);
    //scrollbars in case images exceed map scene dimensions
    var node = me.modelResource.nodeToolbar;
    me.flagHScrollBar = false;
    me.flagVScrollBar = false;
    me.flagNeedHScrollBar = false;
    me.flagNeedVScrollBar = false;
    //
    var vScrollerProps = {
        target:me,
        property:"originY",
        vertical:true
    };
    me.vScrollBar = new ScrollerModel(vScrollerProps);
    //
    var hScrollerProps = {
        target:me,
        property:"originX",
        vertical:false
    };
    me.hScrollBar = new ScrollerModel(hScrollerProps);
    //scene shift using scrollers
    me.origin = new Point(0, 0);
    me.originTL = new Point(0,0);
    me.originBR = new Point(0,0);
    //
    if(node.hasChildNodes()){
        var cNodes = node.getElementsByTagName("ShowHScroll");
        if(cNodes.length >0)
            me.flagHScrollBar = (cNodes[0].getAttribute("on") == "true");
        cNodes = node.getElementsByTagName("ShowVScroll");
        if(cNodes.length >0)
            me.flagVScrollBar = (cNodes[0].getAttribute("on") == "true");
    }
    //
    me.attemptsCount = 0;
    me.startTime = 0;
    //
    MapSceneModel.prototype.CLOSE_CAPTION =
        document.localization.getString("interface.dialogs.button-captions.close", MapSceneModel.prototype.CLOSE_CAPTION);
}
extend(MapSceneModel, NonDOMEventDispatcher);

MapSceneModel.prototype.SCROLL_RATIO = 12;

MapSceneModel.prototype.SCROLL_MARGIN_VERTICAL_THRESHOLD = 1;
MapSceneModel.prototype.SCROLL_MARGIN_HORIZONTAL_THRESHOLD = 1;

MapSceneModel.prototype.HINT_WINDOW_WIDTH = 536;
MapSceneModel.prototype.HINT_WINDOW_HEIGHT = 393;
MapSceneModel.prototype.HINT_TEXT_WIDTH = 420;
MapSceneModel.prototype.HINT_TEXT_HEIGHT = 293;

MapSceneModel.prototype.RESPONSE_WINDOW_WIDTH = 296;
MapSceneModel.prototype.RESPONSE_WINDOW_HEIGHT = 212;
MapSceneModel.prototype.RESPONSE_TEXT_WIDTH = 280;
MapSceneModel.prototype.RESPONSE_TEXT_HEIGHT = 112;

MapSceneModel.prototype.CLOSE_CAPTION = "Закрыть";

MapSceneModel.prototype.aMethodNameForProperty =
    miscFunctions.cloneObject(MapSceneModel.superclass.aMethodNameForProperty);

MapSceneModel.prototype.setMethodNameForProperty(
    MapSceneModel.prototype.aMethodNameForProperty, "originX", false, "getOriginX"
);
MapSceneModel.prototype.setMethodNameForProperty(
    MapSceneModel.prototype.aMethodNameForProperty, "originX", true, "setOriginX"
);

MapSceneModel.prototype.setMethodNameForProperty(
    MapSceneModel.prototype.aMethodNameForProperty, "originY", false, "getOriginY"
);
MapSceneModel.prototype.setMethodNameForProperty(
    MapSceneModel.prototype.aMethodNameForProperty, "originY", true, "setOriginY"
);

//
MapSceneModel.prototype.getZoomRatio = function(){
    return this.zoomRatio;
};

MapSceneModel.prototype.setZoomRatio = function(v){
    this.zoomRatio = v;
    ModelElement.prototype.ZOOM_RATIO = this.zoomRatio;
};

//
MapSceneModel.prototype.getStartTime = function(){
    return this.startTime;
};

//
MapSceneModel.prototype.setStartTime = function(t){
    this.startTime = t;
};

//
MapSceneModel.prototype.deactivate = function(){
    MapSceneModel.superclass.deactivate.call(this);
    var me = this;
    var i;
    for(i=0; i<me.aGroup.length; i++)
        me.aGroup[i].removeAllElements();
    me.aGroup.splice(0);
    for(i=0; i<me.aElementByDepth.length; i++)
        me.aElementByDepth[i].deactivate();
};

MapSceneModel.prototype.isValid = function(){
    return this.flagValid;
};

MapSceneModel.prototype.invalidate = function(){
    if(this.flagValid)
        this.flagValid = false;
};

MapSceneModel.prototype.validate = function(_flagEvent){
    var me = this;
    var flagEvent = (typeof _flagEvent != TYPE_BOOLEAN) ? true : _flagEvent;
    var zoom  = me.zoomRatio;
    var i, n = me.aElementByDepth.length, j, element;
    for(j=0; j<n; j++)
        if(!me.aElementByDepth[j].isValid())
            break;
    var flagValid = me.flagValid && me.scaleFrameModel.isValid() && (j==n);
    if(flagValid) {
        for (i = 0; i < n; i++) {
            element = me.aElementByDepth[i];
            if ((element.isSelected() && element.isBlinkingFrameWhenSelected()) || element.isBlinking()) {
                //for selection frame flinking
                flagValid = false;
                break;
            }
        }
    }

    if(flagValid && me.flagVScrollBar && me.flagNeedVScrollBar && !me.vScrollBar.isValid())
        flagValid = false;
    if(flagValid && me.flagHScrollBar && me.flagNeedHScrollBar && !me.hScrollBar.isValid())
        flagValid = false;

    if(!flagValid){
        me.scaleFrameModel.validate();
        if(j<n)
            for (j = 0; j < n; j++)
                me.aElementByDepth[j].validate();

        var margin, wScroller = 0;
        var scrollMarginLeft = Number.NEGATIVE_INFINITY;
        var scrollMarginRight = Number.NEGATIVE_INFINITY;
        var scrollMarginTop = Number.NEGATIVE_INFINITY;
        var scrollMarginBottom = Number.NEGATIVE_INFINITY;

        var sceneHeightZoom = me.sceneHeight/zoom, sceneWidthZoom = me.sceneWidth/zoom;

        var prevNeedVScrollBar = me.flagNeedVScrollBar;
        var prevNeedHScrollBar = me.flagNeedHScrollBar;

        function fUpdateMargins() {
            var bounds, aBounds = [{x: 0, y: 0, width: me.sceneWidth, height: me.sceneHeight}];
            for (i = 0; i < me.aElementByDepth.length; i++) {
                element = me.aElementByDepth[i];
                if (element.isVisible() && element.isScrollerProvoking())
                    aBounds.push(element.getBounds());
            }

            scrollMarginLeft = Number.NEGATIVE_INFINITY;
            scrollMarginRight = Number.NEGATIVE_INFINITY;
            scrollMarginTop = Number.NEGATIVE_INFINITY;
            scrollMarginBottom = Number.NEGATIVE_INFINITY;

            me.flagNeedVScrollBar = false;
            me.flagNeedHScrollBar = false;

            //are scrollers needed?
            var scrollPaddingLeft = me.origin.x;
            var scrollPaddingRight = sceneWidthZoom - me.sceneWidth - me.origin.x;
            var scrollPaddingTop = me.origin.y;
            var scrollPaddingBottom = sceneHeightZoom - me.sceneHeight - me.origin.y;

            if (me.flagVScrollBar) {
                for (i = 0; i < aBounds.length; i++) {
                    bounds = aBounds[i];
                    margin = -me.origin.y - bounds.y;
                    if (margin > scrollMarginTop)
                        scrollMarginTop = margin;
                    else if (margin < 0 && -margin < scrollPaddingTop)
                        scrollPaddingTop = -margin;

                    margin = (bounds.y + bounds.height) - (sceneHeightZoom - me.origin.y);
                    if (margin > scrollMarginBottom)
                        scrollMarginBottom = margin;
                    else if (margin < 0 && -margin < scrollPaddingBottom)
                        scrollPaddingBottom = -margin;
                }
                me.flagNeedVScrollBar =
                    (
                        scrollMarginTop > me.SCROLL_MARGIN_VERTICAL_THRESHOLD ||
                        scrollMarginBottom > me.SCROLL_MARGIN_VERTICAL_THRESHOLD
                    );
            }

            if(me.flagNeedVScrollBar)
                aBounds[0].width -= ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH/zoom;

            wScroller = me.flagNeedVScrollBar ? ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH : 0;

            if (me.flagHScrollBar) {
                for (i = 0; i < aBounds.length; i++) {
                    bounds = aBounds[i];
                    margin = -me.origin.x - bounds.x;
                    if (margin > scrollMarginLeft)
                        scrollMarginLeft = margin;
                    else if (margin < 0 && -margin < scrollPaddingLeft)
                        scrollPaddingLeft = -margin;

                    margin = (bounds.x + bounds.width) - (sceneWidthZoom - wScroller / zoom - me.origin.x);
                    if (margin > scrollMarginRight)
                        scrollMarginRight = margin;
                    else if (margin < 0 && -margin < scrollPaddingRight)
                        scrollPaddingRight = -margin;
                }
                me.flagNeedHScrollBar =
                    (
                        scrollMarginLeft > me.SCROLL_MARGIN_HORIZONTAL_THRESHOLD ||
                        scrollMarginRight > me.SCROLL_MARGIN_HORIZONTAL_THRESHOLD
                    );
            }

            if(me.flagNeedVScrollBar)
                aBounds[0].height -= ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH/zoom;

            if (me.flagNeedHScrollBar && me.flagVScrollBar) {
                //recalculate flagNeedVScrollBar and bottom margin because
                //the scene has become lower due to horizontal scroller
                wScroller = ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH;
                for (i = 0; i < aBounds.length; i++) {
                    bounds = aBounds[i];
                    margin = (bounds.y + bounds.height) - (sceneHeightZoom - wScroller / zoom - me.origin.y);
                    if (margin > scrollMarginBottom)
                        scrollMarginBottom = margin;
                    else if (margin < 0 && -margin < scrollPaddingBottom)
                        scrollPaddingBottom = -margin;
                }
                me.flagNeedVScrollBar =
                    (
                        scrollMarginTop > me.SCROLL_MARGIN_VERTICAL_THRESHOLD ||
                        scrollMarginBottom > me.SCROLL_MARGIN_VERTICAL_THRESHOLD
                    );
            }

            if (me.flagNeedVScrollBar)
                scrollPaddingBottom -= ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH / zoom;
            if (me.flagNeedHScrollBar)
                scrollPaddingRight -= ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH / zoom;

            if (scrollPaddingLeft < 0 || scrollMarginLeft >= -me.SCROLL_MARGIN_HORIZONTAL_THRESHOLD)
                scrollPaddingLeft = 0;
            if (scrollPaddingRight < 0 || scrollMarginRight >= -me.SCROLL_MARGIN_HORIZONTAL_THRESHOLD)
                scrollPaddingRight = 0;
            if (scrollPaddingTop < 0 || scrollMarginTop >= -me.SCROLL_MARGIN_VERTICAL_THRESHOLD)
                scrollPaddingTop = 0;
            if (scrollPaddingBottom < 0 || scrollMarginBottom >= -me.SCROLL_MARGIN_VERTICAL_THRESHOLD)
                scrollPaddingBottom = 0;

            var flagV = false, flagH = false;

            if (me.flagNeedVScrollBar) {
                if (scrollPaddingBottom > 0 && scrollMarginTop > 0) {
                    me.origin.y += (scrollPaddingBottom < scrollMarginTop) ? scrollPaddingBottom : scrollMarginTop;
                    flagV = true;
                } else if (scrollPaddingTop > 0 && scrollMarginBottom > 0) {
                    me.origin.y -= (scrollPaddingTop < scrollMarginBottom) ? scrollPaddingTop : scrollMarginBottom;
                    flagV = true;
                }
            }

            if (me.flagNeedHScrollBar) {
                if (scrollPaddingRight > 0 && scrollMarginLeft > 0) {
                    me.origin.x += (scrollPaddingRight < scrollMarginLeft) ? scrollPaddingRight : scrollMarginLeft;
                    flagH = true;
                } else if (scrollPaddingLeft > 0 && scrollMarginRight > 0) {
                    me.origin.x -= (scrollPaddingLeft < scrollMarginRight) ? scrollPaddingLeft : scrollMarginRight;
                    flagH = true;
                }
            }

            if(scrollMarginLeft<0)
                scrollMarginLeft = 0;
            if(scrollMarginRight<0)
                scrollMarginRight = 0;
            if(scrollMarginTop<0)
                scrollMarginTop = 0;
            if(scrollMarginBottom<0)
                scrollMarginBottom = 0;

            me.originTL.x = scrollMarginLeft + me.origin.x;
            me.originTL.y = scrollMarginTop + me.origin.y;
            me.originBR.x = -scrollMarginRight + me.origin.x;
            me.originBR.y = -scrollMarginBottom + me.origin.y;

            if (flagV) {
                me.vScrollBar.setValue(me.getOriginY());
            }
            if (flagH) {
                me.hScrollBar.setValue(me.getOriginX());
            }
        }
        if(me.flagVScrollBar || me.flagHScrollBar) {
            var prevOriginX = me.origin.x, prevOriginY = me.origin.y;
            fUpdateMargins();
            if (me.origin.x != prevOriginX || me.origin.y != prevOriginY)
                fUpdateMargins();
        }


        var v, step;
        if(me.flagNeedVScrollBar){
            me.vScrollBar.setTargetZoomRatio(zoom);
            //visible content length
            me.vScrollBar.setVisibleSpan(zoom*sceneHeightZoom-(me.flagNeedHScrollBar ? wScroller : 0));
            //full content length
            me.vScrollBar.setExtent(scrollMarginTop+sceneHeightZoom+scrollMarginBottom);
            //"minimal"(>=0) is above "maximal"(<=0) because originY is <0 when the scene is shifted upwards
            v = me.vScrollBar.getValue();
            me.vScrollBar.setMinValue(scrollMarginTop+v);
            me.vScrollBar.setMaxValue(-scrollMarginBottom+v);
            if(v<-scrollMarginBottom)
                me.vScrollBar.setValue(-scrollMarginBottom);
            else if(v>scrollMarginTop)
                me.vScrollBar.setValue(scrollMarginTop);
            step =
                ((me.vScrollBar.getMaxValue()<me.vScrollBar.getMinValue()) ? (-1) : 1) *
                me.vScrollBar.getVisibleSpan()/me.SCROLL_RATIO;
            me.vScrollBar.setStep(step);
            me.vScrollBar.setX(me.sceneWidth-ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH);
            me.vScrollBar.setY(0);
            me.vScrollBar.setWidth(ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH);
            me.vScrollBar.updateValueFromTarget();
        }
        if(me.flagNeedHScrollBar){
            me.hScrollBar.setTargetZoomRatio(zoom);
            //visible content length
            me.hScrollBar.setVisibleSpan(zoom*sceneWidthZoom-(me.flagNeedVScrollBar ? wScroller : 0));
            //full content length
            me.hScrollBar.setExtent(scrollMarginLeft+sceneWidthZoom+scrollMarginRight);
            //"minimal"(>=0) is above "maximal"(<=0) because originX is <0 when the scene is shifted upwards
            v = me.hScrollBar.getValue();
            me.hScrollBar.setMinValue(scrollMarginLeft+v);
            me.hScrollBar.setMaxValue(-scrollMarginRight+v);
            if(v<-scrollMarginRight)
                me.hScrollBar.setValue(-scrollMarginRight);
            else if(v>scrollMarginLeft)
                me.hScrollBar.setValue(scrollMarginLeft);
            step =
                ((me.hScrollBar.getMaxValue()<me.hScrollBar.getMinValue()) ? (-1) : 1) *
                me.hScrollBar.getVisibleSpan()/me.SCROLL_RATIO;
            me.hScrollBar.setStep(step);
            me.hScrollBar.setX(0);
            me.hScrollBar.setY(me.sceneHeight-ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH);
            me.hScrollBar.setWidth(ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH);
            me.hScrollBar.updateValueFromTarget();
        }

        if(flagEvent) {
            var evt = new ModelEvent(ModelEvent.prototype.VALIDATE_SCENE, false, true);
            if (evt == null)
                return;
            evt.data = {time: new Date().getTime()};
            me.dispatchEvent(evt);

            if (prevNeedVScrollBar != me.flagNeedVScrollBar || prevNeedHScrollBar != me.flagNeedHScrollBar) {
                evt = new ModelEvent(ModelEvent.prototype.SCROLL_TOGGLE, false, true);
                if (evt == null)
                    return;
                evt.data = {hScrollBar: me.flagNeedHScrollBar, vScrollBar: me.flagNeedVScrollBar};
                me.dispatchEvent(evt);
            }
        }

        me.flagValid = true;
    }
};

//
MapSceneModel.prototype.getOrigin = function(){
    return this.origin.clone();
};

//
MapSceneModel.prototype.getOriginX = function(){
    return this.origin.x;
};

//
MapSceneModel.prototype.getOriginY = function(){
    return this.origin.y;
};

//
MapSceneModel.prototype.setOriginX = function(v){
    var me = this;
    var vMin = me.originBR.x, vMax = me.originTL.x;
    me.origin.x = (v<vMin) ? vMin : ((v>vMax) ? vMax : v);
};

//
MapSceneModel.prototype.setOriginY = function(v){
    var me = this;
    var vMin = me.originBR.y, vMax = me.originTL.y;
    me.origin.y = (v<vMin) ? vMin : ((v>vMax) ? vMax : v);
};

//
MapSceneModel.prototype.getSceneWidth = function(){
    return this.sceneWidth;
};

MapSceneModel.prototype.getSceneHeight = function(){
    return this.sceneHeight;
};

MapSceneModel.prototype.getScaleFrame = function(){
    return this.scaleFrameModel;
};

//
MapSceneModel.prototype.isPointWithinScene = function(pnt){
    var me = this;
    var p = pnt.add(me.origin);
    if(p instanceof Point)
        return (p.x>=0.0 && p.x<me.sceneWidth/me.zoomRatio && p.y>=0.0 && p.y>=0 && p.y<me.sceneHeight/me.zoomRatio);
    else
        return false;
};

//
MapSceneModel.prototype.fFillElements = function(elementsData){
	var me = this;
	me.elements = {};
	var aType = ModelResource.prototype.TYPES, i, n = aType.length, j, l;
	var aData, objType, sPrefix;
	
	for(i=0; i<n; i++){
		aData = elementsData[aType[i].family];
        if(Array.isArray(aData)) {
            l = aData.length;
            objType = aType[i];
            sPrefix = (typeof objType.prefixData == TYPE_UNDEFINED) ? objType.prefix : objType.prefixData;
            for (j = 0; j < l; j++)
                me.fAddElementByData(sPrefix, aData[j]);
        }
	}

    me.sortElementsByDepth();
};

//
MapSceneModel.prototype.unblinkAll = function(){
    var me = this;
    var i, n = me.getElementsCount();
    var element;
    for(i=0; i<n; i++){
        element = me.getDepthSortedElementNo(i);
        if(element.isBlinking())
            element.setBlinking(false);
    }
};

//
MapSceneModel.prototype.showHint = function(){
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.SHOW_HINT, false, true);
    if(evt == null)
        return;
    var objProps = {
        id:"hintDialog",
        width:me.HINT_WINDOW_WIDTH, height:me.HINT_WINDOW_HEIGHT,
        textWidth:me.HINT_TEXT_WIDTH, textHeight:me.HINT_TEXT_HEIGHT,
        hAlign:"left", vAlign:"top",
        closeCaption:me.CLOSE_CAPTION
    };
    objProps.nodeText = me.modelResource.nodeHintSolution;
    evt.data = {time:new Date().getTime(), dialogProperties:objProps};
    me.dispatchEvent(evt);
};

//
MapSceneModel.prototype.showResponse = function(correctPercentage, options){
    //correctPercentage is in [0, 1]
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.SHOW_RESPONSE, false, true);
    if(evt == null)
        return;
    var objProps = {
        id:"responseDialog",
        width:me.RESPONSE_WINDOW_WIDTH, height:me.RESPONSE_WINDOW_HEIGHT,
        textWidth:me.RESPONSE_TEXT_WIDTH, textHeight:me.RESPONSE_TEXT_HEIGHT,
        hAlign:"center", vAlign:"middle",
        closeCaption:me.CLOSE_CAPTION
    };
    //
    //CENTER ALIGNED!
    var s, color;
    var objectsAttributes = me.modelResource.objectsAttributes;
    if(options != null && typeof options.message == TYPE_STRING){
        s = options.message;
        if(typeof options.color == TYPE_UNDEFINED)
            if (correctPercentage >= parseFloat(objectsAttributes["errorLevelUpper"]))
                color = "#00AA00";
            else if (correctPercentage >= parseFloat(objectsAttributes["errorLevelLower"]))
                color = "#FF9900";
            else
                color = "#DD0000";
    }else {
        if (correctPercentage >= parseFloat(objectsAttributes["errorLevelUpper"])) {
            s = objectsAttributes["errorText3"];
            color = "#00AA00";
        } else if (correctPercentage >= parseFloat(objectsAttributes["errorLevelLower"])) {
            s = objectsAttributes["errorText2"];
            color = "#FF9900";
        } else {
            s = objectsAttributes["errorText1"];
            color = "#DD0000";
        }
    }
    objProps.nodeText =
        new DOMParser().parseFromString(
            "<font face=\"Arial_bold\" size=\"20pt\" color=\""+color+"\">"+s+"</font>", "text/xml"
        );
    evt.data = {time:new Date().getTime(), dialogProperties:objProps};
    me.dispatchEvent(evt);
};

//
MapSceneModel.prototype.fillWithInitialElements = function(){
    //fills an empty model with initially specified set of elements
    var me = this;
    if(me.aElementByDepth.length>0)
        return false;
    me.fFillElements(me.modelResource[ModelResource.prototype.OBJECTS_PROPERTY]);
    me.invalidate();
    var evt = new ModelEvent(ModelEvent.prototype.RESET_SCENE, false, true);
    if(evt == null)
        return;
    evt.data = {time:new Date().getTime()};
    me.dispatchEvent(evt);
    return true;
};

//
MapSceneModel.prototype.verifyElements = function(){
    //compares user-built and user-editable elements to the verifiable ones
    var me = this;
    var objCountByType = {};
    var aCheckPropertyObj = [
        {name: "checkColor", property:"fillColor", permission:"colorModifiable", objCount:{}},
        {name: "checkColor", property:"lineColor", permission:"colorModifiable", objCount:{}},
        {name: "checkLineColor", property:"lineColor", permission:"colorModifiable", objCount:{}}
    ];
    var ii, nCheck = aCheckPropertyObj.length, objCount, sGetMethod, sCheckMethod, sMethod, sPermission;

    var obj = {total:0, correct:0, extra:0, aWeight:[], aReferenceId:[], aReferenceMatch:[], aReferenceMismatch:[]};
    var objectsAttributes = me.modelResource.objectsAttributes;
    if(
        typeof objectsAttributes["HardEvaluation"] != TYPE_UNDEFINED &&
        objectsAttributes["HardEvaluation"].toLowerCase() == "true"
    )
        obj.evaluationType = "hard";
    else if(
        typeof objectsAttributes["StrictEvaluation"] != TYPE_UNDEFINED &&
        objectsAttributes["StrictEvaluation"].toLowerCase() == "true"
    )
        obj.evaluationType = "strict";
    else
        obj.evaluationType = "simple";
    if(me.aElementByDepth.length>0) {
        var i, n = me.aElementByDepth.length;
        var aVerified = [];
        var aReference = [];
        var element;
        for (i = 0; i < n; i++) {
            element = me.aElementByDepth[i];
            if (element.isVerifiable())
                aReference.push(element);
            else if (
                element.isUserCreated() || element.isColorModifiable() ||
                element.isShapeModifiable() || element.isMovable() || element.isOtherwiseModifiable()
            )
                aVerified.push(element);
        }
        n = aReference.length;
        var j, l, cExtra, cCorrect;
        var refElement, refElementZone, options;
        var sType, sId, flag, fromZoneId, toZoneId;
        var aUsedId = [];
        var aReferenceId = obj.aReferenceId;
        var aReferenceMatch = obj.aReferenceMatch;
        var aReferenceMismatch = obj.aReferenceMismatch;
        var aWeight = obj.aWeight;
        cCorrect = 0;
        //count extra elements
        for(i=0; i<n; i++){
            refElement = aReference[i];
            sType = refElement.getType();
            if(typeof objCountByType[sType] == TYPE_UNDEFINED) {
                objCountByType[sType] = 0;
            }
            if(!refElement.isVerifyOwnState())
                objCountByType[sType]++;
            for(ii=0; ii<nCheck; ii++){
                sCheckMethod = refElement.getMethodNameForProperty(aCheckPropertyObj[ii].name, false);
                sGetMethod = refElement.getMethodNameForProperty(aCheckPropertyObj[ii].property, false);
                if(
                    sCheckMethod != null && sGetMethod != null && refElement[sCheckMethod]()
                    && !refElement.isForbiddenPropertyCheck(aCheckPropertyObj[ii])
                ) {
                    objCount = aCheckPropertyObj[ii].objCount;
                    if(typeof objCount[sType] == TYPE_UNDEFINED)
                        objCount[sType] = 0;
                    objCount[sType]++;
                }
            }
        }
        l = aVerified.length;
        //first take model-provided verified elements into account;
        for(j=0; j<l; j++){
            element = aVerified[j];
            sType = element.getType();
            if(typeof objCountByType[sType] == TYPE_UNDEFINED)
                objCountByType[sType] = 0;
            if(!element.isUserCreated()){
                for(ii=0; ii<nCheck; ii++){
                    objCount = aCheckPropertyObj[ii].objCount;
                    if(typeof objCount[sType] != TYPE_UNDEFINED){
                        //e.g. elements of sType do need to check color: there may be extras among model-provided but user-colored elements
                        sCheckMethod = element.getMethodNameForProperty(aCheckPropertyObj[ii].name, false);
                        sMethod = element.getMethodNameForProperty(aCheckPropertyObj[ii].property, false);
                        sPermission = element.getMethodNameForProperty(aCheckPropertyObj[ii].permission, false);
                        if(
                            sMethod != null && sPermission != null && element[sPermission]() && typeof element[sMethod]() != TYPE_UNDEFINED
                            && !element.isForbiddenPropertyCheck(aCheckPropertyObj[ii])
                        )
                            objCount[sType]--;
                    }
                }
                if(objCountByType[sType]>0)
                    objCountByType[sType]--;
            }

        }
        //now the same check for user-created elements; negative objCountByType[sType] means there are extra of sType
        for(j=0; j<l; j++){
            element = aVerified[j];
            sType = element.getType();
            if(typeof objCountByType[sType] == TYPE_UNDEFINED)
                objCountByType[sType] = 0;
            if(element.isUserCreated())
                objCountByType[sType]--;
        }
        //count total of extra's
        cExtra = 0;
        for(sType in objCountByType){
            i = objCountByType[sType];
            if(i<0)
                cExtra -= i;
        }
        for(ii=0; ii<nCheck; ii++) {
            objCount = aCheckPropertyObj[ii].objCount;
            for(sType in objCount) {
                i = objCount[sType];
                if (i < 0)
                    cExtra -= i;
            }
        }
        //
        for(i=0; i<n; i++){
            flag = false;
            sId = null;
            refElement = aReference[i];
            sType = refElement.getType();
            options = {};
            if(refElement.getErrorMode() == "zone"){
                if(typeof refElement["getFromZoneId"] == TYPE_FUNCTION) {
                    fromZoneId = refElement.getFromZoneId();
                    refElementZone = me.getElementById(fromZoneId);
                    if (refElementZone == null)
                        continue;
                    options.zoneElement = refElementZone;
                }
                if(typeof refElement["getToZoneId"] == TYPE_FUNCTION){
                    toZoneId = refElement.getToZoneId();
                    refElementZone = me.getElementById(toZoneId);
                    if (refElementZone == null)
                        continue;
                    options.zoneDestinationElement = refElementZone;
                }
            }

            aReferenceId.push(refElement.getId());
            aWeight.push(refElement.getObjectWeight());
            if(refElement.isVerifyOwnState()){
                flag = refElement.isMatchForElement(refElement, options);
                if (flag) {
                    sId = refElement.getId();
                    cCorrect++;
                    aUsedId.push(sId);
                }
            } else {
                for (j = 0; j < aVerified.length; j++) {
                    element = aVerified[j];
                    if (element.getType() != sType)
                        continue;
                    flag = element.isMatchForElement(refElement, options);
                    if (flag) {
                        sId = element.getId();
                        cCorrect++;
                        aUsedId.push(sId);
                        aVerified.splice(j, 1);
                        break;
                    }
                }
            }
            aReferenceMatch.push(sId);
        }


        aReference.forEach(
            function(refElement, i, a){
                var sId = refElement.getId();
                if(aReferenceMatch.indexOf(sId) == -1 && refElement.isVerifyOwnState())
                    aReferenceMismatch.push(sId);
            }
        );

        for(i=0; i<aVerified.length; i++)
            aReferenceMismatch.push(aVerified[i].getId());

        obj.total = n;
        obj.extra = cExtra;
        obj.correct = cCorrect;

        //for SCORM
        var correctPercentage = 0;
        var weightSum = 0;
        n = obj.aReferenceMatch.length;
        if(n>0) {
            for (i = 0; i < n; i++) {
                if (obj.aReferenceMatch[i] != null)
                    correctPercentage += obj.aWeight[i];
                weightSum += obj.aWeight[i];
            }
        }
        switch(obj.evaluationType){
            case "hard":
                if(obj.extra>0) {
                    correctPercentage = 0;
                }
                break;
            case "strict":
                if(obj.extra>0){
                    correctPercentage =
                        (correctPercentage>obj.extra) ? (correctPercentage-obj.extra) : 0;
                }
                break;
            case "simple":
            default:
        }
        if(weightSum>0)
            correctPercentage /= weightSum;
        obj.correctPercentage = correctPercentage;
        if(obj.evaluationType == "hard" && obj.extra>0)
            correctPercentage = 0;
        if (correctPercentage >= 0.01*parseFloat(objectsAttributes["errorLevelUpper"]))
            obj.successStatus = "passed";
        else if (correctPercentage >= 0.01*parseFloat(objectsAttributes["errorLevelLower"]))
            obj.successStatus = "partially";
        else
            obj.successStatus = "failed";

    }
    return obj;
};

//
MapSceneModel.prototype.notifyOfElementsVerification = function(obj){
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.VERIFY_ELEMENTS, false, true);
    if(evt != null){
        evt.data = {time:new Date().getTime(), attemptNo:me.attemptsCount, result:obj};
        me.dispatchEvent(evt);
    }
};

//
MapSceneModel.prototype.isWithHorizontalScrollBar = function(){
    return this.flagHScrollBar;
};

//
MapSceneModel.prototype.isNeedHorizontalScrollBar = function(){
    return this.flagNeedHScrollBar;
};

//
MapSceneModel.prototype.getHorizontalScrollBar = function(){
    return this.hScrollBar;
};

//
MapSceneModel.prototype.isWithVerticalScrollBar = function(){
    return this.flagVScrollBar;
};

//
MapSceneModel.prototype.isNeedVerticalScrollBar = function(){
    return this.flagNeedVScrollBar;
};

//
MapSceneModel.prototype.getVerticalScrollBar = function(){
    return this.vScrollBar;
};

//
MapSceneModel.prototype.isWithSwitchableVisibilityElements = function(){
    var me = this;
    var i, n = me.getElementsCount(), element;
    for(i=0; i<n; i++){
        element = me.getDepthSortedElementNo(i);
        if(element.isCanBeTurnedOn() && !element.isVerifiable())
            return true;
    }
    return false;
};

//
MapSceneModel.prototype.getSwitchableVisibilityArray = function(){
    var me = this;
    var a = [];
    var i, n = me.getElementsCount(), element;
    var groupId, flagVisible, el, group;
    var aGr = [];
    for(i=0; i<n; i++) {
        element = me.getDepthSortedElementNo(i);
        if (element.isCanBeTurnedOn() && !element.isVerifiable()) {
            groupId = element.getGroupId();
            if(typeof groupId == TYPE_NUMBER){
                if(aGr.indexOf(groupId) != -1)
                    continue;
                group = me.getGroupById(groupId);
                aGr.push(groupId);
                a.push({groupId: groupId, name: group.getName(), visible: element.isVisible()});
            }else
                a.push({id: element.getId(), name: element.getName(), visible: element.isVisible()});
        }
    }
    return a;
};

//
MapSceneModel.prototype.isWithVerifiableElements = function(){
    var me = this;
    var i, n = me.getElementsCount(), element;
    for(i=0; i<n; i++){
        element = me.getDepthSortedElementNo(i);
        if(element.isVerifiable())
            return true;
    }
    return false;
};

//
MapSceneModel.prototype.getGroupById = function(q){
    var me = this;
    var i, n = me.aGroup.length, group;
    for(i=0; i<n; i++){
        group = me.aGroup[i];
        if(group.getId() == q)
            return group;
    }
    return null;
};

//returns elements of the group that element belongs to except the element itself or an empty array if the element is not in a group
MapSceneModel.prototype.getOtherElementsInGroup = function(element){
    var groupId = element.getGroupId();
    if(typeof groupId === TYPE_UNDEFINED)
        return [];
    var group = this.getGroupById(groupId);
    var aEl = group.getElements();
    aEl.splice(aEl.indexOf(element), 1);
    return  aEl;
};

//
MapSceneModel.prototype.addElement = function (element, flagSort) {
    var me = this;
    var flag = (typeof flagSort == TYPE_BOOLEAN) ? flagSort : true;
    var sType = element.getType(), aElement = me.elements[sType];
    if(!Array.isArray(aElement)){
        me.elements[sType] = [];
        aElement = me.elements[sType];
    }
    aElement.push(element);
    if(flag)
        me.sortElementsByDepth();
    var groupId = element.getGroupId();
    if(typeof groupId == TYPE_NUMBER){
        me.sortElementsByDepth();
        var group = me.getGroupById(groupId);
        if(group == null){
            group = new ElementsGroup(me, groupId);
            me.aGroup.push(group);
        }
        group.addElementById(element.getId());
    }
};

//returns true if the element was removed
MapSceneModel.prototype.removeElement = function (element) {
    var me = this;
    var sType = element.getType(), aElement = me.elements[sType];
    if(!Array.isArray(aElement))
        return;
    var i = aElement.indexOf(element);
    if(i != -1){
        var groupId = element.getGroupId();
        if(typeof groupId == TYPE_NUMBER){
            var group = me.getGroupById(groupId);
            if(group != null)
                if(group.removeElementById(element.getId()))
                    if(group.getCount() == 0)
                        me.aGroup.splice(me.aGroup.indexOf(group), 1);
        }
        element.deactivate();
        aElement.splice(i, 1);
        me.sortElementsByDepth();
        me.invalidate();
        return true;
    }else
        return false;
};

MapSceneModel.prototype.fAddElementByData = function(sTypePrefix, elementData){
    var me = this;
    var aType = ModelResource.prototype.TYPES, i, n = aType.length;
    for(i=0; i<n; i++)
        if(aType[i].prefix == sTypePrefix)
            break;
    if(i==n)
        return;
    var cls = window[aType[i].classId];
    if(typeof cls != TYPE_UNDEFINED)
        me.addElement(new cls(elementData), false);
};

MapSceneModel.prototype.addElementByData = function(sTypePrefix, elementData){
    var me = this;
    me.fAddElementByData(sTypePrefix, elementData);
    me.sortElementsByDepth();
};

MapSceneModel.prototype.sortElementsByDepth = function(){
    var me = this;
	var a = [];
    var aType = ModelResource.prototype.TYPES, i, n = aType.length, sPrefix;
    for(i=0; i<n; i++) {
        sPrefix = aType[i].prefix;
        if(Array.isArray(me.elements[sPrefix]))
            a = a.concat(me.elements[sPrefix]);
    }
    a.sort(function(a, b){
        var d1 = a.getDepth();
        var d2 = b.getDepth();
        return (d1==d2) ? 0 : ((d1<d2) ? (-1) : 1);
    });
    me.aElementByDepth = a;
};

MapSceneModel.prototype.getDepthSortedElementNo = function(iNo){
    var me = this;
    return (iNo<0 || iNo>=me.aElementByDepth.length) ? null : me.aElementByDepth[iNo];
};

MapSceneModel.prototype.getElementsCount = function(){
    return this.aElementByDepth.length;
};

MapSceneModel.prototype.getUniqueId = function(){
    var me = this;
    var i, a = me.aElementByDepth, n = a.length;
    var aId = [];
    for(i=0; i<n; i++)
        aId.push(a[i].getId());
    aId.sort(function(a,b){
        return a-b;
    });
    n = aId.length;
    i = 0;
    while((aId[i]== i) && (i<n))
        i++;
    return i;
};

//
MapSceneModel.prototype.getElementById = function(q){
    var i, a = this.aElementByDepth, n = a.length;
    for(i=0; i<n; i++)
        if(a[i].getId() == q)
            return a[i];
    return null;
};

//
MapSceneModel.prototype.getMaximalDepth = function(){
    var me = this;
    var i, a = me.aElementByDepth, n = a.length;
    var maxDepth = 1, d;
    for(i=0; i<n; i++){
        d = a[i].getDepth();
        if(d>maxDepth)
            maxDepth = d;
    }
    return maxDepth;
};

//
MapSceneModel.prototype.getAttemptsCount = function(){
    return this.attemptsCount;
};

//
MapSceneModel.prototype.resetAttemptsCount = function(){
    this.attemptsCount = 0;
};

//
MapSceneModel.prototype.isSolutionAlways = function(){
    var objectsAttributes = this.modelResource.objectsAttributes;
    return objectsAttributes["SolutionAlways"] != TYPE_UNDEFINED && objectsAttributes["SolutionAlways"].toLowerCase() == "true";
};

//
MapSceneModel.prototype.isHintAlways = function(){
    var objectsAttributes = this.modelResource.objectsAttributes;
    return typeof objectsAttributes["HintAlways"] != TYPE_UNDEFINED && objectsAttributes["HintAlways"].toLowerCase() == "true"
};

//
MapSceneModel.prototype.increaseAttemptsCount = function(){
    this.attemptsCount++;
};

MapSceneModel.prototype.getHitTestInfosForPoint = function(p0){
    var me = this;
    var aElement = me.aElementByDepth;
    var i, n = aElement.length;
    var aResult = [];
    var element, hTInfo;
    //
    for(i=0; i<n; i++){
        element = aElement[i];
        if(!(
            element.isComplete() && element.isVisible() &&
            (element.canBeHighlighted() || element.canBeSelected() ||
            (element.isVerifiable() && element.isShowIfVerifiable()) ||
            element.isOtherwiseModifiable())
            )
        )
            continue;
        hTInfo = element.getHitTestInfoForPoint(p0);
        if(hTInfo.isHit()) {
            element.setHitDetails(hTInfo.getDetails());
            aResult.push(hTInfo);
        }
    }
    //
    aResult.sort(function(a, b){
        var d1 = a.getElement().getDepth();
        var d2 = b.getElement().getDepth();
        return (d2==d1) ? (0) : ((d2<d1) ? (-1) : 1);
    });
    //
    return aResult;
};

MapSceneModel.prototype.resetHitTestInfos = function(){
    var me = this;
    var aElement = me.aElementByDepth;
    var i, n = aElement.length;
    var element;
    for(i=0; i<n; i++) {
        element = aElement[i];
        element.setHitDetails(null);
    }
};

//
//
MapSceneModel.prototype.toggleScreenKeyboard = function(objData){
    var evt = new ModelEvent(ModelEvent.prototype.TOGGLE_SCREEN_KEYBOARD, true, true);
    if(evt != null){
        evt.data = objData;
        this.dispatchEvent(evt);
    }
};
/**
 * @constructor
 */
function MapSceneVisualization(_model, _canvas){
    MapSceneVisualization.superclass.constructor.call(this);
	var me = this;
	me.model = _model;
	me.canvas = (typeof _canvas != TYPE_UNDEFINED) ? _canvas : null;
    //paddings for canvasToSceneSizeRatio == 1, i.e., when pixel scene dimensions equal those of the model
    me.leftPadding100 = 0;
    me.topPadding100 = 0;
    me.rightPadding100 = 0;
    me.bottomPadding100 = 0;
    me.deltaLeftPadding = 0;
    me.deltaTopPadding = 0;

    me.inputDIV = null;
    me.dialogDIV = null;

    me.canvasToSceneSizeRatio = 0.0;
    me.canvasToSceneSizeRatioZoom = 0.0;

    me.flagScaleInterface = ModelMainInstance.prototype.flagScaleInterface;

    me.width = me.defaultWidth;
    me.height = me.defaultHeight;
    me.time = 0;
    me.highlightBlinkPhase = 0;
    me.highlightAlphaMultiplier = 1;
    //
    //me.flagUseBuffer = false для отладки, чтобы сразу отображались
    me.flagUseBuffer = true;
    me.canvasBuffer = document.createElement("CANVAS");
    me.canvasBuffer.width = me.width;
    me.canvasBuffer.height = me.height;
    //
    me.vScrollBarVisualization = new ScrollerVisualization(me.model.getVerticalScrollBar());
    me.hScrollBarVisualization = new ScrollerVisualization(me.model.getHorizontalScrollBar());
    //
    me.fFillVisualizations();
    //
    me.model.addEventListener(ModelEvent.prototype.VALIDATE_SCENE, {listener:me.fOnValidate, target:me}, false);
    me.model.addEventListener(ModelEvent.prototype.RESET_SCENE, {listener:me.fOnReset, target:me}, false);
    //рамка масштабирования
    me.scaleFrameVisualization = new ScaleFrameVisualization(me.model.getScaleFrame(), me);
    //e.g. ctrl was pressed to remove a polyline vertix by click so that a cross has to be shown atop current point
    me.flagCtrl = false;
}
extend(MapSceneVisualization, NonDOMEventDispatcher);

MapSceneVisualization.prototype.defaultWidth = 800;
MapSceneVisualization.prototype.defaultHeight = 600;
MapSceneVisualization.prototype.HIGHLIGHT_BLINK_PERIOD = 1000;
MapSceneVisualization.prototype.VISUALIZATION_SUFFIX = "Visualization";

MapSceneVisualization.prototype.getModel = function(){
    return this.model;
};

MapSceneVisualization.prototype.getWidth = function(){
    return this.width;
};

MapSceneVisualization.prototype.getHeight = function(){
    return this.height;
};

MapSceneVisualization.prototype.deactivate = function(){
    MapSceneVisualization.superclass.deactivate.call(this);
    var me = this;
    if(Array.isArray(me.aVisualization))
        while(me.aVisualization.length>0)
            me.aVisualization.pop().deactivate();

    me.model.removeEventListener(ModelEvent.prototype.VALIDATE_SCENE, {listener:me.fOnValidate, target:me}, false);
    me.model.removeEventListener(ModelEvent.prototype.RESET_SCENE, {listener:me.fOnReset, target:me}, false);
};

MapSceneVisualization.prototype.fOnValidate = function(e){
    var me = this;
    me.time = e.data.time;
    me.update();
};

MapSceneVisualization.prototype.fOnReset = function(e){
    this.fFillVisualizations();
};

MapSceneVisualization.prototype.getCanvas = function(){
    return this.canvas;
};

MapSceneVisualization.prototype.setCanvas = function(canvas){
    this.canvas = canvas;
};

MapSceneVisualization.prototype.getInputDIV = function(){
    return this.inputDIV;
};

MapSceneVisualization.prototype.setInputDIV = function(div){
    this.inputDIV = div;
};

MapSceneVisualization.prototype.getDialogDIV = function(){
    return this.dialogDIV;
};

MapSceneVisualization.prototype.setDialogDIV = function(div){
    this.dialogDIV = div;
};

MapSceneVisualization.prototype.getCanvasContext = function(){
    var me = this;
    return me.flagUseBuffer ?
        me.canvasBuffer.getContext("2d") :
        ((me.canvas != null) ? me.canvas.getContext("2d") : null);
};

//
MapSceneVisualization.prototype.getLeftPadding100 = function(){
    return this.leftPadding100;
};

MapSceneVisualization.prototype.setLeftPadding100 = function(v){
    this.leftPadding100 = v;
};

//
MapSceneVisualization.prototype.getTopPadding100 = function(){
    return this.topPadding100;
};

MapSceneVisualization.prototype.setTopPadding100 = function(v){
    this.topPadding100 = v;
};

//
MapSceneVisualization.prototype.getRightPadding100 = function(){
    return this.rightPadding100;
};

MapSceneVisualization.prototype.setRightPadding100 = function(v){
    this.rightPadding100 = v;
};

//
MapSceneVisualization.prototype.getBottomPadding100 = function(){
    return this.bottomPadding100;
};

MapSceneVisualization.prototype.setBottomPadding100 = function(v){
    this.bottomPadding100 = v;
};

//
MapSceneVisualization.prototype.getLeftPadding = function(){
    var me = this;
    return me.flagScaleInterface ?
        (me.leftPadding100*me.canvasToSceneSizeRatio+me.deltaLeftPadding) :
        (me.leftPadding100+me.deltaLeftPadding);
};

//
MapSceneVisualization.prototype.getRightPadding = function(){
    var me = this;
    return me.flagScaleInterface ?
        (me.rightPadding100*me.canvasToSceneSizeRatio+me.deltaLeftPadding) :
        (me.rightPadding100+me.deltaLeftPadding);
};

//
MapSceneVisualization.prototype.getTopPadding = function(){
    var me = this;
    return me.flagScaleInterface ?
        (me.topPadding100*me.canvasToSceneSizeRatio+me.deltaTopPadding) :
        (me.topPadding100+me.deltaTopPadding);
};

//
MapSceneVisualization.prototype.getBottomPadding = function(){
    var me = this;
    return me.flagScaleInterface ?
        (me.bottomPadding100*me.canvasToSceneSizeRatio+me.deltaTopPadding) :
        (me.bottomPadding100+me.deltaTopPadding);
};

//
MapSceneVisualization.prototype.getOriginX = function(flagZoomCorrection){
    var me = this;
    var q = (typeof flagZoomCorrection == TYPE_BOOLEAN && !flagZoomCorrection) ? 1 : me.model.getZoomRatio();
    return me.model.getOriginX()*me.width*q/me.model.getSceneWidth();
};

//
MapSceneVisualization.prototype.getOriginY = function(flagZoomCorrection){
    var me = this;
    var q = (typeof flagZoomCorrection == TYPE_BOOLEAN && !flagZoomCorrection) ? 1 : me.model.getZoomRatio();
    return me.model.getOriginY()*me.height*q/me.model.getSceneHeight();
};

//
MapSceneVisualization.prototype.getHighlightAlphaMultiplier = function(){
    return this.highlightAlphaMultiplier;
};

//
MapSceneVisualization.prototype.isCtrl = function(){
    return this.flagCtrl;
};

MapSceneVisualization.prototype.setCtrl = function(flag){
    var me = this;
    var prev = me.flagCtrl;
    if(prev != flag){
        me.flagCtrl = flag;
        me.update();
    }
};

MapSceneVisualization.prototype.update = function(){
    var me = this;
    if(me.canvas == null)
        return;
	var canvas = me.canvas;
	var ctx = canvas.getContext("2d");
    //
    var prevWidth = me.width;
    var prevHeight = me.height;
    var sceneWidth = me.model.getSceneWidth();
    var sceneHeight = me.model.getSceneHeight();

    var prevSizeRatio = me.canvasToSceneSizeRatio;
    var prevSizeRatioZoom = me.canvasToSceneSizeRatioZoom;

    var w100, h100;
    var rW, rH, r;
    if(ModelMainInstance.prototype.flagScaleModel){
        if(me.flagScaleInterface){
            //pre-correct ratio if the interface should scale as well (like in the flash version)
            w100 = sceneWidth+me.leftPadding100+me.rightPadding100;
            h100 = sceneHeight+me.topPadding100+me.bottomPadding100;
            rW = canvas.width/w100;
            rH = canvas.height/h100;
            if(rH<rW){
                r = rH;
                me.deltaLeftPadding = (canvas.width-rH*w100)/2;
                me.deltaTopPadding = 0;
            }else{
                r = rW;
                me.deltaLeftPadding = 0;
                me.deltaTopPadding = (canvas.height-rW*h100)/2;
            }
            me.canvasToSceneSizeRatio = r;
        }
    }else{
        w100 = sceneWidth+me.leftPadding100+me.rightPadding100;
        h100 = sceneHeight+me.topPadding100+me.bottomPadding100;
        me.deltaLeftPadding = (canvas.width-w100)/2;
        me.deltaTopPadding = (canvas.height-h100)/2;
        me.canvasToSceneSizeRatio = 1.0;
    }

    var cnvWidth = canvas.width-me.getLeftPadding()-me.getRightPadding();
    var cnvHeight = canvas.height-me.getTopPadding()-me.getBottomPadding();
    var newWidth;
    var newHeight;
    if(ModelMainInstance.prototype.flagScaleModel){
        var pr1 = sceneWidth*cnvHeight;
        var pr2 = sceneHeight*cnvWidth;
        if(pr1>pr2){
            newWidth = cnvWidth;
            newHeight = cnvWidth*sceneHeight/sceneWidth;
        }else{
            newWidth = cnvHeight*sceneWidth/sceneHeight;
            newHeight = cnvHeight;
        }
    }else{
        newWidth = sceneWidth;
        newHeight = sceneHeight;
    }

    newWidth = Math.round(newWidth)|0;
    newHeight = Math.round(newHeight)|0;

    if(newWidth != prevWidth || newHeight != prevHeight){
        me.width = newWidth;
        me.height = newHeight;
    }
    if(me.flagUseBuffer) {
        if(me.canvasBuffer.width != me.width || me.canvasBuffer.height != me.height) {
            me.canvasBuffer.width = me.width;
            me.canvasBuffer.height = me.height;
        } else
            me.canvasBuffer.getContext("2d").clearRect(0, 0, me.width, me.height);
    }
    //
    var zoomRatio = me.model.getZoomRatio();
    me.canvasToSceneWidthRatio = me.width/me.model.getSceneWidth();
    me.canvasToSceneHeightRatio = me.height/me.model.getSceneHeight();
    //alert("rW:"+me.canvasToSceneWidthRatio+" rH:"+me.canvasToSceneHeightRatio+"\nme.width:"+me.width+" .height:"+me.height+"\nsceneWidth:"+sceneWidth+" sceneHeight:"+sceneHeight);
    me.canvasToSceneSizeRatio = Math.sqrt(me.canvasToSceneWidthRatio*me.canvasToSceneHeightRatio);
    //
    me.canvasToSceneSizeRatioZoom = me.canvasToSceneSizeRatio*zoomRatio;
    //
    if(me.canvasToSceneSizeRatioZoom != prevSizeRatioZoom){
        var evtRSize = new ModelEvent(ModelEvent.prototype.CANVAS_TO_SCENE_RATIO_CHANGED, true, true);
        if(evtRSize != null){
            evtRSize.data = {value:me.canvasToSceneSizeRatio, zoom:zoomRatio, prevValue:prevSizeRatio};
            me.dispatchEvent(evtRSize);
        }
    }
    //
    if(!me.flagUseBuffer) {
        ctx.save();
        ctx.strokeStyle = "rgba(255, 255, 255, 0)";
        ctx.lineWidth = 0;
        ctx.rect(me.getLeftPadding(), me.getTopPadding(), me.width, me.height);
        ctx.stroke();
        ctx.clip();
    }
	//clear canvas
    ctx.clearRect(Math.floor(me.getLeftPadding()), Math.floor(me.getTopPadding()), me.width, me.height);

    ctx.beginPath();
    ctx.fillStyle = "#FFFFFF";
    ctx.fillRect(Math.floor(me.getLeftPadding()), Math.floor(me.getTopPadding()), me.width, me.height);

    var rPhase = (me.time % me.HIGHLIGHT_BLINK_PERIOD)/me.HIGHLIGHT_BLINK_PERIOD;
    me.highlightBlinkPhase = 2*Math.PI*rPhase;
    //uncomment for "sharp" blinking
    //me.highlightAlphaMultiplier = 1-rPhase;
    //uncomment for "soft" blinkinh
    me.highlightAlphaMultiplier = 0.5*(1.0+Math.cos(me.highlightBlinkPhase));

    var aVisualization = me.aVisualization;
	var i, n = aVisualization.length;
    var visualization;

    var currCtx = me.flagUseBuffer ? me.canvasBuffer.getContext("2d") : ctx;

    var originX = me.getOriginX();
    var originY = me.getOriginY();
    currCtx.setTransform(1, 0, 0, 1, originX, originY);

    for(i=0; i<n; i++){
        visualization = aVisualization[i];
        if(visualization.getElement().isVisible())
            visualization.update();
    }

    me.scaleFrameVisualization.update();

    currCtx.setTransform(1, 0, 0, 1, 0, 0);

    //are scrollers needed?
    var rSize = me.canvasToSceneSizeRatio;
    var wScroller = rSize*ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH;
    var flagNeededVScroller = me.model.isNeedVerticalScrollBar();
    var flagNeededHScroller = me.model.isNeedHorizontalScrollBar();
    var scroller;
    if(flagNeededVScroller){
        scroller = me.vScrollBarVisualization.getModel();
        me.vScrollBarVisualization.setSizeRatio(rSize);
        if(!scroller.isValid()) {
            me.vScrollBarVisualization.invalidate();
            scroller.validate();
        }
        me.vScrollBarVisualization.update();
        currCtx.drawImage(me.vScrollBarVisualization.getCanvas(), Math.ceil(cnvWidth-wScroller)|0, 0);
    }
    if(flagNeededHScroller){
        scroller = me.hScrollBarVisualization.getModel();
        me.hScrollBarVisualization.setSizeRatio(rSize);
        if(!scroller.isValid()) {
            me.hScrollBarVisualization.invalidate();
            scroller.validate();
        }
        me.hScrollBarVisualization.setSizeRatio(rSize);
        me.hScrollBarVisualization.update();
        currCtx.drawImage(me.hScrollBarVisualization.getCanvas(), 0, Math.ceil(cnvHeight-wScroller)|0);
    }

    if(me.flagUseBuffer)
        ctx.drawImage(me.canvasBuffer, me.getLeftPadding() | 0, me.getTopPadding() | 0);
    else
        ctx.restore();

};

MapSceneVisualization.prototype.fFillVisualizations = function(){
    var me = this;
    me.aVisualization = [];
    var model = me.model;
    var i, n = model.getElementsCount();
    var element;
    var sClassName;
    var cls;
    var visualization;
    for(i=0; i<n; i++){
        element = model.getDepthSortedElementNo(i);
        if(element.isVerifiable() && !element.isShowIfVerifiable())
            continue;
        sClassName = element.getClassName();
        cls = window[sClassName + MapSceneVisualization.prototype.VISUALIZATION_SUFFIX];
        if(typeof cls != TYPE_UNDEFINED){
            visualization = new cls(element, me);
            me.aVisualization.push(visualization);
        }
    }
};

MapSceneVisualization.prototype.addVisualizationForElement = function(element){
    var me = this;
    var visualization;
    var sClassName = element.getClassName();
    var cls = window[sClassName + MapSceneVisualization.prototype.VISUALIZATION_SUFFIX];
    if(typeof cls != TYPE_UNDEFINED){
        visualization = new cls(element, me);
        me.aVisualization.splice(element.getDepth(), 0, visualization);
    }
};

MapSceneVisualization.prototype.removeVisualizationForElement = function(element) {
    var aVisualization = this.aVisualization;
    var i, n = aVisualization.length;
    var visualization;
    for (i = 0; i < n; i++) {
        visualization = aVisualization[i];
        if(visualization.getElement() == element) {
            visualization.deactivate();
            aVisualization.splice(i--, 1);
            n--;
        }
        //no break here since there might be several visualizations for one element
    }
};

MapSceneVisualization.prototype.getVisualizationByElement = function(element){
    var me = this;
    var aVisualization = me.aVisualization;
    var i, n = aVisualization.length;
    for(i=0; i<n; i++)
        if(aVisualization[i].getElement() == element)
            break;
    return (i<n) ? aVisualization[i] : null;
};

MapSceneVisualization.prototype.elementCoordsToCanvasCoords = function(x0, y0, flagZoomCorrection){
    var me = this, scene = me.model;
    var q = (typeof flagZoomCorrection == TYPE_BOOLEAN && !flagZoomCorrection) ? 1 : me.model.getZoomRatio();
    var kx = me.width/scene.getSceneWidth();
    var ky = me.height/scene.getSceneHeight();
    var dx, dy;
    if(me.flagUseBuffer){
        dx = 0;
        dy = 0;
    }else{
        dx = me.getLeftPadding();
        dy = me.getTopPadding();
    }
    return new Point(dx+kx*x0*q, dy+ky*y0*q);
};

MapSceneVisualization.prototype.canvasCoordsToElementCoords = function(x0, y0, flagZoomCorrection){
    var me = this, scene = me.model;
    var q = (typeof flagZoomCorrection == TYPE_BOOLEAN && !flagZoomCorrection) ? 1 : me.model.getZoomRatio();
    var kx = me.width/scene.getSceneWidth();
    var ky = me.height/scene.getSceneHeight();
    return new Point(
        (x0-me.getLeftPadding())/(q*kx)-scene.getOriginX(flagZoomCorrection),
        (y0-me.getTopPadding())/(q*ky)-q*scene.getOriginY(flagZoomCorrection)/q
    );
};

MapSceneVisualization.prototype.getCanvasToSceneSizeRatio = function(){
    return this.canvasToSceneSizeRatio;
};

MapSceneVisualization.prototype.getCanvasToSceneSizeRatioZoom = function(){
    return this.canvasToSceneSizeRatioZoom;
};

/**
 * @constructor
 */
function MapSceneController(_model, _visualization){
    MapSceneController.superclass.constructor.call(this);
    var me = this;
    me.model = _model;
    me.visualization = _visualization;
    me.toolsController = null;
    me.lastToolId = null;
    //
    me.aController = [];
    //
    me.flagMouseEnabled = true;
    //for "touchstart" event only
    me.lastMouseDownTime = 0;
    me.lastMouseDownPoint = new Point(Number.POSITIVE_INFINITY, Number.POSITIVE_INFINITY);
    me.jitterRadius2 = ModelEvent.prototype.DOUBLE_TOUCH_JITTER_RADIUS*ModelEvent.prototype.DOUBLE_TOUCH_JITTER_RADIUS;

    me.mouseMoveListener = function(e){
        if(me.flagMouseEnabled)
            me.fMouseMove.call(me, e);
    };

    me.mouseDownListener = function(e){
        if(me.flagMouseEnabled) {
            //TODO coord-JITTER!!!!
            var flagDoubleTouch = (e.type == "touchstart");
            if(flagDoubleTouch){
                var prevPoint = me.lastMouseDownPoint;
                me.lastMouseDownPoint = miscFunctions.getPageXY(e);
                var prevTime = me.lastMouseDownTime;
                me.lastMouseDownTime = miscFunctions.getTime();
                flagDoubleTouch = (me.lastMouseDownTime-prevTime<=ModelEvent.prototype.DOUBLE_CLICK_INTERVAL);
                if(flagDoubleTouch){
                    var rSize2 = me.visualization.getCanvasToSceneSizeRatioZoom();
                    rSize2 *= rSize2;
                    flagDoubleTouch = (prevPoint.distanceSquared(me.lastMouseDownPoint)<=me.jitterRadius2*rSize2);
                }
            }

            if(flagDoubleTouch)
                me.fDoubleClickListener.call(me, e);
            else
                me.fMouseDown.call(me, e);
        }
    };

    me.mouseUpListener = function(e){
        if(me.flagMouseEnabled)
            me.fMouseUp.call(me, e);
    };

    me.mouseContextMenuListener = function(e){
        e.preventDefault();
        return false;
    };

    me.mouseDoubleClickListener = function(e){
        if(me.flagMouseEnabled)
            me.fDoubleClickListener.call(me, e);
    };

    me.keydownListener = function(e){
        me.fKeydownListener.call(me, e);
    };
    
    me.keyupListener = function(e){
        me.fKeyupListener.call(me, e);
    };

    me.mouseWheelListener = function(e){
        me.fMouseWheelListener.call(me, e);
    };

    window.addEventListener("mousemove", me.mouseMoveListener, true);
    window.addEventListener("touchmove", me.mouseMoveListener, true);
    window.addEventListener("mouseup", me.mouseUpListener, false);
    window.addEventListener("touchend", me.mouseUpListener, false);
    document.body.addEventListener("contextmenu", me.mouseContextMenuListener, false);
    document.body.addEventListener("mousedown", me.mouseDownListener, false);
    document.body.addEventListener("touchstart", me.mouseDownListener, false);
    document.body.addEventListener("dblclick", me.mouseDoubleClickListener, false);
    //useCapture flag is false, otherwise a focused TextModelElement will move by cursor keys
    document.addEventListener("keydown", me.keydownListener, false);
    document.addEventListener("keyup", me.keyupListener, false);

    me.visualization.getCanvas().addEventListener("wheel", me.mouseWheelListener, false);

    me.symbolHighlighted = function(e){
        me.fSymbolHighlighted.call(me, e);
    };
    document.body.addEventListener(ModelEvent.prototype.SYMBOL_HIGHLIGHTED, me.symbolHighlighted, false);

    me.flagPointWithinScene = false;
    me.flagMouseIsDown = false;
    me.mouseDownPoint = new Point(0, 0);
    me.mouseDownOrigin = new Point(0, 0);
    me.flagMoveScene = false;

    me.flagSelectionMovable = false;
    me.selectionMoveMouseStartPoint = null;
    me.aSelectionMoveStartPoint = null;

    me.flagVScrollbarHit = false;
    me.flagHScrollbarHit = false;

    //scale frame
    me.scaleFrameController = new ScaleFrameController(me.model.getScaleFrame(), me);
    me.flagOverScaleHandler = false;
    me.flagMoveScaleHandler = false;
    me.scaleFrameHandlerDetails = null;

    me.fFillControllers();

    me.model.addEventListener(ModelEvent.prototype.RESET_SCENE, {listener:me.fOnReset, target:me}, false);

    var lastTime = -1;
    var flagReady = true;

    function step(timestamp){
        me.model.validate();
        flagReady = true;
        lastTime = new Date().getTime();
    }

    me.unloadListener = function(e){
        window.removeEventListener("unload", me.unloadListener, false);
        //window.removeEventListener("pagehide", me.unloadListener, false);
        if(me.toolsController != null)
            me.toolsController.deactivate();
        me.deactivate();
        me.visualization.deactivate();
        me.model.deactivate();
    };

    window.addEventListener("unload", me.unloadListener, false);
    //window.addEventListener("pagehide", me.unloadListener, false);

}
extend(MapSceneController, NonDOMEventDispatcher);

MapSceneController.prototype.CONTROLLER_SUFFIX = "Controller";

MapSceneController.prototype.DELETE = "Delete";
MapSceneController.prototype.KEY_CODE_DELETE = 46;

MapSceneController.prototype.ARROW_LEFT = "ArrowLeft";
MapSceneController.prototype.KEY_CODE_ARROW_LEFT = 37;
MapSceneController.prototype.ARROW_RIGHT = "ArrowRight";
MapSceneController.prototype.KEY_CODE_ARROW_RIGHT = 39;
MapSceneController.prototype.ARROW_UP = "ArrowUp";
MapSceneController.prototype.KEY_CODE_ARROW_UP = 38;
MapSceneController.prototype.ARROW_DOWN = "ArrowDown";
MapSceneController.prototype.KEY_CODE_ARROW_DOWN = 40;
MapSceneController.prototype.CTRL = "Control";
MapSceneController.prototype.KEY_CODE_CTRL = 17;
MapSceneController.prototype.KEYBOARD_DISPLACEMENT = 1;
MapSceneController.prototype.KEYBOARD_SHIFT_KEY_MULTIPLIER = 10;

//
MapSceneController.prototype.isSelectionMovable = function(){
    return this.flagSelectionMovable;
};

//
MapSceneController.prototype.isKeyInEvent = function(sKeyName, e){
    if(typeof MapSceneController.prototype[sKeyName] != TYPE_STRING)
        return false;
    if(typeof e.keyCode == TYPE_NUMBER)
        return (e.keyCode == MapSceneController.prototype["KEY_CODE_"+sKeyName]);
    else if(typeof e.key == TYPE_STRING){
        return (e.key == sKeyName);
    }
    return false;
};

//
MapSceneController.prototype.isMouseEnabled = function(){
    return this.flagMouseEnabled;
};

//
MapSceneController.prototype.setMouseEnabled = function(flag){
    if(this.flagMouseEnabled != flag)
        this.flagMouseEnabled = flag;
};

//
MapSceneController.prototype.setToolsController = function(toolsController){
    var me = this;
    var prev = me.toolsController;
    if(prev instanceof ModelToolsController) {
        prev.removeEventListener(
            ModelEvent.prototype.SET_TOOL, {listener: me.fSetToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.START_TOOL, {listener: me.fStartToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.TRACK_TOOL, {listener: me.fTrackToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.USE_TOOL, {listener: me.fUseToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.END_TOOL, {listener: me.fEndToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.CANCEL_TOOL, {listener: me.fCancelToolListener, target: me}, false
        );
    }
    me.toolsController = toolsController;
    if(toolsController instanceof ModelToolsController) {
        toolsController.addEventListener(
            ModelEvent.prototype.SET_TOOL, {listener: me.fSetToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.START_TOOL, {listener: me.fStartToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.TRACK_TOOL, {listener: me.fTrackToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.USE_TOOL, {listener: me.fUseToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.END_TOOL, {listener: me.fEndToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.CANCEL_TOOL, {listener: me.fCancelToolListener, target: me}, false
        );
    }
};

//
MapSceneController.prototype.getCurrentTool = function(){
    var toolsController = this.toolsController;
    return (toolsController != null) ? toolsController.getCurrentTool() : null;
};

//
MapSceneController.prototype.getCurrentToolId = function(){
    var toolsController = this.toolsController;
    return (toolsController != null) ? toolsController.getCurrentToolId() : null;
};

MapSceneController.prototype.setCurrentToolId = function(sId){
    var toolsController = this.toolsController;
    if(toolsController != null)
        toolsController.setCurrentToolId(sId);
};

//
MapSceneController.prototype.fUnHighlightAll = function(){
    var model = this.model;
    var cnt = 0;
    var i, n = model.getElementsCount();
    var element;
    for(i=0; i<n; i++){
        element = model.getDepthSortedElementNo(i);
        if(element.isHighlighted()){
            element.setHighlighted(false);
            cnt++;
        }
    }
    return cnt;
};

//
MapSceneController.prototype.fUnSelectAll = function(aExcl){
    var aExclude = Array.isArray(aExcl) ? aExcl : null;
    var model = this.model;
    var cnt = 0;
    var i, n = model.getElementsCount();
    var element;
    for(i=0; i<n; i++){
        element = model.getDepthSortedElementNo(i);
        if(element.isSelected() && (aExclude == null || aExclude.indexOf(element) == -1)){
            this.fSelectModelElement(element, false);
            cnt++;
        }
    }
    return cnt;
};

//
MapSceneController.prototype.fSelectModelElement = function(element, flag, flagCheckGroups){
    if(typeof element == TYPE_UNDEFINED || !(element instanceof ModelElement) || !(typeof flag == TYPE_BOOLEAN))
        return;
    var flagPrev = element.isSelected();
    if(flagPrev == flag)
        return;
    var me = this;
    element.setSelected(flag);
    if(typeof flagCheckGroups != TYPE_BOOLEAN || flagCheckGroups == true) {
        var groupId = element.getGroupId();
        if (typeof groupId == TYPE_NUMBER) {
            var group = me.model.getGroupById(groupId);
            if (group != null) {
                var aElement = group.getElements();
                //none should be verifiable
                if(ModelElement.prototype.checkIfAllAre(aElement, "isVerifiable", true)) {
                    var i, l = aElement.length, el;
                    for (i = 0; i < l; i++) {
                        el = aElement[i];
                        if (el != element)
                            me.fSelectModelElement(el, flag, false);
                    }
                }
            }
        }
    }
    var flagCurr = element.isSelected();
    if(flagCurr == flagPrev)
        return;
    var evt = new ModelEvent(
        flag ? ModelEvent.prototype.MODEL_ELEMENT_SELECTED : ModelEvent.prototype.MODEL_ELEMENT_DESELECTED,
        true, true
    );
    if(evt!=null){
        evt.data = {element:element};
        me.dispatchEvent(evt);
    }
};

//
MapSceneController.prototype.fUnblinkAll = function(){
    this.model.unblinkAll();
};

//
MapSceneController.prototype.notify = function(sName, objData){
    var evt = new ModelEvent(sName, true, true);
    if(evt != null){
        evt.data = objData;
        this.dispatchEvent(evt);
    }
};

//
MapSceneController.prototype.setPointWithinScene = function(flag){
    if(this.flagPointWithinScene == flag)
        return;
    this.flagPointWithinScene = flag;
    if(this.flagPointWithinScene) {
        miscFunctions.disableDefaultTouchAction(document.body);
        document.body.style.touchAction = "pinch-zoom";
    }else
        miscFunctions.enableDefaultTouchAction(document.body);
};

//
MapSceneController.prototype.fOnReset = function(e){
    this.fFillControllers();
};

//
MapSceneController.prototype.fMouseMove = function(e){
    if(typeof e.touches != TYPE_UNDEFINED){
        if(e.touches.length>1) {
            //e.stopImmediatePropagation();
            //e.stopPropagation();
            //e.preventDefault();
            return;
        }
    }
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this, pageX, pageY;
    var model = me.model;
    var pPageXY = miscFunctions.getPageXY(e);
    if(pPageXY == null)
        return;
    pageX = pPageXY.x;
    pageY = pPageXY.y;

    var pos = miscFunctions.getPosition(me.visualization.getCanvas());
    if(pos == null)
        return;

    me.flagVScrollBarHit = false;
    me.flagHScrollBarHit = false;
    var hTInfo, scrollBar;
    var p1 = me.visualization.canvasCoordsToElementCoords(pageX-pos.x, pageY-pos.y, false).add(model.getOrigin());
    var flagMouseWasDownOnScrollBar = false;
    if(model.isWithVerticalScrollBar() && model.isNeedVerticalScrollBar()) {
        scrollBar = model.getVerticalScrollBar();
        flagMouseWasDownOnScrollBar = flagMouseWasDownOnScrollBar || scrollBar.isMouseDown();
        hTInfo = scrollBar.getHitTestInfoForPoint(p1);
        me.flagVScrollBarHit = hTInfo.isHit();
        if(me.flagMouseIsDown && scrollBar.isMouseDown())
            scrollBar.setSelected(hTInfo.isHit(), hTInfo.getDetails());
        if(scrollBar.isDraggerSelected())
            scrollBar.setDraggerPosition(scrollBar.getDraggerOffset()+ p1.y);
    }
    if(model.isWithHorizontalScrollBar() && model.isNeedHorizontalScrollBar()) {
        scrollBar = model.getHorizontalScrollBar();
        flagMouseWasDownOnScrollBar = flagMouseWasDownOnScrollBar || scrollBar.isMouseDown();
        hTInfo = scrollBar.getHitTestInfoForPoint(p1);
        me.flagHScrollBarHit = hTInfo.isHit();
        if(me.flagMouseIsDown && scrollBar.isMouseDown())
            scrollBar.setSelected(hTInfo.isHit(), hTInfo.getDetails());
        if(scrollBar.isDraggerSelected())
            scrollBar.setDraggerPosition(scrollBar.getDraggerOffset()+ p1.x);
    }

    if(!me.flagVScrollBarHit && !me.flagHScrollBarHit && !flagMouseWasDownOnScrollBar) {

        var p = me.visualization.canvasCoordsToElementCoords(pageX - pos.x, pageY - pos.y);

        var evt, evtScene;
        if (!model.isPointWithinScene(p)) {
            if (me.flagPointWithinScene) {
                me.setPointWithinScene(false);
                evtScene = new ModelEvent(ModelEvent.prototype.OUT_SCENE, true, true);
                if (evtScene != null)
                    me.dispatchEvent(evtScene);
            }
            return;
        }
        if (!me.flagPointWithinScene) {
            me.setPointWithinScene(true);
            evtScene = new ModelEvent(ModelEvent.prototype.OVER_SCENE, true, true);
            if (evtScene != null)
                me.dispatchEvent(evtScene);
        }

        var scaleFrame = model.getScaleFrame();
        var scaleFrameHTInfo = scaleFrame.getHitTestInfoForPoint(p);
        me.scaleFrameHandlerDetails = scaleFrameHTInfo.getDetails();

        if (me.flagMoveScaleHandler) {
            evt = new ModelEvent(ModelEvent.prototype.MOVE_SCALE_HANDLE, true, true);
            if (evt != null) {
                evt.data = {
                    x: p.x, y: p.y,
                    pageX: pageX, pageY: pageY,
                    currentToolId: me.getCurrentToolId(),
                    shiftKey: (typeof e.shiftKey != TYPE_UNDEFINED) ? e.shiftKey : false,
                    ctrlKey: (typeof e.ctrlKey != TYPE_UNDEFINED) ? e.ctrlKey : false
                };
                me.dispatchEvent(evt);
            }
        } else {
            if (!me.flagMouseIsDown && scaleFrameHTInfo.isHit() && scaleFrameHTInfo.hasDetails()) {
                me.flagOverScaleHandler = true;
                evt = new ModelEvent(ModelEvent.prototype.OVER_SCALE_HANDLE, true, true);
                if (evt != null) {
                    evt.data = {
                        where: me.scaleFrameHandlerDetails.where,
                        x: p.x, y: p.y, pageX: pageX, pageY: pageY,
                        currentToolId: me.getCurrentToolId()
                    };
                    me.dispatchEvent(evt);
                }
            } else if (me.flagOverScaleHandler) {
                me.flagOverScaleHandler = false;
                me.scaleFrameHandlerDetails = null;
                evt = new ModelEvent(ModelEvent.prototype.OUT_SCALE_HANDLE, true, true);
                if (evt != null) {
                    evt.data = {
                        x: p.x, y: p.y, pageX: pageX, pageY: pageY,
                        currentToolId: me.getCurrentToolId()
                    };
                    me.dispatchEvent(evt);
                }
            } else if (me.flagMouseIsDown) {
                //moving selected model elements
                if (me.flagSelectionMovable && (me.getCurrentToolId() == ModelTool.prototype.TOOL_SELECT)) {
                    var aSelected = scaleFrame.getSelected();
                    var j, l = aSelected.length;
                    var pStart = me.selectionMoveMouseStartPoint, p0;
                    for (j = 0; j < l; j++) {
                        element = aSelected[j];
                        if (typeof element["setX"] != TYPE_FUNCTION || typeof element["setY"] != TYPE_FUNCTION)
                            continue;
                        p0 = me.aSelectionMoveStartPoint[j];
                        element.setX(p0.x + (p.x - pStart.x));
                        element.setY(p0.y + (p.y - pStart.y));
                    }
                    if (l > 0) {
                        evt = new ModelEvent(ModelEvent.prototype.SELECTED_ELEMENTS_MOVED, true, true);
                        if (evt != null) {
                            evt.data = {elements: aSelected};
                            me.dispatchEvent(evt);
                        }
                    }
                }
            }
        }

        var nn = model.getElementsCount(), ii;
        var element, flagNeedValidation = false;
        for (ii = 0; ii < nn; ii++) {
            element = model.getDepthSortedElementNo(ii);
            if (element.getHitDetails() != null) {
                flagNeedValidation = true;
                element.setHitDetails(null);
            }
        }

        var aHTInfo = model.getHitTestInfosForPoint(p);
        var aElement = [];

        var options = {event: e, mouseIsDown: me.flagMouseIsDown};
        var flagToolCanHighlight = me.getCurrentTool().canHighlight(
            p, (aHTInfo.length > 0) ? [aHTInfo[0].getElement()] : [], options
        );
        //highlight
        var cnt = me.fUnHighlightAll();
        if (aHTInfo.length > 0) {
            var i, n = aHTInfo.length;
            var htInfo;
            for (i = 0; i < n; i++) {
                htInfo = aHTInfo[i];
                element = htInfo.getElement();
                aElement.push(element);
                if (!me.flagMouseIsDown && htInfo.isHit() && element.canBeSelected()) {
                    if (flagToolCanHighlight)
                        element.setHighlighted(true);
                    cnt++;
                    break;
                }
            }
        }

        if (!flagToolCanHighlight) {
            if (me.toolsController.isCurrentToolStarted()) {
                me.toolsController.trackCurrentTool(
                    p, aElement, options
                );
            }
        }

        flagNeedValidation = flagNeedValidation || (cnt > 0);
        if (flagNeedValidation)
            model.invalidate();

        evt = new ModelEvent(me.flagMouseIsDown ? ModelEvent.prototype.MOUSE_DRAG : ModelEvent.prototype.MOUSE_MOVE);
        if (evt != null) {
            evt.data = {
                x: p.x, y: p.y, pageX: pageX, pageY: pageY,
                currentToolId: me.getCurrentToolId()
            };
            me.dispatchEvent(evt);
        }

        if (me.flagMoveScene) {
            var rSize = me.visualization.getCanvasToSceneSizeRatioZoom();
            model.setOriginX(me.mouseDownOrigin.x + (pageX - me.mouseDownPoint.x) / rSize);
            model.setOriginY(me.mouseDownOrigin.y + (pageY - me.mouseDownPoint.y) / rSize);
            model.invalidate();
        }
    }else{
        me.fUnHighlightAll();
    }
    e.stopPropagation();
    e.preventDefault();
};

//
MapSceneController.prototype.fMouseHoldDown = function(p, aElement, options){
    var me = this;
    me.toolsController.useCurrentToolHoldDown(p, aElement, options);
};

//
MapSceneController.prototype.fMouseDown = function(e){
    if(typeof e.touches != TYPE_UNDEFINED){
        if(e.touches.length>1) {
            //e.stopPropagation();
            //e.preventDefault();
            return;
        }
    }
    if(!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this, pageX, pageY;
    var model = me.model;
    var pPageXY = miscFunctions.getPageXY(e);
    if(pPageXY == null)
        return;
    pageX = pPageXY.x;
    pageY = pPageXY.y;

    me.fUnblinkAll();

    me.flagMoveScene = false;
    me.flagMouseIsDown = true;
    var pos = miscFunctions.getPosition(me.visualization.getCanvas());
    var p = me.visualization.canvasCoordsToElementCoords(pageX-pos.x, pageY-pos.y);
    if(!me.model.isPointWithinScene(p)) {
        me.setPointWithinScene(false);
        return;
    }
    me.setPointWithinScene(true);

    var shiftKey = (typeof e.shiftKey != TYPE_UNDEFINED) ? e.shiftKey : false;
    var flagToolStartedOrUsed = false;

    var lHT = 0;
    var nn = model.getElementsCount(), i, ii;
    var element, flagNeedValidation = false;
    for(ii=0; ii<nn; ii++) {
        element = model.getDepthSortedElementNo(ii);
        if(element.getHitDetails() != null) {
            flagNeedValidation = true;
            element.setHitDetails(null);
        }
    }

    me.flagVScrollBarHit = false;
    me.flagHScrollBarHit = false;
    var p1 = me.visualization.canvasCoordsToElementCoords(pageX-pos.x, pageY-pos.y, false).add(model.getOrigin());
    var hTInfo = null, scrollBar;
    if(model.isWithVerticalScrollBar() && model.isNeedVerticalScrollBar()) {
        scrollBar = model.getVerticalScrollBar();
        hTInfo = scrollBar.getHitTestInfoForPoint(p1);
        if(hTInfo.isHit()) {
            me.flagVScrollBarHit = true;
            scrollBar.setMouseDown(true);
            scrollBar.setSelected(true, hTInfo.getDetails());
        }
    }
    if(model.isWithHorizontalScrollBar() && model.isNeedHorizontalScrollBar()) {
        scrollBar = model.getHorizontalScrollBar();
        hTInfo = scrollBar.getHitTestInfoForPoint(p1);
        if(hTInfo.isHit()) {
            me.flagHScrollBarHit = true;
            scrollBar.setMouseDown(true);
            scrollBar.setSelected(true, hTInfo.getDetails());
        }
    }

    if(!me.flagVScrollBarHit && !me.flagHScrollBarHit) {
        var scaleFrame = model.getScaleFrame();
        if (e.type == "touchstart") {
            var scaleFrameHTInfo = scaleFrame.getHitTestInfoForPoint(p);
            if (scaleFrameHTInfo.isHit() && scaleFrameHTInfo.hasDetails()) {
                me.flagOverScaleHandler = true;
                me.scaleFrameHandlerDetails = scaleFrameHTInfo.getDetails();
            }
        }

        var aHTInfo = model.getHitTestInfosForPoint(p);
        lHT = aHTInfo.length;
        var aElement = [];
        var options = {event: e, mouseIsDown: me.flagMouseIsDown};
        var flagToolCanSelect = me.getCurrentTool().canSelect(
            p, (lHT > 0) ? [aHTInfo[0].getElement()] : [], options
        );
        var flagToolSelection = (me.getCurrentToolId() == ModelTool.prototype.TOOL_SELECT);
        var flagToolPan = (me.getCurrentToolId() == ModelTool.prototype.TOOL_PAN);
        var flagHitElementIsSelected = lHT > 0 && aHTInfo[0].getElement().isSelected();
        //put selection frame
        var cnt = (flagToolSelection && (shiftKey || flagHitElementIsSelected)) ?
            0 :
            me.fUnSelectAll(
                (me.flagOverScaleHandler && flagToolSelection) ? scaleFrame.getSelected() : null
            );

        if(!me.flagOverScaleHandler) {
            //if there is a scaled element, selection should not change
            if (lHT > 0) {
                var htInfo;
                for (i = 0; i < lHT; i++) {
                    htInfo = aHTInfo[i];
                    element = htInfo.getElement();
                    aElement.push(element);
                    if (element.canBeSelected()) {
                        if (flagToolCanSelect) {
                            me.fSelectModelElement(element, !shiftKey || !element.isSelected());
                        }
                        cnt++;
                        break;
                    }
                }
            }
        }

        if (!flagToolCanSelect && me.toolsController.getCurrentToolId() != ModelTool.prototype.TOOL_SELECT) {
            if (typeof e.button != TYPE_UNDEFINED && e.button == ModelEvent.prototype.RIGHT_BUTTON) {
                me.toolsController.cancelCurrentTool(p, aElement, options);
            } else if (me.toolsController.isCurrentToolStarted() && me.toolsController.currentToolCanBeUsed(p, aElement, options)) {
                me.toolsController.useCurrentTool(p, aElement, options);
                flagToolStartedOrUsed = true;
            } else if (me.toolsController.currentToolCanBeStarted(p, aElement, options)) {
                me.toolsController.startCurrentTool(p, aElement, options);
                flagToolStartedOrUsed = true;
            }
        }

        flagNeedValidation = flagNeedValidation || (cnt > 0);

        if (flagNeedValidation) {
            me.model.invalidate();
        }

        if (!flagToolStartedOrUsed) {
            me.flagMoveScaleHandler = me.flagOverScaleHandler;
            if (me.flagOverScaleHandler) {
                evt = new ModelEvent(ModelEvent.prototype.DOWN_SCALE_HANDLE, true, true);
                if (evt != null) {
                    evt.data = {
                        where: me.scaleFrameHandlerDetails.where,
                        x: p.x, y: p.y, pageX: pageX, pageY: pageY,
                        currentToolId: me.getCurrentToolId()
                    };
                    me.dispatchEvent(evt);
                }
            } else {
                var aSelected = scaleFrame.getSelected();
                me.flagSelectionMovable =
                    ModelElement.prototype.checkIfAllAre(aSelected, "isMovableByMouse") &&
                    ModelElement.prototype.checkIfAllHaveMethod(aSelected, "getX") &&
                    ModelElement.prototype.checkIfAllHaveMethod(aSelected, "getY") &&
                    ModelElement.prototype.checkIfAllHaveMethod(aSelected, "setX") &&
                    ModelElement.prototype.checkIfAllHaveMethod(aSelected, "setY");
                if (me.flagSelectionMovable) {
                    var j, l = aSelected.length;
                    me.selectionMoveMouseStartPoint = p;
                    me.aSelectionMoveStartPoint = [];
                    for (j = 0; j < l; j++) {
                        element = aSelected[j];
                        me.aSelectionMoveStartPoint.push(new Point(element.getX(), element.getY()));
                    }
                }
            }
        }

        //TODO comment "|| e.type == "mousedown"" for release; holdDown is for touch-screen browsers only
        if (e.type == "touchstart" || e.type == "mousedown")
            ModelEvent.prototype.startHoldDown(me.fMouseHoldDown, me, [p, aElement, options]);

        var evt = new ModelEvent(ModelEvent.prototype.MOUSE_DOWN);
        if (evt != null) {
            evt.data = {
                id: (lHT > 0) ? aHTInfo[0].getElement().getId() : null,
                currentToolId: me.getCurrentToolId(),
                x: p.x, y: p.y,
                pageX: pageX, pageY: pageY,
                shiftKey: (typeof e.shiftKey != TYPE_UNDEFINED) ? e.shiftKey : false,
                ctrlKey: (typeof e.ctrlKey != TYPE_UNDEFINED) ? e.ctrlKey : false
            };
            me.dispatchEvent(evt);
        }

        me.flagMoveScene =
            (model.isNeedHorizontalScrollBar() || model.isNeedVerticalScrollBar()) &&
            (((lHT == 0) && flagToolSelection && !me.flagOverScaleHandler) || flagToolPan) &&
            !(me.flagVScrollBarHit || me.flagHScrollBarHit) &&
            (typeof e.button == TYPE_UNDEFINED || e.button == ModelEvent.prototype.LEFT_BUTTON);

        if (me.flagMoveScene) {
            me.mouseDownPoint.x = pageX;
            me.mouseDownPoint.y = pageY;
            me.mouseDownOrigin.x = model.getOriginX();
            me.mouseDownOrigin.y = model.getOriginY();
        }
    }
    e.stopPropagation();
    e.preventDefault();
};

//
MapSceneController.prototype.fMouseUp = function(e){
    if(typeof e.touches != TYPE_UNDEFINED){
        if(e.touches.length>1) {
            //e.stopPropagation();
            //e.preventDefault();
            return;
        }
    }
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this, pageX, pageY, evt;
    var pPageXY = miscFunctions.getPageXY(e);
    if(pPageXY == null)
        return;
    pageX = pPageXY.x;
    pageY = pPageXY.y;

    var model = me.model;

    me.flagMouseIsDown = false;
    me.flagMoveScene = false;
    me.flagVScrollBarHit = false;
    me.flagHScrollBarHit = false;

    var pos = miscFunctions.getPosition(me.visualization.getCanvas());
    var p = me.visualization.canvasCoordsToElementCoords(pageX-pos.x, pageY-pos.y);

    if(e.type == "touchend"){
        me.flagOverScaleHandler = false;
        me.scaleFrameHandlerDetails = null;
        model.resetHitTestInfos();
        model.invalidate();
    }

    var scrollBar;
    if(model.isWithVerticalScrollBar() && model.isNeedVerticalScrollBar()) {
        scrollBar = model.getVerticalScrollBar();
        scrollBar.setSelected(false);
        scrollBar.setMouseDown(false);
    }
    if(model.isWithHorizontalScrollBar() && model.isNeedHorizontalScrollBar()) {
        scrollBar = model.getHorizontalScrollBar();
        scrollBar.setSelected(false);
        scrollBar.setMouseDown(false);
    }

    if(me.flagMoveScaleHandler){
        me.flagMoveScaleHandler = false;
        evt = new ModelEvent(ModelEvent.prototype.UP_SCALE_HANDLE, true, true);
        if(evt!=null){
            evt.data = {
                x: p.x, y: p.y,
                pageX:pageX, pageY:pageY,
                currentToolId:me.getCurrentToolId(),
                shiftKey:(typeof e.shiftKey != TYPE_UNDEFINED) ? e.shiftKey : false,
                ctrlKey:(typeof e.ctrlKey != TYPE_UNDEFINED) ? e.ctrlKey : false
            };
            me.dispatchEvent(evt);
        }
    }

    e.stopPropagation();
    e.preventDefault();

    var options = {event: e, mouseIsDown: me.flagMouseIsDown};
    if(me.toolsController.isCurrentToolStarted() && me.toolsController.currentToolCanBeUsed(p, [], options))
        me.toolsController.useCurrentToolMouseUp(p, [], options);

    evt = new ModelEvent(ModelEvent.prototype.MOUSE_UP);
    if(evt!=null) {
        evt.data = {
            x: p.x, y: p.y,
            pageX:pageX, pageY:pageY,
            currentToolId:me.getCurrentToolId(),
            shiftKey:(typeof e.shiftKey != TYPE_UNDEFINED) ? e.shiftKey : false,
            ctrlKey:(typeof e.ctrlKey != TYPE_UNDEFINED) ? e.ctrlKey : false
        };
        me.dispatchEvent(evt);
    }
};

MapSceneController.prototype.fDoubleClickListener = function(e){
    if(!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this, pageX, pageY;
    var pPageXY = miscFunctions.getPageXY(e);
    if(pPageXY == null)
        return;
    pageX = pPageXY.x;
    pageY = pPageXY.y;

    var pos = miscFunctions.getPosition(me.visualization.getCanvas());
    var p = me.visualization.canvasCoordsToElementCoords(pageX-pos.x, pageY-pos.y);
    if(!me.model.isPointWithinScene(p)) {
        me.setPointWithinScene(false);
        return;
    }
    me.setPointWithinScene(true);

    var aHTInfo = me.model.getHitTestInfosForPoint(p);

    var element, aElement = [];
    if (aHTInfo.length > 0) {
        var i, n = aHTInfo.length;
        var htInfo;
        for (i = 0; i < n; i++) {
            htInfo = aHTInfo[i];
            element = htInfo.getElement();
            aElement.push(element);
            if (element.canBeSelected()) {
                break;
            }
        }
    }

    var options = {event: e, mouseIsDown: me.flagMouseIsDown};
    if(me.toolsController.isCurrentToolStarted() && me.toolsController.currentToolCanBeUsed(p, aElement, options))
        me.toolsController.useCurrentToolDoubleClick(p, aElement, options);

    var evt = new ModelEvent(ModelEvent.prototype.DOUBLE_CLICK, true, true);
    if(evt!=null){
        evt.data = {
            id:(aHTInfo.length>0) ? aHTInfo[0].getElement().getId() : (-1),
            x: p.x, y: p.y,
            pageX:pageX, pageY:pageY,
            currentToolId:me.getCurrentToolId(),
            shiftKey:(typeof e.shiftKey != TYPE_UNDEFINED) ? e.shiftKey : false,
            ctrlKey:(typeof e.ctrlKey != TYPE_UNDEFINED) ? e.ctrlKey : false
        };
        me.dispatchEvent(evt);
    }
};

MapSceneController.prototype.fKeydownListener = function(e){
    var me = this;
    var sKey;

    me.visualization.setCtrl((typeof e.ctrlKey != TYPE_UNDEFINED) ? e.ctrlKey : false);

    if(typeof e.keyCode == TYPE_NUMBER)
        switch(e.keyCode){
            case MapSceneController.prototype.KEY_CODE_ARROW_LEFT:
                sKey = MapSceneController.prototype.ARROW_LEFT;
                break;
            case MapSceneController.prototype.KEY_CODE_ARROW_DOWN:
                sKey = MapSceneController.prototype.ARROW_DOWN;
                break;
            case MapSceneController.prototype.KEY_CODE_ARROW_RIGHT:
                sKey = MapSceneController.prototype.ARROW_RIGHT;
                break;
            case MapSceneController.prototype.KEY_CODE_ARROW_UP:
                sKey = MapSceneController.prototype.ARROW_UP;
                break;
            case MapSceneController.prototype.KEY_CODE_DELETE:
                sKey = MapSceneController.prototype.DELETE;
                break;
            default:
                sKey = null;
        }
    else if(typeof e.key == TYPE_STRING){
        switch(e.key){
            case MapSceneController.prototype.ARROW_LEFT:
            case MapSceneController.prototype.ARROW_DOWN:
            case MapSceneController.prototype.ARROW_RIGHT:
            case MapSceneController.prototype.ARROW_UP:
            case MapSceneController.prototype.DELETE:
                sKey = e.key;
                break;
            default:
                sKey = null;
        }
    }else
        sKey = null;

    if(sKey == null)
        return;

    var scaleFrame = me.model.getScaleFrame();
    var aSelected = scaleFrame.getSelected();
    var i, n = aSelected.length, cnt, flag;
    var element;
    var evt;
    if(n>0) {
        switch(sKey){
            case MapSceneController.prototype.ARROW_LEFT:
            case MapSceneController.prototype.ARROW_DOWN:
            case MapSceneController.prototype.ARROW_RIGHT:
            case MapSceneController.prototype.ARROW_UP:
                var flagSelectionMovable =
                    ModelElement.prototype.checkIfAllAre(aSelected, "isMovableByKeyboard") &&
                    ModelElement.prototype.checkIfAllHaveMethod(aSelected, "getX") &&
                    ModelElement.prototype.checkIfAllHaveMethod(aSelected, "getY") &&
                    ModelElement.prototype.checkIfAllHaveMethod(aSelected, "setX") &&
                    ModelElement.prototype.checkIfAllHaveMethod(aSelected, "setY");
                if (flagSelectionMovable) {
                    var sGetName, sSetName;
                    var shiftKey = (typeof e.shiftKey != TYPE_UNDEFINED) ? e.shiftKey : false;
                    var d =
                        (shiftKey ?
                                MapSceneController.prototype.KEYBOARD_SHIFT_KEY_MULTIPLIER : 1
                        ) * MapSceneController.prototype.KEYBOARD_DISPLACEMENT;
                    switch (sKey) {
                        case MapSceneController.prototype.ARROW_LEFT:
                            sGetName = "getX";
                            sSetName = "setX";
                            d = -d;
                            break;
                        case MapSceneController.prototype.ARROW_DOWN:
                            sGetName = "getY";
                            sSetName = "setY";
                            break;
                        case MapSceneController.prototype.ARROW_RIGHT:
                            sGetName = "getX";
                            sSetName = "setX";
                            break;
                        case MapSceneController.prototype.ARROW_UP:
                            sGetName = "getY";
                            sSetName = "setY";
                            d = -d;
                            break;
                    }
                    for (i = 0; i < n; i++) {
                        element = aSelected[i];
                        if(typeof element[sSetName] == TYPE_FUNCTION && typeof element[sGetName] == TYPE_FUNCTION)
                            element[sSetName].call(element, element[sGetName].call(element) + d);
                    }
                    evt = new ModelEvent(ModelEvent.prototype.SELECTED_ELEMENTS_MOVED, true, true);
                    if (evt != null) {
                        evt.data = {elements: aSelected};
                        me.dispatchEvent(evt);
                    }
                }
                break;

            case MapSceneController.prototype.DELETE:
                n = me.model.getElementsCount();
                cnt = 0;
                for(i=0; i<n; i++) {
                    element = me.model.getDepthSortedElementNo(i);
                    if(element.isSelected() && element.canBeRemoved()){
                        flag = me.toolsController.removeElementFromModel(element);
                        if(flag) {
                            i--;
                            n--;
                            cnt++;
                        }
                    }
                }
                me.flagSelectionMovable = false;
                break;

            default:
        }
    }
};

//
MapSceneController.prototype.resetMapScene = function(){
    var me = this;
    var i, n, flag;
    var element;
    n = me.model.getElementsCount();
    for(i=0; i<n; i++) {
        element = me.model.getDepthSortedElementNo(i);
        flag = me.toolsController.removeElementFromModel(element);
        if(flag) {
            i--;
            n--;
        }
    }
    me.flagSelectionMovable = false;
};


MapSceneController.prototype.fKeyupListener = function(e) {
    var me = this;
    me.visualization.setCtrl((typeof e.ctrlKey != TYPE_UNDEFINED) ? e.ctrlKey : false);
};


MapSceneController.prototype.fMouseWheelListener = function(e){
    var me = this, pageX, pageY;
    var model = me.model;
    var pPageXY = miscFunctions.getPageXY(e);
    if(pPageXY == null)
        return;
    pageX = pPageXY.x;
    pageY = pPageXY.y;

    var pos = miscFunctions.getPosition(me.visualization.getCanvas());
    if(pos == null)
        return;
    var p = me.visualization.canvasCoordsToElementCoords(pageX-pos.x, pageY-pos.y, false).add(model.getOrigin());
    var scrollBar, hTInfo, flagScrollProcessed = false;
    if(p.x>=0 && p.y>=0 && p.x<model.getSceneWidth() && p.y<model.getSceneHeight()) {
        if (model.isWithHorizontalScrollBar() && model.isNeedHorizontalScrollBar()) {
            scrollBar = model.getHorizontalScrollBar();
            hTInfo = scrollBar.getHitTestInfoForPoint(p);
            if (hTInfo.isHit()) {
                flagScrollProcessed = true;
            }
        }
        if (!flagScrollProcessed && model.isWithVerticalScrollBar() && model.isNeedVerticalScrollBar()) {
            scrollBar = model.getVerticalScrollBar();
            flagScrollProcessed = true;
        }
        if(flagScrollProcessed){
            if (e.deltaY > 0)
                scrollBar.increase();
            else if (e.deltaY < 0)
                scrollBar.decrease();
        }
    }
};


MapSceneController.prototype.fSetToolListener = function(e) {
    var me = this;
    var toolId = e.data.id;
    if (toolId != ModelTool.prototype.TOOL_SELECT && toolId != me.lastToolId) {
        me.fUnHighlightAll();
        me.fUnSelectAll();
        me.model.invalidate();
    }
    me.lastToolId = toolId;
    var evt = new ModelEvent(ModelEvent.prototype.SET_TOOL, true, true);
    if(evt != null) {
        evt.data = {id: e.data.id, details:(typeof e.data.details != TYPE_UNDEFINED) ? e.data.details : null};
        me.dispatchEvent(evt);
    }
};

MapSceneController.prototype.fStartToolListener = function(e){
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.START_TOOL, true, true);
    if(evt != null) {
        evt.data = {id: e.data.id};
        me.dispatchEvent(evt);
    }
};

MapSceneController.prototype.fTrackToolListener = function(e){
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.TRACK_TOOL, true, true);
    if(evt != null) {
        evt.data = {id: e.data.id};
        me.dispatchEvent(evt);
    }
};

MapSceneController.prototype.fUseToolListener = function(e){
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.USE_TOOL, true, true);
    if(evt != null) {
        evt.data = {id: e.data.id};
        me.dispatchEvent(evt);
    }
};

MapSceneController.prototype.fEndToolListener = function(e){
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.END_TOOL, true, true);
    if(evt != null) {
        evt.data = {id: e.data.id};
        me.dispatchEvent(evt);
    }
};

MapSceneController.prototype.fCancelToolListener = function(e){
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.CANCEL_TOOL, true, true);
    if(evt != null) {
        evt.data = {id: e.data.id};
        me.dispatchEvent(evt);
    }
};

MapSceneController.prototype.fFillControllers = function(){
    var me = this;
    var model = me.model;
    var visualization = me.visualization;
    var i, n = model.getElementsCount();
    var element;
    var sClassName;
    var cls;
    var elementController;
    var elementVisualization;
    for(i=0; i<n; i++){
        element = model.getDepthSortedElementNo(i);
        if(element.isVerifiable() && !element.isShowIfVerifiable())
            continue;
        sClassName = element.getClassName();
        cls = window[sClassName + MapSceneController.prototype.CONTROLLER_SUFFIX];
        if(typeof cls != TYPE_UNDEFINED){
            elementVisualization = visualization.getVisualizationByElement(element);
            elementController = new cls(element, elementVisualization, me);
            me.aController.push(elementController);
        }
    }

};

//
MapSceneController.prototype.fSymbolHighlighted = function(e){
    //TODO symbol replacement for arrows and points by ctrl+rollover in the menu droplist
};

//
MapSceneController.prototype.getControllerByElement = function(element){
    var me = this;
    var aController = me.aController;
    var i, n = aController.length;
    var controller;
    for(i=0; i<n; i++){
        controller = aController[i];
        if(controller.getElement() == element)
            return controller;
    }
    return null;
};

//elements created at model runtime are added and removed via controller
MapSceneController.prototype.addElementToModel = function(element){
    var me = this;
    var model = me.model;
    var visualization = me.visualization;
    var elementVisualization;
    var elementController;

    model.addElement(element);
    visualization.addVisualizationForElement(element);
    var sClassName = element.getClassName();

    var cls = window[sClassName + MapSceneController.prototype.CONTROLLER_SUFFIX];
    if(typeof cls != TYPE_UNDEFINED){
        elementVisualization = visualization.getVisualizationByElement(element);
        elementController = new cls(element, elementVisualization, me);
        me.aController.push(elementController);
    }

    if(element.isSelected()){
        model.getScaleFrame().addToSelected(element);
    }
};

//returns true if the element was removed
MapSceneController.prototype.removeElementFromModel = function(element){
    var me = this;
    var aController = me.aController;
    var controller = me.getControllerByElement(element);
    if(controller instanceof ModelElementController){
        controller.deactivate();
        aController.splice(aController.indexOf(controller), 1);
    }
    me.visualization.removeVisualizationForElement(element);
    me.fSelectModelElement(element, false);
    return me.model.removeElement(element);
};

//
MapSceneController.prototype.updateVisualization = function(){
    this.visualization.update();
};

//
MapSceneController.prototype.deactivate = function(){
    MapSceneController.superclass.deactivate.call(this);
    var me = this;
    var i, n = me.aController.length;
    for(i=0; i<n; i++)
        me.aController[i].deactivate();
    me.scaleFrameController.deactivate();
    me.visualization.getCanvas().removeEventListener("wheel", me.mouseWheelListener, false);
    me.model.removeEventListener(ModelEvent.prototype.RESET_SCENE, {listener:me.fOnReset, target:me}, false);
    window.removeEventListener("mousemove", me.mouseMoveListener, true);
    window.removeEventListener("touchmove", me.mouseMoveListener, true);
    window.removeEventListener("mouseup", me.mouseUpListener, false);
    window.removeEventListener("touchend", me.mouseUpListener, false);
    document.body.removeEventListener("contextmenu", me.mouseContextMenuListener, false);
    document.body.removeEventListener("mousedown", me.mouseDownListener, false);
    document.body.removeEventListener("touchstart", me.mouseDownListener, false);
    document.body.removeEventListener("dblclick", me.mouseDoubleClickListener, false);
    //
    document.removeEventListener("keydown", me.keydownListener, true);
    document.removeEventListener("keyup", me.keyupListener, true);
    //
    document.body.removeEventListener(ModelEvent.prototype.SYMBOL_HIGHLIGHTED, me.symbolHighlighted, false);
    //
    if(me.toolsController != null)
        me.setToolsController(null);
};
/**
 * @constructor
 */
function ElementsGroup(mapSceneModel, id){
    var me = this;
    me.mapSceneModel = mapSceneModel;
    me.id = id;
    me.aId = [];
    me.name = null;
}
extend(ElementsGroup, Object);

//
ElementsGroup.prototype.getId = function(){
    return this.id;
};

//
ElementsGroup.prototype.getName = function(){
    return this.name;
};

//
ElementsGroup.prototype.getCount = function(){
    return this.aId.length;
};

//
ElementsGroup.prototype.getElementNo = function(n){
    return (n>=0 && n<this.aId.length) ? me.mapSceneModel.getElementById(this.aId[n]) : null;
};

//
ElementsGroup.prototype.fUpdateName = function(){
    var me = this;
    var maxDepth = -1, depth;
    var element;
    var i, n = me.aId.length, iMax = -1, elementMax = null;
    for(i=0; i<n; i++){
        element = me.mapSceneModel.getElementById(me.aId[i]);
        depth = element.getDepth();
        if(depth>maxDepth){
            elementMax = element;
            maxDepth = depth;
        }
    }
    if(elementMax != null)
        me.name = elementMax.getName();
};

//
ElementsGroup.prototype.isContainsElementWithId = function(q){
    return this.aId.indexOf(q) != -1;
};

//
ElementsGroup.prototype.addElementById = function(q){
    var me = this;
    if(me.aId.indexOf(q) != -1)
        return false;
    me.aId.push(q);
    me.fUpdateName();
    return true;
};

//
ElementsGroup.prototype.removeElementById = function(q){
    var me = this;
    var i = me.aId.indexOf(q);
    if(i == -1)
        return false;
    me.aId.splice(i, 1);
    me.fUpdateName();
    return true;
};

//
ElementsGroup.prototype.getElements = function(){
    var me = this;
    var a= [];
    var i, n = me.aId.length;
    for(i=0; i<n; i++)
        a.push(me.mapSceneModel.getElementById(me.aId[i]));
    return a;
};

//
ElementsGroup.prototype.callMethodOnElements = function(sMethodName, args){
    var me = this;
    var a = me.getElements();
    var i, n = a.length, element;
    for(i=0; i<n; i++){
        element = a[i];
        if(typeof element[sMethodName] == TYPE_FUNCTION)
            element[sMethodName].apply(element, args);
    }
};

//
ElementsGroup.prototype.removeAllElements = function(){
    var me = this;
    me.aId.splice(0);
    me.name = null;
};
/**
 * @constructor
 */
function ScaleFrameModel(mapSceneModel){
    var me = this;
    me.mapSceneModel = mapSceneModel;
    me.aSelected = [];
    //top left corners of handlers
    me.aHandlerX = new Array(8);
    me.aHandlerY = new Array(8);
    me.bounds = null;
    me.flagBoundsChanged = false;
    me.flagValid = true;
}
extend(ScaleFrameModel, Object);

//factor for increasing the sensitive area for touch-screens
ScaleFrameModel.prototype.TOUCH_SIZE_FACTOR = 10;

ScaleFrameModel.prototype.DEFAULT_LINE_WIDTH = 1;
ScaleFrameModel.prototype.DEFAULT_LINE_BACKGROUND_WIDTH = 2;
ScaleFrameModel.prototype.DEFAULT_HANDLER_SIZE = 4;
ScaleFrameModel.prototype.DEFAULT_HIT_MARGIN = 2;

ScaleFrameModel.prototype.getSelected = function(){
    return this.aSelected;
};

ScaleFrameModel.prototype.addToSelected = function(element){
    var me = this;
    var aSelected = me.aSelected;
    var i = aSelected.indexOf(element);
    if(i == -1) {
        aSelected.push(element);
        me.updateBounds();
    }
};

ScaleFrameModel.prototype.removeFromSelected = function(element){
    var me = this;
    var aSelected = me.aSelected;
    var i = aSelected.indexOf(element);
    if(i != -1) {
        aSelected.splice(i, 1);
        me.updateBounds();
    }
};

ScaleFrameModel.prototype.countSelected = function(){
    return this.aSelected.length;
};

ScaleFrameModel.prototype.isValid = function(){
    return this.flagValid;
};

ScaleFrameModel.prototype.validate = function(){
    var me = this;
    if(!me.flagValid) {
        me.flagBoundsChanged = false;
        me.flagValid = true;
    }
};

ScaleFrameModel.prototype.getBounds = function(){
    return this.bounds;
};

ScaleFrameModel.prototype.getHandlerX = function(n){
    return this.aHandlerX[n];
};

ScaleFrameModel.prototype.getHandlerY = function(n){
    return this.aHandlerY[n];
};

ScaleFrameModel.prototype.getHandlerNoByWhere = function(where){
    switch (where) {
        case "TL": return 0;
        case "ML": return 1;
        case "BL": return 2;
        case "BM": return 3;
        case "BR": return 4;
        case "MR": return 5;
        case "TR": return 6;
        case "TM": return 0;
        default: return -1;
    }
};

ScaleFrameModel.prototype.getHandlerShiftForPoint = function(where, x, y){
    var me = this;
    var no = me.getHandlerNoByWhere(where);
    return new Point(
        x - me.bounds.x - ((no!=-1) ?  me.aHandlerX[no] : 0),
        y - me.bounds.y - ((no!=-1) ?  me.aHandlerY[no] : 0)
    );
};

ScaleFrameModel.prototype.setHandlerPosition = function(where, x, y){
    var me = this;
    var c = me.countSelected();
    if(c == 1) {
        var hSize = ScaleFrameModel.prototype.DEFAULT_HANDLER_SIZE;
        var lbWidth = ScaleFrameModel.prototype.DEFAULT_LINE_BACKGROUND_WIDTH;
        var wMin = 3*hSize+2*lbWidth;
        var hMin = wMin;
        var element = me.getSelected()[0];
        if(typeof element.setBounds == TYPE_FUNCTION && typeof element.getExtremeBounds == TYPE_FUNCTION) {
            me.flagValid = false;
            var bounds = me.bounds;
            var boundsNew = {};
            var boundsExtreme = element.getExtremeBounds();
            if(boundsExtreme.widthMin<wMin)
                boundsExtreme.widthMin = wMin;
            if(boundsExtreme.heightMin<wMin)
                boundsExtreme.heightMin = hMin;
            var xNew, yNew, wNew, hNew;
            var xMax, yMax, yMin, xMin;

            function correctWidthAndHeightPerRatio(){
                var pr1, pr2;
                var eZero = 1E-5;
                pr1 = wNew*bounds.height;
                pr2 = hNew*bounds.width;
                if(pr1<pr2){
                    hNew = pr1/bounds.width;
                    if(hNew<boundsExtreme.heightMin){
                        hNew = boundsExtreme.heightMin;
                        wNew = hNew*bounds.width/((bounds.height>eZero)?bounds.height:eZero);
                    }
                }else if(pr1>pr2){
                    wNew = pr2/bounds.height;
                    if(wNew<boundsExtreme.widthMin){
                        wNew = boundsExtreme.widthMin;
                        hNew = wNew*bounds.height/((bounds.width>eZero)?bounds.width:eZero);
                    }
                }
            }

            switch (where) {
                case "TL":
                    xMax = bounds.x+bounds.width-boundsExtreme.widthMin;
                    yMax = bounds.y+bounds.height-boundsExtreme.heightMin;
                    xNew = (x<=xMax) ? x : xMax;
                    yNew = (y<=yMax) ? y : yMax;
                    wNew = bounds.x+bounds.width-xNew;
                    hNew = bounds.y+bounds.height-yNew;
                    correctWidthAndHeightPerRatio();
                    xNew = bounds.x+bounds.width-wNew;
                    yNew = bounds.y+bounds.height-hNew;
                    break;
                case "ML":
                    xMax = bounds.x+bounds.width-boundsExtreme.widthMin;
                    xNew = (x<=xMax) ? x : xMax;
                    yNew = bounds.y;
                    wNew = bounds.x+bounds.width-xNew;
                    hNew = bounds.height;
                    break;
                case "BL":
                    xMax = bounds.x+bounds.width-boundsExtreme.widthMin;
                    yMin = bounds.y+boundsExtreme.heightMin;
                    xNew = (x<=xMax) ? x : xMax;
                    wNew = bounds.x+bounds.width-xNew;
                    hNew = y+hSize-bounds.y;
                    if(hNew<boundsExtreme.heightMin)
                        hNew = boundsExtreme.heightMin;
                    correctWidthAndHeightPerRatio();
                    xNew = bounds.x+bounds.width-wNew;
                    yNew = bounds.y;
                    break;
                case "BM":
                    xNew = bounds.x;
                    yNew = bounds.y;
                    wNew = bounds.width;
                    hNew = y+hSize-bounds.y;
                    if(hNew<boundsExtreme.heightMin)
                        hNew = boundsExtreme.heightMin;
                    break;
                case "BR":
                    xMin = bounds.x+boundsExtreme.widthMin;
                    yMin = bounds.y+boundsExtreme.heightMin;
                    wNew = x+hSize-bounds.x;
                    if(wNew<boundsExtreme.widthMin)
                        wNew = boundsExtreme.widthMin;
                    hNew = y+hSize-bounds.y;
                    if(hNew<boundsExtreme.heightMin)
                        hNew = boundsExtreme.heightMin;
                    correctWidthAndHeightPerRatio();
                    xNew = bounds.x;
                    yNew = bounds.y;
                    break;
                case "MR":
                    xNew = bounds.x;
                    yNew = bounds.y;
                    wNew = x+hSize-bounds.x;
                    if(wNew<boundsExtreme.widthMin)
                        wNew = boundsExtreme.widthMin;
                    hNew = bounds.height;
                    break;
                case "TR":
                    xNew = bounds.x;
                    yMax = bounds.y+bounds.height-boundsExtreme.heightMin;
                    yNew = (y<yMax) ? y : yMax;
                    wNew = x+hSize-bounds.x;
                    if(wNew<boundsExtreme.widthMin)
                        wNew = boundsExtreme.widthMin;
                    hNew = bounds.y+bounds.height-yNew;
                    correctWidthAndHeightPerRatio();
                    yNew = bounds.y+bounds.height-hNew;
                    break;
                case "TM":
                    xNew = bounds.x;
                    yMax = bounds.y+bounds.height-boundsExtreme.heightMin;
                    yNew = (y<yMax) ? y : yMax;
                    wNew = bounds.width;
                    hNew = bounds.y+bounds.height-yNew;
                    break;
            }
            boundsNew.x = xNew;
            boundsNew.y = yNew;
            boundsNew.width = wNew;
            boundsNew.height = hNew;
            element.setBounds(boundsNew);
            me.updateBounds();
        }
    }
};

ScaleFrameModel.prototype.fUpdateHandlerCoordinates = function(){
    var me = this;
    var hSize = ScaleFrameModel.prototype.DEFAULT_HANDLER_SIZE;
    var aHX = me.aHandlerX;
    var aHY = me.aHandlerY;
    var bounds = me.getBounds();

    if(bounds == null)
        return;

    var xL = 0, xM = (bounds.width - hSize)/ 2, xR = bounds.width - hSize;
    var yT = 0, yM = (bounds.height - hSize)/ 2, yB = bounds.height - hSize;
    //CCW
    aHX[0] = xL;
    aHY[0] = yT;
    aHX[1] = xL;
    aHY[1] = yM;
    aHX[2] = xL;
    aHY[2] = yB;
    aHX[3] = xM;
    aHY[3] = yB;
    aHX[4] = xR;
    aHY[4] = yB;
    aHX[5] = xR;
    aHY[5] = yM;
    aHX[6] = xR;
    aHY[6] = yT;
    aHX[7] = xM;
    aHY[7] = yT;
};

ScaleFrameModel.prototype.updateBounds = function(){
    var me = this;
    var boundsPrev = me.bounds;
    var bounds;
    var c = me.countSelected();
    if(c == 1) {
        var element = me.getSelected()[0];
        if(!(element.canBeScaled() && element.isShapeModifiable()))
            bounds = null;
        else{
            bounds = element.getBounds();
            if(bounds != null){
                if(bounds.width<1)
                    bounds.width = 1;
                if(bounds.height<1)
                    bounds.height = 1;
            }
        }
    }else
        bounds = null;

    me.flagBoundsChanged =
        (boundsPrev == null && bounds != null) ||
        (boundsPrev != null && bounds == null) ||
        (boundsPrev != null && bounds != null &&
            (
                boundsPrev.x != bounds.x || boundsPrev.y != bounds.y ||
                boundsPrev.width != bounds.width || boundsPrev.height != bounds.height
            )
        );

    me.bounds = bounds;

    if(me.flagBoundsChanged)
        me.fUpdateHandlerCoordinates();
};

ScaleFrameModel.prototype.getHitTestInfoForPoint = function(p0){
    var me = this;
    var flagHit = false, flagInside = false, objDetails = null;
    var bounds = this.bounds;
    var hitMargin = ScaleFrameModel.prototype.DEFAULT_HIT_MARGIN;
    var rTouch = me.TOUCH_SIZE_FACTOR/ModelElement.prototype.ZOOM_RATIO;
    if(MOUSE_OR_TOUCH>0)
        hitMargin *= rTouch;
    var pX = p0.x, pY = p0.y;
    if(
        bounds != null &&
        pX >= bounds.x-hitMargin && pX < bounds.x+bounds.width+hitMargin &&
        pY >= bounds.y-hitMargin && pY < bounds.y+bounds.height+hitMargin
    ){
        var hSize = ScaleFrameModel.prototype.DEFAULT_HANDLER_SIZE;
        if(MOUSE_OR_TOUCH>0)
            hSize *= rTouch;
        var aHX = me.aHandlerX, aHY = me.aHandlerY;
        var i, x, y;
        for(i=0; i<8; i++){
            x = bounds.x+aHX[i];
            y = bounds.y+aHY[i];
            if(pX >= x-hitMargin && pY >= y-hitMargin && pX < x+hSize+hitMargin && pY < y+hSize+hitMargin)
                break;
        }
        if(i<8){
            flagHit = true;
            objDetails = {};
            switch(i){
                case 0:
                    objDetails.where = "TL";
                    break;
                case 1:
                    objDetails.where = "ML";
                    break;
                case 2:
                    objDetails.where = "BL";
                    break;
                case 3:
                    objDetails.where = "BM";
                    break;
                case 4:
                    objDetails.where = "BR";
                    break;
                case 5:
                    objDetails.where = "MR";
                    break;
                case 6:
                    objDetails.where = "TR";
                    break;
                case 7:
                    objDetails.where = "TM";
                    break;
                default:
            }
        }
    }
    return new HitTestInfo(me, flagHit, flagInside, objDetails);
};
/**
 * @constructor
 */
function ScaleFrameController(scaleFrameModel, mapSceneController){
    ScaleFrameController.superclass.constructor.call(this);
    var me = this;
    me.model = scaleFrameModel;
    me.mapSceneController = mapSceneController;
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MODEL_ELEMENT_SELECTED, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MODEL_ELEMENT_DESELECTED, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.OVER_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.OUT_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.DOWN_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.UP_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOVE_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.SELECTED_ELEMENTS_MOVED, {target:me, listener:me.mapSceneControllerListener}, false
    );

    me.aSelected = [];
    me.idMovingHandle = null;
    me.handlerShift = null;
    me.idOverHandle = null;
}
extend(ScaleFrameController, NonDOMEventDispatcher);

ScaleFrameController.prototype.mapSceneControllerListener = function(e){
    var me = this;
    var model = me.model;
    var objData = e.data;
    switch(e.type){
        case ModelEvent.prototype.MODEL_ELEMENT_SELECTED:
            me.model.addToSelected(e.data.element);
            break;
        case ModelEvent.prototype.MODEL_ELEMENT_DESELECTED:
            me.model.removeFromSelected(e.data.element);
            break;
        case ModelEvent.prototype.OVER_SCALE_HANDLE:
            me.idOverHandle = objData.where;
            break;
        case ModelEvent.prototype.OUT_SCALE_HANDLE:
            me.idOverHandle = null;
            me.handlerShift = null;
            break;
        case ModelEvent.prototype.DOWN_SCALE_HANDLE:
            me.idMovingHandle = objData.where;
            me.handlerShift = model.getHandlerShiftForPoint(objData.where, objData.x, objData.y);
            break;
        case ModelEvent.prototype.UP_SCALE_HANDLE:
            me.idMovingHandle = null;
            break;
        case ModelEvent.prototype.MOVE_SCALE_HANDLE:
            var handlerShift = me.handlerShift;
            model.setHandlerPosition(me.idMovingHandle, objData.x-handlerShift.x, objData.y-handlerShift.y);
            break;
        case ModelEvent.prototype.SELECTED_ELEMENTS_MOVED:
            model.updateBounds();
            break;
        default:
    }
};

ScaleFrameController.prototype.deactivate = function(){
    ScaleFrameController.superclass.deactivate.call(this);
    var me = this;
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MODEL_ELEMENT_SELECTED, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MODEL_ELEMENT_DESELECTED, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.OVER_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.OUT_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.DOWN_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.UP_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MOVE_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.SELECTED_ELEMENTS_MOVED, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.aSelected.splice(0);
};
/**
 * @constructor
 */
function ScaleFrameVisualization(scaleFrameModel, _sceneVisualization){
    var me = this;
    me.model = scaleFrameModel;
    me.mapSceneVisualization = _sceneVisualization;
    me.canvas = document.createElement("CANVAS");
    me.boundsPrev = null;
    me.sizeRatioPrev = -1;
}
extend(ScaleFrameVisualization, Object);

ScaleFrameVisualization.prototype.DEFAULT_FRAME_COLOR = 0x2000A2;
ScaleFrameVisualization.prototype.DEFAULT_FRAME_BACKGROUND_COLOR = 0xFFFFFF;

//ScaleFrameVisualization.prototype.

ScaleFrameVisualization.prototype.update = function(){
    var me = this;
    var model = me.model;
    var bounds = model.getBounds();
    if(bounds != null){
        var sV = me.mapSceneVisualization;
        var rSize = sV.getCanvasToSceneSizeRatioZoom();
        var lW = ScaleFrameModel.prototype.DEFAULT_LINE_WIDTH;
        var canvas = me.canvas;
        var ctx = sV.getCanvasContext();
        var sizeRatioChanged = rSize != me.sizeRatioPrev;
        var boundsChanged = false;
        if(sizeRatioChanged)
            me.sizeRatioPrev = rSize;
        else{
            if (me.boundsPrev != null) {
                var boundsPrev = me.boundsPrev;
                boundsChanged =
                    (boundsPrev.x != bounds.x) || (boundsPrev.y != bounds.y) ||
                    (boundsPrev.width != bounds.width) || (boundsPrev.height != bounds.height);
            } else
                boundsChanged = true;
        }
        if(sizeRatioChanged || boundsChanged) {
            var lBW = ScaleFrameModel.prototype.DEFAULT_LINE_BACKGROUND_WIDTH;
            var handlerSize = ScaleFrameModel.prototype.DEFAULT_HANDLER_SIZE;
            var hBW = lBW-lW;
            var handlerBSize = handlerSize+2*hBW;

            var canvasLW = lW*rSize;
            var canvasLBW = lBW*rSize;
            var canvasHandlerSize = handlerSize*rSize;
            var canvasHandlerBSize = handlerBSize*rSize;
            var offset2 = canvasLW/2;
            var offsetB2 = canvasLBW/2;

            var ctxFrm = canvas.getContext("2d");

            var canvasBW = bounds.width*rSize;
            var canvasBH = bounds.height*rSize;

            var offset2Trunc = ((2*offset2)|0)/2;

            var cW = ((Math.round(canvasBW-canvasLW)|0)+ canvasLW)|0;
            var cH = ((Math.round(canvasBH-canvasLW)|0)+ canvasLW)|0;

            if(canvas.width != bounds.width || canvas.height != bounds.height){
                canvas.width = cW;
                canvas.height = cH;
            }else
                ctxFrm.clearRect(0, 0, canvas.width, canvas.height);

            ctxFrm.beginPath();
            ctxFrm.lineWidth = canvasLBW;
            ctxFrm.strokeStyle = miscFunctions.intAlpha2RGBAFunction(ScaleFrameVisualization.prototype.DEFAULT_FRAME_BACKGROUND_COLOR, 1);
            var offsetB2Trunc = ((2*offsetB2)|0)/2;
            ctxFrm.strokeRect(offsetB2Trunc, offsetB2Trunc, Math.round(canvasBW-canvasLBW)|0, Math.round(canvasBH-canvasLBW)|0);

            var i;
            ctxFrm.fillStyle = miscFunctions.intAlpha2RGBAFunction(ScaleFrameVisualization.prototype.DEFAULT_FRAME_BACKGROUND_COLOR, 1);

            for(i=0; i<8; i++){
                ctxFrm.beginPath();
                ctxFrm.fillRect(
                    ((model.getHandlerX(i)-hBW)*rSize), ((model.getHandlerY(i)-hBW)*rSize),
                    canvasHandlerBSize, canvasHandlerBSize
                );
            }

            ctxFrm.fillStyle = miscFunctions.intAlpha2RGBAFunction(ScaleFrameVisualization.prototype.DEFAULT_FRAME_COLOR, 1);

            for(i=0; i<8; i++) {
                ctxFrm.beginPath();
                ctxFrm.fillRect(
                    (model.getHandlerX(i) * rSize), (model.getHandlerY(i) * rSize),
                    canvasHandlerSize, canvasHandlerSize
                );
            }

            ctxFrm.beginPath();
            ctxFrm.lineWidth = canvasLW;
            ctxFrm.strokeStyle = miscFunctions.intAlpha2RGBAFunction(ScaleFrameVisualization.prototype.DEFAULT_FRAME_COLOR, 1);
            ctxFrm.strokeRect(offset2Trunc, offset2Trunc, Math.round(canvasBW-canvasLW)|0, Math.round(canvasBH-canvasLW)|0);

            me.boundsPrev = bounds;
        }
        var pTL = sV.elementCoordsToCanvasCoords(bounds.x, bounds.y);
        ctx.drawImage(canvas, pTL.x|0, pTL.y|0);
    }
};
/**
 * @constructor
 */
function MousePointerModel(){
    var me = this;
    me.iconId = null;
    me.flagOverScene = false;
    me.flagOverSelectableText = false;
    me.x = 0;
    me.y = 0;
}
extend(MousePointerModel, Object);

MousePointerModel.prototype.getIconId = function(){
    return this.iconId;
};

MousePointerModel.prototype.setIconId = function(sIconId){
    var me = this;
    var prevId = me.iconId;
    if(sIconId != prevId){
        me.iconId = sIconId;
    }
};

MousePointerModel.prototype.getX = function(){
    return this.x;
};

MousePointerModel.prototype.setX = function(v){
    this.x = v;
};

MousePointerModel.prototype.getY = function(){
    return this.y;
};

MousePointerModel.prototype.setY = function(v){
    this.y = v;
};

MousePointerModel.prototype.isOverScene = function(){
    return this.flagOverScene;
};

MousePointerModel.prototype.setOverScene = function(flag){
    this.flagOverScene = flag;
};

MousePointerModel.prototype.isOverSelectableText = function(){
    return this.flagOverSelectableText;
};

MousePointerModel.prototype.setOverSelectableText = function(flag){
    this.flagOverSelectableText = flag;
};
/**
 * @constructor
 */
function MousePointerController(mousePointerModel, mousePointerVisualization, mapSceneController, mainInterfaceController){
    //these should also be references to elements beyond MapSceneController like toolbars to listen to
    MousePointerController.superclass.constructor.call(this);
    var me = this;
    me.model = mousePointerModel;
    me.visualization = mousePointerVisualization;
    me.mapSceneController = mapSceneController;
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.OUT_SCENE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.OVER_SCENE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.OUT_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.OVER_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.OUT_SELECTABLE_TEXT, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.OVER_SELECTABLE_TEXT, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.SET_TOOL, {target:me, listener:me.mapSceneControllerListener}, false
    );

    me.mainInterfaceController = mainInterfaceController;
    me.flagOverInterface = false;
    me.mainInterfaceController.addEventListener(
        ModelEvent.prototype.OVER_INTERFACE, {target:me, listener:me.mainInterfaceControllerListener}, false
    );
    me.mainInterfaceController.addEventListener(
        ModelEvent.prototype.OUT_INTERFACE, {target:me, listener:me.mainInterfaceControllerListener}, false
    );
    me.mouseMoveListener = function(e){
        me.fMouseMove.call(me, e);
    };
    me.mouseDownListener = function(e){
        me.fMouseDown.call(me, e);
    };
    me.mouseUpListener = function(e){
        me.fMouseUp.call(me, e);
    };
    me.mouseOverDroplist = function(e){
        me.droplistListener.call(me, e);
    };
    me.mouseOutDroplist = function(e){
        me.droplistListener.call(me, e);
    };
    window.addEventListener("mousemove", me.mouseMoveListener, true);
    window.addEventListener("mouseup", me.mouseUpListener, false);
    window.addEventListener("touchend", me.mouseUpListener, false);
    document.body.addEventListener("mousedown", me.mouseDownListener, false);

    document.body.addEventListener(ModelEvent.prototype.OVER_DROPLIST, me.mouseOverDroplist, false);
    document.body.addEventListener(ModelEvent.prototype.OUT_DROPLIST, me.mouseOutDroplist, false);

    me.flagOverScaleHandler = false;
    me.flagMouseDown = false;
    me.iconIdBeforeScaleHandle = null;

    me.unloadListener = function(e){
        window.removeEventListener("unload", me.unloadListener, false);
        //window.removeEventListener("pagehide", me.unloadListener, false);
        me.deactivate();
    };

    window.addEventListener("unload", me.unloadListener, false);
    //window.addEventListener("pagehide", me.unloadListener, false);
}
extend(MousePointerController, NonDOMEventDispatcher);

MousePointerController.prototype.mapSceneControllerListener = function(e) {
    var me = this;
    var model = me.model;
    var visualization = me.visualization;
    var flagChange = true;
    switch (e.type) {
        case ModelEvent.prototype.OVER_SCENE:
            model.setOverScene(true);
            break;

        case ModelEvent.prototype.OVER_DROPLIST:
        case ModelEvent.prototype.OUT_SCENE:
            model.setOverScene(false);
            break;

        case ModelEvent.prototype.OVER_SCALE_HANDLE:
            if(typeof e.data.currentToolId != TYPE_STRING || e.data.currentToolId != ModelTool.prototype.TOOL_SELECT)
                break;

            if(me.flagOverScaleHandler) {
                flagChange = false;
                break;
            }
            me.flagOverScaleHandler = true;
            var where = e.data.where;
            if(!me.flagMouseDown) {
                var iconId;
                me.iconIdBeforeScaleHandle = model.getIconId();

                switch (where) {
                    case "TL":
                    case "BR":
                        iconId = "mouse-pointer.scale.tl";
                        break;
                    case "ML":
                    case "MR":
                        iconId = "mouse-pointer.scale.ml";
                        break;
                    case "BL":
                    case "TR":
                        iconId = "mouse-pointer.scale.bl";
                        break;
                    case "TM":
                    case "BM":
                        iconId = "mouse-pointer.scale.tm";
                        break;
                    default:
                        iconId = null;
                }
                model.setIconId(iconId);
            }
            break;

        case ModelEvent.prototype.OUT_SCALE_HANDLE:
            if(typeof e.data.currentToolId != TYPE_STRING || e.data.currentToolId != ModelTool.prototype.TOOL_SELECT)
                break;
            if(!me.flagMouseDown) {
                model.setIconId(me.iconIdBeforeScaleHandle);
                me.iconIdBeforeScaleHandle = null;
            }
            me.flagOverScaleHandler = false;
            break;

        case ModelEvent.prototype.UP_SCALE_HANDLE:
            break;

        case ModelEvent.prototype.OUT_SELECTABLE_TEXT:
            model.setOverSelectableText(false);
            break;

        case ModelEvent.prototype.OVER_SELECTABLE_TEXT:
            model.setOverSelectableText(true);
            break;

        case ModelEvent.prototype.SET_TOOL:
            model.setIconId(MousePointerLibrary.prototype.getPointerIdByToolId(e.data.id, e.data.details));
            break;

        default:
            flagChange = false;
    }
    if (flagChange) {
        visualization.update();
    }
};

//
MousePointerController.prototype.mainInterfaceControllerListener = function(e){
    var me = this;
};

//
MousePointerController.prototype.droplistListener = function(e){
    var me = this;
    var model = me.model;
    var visualization = me.visualization;
    var flagChange = true;
    switch (e.type) {
        case ModelEvent.prototype.OVER_DROPLIST:
            me.mapSceneController.setPointWithinScene(false);
            model.setOverScene(false);
            break;
        default:
            flagChange = false;
    }
    if (flagChange) {
        visualization.update();
    }

};

//
MousePointerController.prototype.fMouseMove = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this;
    var model = me.model;
    var pPageXY = miscFunctions.getPageXY(e);
    if(pPageXY == null)
        return;
    model.setX(pPageXY.x);
    model.setY(pPageXY.y);
    me.visualization.update();
};

MousePointerController.prototype.fMouseDown = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    this.flagMouseDown = true;
};

MousePointerController.prototype.fMouseUp = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this;
    var model = me.model;
    me.flagMouseDown = false;
    if(!me.flagOverScaleHandler && me.iconIdBeforeScaleHandle != null) {
        model.setIconId(me.iconIdBeforeScaleHandle);
        me.iconIdBeforeScaleHandle = null;
    }
};

MousePointerController.prototype.deactivate = function(){
    MousePointerController.superclass.deactivate.call(this);
    var me = this;
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.OUT_SCENE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.OVER_SCENE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.OUT_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.OVER_SCALE_HANDLE, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.OUT_SELECTABLE_TEXT, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.OVER_SELECTABLE_TEXT, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.SET_TOOL, {target:me, listener:me.mapSceneControllerListener}, false
    );
    me.mainInterfaceController.removeEventListener(
        ModelEvent.prototype.OVER_INTERFACE, {target:me, listener:me.mainInterfaceControllerListener}, false
    );
    me.mainInterfaceController.removeEventListener(
        ModelEvent.prototype.OUT_INTERFACE, {target:me, listener:me.mainInterfaceControllerListener}, false
    );

    window.removeEventListener("mousemove", me.mouseMoveListener, false);
    window.removeEventListener("mouseup", me.mouseUpListener, false);
    window.removeEventListener("touchend", me.mouseUpListener, false);
    document.body.removeEventListener("mousedown", me.mouseDownListener, false);
    document.body.removeEventListener(ModelEvent.prototype.OVER_DROPLIST, me.mouseOverDroplist, false);
    document.body.removeEventListener(ModelEvent.prototype.OUT_DROPLIST, me.mouseOutDroplist, false);
};
/**
 * @constructor
 */
function MousePointerVisualization(mousePointerModel, mousePointerCanvas){
    var me = this;
    me.model = mousePointerModel;
    me.mousePointerCanvas = mousePointerCanvas;
    me.canvasIcon = document.createElement("CANVAS");
    me.objSize = null;
    var ctxIcon = me.canvasIcon.getContext("2d");
    ctxIcon.fillStyle = "rgba(255, 255, 255, 0.0)";
    ctxIcon.fillRect(0, 0, me.canvasIcon.width, me.canvasIcon.height);
    me.iconId = null;
    me.flagCanvasClear = false;
}
extend(MousePointerVisualization, Object);

MousePointerVisualization.prototype.update = function() {
    var me = this;

    if(MOUSE_OR_TOUCH>0 && me.flagCanvasClear)
        return;

    var model = me.model;
    var canvasIcon = me.canvasIcon;
    me.mousePointerCanvas.getContext("2d").clearRect(0, 0, me.mousePointerCanvas.width, me.mousePointerCanvas.height);
    me.flagCanvasClear = true;

    if(MOUSE_OR_TOUCH>0)
        return;

    var iconId = model.getIconId();
    if (iconId != null && model.isOverScene() && !model.isOverSelectableText()) {
        if (canvasIcon.style.display == "none") {
            canvasIcon.style.display = "block";
            document.body.style.cursor = "none";
        }
        var objSize = MousePointerLibrary.prototype.getSizeById(iconId);
        var canvasBounds = me.mousePointerCanvas.getBoundingClientRect();
        if(iconId != me.iconId){
            //redraw the canvasIcon contents
            var ctxIcon = me.canvasIcon.getContext("2d");
            var img = MousePointerLibrary.prototype.getCachedSVG(iconId);
            if(canvasIcon.width != objSize.width || canvasIcon.height != objSize.height){
                canvasIcon.width = objSize.width;
                canvasIcon.height = objSize.height;
            }else
                ctxIcon.clearRect(0, 0, canvasIcon.width, canvasIcon.height);
            ctxIcon.drawImage(img, 0, 0);
            me.objSize = objSize;
            me.iconId = iconId;
        }
        me.mousePointerCanvas.getContext("2d").drawImage(
            me.canvasIcon,
            (model.getX()+objSize.xShift-canvasBounds.left-miscFunctions.getScrollLeft()),
            (model.getY()+objSize.yShift-canvasBounds.top-miscFunctions.getScrollTop())
        );
    } else {
        if (canvasIcon.style.display != "none") {
            canvasIcon.style.display = "none";
            document.body.style.cursor = "auto";
        }
    }
    me.flagCanvasClear = false;
};
