/**
 * @constructor
 */
function MainInterfaceController(model, visualization, mapSceneController){
    MainInterfaceController.superclass.constructor.call(this);
    var me = this;
    me.model = model;
    me.visualization = visualization;
    //
    me.mapSceneController = mapSceneController;
    //
    me.sceneToolsController = null;
    //
    //20190123 model_tool_show_hidden list should only close on its button click, not on misclick
    var s = miscFunctions.stripSpaces(document.localization.getParameter("droplists.ignore-misclick-ids", ModelTool.prototype.TOOL_SHOW_HIDDEN));
    me.aToolToIgnoreMisclick = (s.length>0) ? s.split(",") : [];
    me.aToolToIgnoreMisclick.forEach(function(s, i, a){
        a[i] = miscFunctions.stripSpaces(s);
    });
    //
    me.mouseMoveListener = function(e){
        if(!me.visualization.isModalDialogPresent())
            me.fMouseMove.call(me, e);
    };

    me.mouseDownListener = function(e){
        if(!me.visualization.isModalDialogPresent())
            me.fMouseDown.call(me, e);
        //e.stopPropagation();
    };

    me.mouseUpListener = function(e){
        if(!me.visualization.isModalDialogPresent())
            me.fMouseUp.call(me, e);
    };
    //
    me.keydownListener = function(e){
        var sKey;
        if(typeof e.keyCode == TYPE_NUMBER)
            switch(e.keyCode){
                case MapSceneController.prototype.KEY_CODE_CTRL:
                 sKey = MapSceneController.prototype.CTRL;
                 break;
                default:
                    sKey = null;
            }
        else if(typeof e.key == TYPE_STRING){
            switch(e.key){
                case MapSceneController.prototype.CTRL:
                    sKey = e.key;
                    break;
                default:
                    sKey = null;
            }
        }else
            sKey = null;

        if(sKey == null)
            return;
        if(sKey == MapSceneController.prototype.CTRL){
            me.model.setCtrl(true);
        }
    };
    //
    me.keyupListener = function(e){
        var sKey;
        if(typeof e.keyCode == TYPE_NUMBER)
            switch(e.keyCode){
                case MapSceneController.prototype.KEY_CODE_CTRL:
                    sKey = MapSceneController.prototype.CTRL;
                    break;
                default:
                    sKey = null;
            }
        else if(typeof e.key == TYPE_STRING){
            switch(e.key){
                case MapSceneController.prototype.CTRL:
                    sKey = e.key;
                    break;
                default:
                    sKey = null;
            }
        }else
            sKey = null;

        if(sKey == null)
            return;
        if(sKey == MapSceneController.prototype.CTRL){
            me.model.setCtrl(false);
        }
    };
    //
    me.unloadListener = function(e){
        window.removeEventListener("unload", me.unloadListener, false);
        //window.removeEventListener("pagehide", me.unloadListener, false);
        me.deactivate();
        me.visualization.deactivate();
        me.model.deactivate();
    };

    window.addEventListener("unload", me.unloadListener, false);
    //window.addEventListener("pagehide", me.unloadListener, false);
    //
    me.flagPointWithinInterface = false;
    me.flagMouseIsDown = false;
    me.objHitTestInfo = null;
    //
    window.addEventListener("mousemove", me.mouseMoveListener, false);
    window.addEventListener("mouseup", me.mouseUpListener, false);
    document.body.addEventListener("contextmenu", me.mouseContextMenuListener, false);
    window.addEventListener("touchend", me.mouseUpListener, false);
    document.body.addEventListener("mousedown", me.mouseDownListener, false);
    document.body.addEventListener("touchstart", me.mouseDownListener, false);
    //
    me.aSelectedButton = [];
    //
    //document.body.addEventListener("dblclick", me.mouseDoubleClickListener, false);
    //useCapture flag is false, otherwise a focused TextModelElement will move by cursor keys
    document.addEventListener("keydown", me.keydownListener, false);
    document.addEventListener("keyup", me.keyupListener, false);
    //
    me.model.addEventListener(
        ModelEvent.prototype.ADD_DIALOG, {target:me, listener:me.modelListener}, false
    );

    me.model.addEventListener(
        ModelEvent.prototype.REMOVE_DIALOG, {target:me, listener:me.modelListener}, false
    );
}
extend(MainInterfaceController, NonDOMEventDispatcher);

//
MainInterfaceController.prototype.getModel = function(){
    return this.model;
};

//
MainInterfaceController.prototype.getHitDialog = function(target){
    return this.visualization.getDialogDIV().contains(target) ? this.visualization.getHitDialog(target) : null;
};

//
MainInterfaceController.prototype.setSceneToolsController = function(toolsController){
    var me = this;
    var prev = me.sceneToolsController;
    if(prev instanceof ModelToolsController) {
        prev.removeToolEventsSource(me.model);
        prev.removeEventListener(
            ModelEvent.prototype.SET_TOOL, {listener: me.fSetSceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.START_TOOL, {listener: me.fStartSceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.TRACK_TOOL, {listener: me.fTrackSceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.USE_TOOL, {listener: me.fUseSceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.END_TOOL, {listener: me.fEndSceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.CANCEL_TOOL, {listener: me.fCancelSceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.ENABLE_TOOL, {listener: me.fEnableSceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.DISABLE_TOOL, {listener: me.fDisableSceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.CONFIGURE_TOOL, {listener: me.fConfigureSceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.APPLY_TOOL, {listener: me.fApplySceneToolListener, target: me}, false
        );
        prev.removeEventListener(
            ModelEvent.prototype.RESET_SCENE, {listener: me.fResetSceneListener, target: me}, false
        );
    }
    me.sceneToolsController = toolsController;
    if(toolsController instanceof ModelToolsController) {
        toolsController.addEventListener(
            ModelEvent.prototype.SET_TOOL, {listener: me.fSetSceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.START_TOOL, {listener: me.fStartSceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.TRACK_TOOL, {listener: me.fTrackSceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.USE_TOOL, {listener: me.fUseSceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.END_TOOL, {listener: me.fEndSceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.CANCEL_TOOL, {listener: me.fCancelSceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.ENABLE_TOOL, {listener: me.fEnableSceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.DISABLE_TOOL, {listener: me.fDisableSceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.CONFIGURE_TOOL, {listener: me.fConfigureSceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.APPLY_TOOL, {listener: me.fApplySceneToolListener, target: me}, false
        );
        toolsController.addEventListener(
            ModelEvent.prototype.RESET_SCENE, {listener: me.fResetSceneListener, target: me}, false
        );
        //
        toolsController.addToolEventsSource(me.model);
        //
        me.resetTools();
    }
};

//
MainInterfaceController.prototype.resetTools = function(){
    var me = this;
    var toolsController = me.sceneToolsController;
    var aToolbar = me.model.getToolbars();
    var toolbar, i, j, flag = false;
    for(j=0; j<aToolbar.length; j++) {
        toolbar = aToolbar[j];
        toolbar.reset();
        var n = toolbar.getToolCount(), s, sId, sButtonId, objData;
        for (i = 0; i < n; i++) {
            s = toolbar.getToolClassNameByNo(i);
            objData = toolbar.getToolDataByNo(i);
            sId = toolsController.getIdByClassName(s);
            sButtonId = toolbar.getButtonIdByToolClassName(s);
            toolsController.resetTool(sId);
            toolsController.configureTool(sId, objData);
            if (toolsController.isToolEnabled(sId))
                toolbar.enableToolButtonById(sButtonId);
            else
                toolbar.disableToolButtonById(sButtonId);
        }
        if(!flag && i>0)
            flag = true;
    }
    //MapInterfaceVisualization should be updated if at least one tool was updated;
    if(flag)
        me.model.invalidate();
};

//
MainInterfaceController.prototype.setCurrentToolId = function(sId, objDetails){
    var sceneToolsController = this.sceneToolsController;
    if(sceneToolsController != null)
        sceneToolsController.setCurrentToolId(sId, objDetails);
};

//
MainInterfaceController.prototype.toggleToolDroplist = function(sToolId){
    var buttonId = ToolbarButtonLibrary.prototype.getNameByToolId(sToolId);
    //there might be other toolbars, which should be iterated through
    var aToolbar = this.model.getToolbars();
    var toolbar, j;
    for(j=0; j<aToolbar.length; j++) {
        toolbar = aToolbar[j];
        if(toolbar.getButtonById(buttonId) != null) {
            toolbar.toggleToolDroplistById(buttonId);
            break;
        }
    }
};

//
MainInterfaceController.prototype.hideDroplists = function(aToolIdEx){
    //aToolIdEx – tool id's that should be excepted from the hiding
    var aIdEx, i, n;
    if((typeof aToolIdEx != TYPE_UNDEFINED) && Array.isArray(aToolIdEx)){
        n = aToolIdEx.length;
        aIdEx = [];
        for(i=0; i<n; i++)
            aIdEx.push(ToolbarButtonLibrary.prototype.getNameByToolId(aToolIdEx[i]));
    }
    var aToolbar = this.model.getToolbars();
    for(i=0; i<aToolbar.length; i++)
        aToolbar[i].hideToolDroplists(aIdEx);
};

//
MainInterfaceController.prototype.fMouseMove = function(e) {
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this, pageX, pageY;
    var model = me.model;
    var pPageXY = miscFunctions.getPageXY(e);
    if(pPageXY == null)
        return;
    pageX = pPageXY.x;
    pageY = pPageXY.y;
    var pos = miscFunctions.getPosition(me.visualization.getCanvas());
    if(pos == null)
        return;

    model.resetHitTestInfos();
    me.objHitTestInfo = null;

    var p = me.visualization.canvasCoordsToInterfaceCoords(pageX-pos.x, pageY-pos.y);
    var evtInterface;

    if(!model.isPointWithinInterface(p)) {
        if(me.flagPointWithinInterface){
            me.flagPointWithinInterface = false;
            evtInterface = new ModelEvent(ModelEvent.prototype.OUT_INTERFACE, true, true);
            if(evtInterface!=null)
                me.dispatchEvent(evtInterface);
        }
        return;
    }
    if(!me.flagPointWithinInterface){
        me.flagPointWithinInterface = true;
        evtInterface = new ModelEvent(ModelEvent.prototype.OVER_INTERFACE, true, true);
        if(evtInterface!=null)
            me.dispatchEvent(evtInterface);
    }

    var attr, aHTInfo;
    var flagChange = false;

    me.objHitTestInfo = model.getHitTestInfosForPoint(p);
    for (attr in me.objHitTestInfo) {
        aHTInfo = me.objHitTestInfo[attr];
        flagChange = aHTInfo.length > 0;
        var button =
            (flagChange && me.flagMouseIsDown && me.aSelectedButton.length>0) ? aHTInfo[0].getElement() : null;
        var i, n = me.aSelectedButton.length, curr;
        for(i=0; i<n; i++){
            curr = me.aSelectedButton[i];
            curr.setSelected(curr == button);
        }
        if (flagChange)
            break;
    }

    if(flagChange)
        model.invalidate();

};

//
MainInterfaceController.prototype.fMouseDown = function(e) {
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this, pageX, pageY;
    var model = me.model;
    var pPageXY = miscFunctions.getPageXY(e);
    if(pPageXY == null)
        return;
    var pos = miscFunctions.getPosition(me.visualization.getCanvas());
    if(pos == null)
        return;

    model.resetHitTestInfos();
    me.objHitTestInfo = null;

    pageX = pPageXY.x;
    pageY = pPageXY.y;

    var p = me.visualization.canvasCoordsToInterfaceCoords(pageX-pos.x, pageY-pos.y);
    var objHitTestInfo = me.objHitTestInfo = model.getHitTestInfosForPoint(p);//me.objHitTestInfo;

    var aToolIdEx = me.aToolToIgnoreMisclick.slice(0);

    if(objHitTestInfo != null){
        var aHTInfo, htInfo, details;
        var button;
        var toolId;
        var sBar;
        for(sBar in objHitTestInfo){
            if(typeof objHitTestInfo[sBar] != TYPE_UNDEFINED && Array.isArray(objHitTestInfo[sBar])){
                aHTInfo = objHitTestInfo[sBar];
                if(aHTInfo.length>0){
                    htInfo = aHTInfo[0];
                    button = htInfo.getElement();
                    toolId = ToolbarButtonLibrary.prototype.getToolIdByName(button.getId());
                    details = htInfo.getDetails();

                    if(e.type=="touchstart" && button.isWithHoldDown()){
                        ModelEvent.prototype.startHoldDown(button.doHoldDown, button, [details]);
                    }

                    if(details.flagHitOpenButton || details.flagHitListOpener) {
                        me.toggleToolDroplist(toolId);
                        if(aToolIdEx.indexOf(toolId)==-1)
                            aToolIdEx.push(toolId);
                    }else if(htInfo.isInside() && me.sceneToolsController.canSetTool(toolId)) {
                        if(!button.isApplyOnRelease())
                            me.setCurrentToolId(toolId, details);
                        if(button.isDeselectOnRelease()) {
                            button.setSelected(true);
                            if(me.aSelectedButton.indexOf(button) == -1)
                                me.aSelectedButton.push(button);
                        }
                    }
                }
            }
        }
    }

    me.hideDroplists(aToolIdEx);

    me.flagMouseIsDown = true;

};

//
MainInterfaceController.prototype.fMouseUp = function(e) {
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this, button;
    var pPageXY = miscFunctions.getPageXY(e);

    if(pPageXY == null)
        return;

    if(e.type == "touchend"){
        me.model.resetHitTestInfos();
        me.objHitTestInfo = null;
    }

    me.flagMouseIsDown = false;
    if(me.aSelectedButton.length>0){
        var i, n = me.aSelectedButton.length;
        for(i=0; i<n; i++) {
            button = me.aSelectedButton[i];
            if (button.isSelected()) {
                button.setSelected(false);
                //for "tools" like reset or confirm
                if(button.isApplyOnRelease())
                    me.setCurrentToolId(ToolbarButtonLibrary.prototype.getToolIdByName(button.getId()), button.getHitDetails());
            }
        }
        me.aSelectedButton.splice(0);
    }
};

//
MainInterfaceController.prototype.modelListener = function(e){
    var me = this;
    switch(e.type){
        case ModelEvent.prototype.ADD_DIALOG:
            me.visualization.removeDialogVisualization(e.data.id);
            me.visualization.addDialogVisualization(e.data.id);
            if(me.mapSceneController.isMouseEnabled() && me.visualization.isModalDialogPresent())
                me.mapSceneController.setMouseEnabled(false);
            break;
        case ModelEvent.prototype.REMOVE_DIALOG:
            me.visualization.removeDialogVisualization(e.data.id);
            if(!me.mapSceneController.isMouseEnabled() && !me.visualization.isModalDialogPresent())
                me.mapSceneController.setMouseEnabled(true);
            break;
        default:
    }
};

//
MainInterfaceController.prototype.fSetSceneToolListener = function(e){
    var buttonId = ToolbarButtonLibrary.prototype.getNameByToolId(e.data.id);
    this.model.getTopToolbar().selectToolButtonById(buttonId, e.data.details);
};

//
MainInterfaceController.prototype.fStartSceneToolListener = function(e){
    var me = this;
};

//
MainInterfaceController.prototype.fTrackSceneToolListener = function(e){
    var me = this;
};

//
MainInterfaceController.prototype.fUseSceneToolListener = function(e){
    var me = this;
};

//
MainInterfaceController.prototype.fEndSceneToolListener = function(e){
    var me = this;
};

//
MainInterfaceController.prototype.fCancelSceneToolListener = function(e){
    var me = this;
};

//
MainInterfaceController.prototype.fEnableSceneToolListener = function(e){
    var buttonId = ToolbarButtonLibrary.prototype.getNameByToolId(e.data.id);
    this.model.getTopToolbar().enableToolButtonById(buttonId);
};

//
MainInterfaceController.prototype.fDisableSceneToolListener = function(e){
    var buttonId = ToolbarButtonLibrary.prototype.getNameByToolId(e.data.id);
    this.model.getTopToolbar().disableToolButtonById(buttonId);
};

//
MainInterfaceController.prototype.fConfigureSceneToolListener = function(e){
    var buttonId = ToolbarButtonLibrary.prototype.getNameByToolId(e.data.id);
    var aToolbar = this.model.getToolbars();
    var i, n = aToolbar.length;
    for(i=0; i<n; i++)
        if(aToolbar[i].configureToolButtonById(buttonId, e.data.configuration))
            return;
};

//
MainInterfaceController.prototype.fApplySceneToolListener = function(e){
    this.model.getTopToolbar().invalidate();
};

//
MainInterfaceController.prototype.fResetSceneListener = function(e){
    this.resetTools();
    //this.model.getRightToolbar().invalidate();
};

//
MainInterfaceController.prototype.deactivate = function(){
    MainInterfaceController.superclass.deactivate.call(this);
    var me = this;
    me.model.removeEventListener(
        ModelEvent.prototype.ADD_DIALOG, {target:me, listener:me.modelListener}, false
    );

    me.model.removeEventListener(
        ModelEvent.prototype.REMOVE_DIALOG, {target:me, listener:me.modelListener}, false
    );
    me.aSelectedButton.splice(0);
    if(me.sceneToolsController != null)
        me.setSceneToolsController(null);
    window.removeEventListener("mousemove", me.mouseMoveListener, false);
    window.removeEventListener("mouseup", me.mouseUpListener, false);
    window.removeEventListener("touchend", me.mouseUpListener, false);
    document.body.removeEventListener("mousedown", me.mouseDownListener, false);
    document.body.removeEventListener("touchstart", me.mouseDownListener, false);
    //document.body.removeEventListener("dblclick", me.mouseDoubleClickListener, false);
    //
    document.removeEventListener("keydown", me.keydownListener, false);
    document.removeEventListener("keyup", me.keyupListener, false);
};
const workSheetContainerDIVIdPrefix = "workSheet_container_";

function ModelMainInstance(p) {
	
	const int2RGBString = miscFunctions.int2RGBString;
	const customPatternFromObject = miscFunctions.customPatternFromObject;
	
	var prototypeME = ModelEvent.prototype;
	
	var me;
	//var modelTabs;

	var modelLocalization;
	var modelResource = null;
	var mapSceneModel = null;
	var mapSceneVisualization = null;
    var mapSceneController = null;

    var mainInterfaceModel = null;
    var mainInterfaceVisualization = null;
    var mainInterfaceController = null;

	var workspaceCanvas;
	var workspaceDIV;
    //canvas for custom mouse cursors
    var mousePointerCanvas;
    var inputDIV;
	//div for dialogs that overlay workspace
	var dialogDIV;

    var mousePointerModel;
    var mousePointerVisualization;
    var mousePointerController;

	var modelToolsController;

	var asynchronousReadyId;
	
	
	if (!Math.log10) Math.log10 = function(t){ return Math.log(t)/Math.LN10;};
	
	function ModelMain(){
	
		this.props = p;
		me = this;

		miscFunctions.modelMain = this;
		
		var i;
		var s;
		var aCSS = p.aCSS, objLink;
		
		for(i=0; i<aCSS.length; i++){
			objLink = document.createElement("link");
			objLink.setAttribute("rel", "stylesheet");
			objLink.setAttribute("href", aCSS[i]);
			document.head.appendChild(objLink);
		}

        //строки с локализованными свойствами
        var sLocalizationXML;
        try{
            sLocalizationXML =
                document.getElementById("localizationData").textContent ||
                document.getElementById("localizationData").innerText;
        }catch(e){
            sLocalizationXML = "";
        }
        document.localization = new ModelLocalization(new DOMParser().parseFromString(sLocalizationXML, "text/xml"));
        ModelMainInstance.prototype.flagScaleModel = document.localization.getParameter("scale-model", true);
		ModelMainInstance.prototype.flagAllowScrollers = document.localization.getParameter("allow-scrollers", false);
		ModelMainInstance.prototype.defaultModelWidth = document.localization.getParameter("model-width", 800);
		ModelMainInstance.prototype.defaultModelHeight = document.localization.getParameter("model-height", 600);
        if(!ModelMainInstance.prototype.flagScaleModel)
            ModelMainInstance.prototype.flagScaleInterface = false;

        //сама модель
		var sModelXML;
        try{
            sModelXML =
                document.getElementById("modelData").textContent ||
                document.getElementById("modelData").innerText;
        }catch(e){
            sModelXML = "";
        }

		var modelXML = new DOMParser().parseFromString(sModelXML, "text/xml");
		
		modelResource = new ModelResource(modelXML);

		//======================================================PRELOADER=========================================================
		if(p.hasOwnProperty("loaderImage") && p.loaderImage.hasOwnProperty("src") && p.loaderImage.hasOwnProperty("width") && p.loaderImage.hasOwnProperty("height")){
			var w = parseInt(p.loaderImage.width), h = parseInt(p.loaderImage.height);
            var loaderImg = document.createElement("IMG");
            loaderImg.src = p.loaderImage.src;
            loaderImg.width = w;
            loaderImg.height = h;
            loaderImg.style.position = "relative";
            loaderImg.style.marginLeft = "auto";
            loaderImg.style.marginRight = "auto";
            loaderImg.style.display = "block";
            this.loader = loaderImg;
            document.body.appendChild(this.loader);
		}else{
			var loaderCanvas = objLoaderIndicator.loaderCanvas;
			var fragmentCanvas = objLoaderIndicator.loaderFragmentCanvas;
            this.loader = loaderCanvas;
            document.body.appendChild(this.loader);
			this.loaderIndicator = {
									canvas:loaderCanvas, tickFragment:fragmentCanvas, 
									start:objLoaderIndicator.currentLoaderProgress, c:-1,
									n:objLoaderIndicator.totalLoaderFragments, share: (1.0-objLoaderIndicator.scriptLoaderShare),
									transforms:objLoaderIndicator.objLoaderFragmentTransforms
								};
		}
		//preload images
		var aSrc = p.hasOwnProperty("imagesToPreload") ? p.imagesToPreload.slice(0, p.imagesToPreload.length) : [];
		//
		modelResource.makeImageURLsLocal();
        var aIMG = modelResource.getImageFileList(false);

		aSrc = aSrc.concat(aIMG);

        var pointModelLibrary = new PointModelLibrary();
        var mousePointerLibrary = new MousePointerLibrary();
        var checkboxSetModelLibrary = new CheckboxSetModelLibrary();
        var toolbarButtonLibrary = new ToolbarButtonLibrary();
        var colorPaletteLibrary = new ColorPaletteLibrary();

		//
		var queue = new createjs.LoadQueue(false);
		queue.on("complete", this.createObjects, this);
		queue.on("progress", this.loadProgress, this);
		queue.loadManifest(aSrc);
		me.queue = queue;
	}
	
	ModelMain.prototype.getEntity = function(){
		return this.workspaceDIV;
	};
	
	ModelMain.prototype.loadProgress = function(e){
		if(typeof this.loaderIndicator != TYPE_UNDEFINED && this.loaderIndicator != null){
			var lI = this.loaderIndicator, cPrev = lI.c, n = lI.n, i, ii, ctx = lI.canvas.getContext("2d"), tickFragment = lI.tickFragment;
			var start = lI.start, cCurr = Math.round(e.progress*n*lI.share);
			if(cPrev == -1)
				cPrev = lI.c = 0;
			
			for(i=cPrev; i<cCurr; i++){
				ii = i+start;
				ctx.setTransform(lI.transforms.aT1[ii], lI.transforms.aT2[ii], lI.transforms.aT3[ii], lI.transforms.aT4[ii], lI.transforms.aT5[ii], lI.transforms.aT6[ii]);
				ctx.drawImage(tickFragment, 0, 0);
				ctx.setTransform(1, 0, 0, 1, 0, 0);
			}
			lI.c = cCurr;
		}
	};
	
	//checks if all the "asynchronous" resources have been loaded and permits further initialization
	function fAsynchronousReady(me){
		if(objLoaderIndicator.isAsynchronousReady()){
			clearInterval(asynchronousReadyId);
			me.createObjects();
		}
	}
	
	ModelMain.prototype.fSetupWorkspace = function(flagModifyCanvas){
		var flagCanvas = (typeof flagModifyCanvas == TYPE_BOOLEAN) ? flagModifyCanvas : true;
		var workspaceTableContainer = document.getElementById("workspaceTableContainer");
		var workspace = document.getElementById("workspace");
		var info = document.getElementById("info");
		var containerParent = workspaceTableContainer.parentNode;
		var hInfo = (info != null) ? info.offsetHeight : 0;
		var pStyle = window.getComputedStyle(document.body);
		var w, h, xLeft, yTop;
		if(ModelMainInstance.prototype.flagAllowScrollers && !ModelMainInstance.prototype.flagScaleModel){
			var tableStyle = document.getElementById("workspaceTable").style;
			if(flagCanvas) {
				w =
					ModelMainInstance.prototype.defaultModelWidth +
					MainInterfaceModel.prototype.leftWidth + MainInterfaceModel.prototype.rightWidth;
				h = ModelMainInstance.prototype.defaultModelHeight +
					MainInterfaceModel.prototype.topHeight + MainInterfaceModel.prototype.bottomHeight;
			}else{
				w = workspaceCanvas.width;
				h = workspaceCanvas.height;
			}
			workspace.style.height = h+"px";
			workspaceCanvas.width = w;
			workspaceCanvas.height = h;
			yTop =
				((containerParent == document.body) ?
					(Math.round((window.innerHeight - h) / 2) | 0) :
					(Math.round((parseFloat(pStyle.height) - h) / 2) | 0));
			if(yTop<0)
				yTop = 0;
			tableStyle.position = "absolute";
			tableStyle.removeProperty("height");
			tableStyle.top = yTop+"px";
			tableStyle.left = 0+"px";
			workspaceCanvas.style.top = mousePointerCanvas.style.top = dialogDIV.style.top = 0+"px";//yTop+"px";
		}else {
			if (containerParent == document.body) {
				//|0 otherwise in IE scrollbars may appear
				if(flagCanvas) {
					w = Math.floor(window.innerWidth - parseFloat(pStyle.marginLeft) - parseFloat(pStyle.marginRight)) | 0;
					h = Math.floor(window.innerHeight - parseFloat(pStyle.marginTop) - parseFloat(pStyle.marginBottom)) | 0;
				}else{
					w = workspaceCanvas.width;
					h = workspaceCanvas.height+hInfo;
				}
				yTop = (Math.round((window.innerHeight - h) / 2) | 0);
			} else {
				if(flagCanvas) {
					w = window.innerWidth - parseFloat(pStyle.marginLeft) - parseFloat(pStyle.marginRight);
					h = parseFloat(pStyle.height) - parseFloat(pStyle.marginTop) - parseFloat(pStyle.marginBottom);
				}else{
					w = workspaceCanvas.width;
					h = workspaceCanvas.height+hInfo;
				}
				yTop = (Math.round((parseFloat(pStyle.height) - h) / 2) | 0);
			}
			workspaceCanvas.style.top = mousePointerCanvas.style.top = dialogDIV.style.top = yTop+"px";
			if(h<hInfo+8)
				h = hInfo+8;
			workspace.style.height = (h-hInfo)+"px";
			workspaceCanvas.width = w;
			workspaceCanvas.height = h-hInfo;
		}

		var ctx = workspaceCanvas.getContext("2d");
		ctx.fillStyle = "rgba(240, 240, 250, 1)";
		ctx.fillRect(0, 0, workspaceCanvas.width, workspaceCanvas.height);

        mousePointerCanvas.width = workspaceCanvas.width;
        mousePointerCanvas.height = workspaceCanvas.height;

        dialogDIV.style.width = 1+"px";
        dialogDIV.style.height = 1+"px";

		xLeft = Math.round((window.innerWidth-w)/2);
		if(ModelMainInstance.prototype.flagAllowScrollers && !ModelMainInstance.prototype.flagScaleModel && xLeft<0)
			xLeft = 0;
		workspaceCanvas.style.left = mousePointerCanvas.style.left = dialogDIV.style.left = xLeft+"px";
	};
	
	
	ModelMain.prototype.createObjects = function(e){
		try{
            document.getElementById("workspaceTable").style.display = "table";
		}catch(e){
			//no infotable div
		}

		var divDebug = document.createElement("DIV");
		divDebug.id = "div-debug";
		divDebug.setAttribute("style", "position:absolute; left:0px; top:0px; width:200px; height:200px; border:1px solid #00FF00; whitespace:pre-wrap; z-index:100; display:none");
		document.body.appendChild(divDebug);
        document.getElementById("div-debug").textContent = "debug";

		//
        modelResource.addImagesDataFromQueue(me.queue);

		workspaceCanvas = document.createElement("CANVAS");
        workspaceCanvas.id = "workspaceCanvas";
		workspaceCanvas.style.display = "block";
        workspaceCanvas.style.position = "absolute";
        workspaceCanvas.style.zIndex = 1;
		workspaceDIV = document.getElementById("workspaceDIV");
		workspaceDIV.appendChild(workspaceCanvas);

		inputDIV = document.createElement("DIV");
		inputDIV.style.display = "block";
		inputDIV.style.position = "absolute";
		inputDIV.style.zIndex = 2;
		inputDIV.style.overflow = "hidden";
		inputDIV.style.pointerEvents = "none";
		inputDIV.style.backgroundColor = "rgba(0, 0, 0, 0.0)";
		inputDIV.id = "inputDIV";
		workspaceDIV.appendChild(inputDIV);

        dialogDIV = document.createElement("DIV");
        dialogDIV.style.display = "block";
        dialogDIV.style.position = "absolute";
        dialogDIV.style.zIndex = 3;
        dialogDIV.style.overflow = "visible";
        dialogDIV.id = "dialogDIV";
        //dialogDIV.style.pointerEvents = "none";
        workspaceDIV.appendChild(dialogDIV);

        mousePointerCanvas = document.createElement("CANVAS");
        mousePointerCanvas.id = "mousePointerCanvas";
        mousePointerCanvas.style.display = "block";
        mousePointerCanvas.style.position = "absolute";
        mousePointerCanvas.style.zIndex = 4;
        mousePointerCanvas.style.pointerEvents = "none";
        workspaceDIV.appendChild(mousePointerCanvas);

		me.fSetupWorkspace();

		if(this.loader != null){
            this.loader.parentNode.removeChild(this.loader);
			//this.loader.remove();
			if(typeof this.loaderIndicator != TYPE_UNDEFINED){
				this.loaderIndicator.canvas = null;
				this.loaderIndicator = null;
			}
		}

		//to initiate scaling factors
		new TextFunctions();

		mapSceneModel = new MapSceneModel(modelResource);
        mapSceneVisualization = new MapSceneVisualization(mapSceneModel, workspaceCanvas);
		mapSceneVisualization.setInputDIV(inputDIV);
        mapSceneVisualization.setDialogDIV(dialogDIV);

        mapSceneController = new MapSceneController(mapSceneModel, mapSceneVisualization);

        mainInterfaceModel = new MainInterfaceModel(modelResource, mapSceneModel);
        mainInterfaceVisualization = new MainInterfaceVisualization(mainInterfaceModel, mapSceneVisualization);
        mainInterfaceVisualization.setCanvas(workspaceCanvas);
        mainInterfaceVisualization.setDialogDIV(dialogDIV);

        mainInterfaceController = new MainInterfaceController(
            mainInterfaceModel, mainInterfaceVisualization, mapSceneController
        );

		mousePointerModel = new MousePointerModel();
		mousePointerVisualization = new MousePointerVisualization(mousePointerModel, mousePointerCanvas);
		mousePointerController = new MousePointerController(
			mousePointerModel, mousePointerVisualization, mapSceneController, mainInterfaceController
		);
		mousePointerModel.setIconId(null);

        var resizeHandler = new ResizeHandler(window);

		function fResizeHandler(){
			me.fSetupWorkspace();
			var pStyle = window.getComputedStyle(document.body);
			mapSceneVisualization.update();
			mainInterfaceVisualization.update();
			me.fUpdateInputDIV();
		}

        resizeHandler.addEventListener(ModelEvent.prototype.WINDOW_RESIZE,function(e) {
            fResizeHandler();
        });

		modelToolsController = new ModelToolsController(mapSceneModel, mapSceneController, modelResource);
		mapSceneController.setToolsController(modelToolsController);
        mainInterfaceController.setSceneToolsController(modelToolsController);

        mapSceneModel.validate(false);
		mainInterfaceVisualization.update();
		mapSceneModel.invalidate();

        modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_SELECT);

		mapSceneModel.setStartTime(new Date().getTime());
		//
        if(!SCORM_INITIALIZED)
            SCORM.initialize();
        SCORM.setDataValue("cmi.success_status", "unknown");
		SCORM.setDataValue("cmi.completion_status", "not attempted");
		SCORM.setDataValue("cmi.progress_measure", 0.0);
        //
        //modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_POLYLINE);
        //modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_ARROW);
        //modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_RECTANGLE, {buttonNumber:0});
		//modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_RECTANGLE, {buttonNumber:1});
		///modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_TEXT);
		//modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_BOUND_TEXT);
		//modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_CHECKBOX_SET);
        //modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_DIAGRAM);
        //modelToolsController.setCurrentToolId(ModelTool.prototype.TOOL_POINT);

		//alert("modelMain:"+p.title+" RGB:"+int2RGBString(1234567));

		var flagReady = true;
		var lastValidateTime = -1;
		//for mobile browser zooming
		var prevWindowInnerWidth = window.innerWidth;
		var prevWindowInnerHeight = window.innerHeight;
		function step(timestamp){
			if(prevWindowInnerWidth != window.innerWidth || prevWindowInnerHeight != window.innerHeight){
				prevWindowInnerWidth = window.innerWidth;
				prevWindowInnerHeight = window.innerHeight;
				//fResizeHandler();
				//window.dispatchEvent(new E)
			}
			mapSceneModel.validate();
			mainInterfaceModel.validate();
			flagReady = true;
			lastValidateTime = new Date().getTime();
		}
		var validateIntervalId = window.setInterval(
			function(){
				if(flagReady) {
					flagReady = false;
					window.requestAnimationFrame(step);
				}
				//me.model.validate();
			}, ModelMainInstance.prototype.validateDelay
		);

		function unloadListener(e){
			window.clearInterval(validateIntervalId);
		}
		window.addEventListener("unload", unloadListener, false);

		me.fUpdateInputDIV();
	};


	ModelMain.prototype.fUpdateInputDIV = function(){
		if(workspaceCanvas == null || mapSceneVisualization==null)
			return;
		inputDIV.style.left = parseInt(workspaceCanvas.style.left)+mapSceneVisualization.getLeftPadding()+"px";
		inputDIV.style.top = parseInt(workspaceCanvas.style.top)+mapSceneVisualization.getTopPadding()+"px";
		var w = mapSceneVisualization.getWidth();
		var h = mapSceneVisualization.getHeight();
		var wScroller = mapSceneVisualization.canvasToSceneSizeRatio*ScrollerModel.prototype.DEFAULT_SCROLLBAR_WIDTH;
		if(mapSceneModel.isWithVerticalScrollBar() && mapSceneModel.isNeedVerticalScrollBar())
			w -= wScroller;
		if(mapSceneModel.isWithHorizontalScrollBar() && mapSceneModel.isNeedHorizontalScrollBar())
			h -= wScroller;
		inputDIV.style.width = w+"px";
		inputDIV.style.height = h+"px";
	};

	return new ModelMain();
}

//whether it is a player or editor
ModelMainInstance.prototype.flagPlayerMode = true;
//whether toolbars etc. should be scaled
ModelMainInstance.prototype.flagScaleInterface = true;
//whether model itself should be scaled
ModelMainInstance.prototype.flagScaleModel = false;
//
ModelMainInstance.prototype.defaultModelWidth = 800;
ModelMainInstance.prototype.defaultModelHeight = 600;
//60 fps
ModelMainInstance.prototype.validateDelay = 17;