/**
 * @constructor
 */
function DiagramModelElement(_objResourceData) {
    DiagramModelElement.superclass.constructor.call(this, _objResourceData);
    var me = this;
    var objData = me.objData;
    //
    me.aColor = miscFunctions.splitNumericalArrayString(me.objData.sSectorColor);
    me.aAlpha = miscFunctions.splitNumericalArrayString(me.objData.sSectorAlpha);
    //aSectorWeight sum equals to the number of sectors
    me.aSectorWeight = miscFunctions.splitNumericalArrayString(me.objData.sSectorWeight);
    //
    me.sectorCount = 0;
    me.diameter = 0;
    me.aSector = [];
    //initial construction of sectors
    me.setSymbolId(objData.symbolId);
    //
    me.aStartAngle = [];
    me.aEndAngle = [];
    me.fUpdateSectorAnglesByWeights();
    //
    me.pressedHandlerNumber = -1;
    //
    me.flagHandlersMovable = false;
    //
    //current sector fill color can be changed
    me.flagFillColored = true;
}
extend(DiagramModelElement, PointModelElement);

DiagramModelElement.prototype.aMethodNameForProperty =
    miscFunctions.cloneObject(DiagramModelElement.superclass.aMethodNameForProperty);

DiagramModelElement.prototype.setMethodNameForProperty(
    DiagramModelElement.prototype.aMethodNameForProperty, "fillColor", false, "getSelectedSectorColor"
);
DiagramModelElement.prototype.setMethodNameForProperty(
    DiagramModelElement.prototype.aMethodNameForProperty, "fillColor", true, "setSelectedSectorColor"
);
DiagramModelElement.prototype.setMethodNameForProperty(
    DiagramModelElement.prototype.aMethodNameForProperty, "fillAlpha", false, "getSelectedSectorAlpha"
);
DiagramModelElement.prototype.setMethodNameForProperty(
    DiagramModelElement.prototype.aMethodNameForProperty, "fillAlpha", true, "setSelectedSectorAlpha"
);

DiagramModelElement.prototype.typePrefix = ModelResource.prototype.TYPE_DIAGRAM;
DiagramModelElement.prototype.familyPrefix = ModelResource.prototype.TYPE_DIAGRAMS;
DiagramModelElement.prototype.className = ModelResource.prototype.CLASSID_DIAGRAM;
DiagramModelElement.prototype.localizedNamePrefix = "Диаграмма ";

DiagramModelElement.prototype.centralDiameter = 10;
DiagramModelElement.prototype.handlerDiameter = 8;
DiagramModelElement.prototype.sectorBorderThickness = 1;
DiagramModelElement.prototype.sectorSelectedBorderThickness = 2;
DiagramModelElement.prototype.sectorSelectedBorderBackgroundThickness = 4;
DiagramModelElement.prototype.percentageSectorInnerRadiusOffset = 6;
DiagramModelElement.prototype.percentageSectorOuterRadiusOffset = 36;
DiagramModelElement.prototype.percentageSectorStartAngleOffset = 3.0;
DiagramModelElement.prototype.percentageSectorEndAngleOffset = 31.0;

DiagramModelElement.prototype.symbolParametersMatches = [
    {id:"mcSymbolD0_Size0", parameters:{sectorCount:2, diameter:70}},
    {id:"mcSymbolD0_Size1", parameters:{sectorCount:2, diameter:42}},
    {id:"mcSymbolD0_Size2", parameters:{sectorCount:2, diameter:102}},
    {id:"mcSymbolD1_Size0", parameters:{sectorCount:3, diameter:70}},
    {id:"mcSymbolD1_Size1", parameters:{sectorCount:3, diameter:42}},
    {id:"mcSymbolD1_Size2", parameters:{sectorCount:3, diameter:102}},
    {id:"mcSymbolD2_Size0", parameters:{sectorCount:4, diameter:70}},
    {id:"mcSymbolD2_Size1", parameters:{sectorCount:4, diameter:42}},
    {id:"mcSymbolD2_Size2", parameters:{sectorCount:4, diameter:102}},
    {id:"mcSymbolD3_Size0", parameters:{sectorCount:5, diameter:70}},
    {id:"mcSymbolD3_Size1", parameters:{sectorCount:5, diameter:42}},
    {id:"mcSymbolD3_Size2", parameters:{sectorCount:5, diameter:102}},
    {id:"mcSymbolD4_Size0", parameters:{sectorCount:6, diameter:70}},
    {id:"mcSymbolD4_Size1", parameters:{sectorCount:6, diameter:42}},
    {id:"mcSymbolD4_Size2", parameters:{sectorCount:6, diameter:102}}
];

DiagramModelElement.prototype.aPropretyNameMap = DiagramModelElement.superclass.aPropretyNameMap.slice(0);
(function(){
    var a = DiagramModelElement.prototype.aPropretyNameMap;
    a.splice(a.length, 0,
        ["Pl_Angles", "flagAnglesModifiable",  TYPE_BOOLEAN],
        ["angleStep", "angleStep", TYPE_NUMBER],
        ["angleError", "angleError", TYPE_NUMBER],
        ["diagramColor", "sSectorColor", TYPE_STRING],
        ["diagramAlpha", "sSectorAlpha", TYPE_STRING],
        ["sectorWeight", "sSectorWeight", TYPE_STRING]
    );
})();

//
DiagramModelElement.prototype.fGetParameterBySymbolId = function(sId, sParameter){
    var a = DiagramModelElement.prototype.symbolParametersMatches, i, n = a.length, obj;
    for(i=0; i<n; i++){
        obj = a[i];
        if(obj.id == sId)
            break;
    }
    return (i!=n && typeof obj.parameters[sParameter] != TYPE_UNDEFINED) ? obj.parameters[sParameter] : null;

};

//
DiagramModelElement.prototype.fUpdateSectorAnglesByWeights = function(){
    var me = this;
    var i, n = me.sectorCount;
    var aStart = me.aStartAngle;
    var aEnd = me.aEndAngle;
    var aWeight = me.aSectorWeight;

    var startAngle = 0, endAngle = 0, weightSum = 0, weightFactor;
    for(i=0; i<n; i++)
        weightSum += me.aSectorWeight[i];
    weightFactor = miscFunctions.TWO_PI/weightSum;

    for(i=0; i<n; i++){
        aStart[i] = startAngle;
        endAngle = startAngle + weightFactor*aWeight[i];
        aEnd[i] = endAngle;
        startAngle = endAngle;
    }
    aEnd[i-1] = miscFunctions.TWO_PI;
    aStart.splice(n);
    aEnd.splice(n);
};

//
DiagramModelElement.prototype.fUpdateSectorWeightsByAngles = function(){
    var me = this;
    var i, n = me.sectorCount;
    var aStart = me.aStartAngle;
    var aEnd = me.aEndAngle;
    var aWeight = me.aSectorWeight;

    var angleFactor = n/miscFunctions.TWO_PI, weightCorr = 0, weight;

    for(i=0; i<n; i++){
        weight = angleFactor*(aEnd[i]-aStart[i]);
        aWeight[i] = weight;
        weightCorr += weight;
    }
    //to normalize weights sum to n
    weightCorr = n/weightCorr;
    for(i=0; i<n; i++)
        aWeight[i] = aWeight[i]*weightCorr;
    aWeight.splice(n);
};

//if symbolId changes, the whole set of sectors should be rebuilt
DiagramModelElement.prototype.fUpdateSectors = function(){
    var me = this;
    var aSector = me.aSector;
    var prevSectorCount = aSector.length;
    var i;

    var sectorCount = me.sectorCount;
    if(prevSectorCount != me.sectorCount) {
        if(sectorCount>prevSectorCount) {
            for (i = prevSectorCount; i < sectorCount; i++)
                aSector[i] = new PartDiagramSector(me, i);
        }else if (sectorCount < prevSectorCount) {
                for (i = sectorCount; i < prevSectorCount; i++)
                    aSector[i].deactivate();
                aSector.splice(sectorCount);
            }
        }

};

//
DiagramModelElement.prototype.getSectorCount = function(){
    return this.sectorCount;
};

//
DiagramModelElement.prototype.getSector = function(no){
    return (no>=0 && no<this.sectorCount) ? this.aSector[no] : null;
};

//
DiagramModelElement.prototype.getDiameter = function(){
    return this.diameter;
};

//
DiagramModelElement.prototype.getCentralDiameter = function(){
    return this.centralDiameter;
};

//
DiagramModelElement.prototype.getHandlerDiameter = function(){
    return this.handlerDiameter;
};

//
DiagramModelElement.prototype.getSectorBorderThickness = function(){
    return this.sectorBorderThickness;
};

//
DiagramModelElement.prototype.getSectorSelectedBorderThickness = function(){
    return this.sectorSelectedBorderThickness;
};

//
DiagramModelElement.prototype.getSectorSelectedBorderBackgroundThickness = function(){
    return this.sectorSelectedBorderBackgroundThickness;
};

//
DiagramModelElement.prototype.getPercentageSectorInnerRadiusOffset = function(){
    return DiagramModelElement.prototype.percentageSectorInnerRadiusOffset;
};

//
DiagramModelElement.prototype.getPercentageSectorOuterRadiusOffset = function(){
    return DiagramModelElement.prototype.percentageSectorOuterRadiusOffset;
};

//
DiagramModelElement.prototype.getPercentageSectorStartAngleOffset = function(){
    return DiagramModelElement.prototype.percentageSectorStartAngleOffset;
};

//
DiagramModelElement.prototype.getPercentageSectorEndAngleOffset = function(){
    return DiagramModelElement.prototype.percentageSectorEndAngleOffset;
};

//override
DiagramModelElement.prototype.setSelected = function(flag){
    var me = this;
    me.flagSelect = flag;
    if(!me.flagSelect)
        me.setSelectedSectorNumber(-1);
};

//
DiagramModelElement.prototype.isHandlersMovable = function(){
    return this.flagHandlersMovable;
};

//
DiagramModelElement.prototype.setHandlersMovable = function(flag){
    this.flagHandlersMovable = flag;
};

//
DiagramModelElement.prototype.getPressedHandlerNo = function(){
    return this.pressedHandlerNumber;
};

//
DiagramModelElement.prototype.setPressedHandlerNo = function(no){
    this.pressedHandlerNumber = no;
};

//
DiagramModelElement.prototype.getSelectedSectorNumber = function(){
    var me = this;
    var aSector = me.aSector, i, n = aSector.length;
    for(i=0; i<n; i++)
        if(aSector[i].isSelected())
            break;
    return (i<n) ? i: (-1);
};

//
DiagramModelElement.prototype.setSelectedSectorNumber = function(no){
    var me = this;
    var currNo = me.getSelectedSectorNumber();
    if(currNo == no)
        return;
    var aSector = me.aSector, i, n = aSector.length, sector;
    for(i=0; i<n; i++){
        sector = aSector[i];
        if(i==no)
            sector.setSelected(true);
        else if(sector.isSelected())
            sector.setSelected(false);
    }
};

//
DiagramModelElement.prototype.getSAngles = function(){
    return this.aStartAngle.join(",");
};

//
DiagramModelElement.prototype.getSColors = function(){
    return this.aColor.join(",");
};

//
DiagramModelElement.prototype.getSAlphas = function(){
    return this.aAlpha.join(",");
};

//
DiagramModelElement.prototype.setSymbolId = function(sId){
    var objData = this.objData;
    objData.symbolId = sId;
    this.sectorCount = this.fGetParameterBySymbolId(objData.symbolId, "sectorCount");
    this.diameter = this.fGetParameterBySymbolId(objData.symbolId, "diameter");
    this.fUpdateSectors();
};

//
DiagramModelElement.prototype.isAnglesModifiable = function(){
    return this.objData.flagAnglesModifiable;
};

DiagramModelElement.prototype.setAnglesModifiable = function(flag){
    this.objData.flagAnglesModifiable = flag;
};

//override
DiagramModelElement.prototype.isOtherwiseModifiable = function(){
    return this.isAnglesModifiable();
};

//
DiagramModelElement.prototype.getAngleStep = function(){
    return this.objData.angleStep;
};

DiagramModelElement.prototype.setAngleStep = function(v){
    this.objData.angleStep = v;
};

//
DiagramModelElement.prototype.getAngleError = function(){
    return this.objData.angleError;
};

DiagramModelElement.prototype.setAngleError = function(v){
    this.objData.angleError = v;
};

//
DiagramModelElement.prototype.getSectorColor = function(no){
    return (no>=0 && no<this.aColor.length) ? this.aColor[no] : undefined;
};

DiagramModelElement.prototype.setSectorColor = function(no, c){
    var me = this;
    if(no>=0 && no<this.aColor.length){
        var objData = me.objData;
        me.aColor[no] = c;
        objData.sSectorColor = me.aColor.join(",");
    }
};

//
DiagramModelElement.prototype.getSectorAlpha = function(no){
    return (no>=0 && no<this.aAlpha.length) ? this.aAlpha[no] : 0;
};

DiagramModelElement.prototype.setSectorAlpha = function(no, v){
    var me = this;
    if(no>=0 && no<this.aAlpha.length){
        var objData = me.objData;
        me.aAlpha[no] = v;
        objData.sSectorAlpha = me.aAlpha.join(",");
    }
};

//
DiagramModelElement.prototype.getSectorWeight = function(no){
    return (no>=0 && no<this.aSectorWeight.length) ? this.aSectorWeight[no] : 0;
};

DiagramModelElement.prototype.setSectorWeight = function(no, v){
    var me = this;
    if(no>=0 && no<this.aSectorWeight.length){
        var objData = me.objData;
        var prev = me.aSectorWeight[no];
        me.aSectorWeight[no] = v;
        if(prev != v)
            me.fUpdateSectorAnglesByWeights();
        objData.sSectorWeight = me.aSectorWeight.join(",");
    }
};

//
DiagramModelElement.prototype.getSectorStartAngle = function(no){
    return (no>=0 && no<this.aStartAngle.length) ? this.aStartAngle[no] : 0;
};

//
DiagramModelElement.prototype.setSectorStartAngle = function(no, v){
    if(no>=0 && no<this.aStartAngle.length) {
        var prev = this.aStartAngle[no];
        this.aStartAngle[no] = v;
        if(prev != v)
            this.fUpdateSectorWeightsByAngles();
    }
};

//
DiagramModelElement.prototype.getSectorEndAngle = function(no){
    return (no>=0 && no<this.aEndAngle.length) ? this.aEndAngle[no] : 0;
};

DiagramModelElement.prototype.setSectorEndAngle = function(no, v){
    if(no>=0 && no<this.aEndAngle.length) {
        var prev = this.aEndAngle[no];
        this.aEndAngle[no] = v;
        if(prev != v)
            this.fUpdateSectorWeightsByAngles();
    }
};

//
DiagramModelElement.prototype.getSelectedSectorColor = function(){
    return this.getSectorColor(this.getSelectedSectorNumber());
};

DiagramModelElement.prototype.setSelectedSectorColor = function(c){
    this.setSectorColor(this.getSelectedSectorNumber(), c);
};

//
DiagramModelElement.prototype.getSelectedSectorAlpha = function(){
    return this.getSectorAlpha(this.getSelectedSectorNumber());
};

DiagramModelElement.prototype.setSelectedSectorAlpha = function(v){
    this.setSectorAlpha(this.getSelectedSectorNumber(), v);
};

//override
DiagramModelElement.prototype.isMovableByMouse = function(){
    var flag = DiagramModelElement.superclass.isMovableByMouse.call(this);
    var hitDetails = this.hitDetails;
    return flag && (hitDetails == null || hitDetails.handlerNumber == -1);
};

//override
DiagramModelElement.prototype.getHitTestInfoForPoint = function(p0){
    var me = this;
    var sectorCount = me.getSectorCount();
    var flagHit = false, flagInside = false;
    //handlerNumber == 1 is between the sector 0 and sector 1, == 0 (not shown) is between sectorCount-1 and 0
    //-1 – not hit

    var xC = me.getX();
    var yC = me.getY();
    var radius = me.getDiameter()/2;
    var centralRadius = me.getCentralDiameter()/2;
    var handlerRadius = me.getHandlerDiameter()/2;

    if(MOUSE_OR_TOUCH>0){
        var rTouch = me.TOUCH_SIZE_FACTOR/ModelElement.prototype.ZOOM_RATIO;
        centralRadius *= rTouch;
        handlerRadius *= rTouch;
    }

    var angle, cosAngle, sinAngle;
    var xCHandler, yCHandler;
    var i;

    var flagMayHitHandler =
        me.flagHandlersMovable &&
            miscFunctions.isPointWithinEllipse(
            p0.x, p0.y, xC, yC, radius+handlerRadius, radius+handlerRadius
        );

    if(flagMayHitHandler) {
        for (i = 1; i < sectorCount; i++) {
            angle = -miscFunctions.PI_OVER_2 + me.getSectorStartAngle(i);
            cosAngle = Math.cos(angle);
            sinAngle = Math.sin(angle);
            xCHandler = xC + radius * cosAngle;
            yCHandler = yC + radius * sinAngle;
            if (miscFunctions.isPointWithinEllipse(p0.x, p0.y, xCHandler, yCHandler, handlerRadius, handlerRadius))
                break;
        }
    }

    var flagHitHandler = flagMayHitHandler && (i<sectorCount);
    var handlerNumber = flagHitHandler ? i : (-1);

    var sectorNumber;

    if(flagHitHandler){
        flagHit = true;
        flagInside = false;
        sectorNumber = -1;
    }else{
        flagHit = miscFunctions.isPointWithinEllipse(p0.x, p0.y, xC, yC, radius, radius);
        flagInside = flagHit && !miscFunctions.isPointWithinEllipse(p0.x, p0.y, xC, yC, centralRadius, centralRadius);
        if(flagInside){
            //which sector
            var dX = p0.x-xC;
            var dY = p0.y-yC;
            angle = Math.atan2(dY, dX) + miscFunctions.PI_OVER_2;
            if(angle<0)
                angle += miscFunctions.TWO_PI;
            for(i = 0; i < sectorCount; i++)
                if(angle>me.getSectorStartAngle(i) && angle<=me.getSectorEndAngle(i))
                    break;
            sectorNumber = (i<sectorCount) ? i : (-1);
        }else
            sectorNumber = -1;
    }

    return new HitTestInfo(me, flagHit, flagInside, {sectorNumber:sectorNumber, handlerNumber:handlerNumber});
};

//
DiagramModelElement.prototype.isPointWithin = function(p0){
    var me = this;
    var radius = me.getDiameter()/2;
    return miscFunctions.isPointWithinEllipse(p0.x, p0.y, me.getX(), me.getY(), radius, radius);
};

//override
DiagramModelElement.prototype.isMatchForElement = function(refElement, options) {
    var me = this;
    var i, j, n, s, sRef, dMax, cRef, wRef, flagMatch;
    if (refElement.getType() != me.getType())
        return false;
    var zoneElement =
        (typeof options != TYPE_UNDEFINED && typeof options.zoneElement != TYPE_UNDEFINED ) ?
            options.zoneElement : refElement;
    var flag = me.fPositionsMatch(zoneElement, zoneElement.getErrorZoneRadius());

    //size
    if(flag && refElement.isCheckSize()){
        s = me.getSymbolId();
        i = s.indexOf("_Size");
        if(i != -1)
            s = s.slice(i);
        sRef = refElement.getSymbolId();
        i = sRef.indexOf("_Size");
        if(i != -1)
            sRef = sRef.slice(i);
        flag = (s==sRef);
    }

    //angles
    if(flag && refElement.isCheckColor()){
        n = refElement.getSectorCount();
        if(me.getSectorCount() == n){
            //weights sum is normalized on 1
            dMax = n*refElement.getAngleError()/360;
            for(i=0; i<n && flag; i++){
                cRef = refElement.getSectorColor(i);
                wRef = refElement.getSectorWeight(i);
                for(j=0; j<n; j++){
                    if(me.getSectorColor(j) == cRef){
                        //n/2 because of round-off error
                        flag = Math.abs(wRef-me.getSectorWeight(j))<=(dMax+0.005*n);
                        break;
                    }
                }
                flag = flag && (j<n);
            }
        }else
            flag = false;
    }

    return flag;
};
/**
 * @constructor
 */
function PartDiagramSector(parentElement, orderNo){
    PartDiagramSector.superclass.constructor.call(this, parentElement, {});
    var me = this;
    //everything can be requested from the parentElement by the sector orderNo
    me.orderNo = orderNo;
    //
    me.flagFillColored = true;
}
extend(PartDiagramSector, ElementPart);

//
PartDiagramSector.prototype.getOrderNo = function(){
    return this.orderNo;
};

/**
 * @constructor
 */
function ArrowModelElement(_objResourceData){
    ArrowModelElement.superclass.constructor.call(this, _objResourceData);
    var me = this;
    me.flagFillColored = true;
    //arrow start/end circle diameter
    me.objDataConst.circleDiameter = 6;
}
extend(ArrowModelElement, ModelElement);

ArrowModelElement.prototype.typePrefix = ModelResource.prototype.TYPE_ARROW;
ArrowModelElement.prototype.familyPrefix = ModelResource.prototype.TYPE_ARROWS;
ArrowModelElement.prototype.className = ModelResource.prototype.CLASSID_ARROW;
ArrowModelElement.prototype.localizedNamePrefix = "Стрелка ";

ArrowModelElement.prototype.TOUCH_SIZE_FACTOR = 5;
ArrowModelElement.prototype.HANDLER_DIAMETER = 6;
ArrowModelElement.prototype.HANDLER_BORDER_WIDTH = 1;
ArrowModelElement.prototype.HANDLER_OVER_DIAMETER = 12;
ArrowModelElement.prototype.HANDLER_OVER_BORDER_WIDTH = 2;

ArrowModelElement.prototype.aMethodNameForProperty =
    miscFunctions.cloneObject(ArrowModelElement.superclass.aMethodNameForProperty);
ArrowModelElement.prototype.setMethodNameForProperty(
    ArrowModelElement.prototype.aMethodNameForProperty, "fillColor", false, "getFillColor"
);
ArrowModelElement.prototype.setMethodNameForProperty(
    ArrowModelElement.prototype.aMethodNameForProperty, "fillColor", true, "setFillColor"
);
ArrowModelElement.prototype.setMethodNameForProperty(
    ArrowModelElement.prototype.aMethodNameForProperty, "fillAlpha", false, "getFillAlpha"
);
ArrowModelElement.prototype.setMethodNameForProperty(
    ArrowModelElement.prototype.aMethodNameForProperty, "fillAlpha", true, "setFillAlpha"
);
ArrowModelElement.prototype.setMethodNameForProperty(
    ArrowModelElement.prototype.aMethodNameForProperty, "symbolId", false, "getSymbolId"
);
ArrowModelElement.prototype.setMethodNameForProperty(
    ArrowModelElement.prototype.aMethodNameForProperty, "symbolId", true, "setSymbolId"
);
ArrowModelElement.prototype.setMethodNameForProperty(
    ArrowModelElement.prototype.aMethodNameForProperty, "checkType", false, "isCheckType"
);

ArrowModelElement.prototype.aPropretyNameMap = ArrowModelElement.superclass.aPropretyNameMap.slice(0);
(function(){
    var a = ArrowModelElement.prototype.aPropretyNameMap;
    a.splice(a.length, 0,
        ["CHCL", "flagCheckColor", TYPE_BOOLEAN],
        ["CHTY", "flagCheckType",  TYPE_BOOLEAN],
        ["CHSZ", "flagCheckSize",  TYPE_BOOLEAN],
        ["FA", "fillAlpha",  TYPE_NUMBER],
        ["FC", "fillColor",  TYPE_INTEGER],
        //to make difference between arrow types; no actual symbol is taken from the library
        ["Symbol", "symbolId", TYPE_STRING],
        ["toZoneId", "toZoneId", TYPE_INTEGER],
        ["toZoneType", "toZoneType", TYPE_STRING],
        ["XS", "xFrom", TYPE_NUMBER],
        ["YS", "yFrom", TYPE_NUMBER],
        ["XF", "xTo", TYPE_NUMBER],
        ["YF", "yTo", TYPE_NUMBER]
    );
})();

//
ArrowModelElement.prototype.isCheckColor = function(){
    return this.objData.flagCheckColor;
};

//
ArrowModelElement.prototype.isCheckType = function(){
    return this.objData.flagCheckType;
};

//
ArrowModelElement.prototype.isCheckSize = function(){
    return this.objData.flagCheckSize;
};

//
ArrowModelElement.prototype.getFillAlpha = function(){
    return this.objData.fillAlpha;
};

//
ArrowModelElement.prototype.setFillAlpha = function(v){
    this.objData.fillAlpha = v;
};

//
ArrowModelElement.prototype.getFillColor = function(){
    return this.objData.fillColor;
};

//
ArrowModelElement.prototype.setFillColor = function(v){
    this.objData.fillColor = v;
};

//
ArrowModelElement.prototype.getSymbolId = function(){
    return this.objData.symbolId;
};

//
ArrowModelElement.prototype.setSymbolId = function(sId){
    this.objData.symbolId = sId;
};

//
ArrowModelElement.prototype.getToZoneId = function(){
    return this.objData.toZoneId;
};

//
ArrowModelElement.prototype.getToZoneType = function(){
    return this.objData.toZoneType;
};

//
ArrowModelElement.prototype.getXFrom = function(){
    return this.objData.xFrom;
};

//
ArrowModelElement.prototype.setXFrom = function(v){
    this.objData.xFrom = v;
};

//
ArrowModelElement.prototype.getYFrom = function(){
    return this.objData.yFrom;
};

//
ArrowModelElement.prototype.setYFrom = function(v){
    this.objData.yFrom = v;
};

//
ArrowModelElement.prototype.getXTo = function(){
    return this.objData.xTo;
};

//
ArrowModelElement.prototype.setXTo = function(v){
    this.objData.xTo = v;
};

//
ArrowModelElement.prototype.getYTo = function(){
    return this.objData.yTo;
};

//
ArrowModelElement.prototype.setYTo = function(v){
    this.objData.yTo = v;
};

//
ArrowModelElement.prototype.getCircleDiameter = function(){
    return this.objDataConst.circleDiameter;
};

//in order to move by mouse
ArrowModelElement.prototype.getX = function(){
    return this.objData.xFrom;
};

//in order to move by mouse
ArrowModelElement.prototype.setX = function(v){
    var me = this;
    var w = me.objData.xTo - me.objData.xFrom;
    me.objData.xFrom = v;
    me.objData.xTo = v+w;
};

//in order to move by mouse
ArrowModelElement.prototype.getY = function(){
    return this.objData.yFrom;
};

//in order to move by mouse
ArrowModelElement.prototype.setY = function(v){
    var me = this;
    var w = me.objData.yTo - me.objData.yFrom;
    me.objData.yFrom = v;
    me.objData.yTo = v+w;
};

//override
ArrowModelElement.prototype.isMovableByMouse = function(){
    var flag = ArrowModelElement.superclass.isMovableByMouse.call(this);
    var hitDetails = this.hitDetails;
    return flag && (hitDetails == null || !(hitDetails.hitStart || hitDetails.hitEnd));
};

//override
ArrowModelElement.prototype.getHitTestInfoForPoint = function (p0) {
    var me = this;
    var flagHit = false, flagInside = false;
    var flagHitStart = false, flagHitEnd = false;

    if(me.isMovable()){
        var handlerRadius = me.HANDLER_DIAMETER/2;
        var rTouch = me.TOUCH_SIZE_FACTOR/ModelElement.prototype.ZOOM_RATIO;
        if(MOUSE_OR_TOUCH>0)
            handlerRadius *= rTouch;
        if(miscFunctions.isPointWithinEllipse(
                p0.x, p0.y, me.objData.xFrom, me.objData.yFrom, handlerRadius, handlerRadius
            )){
            flagHitStart = true;
        }else if(miscFunctions.isPointWithinEllipse(
                p0.x, p0.y, me.objData.xTo, me.objData.yTo, handlerRadius, handlerRadius
            )){
            flagHitEnd = true;
        }
    }

    if(flagHitStart || flagHitEnd){
        flagHit = true;
        flagInside = false;
    }else {
        var aPoint = ArrowModelLibrary.prototype.getArrowOutlineArray(me);
        var pA = new Point(p0.x - me.objData.xFrom, p0.y - me.objData.yFrom);
        flagHit = flagInside = miscFunctions.isPointWithinPolygon(pA, aPoint);
    }

    return new HitTestInfo(me, flagHit, flagInside, {hitStart:flagHitStart, hitEnd:flagHitEnd});
};

//
ArrowModelElement.prototype.isPointWithin = function(p0){
    var me = this;
    var aPoint = ArrowModelLibrary.prototype.getArrowOutlineArray(me);
    return miscFunctions.isPointWithinPolygon(p0.subtract(new Point(me.getX(), me.getY())), aPoint);
};

//
ArrowModelElement.prototype.fPositionsMatch = function(refElement, refDestinationElement, r, rDestination){
    //return new Point(this.getX(), this.getY()).distanceSquared(new Point(refElement.getX(), refElement.getY()))<=r*r;
    var me = this;
    var pFrom = new Point(me.getXFrom(), me.getYFrom());
    var pTo = new Point(me.getXTo(), me.getYTo());
    var flagMatchFrom, flagMatchTo;
    var pRefFrom, pRefTo;
    var r2 = r*r, rD2 = rDestination*rDestination;
    if(refElement == refDestinationElement && refElement.getType() == me.getType()){
        pRefFrom = new Point(refElement.getXFrom(), refElement.getYFrom());
        pRefTo = new Point(refElement.getXTo(), refElement.getYTo());
        flagMatchFrom = pFrom.distanceSquared(pRefFrom)<=r2;
        if(!flagMatchFrom)
            return false;
        flagMatchTo = pTo.distanceSquared(pRefTo)<=rD2;
        return flagMatchTo;
    }else{
        if(typeof refElement[me.POINT_WITHIN_METHOD] == TYPE_FUNCTION){
            flagMatchFrom = refElement[me.POINT_WITHIN_METHOD](pFrom);
        }else if(typeof refElement["getX"] == TYPE_FUNCTION && typeof refElement["getY"] == TYPE_FUNCTION){
            pRefFrom = new Point(refElement.getX(), refElement.getY());
            flagMatchFrom = pFrom.distanceSquared(pRefFrom)<=r2;
        }else
            return false;

        if(!flagMatchFrom)
            return false;

        if(typeof refDestinationElement[me.POINT_WITHIN_METHOD] == TYPE_FUNCTION){
            flagMatchTo = refDestinationElement[me.POINT_WITHIN_METHOD](pTo);
        }else if(typeof refDestinationElement["getX"] == TYPE_FUNCTION && typeof refDestinationElement["getY"] == TYPE_FUNCTION){
            pRefTo = new Point(refDestinationElement.getX(), refDestinationElement.getY());
            flagMatchTo = pTo.distanceSquared(pRefTo)<=rD2;
        }else
            return false;

        return flagMatchTo;
    }
};

//override
ArrowModelElement.prototype.isMatchForElement = function(refElement, options){
    var me = this;
    if(refElement.getType() != me.getType())
        return false;
    var zoneElement =
        (typeof options != TYPE_UNDEFINED && typeof options.zoneElement != TYPE_UNDEFINED ) ?
            options.zoneElement : refElement;
    var zoneDestinationElement =
        (typeof options != TYPE_UNDEFINED && typeof options.zoneDestinationElement != TYPE_UNDEFINED ) ?
        options.zoneDestinationElement : refElement;
    var flag = me.fPositionsMatch(
        zoneElement, zoneDestinationElement,
        zoneElement.getErrorZoneRadius(), zoneDestinationElement.getErrorZoneRadius()
    );
    if(flag && refElement.isCheckColor())
        flag = me.getFillColor() == refElement.getFillColor();
    if(flag && refElement.isCheckType())
        flag = me.getSymbolId() == refElement.getSymbolId();
    return flag;
};
/**
 * @constructor
 */
function CheckboxSetModelElement(_objResourceData) {
    CheckboxSetModelElement.superclass.constructor.call(this, _objResourceData);
    var me = this;
    //
    me.flagShowIfVerified = true;
    me.flagVerifyOwnState = true;
    //
    me.objData.flagCanBeHighlighted = false;
    me.objData.flagColorModifiable = false;
    me.objData.flagShapeModifiable = false;
    me.objData.flagMovable = false;
    me.aCheckbox = [];
    me.aCheckedCorrect = [];
    var aNode = _objResourceData.aNode, node, i, n, a = null, j, l;
    if(Array.isArray(aNode)){
        n = aNode.length;
        for(i=0; i<n; i++){
            node = aNode[i];
            if(node.nodeType == Node.ELEMENT_NODE && node.nodeName == "Items") {
                a = ModelResource.prototype.retrieveObjectsWithinTypeNode("Item", node);
                break;
            }
        }
    }

    var checkbox;
    if(a!=null) {
        n = a.length;
        for (i = 0; i < n; i++) {
            checkbox = new PartCheckbox(me, i, a[i]);
            me.aCheckbox.push(checkbox);
            me.aCheckedCorrect.push(checkbox.isChecked());
        }
        me.arrange();
        if (ModelMainInstance.prototype.flagPlayerMode && !me.isUserCreated()) {
            //correct choices should not be shown from the start
            for (i = 0; i < n; i++) {
                checkbox = me.aCheckbox[i];
                checkbox.setChecked(false);
            }
        }
    }
    //text color set for all the elements
    me.flagFillColored = true;
}
extend(CheckboxSetModelElement, ModelElement);

CheckboxSetModelElement.prototype.paddingTop = 2;
CheckboxSetModelElement.prototype.paddingBottom = 2;
CheckboxSetModelElement.prototype.paddingLeft = 0;
CheckboxSetModelElement.prototype.paddingRight = 5;
CheckboxSetModelElement.prototype.verticalGap = 7;


CheckboxSetModelElement.prototype.typePrefix = ModelResource.prototype.TYPE_CHECKBOXSET;
CheckboxSetModelElement.prototype.familyPrefix = ModelResource.prototype.TYPE_CHECKBOXSETS;
CheckboxSetModelElement.prototype.className = ModelResource.prototype.CLASSID_CHECKBOXSET;
CheckboxSetModelElement.prototype.localizedNamePrefix = "Выборка ";

CheckboxSetModelElement.prototype.aPropretyNameMap = CheckboxSetModelElement.superclass.aPropretyNameMap.slice(0);
(function(){
    var a = CheckboxSetModelElement.prototype.aPropretyNameMap;
    a.splice(a.length, 0,
        ["X", "x", TYPE_NUMBER],
        ["Y", "y", TYPE_NUMBER],
        ["shuffle", "flagShuffle", TYPE_BOOLEAN],
        ["commonFormat", "flagCommonFormat", TYPE_BOOLEAN],
        ["count", "count", TYPE_INTEGER],
        ["radioButtons", "flagRadiobuttons", TYPE_BOOLEAN]
    );
})();

//
CheckboxSetModelElement.prototype.arrange = function(){
    var me = this;
    var aNumber = [], i, n = me.aCheckbox.length;
    for(i=0; i<n; i++)
        aNumber.push(i);
    if(me.isShuffle())
        aNumber = MiscFunctions.prototype.randomizeArray(aNumber, n, true);
    var checkbox;
    var xCurr = CheckboxSetModelElement.prototype.paddingLeft;
    var yCurr = CheckboxSetModelElement.prototype.paddingTop;
    for(i=0; i<n; i++){
        checkbox = me.aCheckbox[aNumber[i]];
        checkbox.setX(xCurr);
        checkbox.setY(yCurr);
        yCurr += CheckboxSetModelElement.prototype.verticalGap+checkbox.getHeight();
    }
};

//override
CheckboxSetModelElement.prototype.reset = function(){
    this.arrange();
};

//
CheckboxSetModelElement.prototype.getCheckboxCount = function(){
    return this.aCheckbox.length;
};

//
CheckboxSetModelElement.prototype.isCorrectChoice = function(no){
    return no>=0 && no<this.aCheckbox.length && this.aCheckedCorrect[no];
};

//
CheckboxSetModelElement.prototype.getCheckboxByOrderNo = function(q){
    var a = this.aCheckbox, checkbox, i, n = a.length;
    for(i=0; i<n; i++){
        checkbox = a[i];
        if(checkbox.getId() == q)
            return checkbox;
    }
    return null;
};

//
CheckboxSetModelElement.prototype.checkByOrderNo = function(q){
    var me = this;
    var checkbox = me.getCheckboxByOrderNo(q);
    if(checkbox != null){
        if(checkbox.isChecked())
            return false;
        checkbox.setChecked(true);
        if(me.isRadiobuttons()){
            var i, n = me.getCheckboxCount();
            for(i=0; i<n; i++){
                if(i == q)
                    continue;
                me.getCheckboxByOrderNo(i).setChecked(false);
            }
        }
        return true;
    }else
        return false;
};

//
CheckboxSetModelElement.prototype.uncheckByOrderNo = function(q){
    var me = this;
    if(ModelMainInstance.prototype.flagPlayerMode && me.isRadiobuttons())
        return false;
    var checkbox = me.getCheckboxByOrderNo(q);
    if(checkbox != null){
        if(!checkbox.isChecked())
            return false;
        checkbox.setChecked(false);
        return true;
    }else
        return false;
};

//
CheckboxSetModelElement.prototype.toggleByOrderNo = function(q){
    var me = this;
    var checkbox = me.getCheckboxByOrderNo(q);
    return (checkbox != null) ? (checkbox.isChecked() ? me.uncheckByOrderNo(q) : me.checkByOrderNo(q)) : false;
};

//string of "0" and "1" for unchecked and checked buttons
CheckboxSetModelElement.prototype.getSChecked = function(){
    var a = this.aCheckbox, i, n = a.length, aChecked = [];
    for(i=0; i<n; i++)
        aChecked.push(a[i].isChecked() ? 1 : 0);
    return aChecked.join("");
};

//override
CheckboxSetModelElement.prototype.isInitiallyTurnedOn = function(){
    return true;
};

//override
CheckboxSetModelElement.prototype.isMovableByMouse = function(){
    var me = this;
    if(!me.isMovable())
        return false;
    var hitDetails = me.getHitDetails();
    //not movable in player
    return false;
};

//override
CheckboxSetModelElement.prototype.getX = function(){
    return this.objData.x;
};

//override
CheckboxSetModelElement.prototype.setX = function(v){
    this.objData.x = v;
};

//override
CheckboxSetModelElement.prototype.getY = function(){
    return this.objData.y;
};

//override
CheckboxSetModelElement.prototype.setY = function(v){
    this.objData.y = v;
};

//override
CheckboxSetModelElement.prototype.isShuffle = function(){
    return this.objData.flagShuffle;
};

//override
CheckboxSetModelElement.prototype.isCommonFormat = function(){
    return this.objData.flagCommonFormat;
};

//override
CheckboxSetModelElement.prototype.getCount = function(){
    return this.objData.count;
};

//override
CheckboxSetModelElement.prototype.isRadiobuttons = function(){
    return this.objData.flagRadiobuttons;
};

//
CheckboxSetModelElement.prototype.addCheckbox = function(objData){
    var me = this;
    var aCheckbox = me.aCheckbox;
    var objCBData = {};
    var objCB = new PartCheckbox(me, aCheckbox.length, objData);
    aCheckbox.push(objCB);
};

//override
CheckboxSetModelElement.prototype.deactivate = function(){
    CheckboxSetModelElement.superclass.deactivate.call(this);
    var me = this;
    var aCheckbox = me.aCheckbox;
    var i, n = aCheckbox.length;
    var checkbox;
    for(i=0; i<n; i++){
        checkbox = aCheckbox[i];
        checkbox.deactivate();
    }
    aCheckbox.splice(0);
};

//override
CheckboxSetModelElement.prototype.getHitTestInfoForPoint = function(p0){
    var me = this;
    var aCheckBox = me.aCheckbox;
    var i, n = aCheckBox.length;
    var checkbox, htCheckbox;
    for(i=0; i<n; i++){
        checkbox = aCheckBox[i];
        htCheckbox = checkbox.getHitTestInfoForPoint(p0);
        if(htCheckbox.isHit())
            break;
    }
    var flagHit = (i<n);
    var flagInside = flagHit && htCheckbox.isInside();
    var objDetails;
    if(flagHit)
        objDetails = {orderNo: checkbox.getId()};
    else
        objDetails = null;
    return new HitTestInfo(me, flagHit, flagInside, objDetails);
};

//
CheckboxSetModelElement.prototype.isMatchForElement = function(refElement, options){
    var me = this;
    if(refElement.getType() != me.getType())
        return false;
    var flag = true;
    var i, n = me.getCheckboxCount(), cb;
    for(i=0; i<n && flag; i++){
        cb = me.getCheckboxByOrderNo(i);
        if(cb.isChecked())
            flag = refElement.isCorrectChoice(i);
        else
            flag = !refElement.isCorrectChoice(i);
    }
    return flag;
};
/**
 * @constructor
 */
function PartCheckbox(parentElement, orderNo, objData){
    PartCheckbox.superclass.constructor.call(this, parentElement, objData);
    var me = this;
    me.orderNo = orderNo;

    me.objData.x = 0;
    me.objData.y = 0;
    me.objData.id = me.orderNo;

    //default parameters
    var objNewResourceData = {
        Id: ""+me.orderNo,
        Depth: me.getParentElement().getDepth(),
        CHCL: "false",
        CHCZ: "false",
        CHFC: "false",
        CHCS: "false",
        CHCT: "true",
        IsVisible: "true",
        Is_Ver: "false",
        Name: PartCheckbox.prototype.getLocalizedNamePrefix() + (me.orderNo + 1),
        NoEdit: "false",
        OW: "1",
        Pl_ECSt: "false",
        Pl_ESSt: "false",
        Pl_HSw: "false",
        Pl_MSt: "false",
        Pl_VSw: "false",
        Pl_VSwOn: "true",
        vAlign: objData.vAlign,
        hAlign: objData.hAlign,
        wordWrap: objData.wordWrap,
        BorderAlpha: objData.BorderAlpha,
        BorderColor: objData.BorderColor,
        BackgroundAlpha: objData.BackgroundAlpha,
        BackgroundColor: objData.BackgroundColor,
        fontAlpha: objData.fontAlpha,
        fontColor: objData.fontColor,
        fontSize: objData.fontSize,
        fontFace: objData.fontFace,
        text: objData.text,
        RZ: "15",
        X: "" + PartCheckbox.prototype.captionXShift,
        Y: "" + 0,
        Width: objData.Width,
        Height: objData.Height,
        type_id: "0"
    };
    me.caption = new TextModelElement(objNewResourceData);
    me.caption.setUserCreated(true);
    me.caption.setComplete(true);
    //
    me.flagFillColored = true;
    me.flagLineColored = true;

}
extend(PartCheckbox, ElementPart);

PartCheckbox.prototype.localizedNamePrefix = "выбор ";
PartCheckbox.prototype.captionXShift = 21;

PartCheckbox.prototype.aPropretyNameMap = PartCheckbox.superclass.aPropretyNameMap.slice(0);
(function(){
    var a = PartCheckbox.prototype.aPropretyNameMap;
    a.splice(a.length, 0,
        ["checked", "flagChecked",  TYPE_BOOLEAN],
        ["X", "x", TYPE_NUMBER],
        ["Y", "y", TYPE_NUMBER]
    );
})();


//
PartCheckbox.prototype.isChecked = function () {
    return this.objData.flagChecked;
};

//
PartCheckbox.prototype.setChecked = function (flag) {
    this.objData.flagChecked = flag;
};

//override
PartCheckbox.prototype.getX = function(){
    return this.objData.x;
};

//override
PartCheckbox.prototype.setX = function(v){
    this.objData.x = v;
    this.caption.setX(PartCheckbox.prototype.captionXShift+v);
};

//override
PartCheckbox.prototype.getY = function(){
    return this.objData.y;
};

//override
PartCheckbox.prototype.setY = function(v){
    this.objData.y = v;
    this.caption.setY(v);
};

//
PartCheckbox.prototype.getCaption = function(){
    return this.caption;
};

//override
PartCheckbox.prototype.getWidth = function(){
    return PartCheckbox.prototype.captionXShift+this.caption.getWidth();
};

//
PartCheckbox.prototype.getButtonSize = function(){
    return this.parentElement.isRadiobuttons() ? 19 : 17;
};

//override
PartCheckbox.prototype.getHeight = function(){
    var me = this;
    var captionHeight = me.caption.getHeight();
    var buttonHeight = me.getButtonSize();
    return (captionHeight>buttonHeight) ? captionHeight : buttonHeight;
};

//override
PartCheckbox.prototype.getHitTestInfoForPoint = function(p0){
    var me = this;
    var flagHit;
    var flagInside;
    var flagHitCorner;
    var x = me.getX();
    var y = me.getY();
    var xParent = me.parentElement.getX();
    var yParent = me.parentElement.getY();
    var p = new Point(p0.x-xParent, p0.y-yParent);
    var d = me.getButtonSize();
    var rButton = d/2;
    var fontSize = me.getCaption().getFontSize();
    var lineHeight =
        TextFunctions.prototype.getLineHeightByFontSize(fontSize)+
        2*TextFunctions.prototype.getVerticalPaddingByFontSize(fontSize);

    var rTouch = me.TOUCH_SIZE_FACTOR/ModelElement.prototype.ZOOM_RATIO;
    var xM = d/2;
    if(MOUSE_OR_TOUCH>0){
        d *= rTouch;
        rButton *= rTouch;
    }

    var flagHitButton;
    if(me.parentElement.isRadiobuttons())
        flagHitButton = miscFunctions.isPointWithinEllipse(p.x, p.y, x+rButton, y+lineHeight/2, rButton, rButton);
    else
        flagHitButton = (p.x>=(xM-d/2)) && (p.x<=(xM+d/2)) && (p.y>=y+(lineHeight - d)/2) && (p.y<=y+(lineHeight+d)/2);

    if(flagHitButton){
        flagHit = true;
        flagInside = false;
        me.caption.setHitDetails({flagHit:false, flagInside:false});
    }else{
        var caption = me.caption.getHitTestInfoForPoint(p);
        flagHit = caption.isHit();
        flagInside = caption.isInside();
        flagHitCorner = caption.getDetails().hitCorner;
        me.caption.setHitDetails({flagHit:flagHit, flagInside:flagInside, hitCorner:flagHitCorner});
    }
    return new HitTestInfo(me, flagHit, flagInside, {hitButton:flagHitButton});
};
/**
 * @constructor
 */
function ImageModelElementVisualization(_element,_sceneVisualization){
    ImageModelElementVisualization.superclass.constructor.call(this, _element, _sceneVisualization);
    var me = this;
    //there is a limitation for canvas sizes; hence actual canvas is only as large as scene visualization
    me.canvasWidth = 0;
    me.canvasHeight = 0;
    me.cnvImg = document.createElement("CANVAS");
    //we use prevDrawParams to compare the way the img was previously drawn onto cnvImg
    // not to redraw it if nothing has changed
    me.prevDrawParams = {w:0, h:0, scaleX:1, scaleY: 1, dWidth2:0, dHeight2:0};
}
extend(ImageModelElementVisualization, ModelElementVisualization);

ImageModelElementVisualization.prototype.update = function() {
    ImageModelElementVisualization.superclass.update.call(this);

    var me = this;
    var ctx = this.ctx;
    var element = this.element;
    var x = element.getX();
    var y = element.getY();
    var xTL = element.getXTL();
    var yTL = element.getYTL();
    var xBR = element.getXBR();
    var yBR = element.getYBR();
    var sV = me.sceneVisualization;

    var rSize = sV.getCanvasToSceneSizeRatioZoom();

    var pTL = sV.elementCoordsToCanvasCoords(xTL+x, yTL+y);
    var pBR = sV.elementCoordsToCanvasCoords(xBR+x, yBR+y);

    var prevDrawParams = me.prevDrawParams;

    var img = element.objData.img;
    var cnvImg = this.cnvImg;
    var ctxImg = cnvImg.getContext("2d");

    var offset;
    var dWidth2;
    var dHeight2;

    //highlight and selection
    offset = element.getHighlightWidth()*rSize;
    dWidth2 = dHeight2 = Math.ceil(offset);
    var w = (Math.round(pBR.x-pTL.x)|0)+2*dWidth2;
    var h = (Math.round(pBR.y-pTL.y)|0)+2*dHeight2;
    pTL.x -= dWidth2;
    pTL.y -= dHeight2;
    var scaleX = (w-2*dWidth2)/img.width;
    var scaleY = (h-2*dHeight2)/img.height;

    var sModel = sV.getModel();
    var originX = sModel.getOriginX()*rSize;
    var originY = sModel.getOriginY()*rSize;
    var pTLOrigin = new Point(pTL.x+originX, pTL.y+originY);
    var wActual = w;
    var hActual = h;
    var wVis = sV.getWidth();
    var hVis = sV.getHeight();

    var dX;
    var dY;

    if(pTLOrigin.x<0){
        dX = pTLOrigin.x;
        wActual += pTLOrigin.x;
        if(wActual>wVis)
            wActual = wVis;
    }else{
        dX = 0;
        if(wActual+pTLOrigin.x>wVis)
            wActual = wVis-pTLOrigin.x;
    }
    if(pTLOrigin.y<0){
        dY = pTLOrigin.y;
        hActual += pTLOrigin.y;
        if(hActual>hVis)
            hActual = hVis;
    }else{
        dY = 0;
        if(hActual+pTLOrigin.y>hVis)
            hActual = hVis-pTLOrigin.y;
    }
    if(wActual < 1)
        wActual = 0;
    if(hActual < 1)
        hActual = 0;

    var flagRedrawImg =
        (prevDrawParams.w != w) || (prevDrawParams.h != h) ||
        (prevDrawParams.scaleX != scaleX) || (prevDrawParams.scaleY != scaleY) ||
        (prevDrawParams.dWidth2 != dWidth2) || (prevDrawParams.dHeight2 != dHeight2) ||
        (prevDrawParams.dX != dX) || (prevDrawParams.dY != dY) ||
        (prevDrawParams.wActual != wActual) || (prevDrawParams.hActual != hActual);

    //flagRedrawImg = true;
    if(flagRedrawImg) {
        if (me.canvasWidth != w || me.canvasHeight != h) {
            me.canvasWidth = w;
            me.canvasHeight = h;
        }
        if(cnvImg.width != wActual || cnvImg.height != hActual) {
            if (wActual > 0 && hActual > 0) {
                cnvImg.width = wActual;
                cnvImg.height = hActual;
            }
        }
        ctxImg.clearRect(0, 0, cnvImg.width, cnvImg.height);

        ctxImg.setTransform(scaleX, 0, 0, scaleY, 0, 0);
        ctxImg.drawImage(img, (dWidth2+dX)/scaleX, (dHeight2+dY)/scaleY);
        ctxImg.setTransform(1, 0, 0, 1, 0, 0);

        prevDrawParams.w = w;
        prevDrawParams.h = h;
        prevDrawParams.scaleX = scaleX;
        prevDrawParams.scaleY = scaleY;
        prevDrawParams.dWidth2 = dWidth2;
        prevDrawParams.dHeight2 = dHeight2;
        prevDrawParams.dX = dX;
        prevDrawParams.dY = dY;
        prevDrawParams.wActual = wActual;
        prevDrawParams.hActual = hActual;
    }

    if(element.isHighlighted() || element.isBlinking() || element.isSelected()) {
        var alpha = me.getHighlightAlpha()*0.01;
        if ((element.isSelected() || element.isBlinking()) && !element.isHighlighted())
            alpha *= sV.getHighlightAlphaMultiplier(); //0.5*(1.0+Math.cos(me.sceneVisualization.getHighlightBlinkPhase()));
        ctx.beginPath();
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.getHighlightColor(), alpha);
        ctx.lineWidth = offset;
        var offset2 = offset/2;
        //width and height of the hl-frame are "former" canvas width plus double line offset (=line width/2)
        ctx.strokeRect((pTL.x+dWidth2-offset2), (pTL.y+dHeight2-offset2), ((w-2*dWidth2)+2*offset2)|0, ((h-2*dHeight2)+2*offset2)|0);
    }

    ctx.drawImage(cnvImg, (pTL.x-dX)|0, (pTL.y-dY)|0);
};

/**
 * @constructor
 */
function PointModelElementVisualization(_element,_sceneVisualization){
    PointModelElementVisualization.superclass.constructor.call(this, _element, _sceneVisualization);
    var me = this;
    me.cnvImg = null;
    me.data = null;
    me.prevData = {};
}
extend(PointModelElementVisualization, ModelElementVisualization);

PointModelElementVisualization.prototype.SHOW_CORRECT_ALPHA = 100;

PointModelElementVisualization.prototype.update = function() {
    PointModelElementVisualization.superclass.update.call(this);
    var me = this;
    var prevData = me.prevData;
    var ctx = this.ctx;
    var element = this.element;
    var x = element.getX();
    var y = element.getY();
    var sV = me.sceneVisualization;

    var rSize = sV.getCanvasToSceneSizeRatioZoom();

    var p = sV.elementCoordsToCanvasCoords(x, y);

    var sId = element.getSymbolId();
    var fillColor = element.getFillColor();
    var fillAlpha = element.getFillAlpha();
    var flagHighlight = element.isHighlighted();
    var flagSelect = element.isSelected();
    var flagBlink = element.isBlinking();

    var highlightAlpha = 0;

    if(flagHighlight || flagBlink || flagSelect) {
        highlightAlpha = me.getHighlightAlpha() * 0.01;
        if ((flagSelect || flagBlink) && !flagHighlight)
            highlightAlpha *= sV.getHighlightAlphaMultiplier(); //0.5 * (1.0 + Math.cos(me.sceneVisualization.getHighlightBlinkPhase()));
    }

    var flagMakeCanvas = false;

    if(typeof prevData.symbolId == TYPE_UNDEFINED || prevData.symbolId != sId){
        me.data = PointModelLibrary.prototype.getDataById(sId);
        flagMakeCanvas = true;
    }else if(
        typeof prevData.rSize == TYPE_UNDEFINED || prevData.rSize != rSize ||
        typeof prevData.fillColor == TYPE_UNDEFINED || prevData.fillColor != fillColor ||
        typeof prevData.fillAlpha == TYPE_UNDEFINED || prevData.fillAlpha != fillAlpha ||
        typeof prevData.flagHighlight == TYPE_UNDEFINED || prevData.flagHighlight != flagHighlight ||
        typeof prevData.flagSelect == TYPE_UNDEFINED || prevData.flagSelect != flagSelect ||
        typeof prevData.highlightAlpha == TYPE_UNDEFINED || prevData.highlightAlpha != highlightAlpha){
        flagMakeCanvas = true;
    }

    if(flagMakeCanvas){
        prevData.symbolId = sId;
        prevData.rSize = rSize;
        prevData.fillColor = fillColor;
        prevData.fillAlpha = fillAlpha;
        prevData.flagHighlight = flagHighlight;
        prevData.flagSelect = flagSelect;
        prevData.highlightAlpha = highlightAlpha;

        //if stock canvas is used, cnv needs to be copied to cnv of the symbol

        //shadow
        var shadowShift = 1*rSize, sizeShift = Math.ceil(shadowShift);
        var cnv = PointModelLibrary.prototype.makeCanvasByData(
                                            me.data, rSize,
                                            0x000000, 0.01*15,
                                            0xFFFFFF, 0.0,
                                            0xFFFFFF, 0.0,
                                            false, true
        );

        if(me.cnvImg == null)
            me.cnvImg = document.createElement("CANVAS");
        if(me.cnvImg.width != (cnv.width+shadowShift) || me.cnvImg.height != (cnv.height+shadowShift)){
            me.cnvImg.width = cnv.width+shadowShift;
            me.cnvImg.height = cnv.height+shadowShift;
        }
        me.cnvImg.getContext("2d").clearRect(0, 0, me.cnvImg.width, me.cnvImg.height);
        me.cnvImg.getContext("2d").drawImage(cnv, shadowShift, shadowShift);

        cnv = PointModelLibrary.prototype.makeCanvasByData(
                                            me.data, rSize,
                                            fillColor, 0.01*fillAlpha,
                                            0xFFFFFF, 0.0,
                                            me.getHighlightColor(), highlightAlpha,
                                            flagSelect && (element.isShapeModifiable() || element.isColorModifiable()),
            false
        );

        me.cnvImg.getContext("2d").drawImage(cnv, 0, 0);
    }

    var cnvImg = me.cnvImg;
    var w = cnvImg.width;
    var h = cnvImg.height;

    //+1 for coordinates were added emprirically, need to be justified
    ctx.drawImage(cnvImg, (p.x-w/2+1)|0, (p.y-h/2+1)|0);
};
/**
 * @constructor
 */
function RectangleModelElementVisualization(_element, _sceneVisualization){
    RectangleModelElementVisualization.superclass.constructor.call(this, _element, _sceneVisualization);
    //
}
extend(RectangleModelElementVisualization, ModelElementVisualization);

RectangleModelElementVisualization.prototype.update = function(){
    RectangleModelElementVisualization.superclass.update.call(this);
    var me = this;
    var ctx = this.ctx;
    var element = this.element;
    var xTL = element.getXTL();
    var yTL = element.getYTL();
    var xBR = element.getXBR();
    var yBR = element.getYBR();
    var sV = me.sceneVisualization;

    var rSize = sV.getCanvasToSceneSizeRatioZoom();

    var bW = element.getBorderWidth(), bW2 = bW/2;
    var hW = element.getHighlightWidth();

    var pTL = sV.elementCoordsToCanvasCoords(xTL+bW2, yTL+bW2);
    var pBR = sV.elementCoordsToCanvasCoords(xBR-bW2, yBR-bW2);

    var w = pBR.x-pTL.x;
    var h = pBR.y-pTL.y;

    if(element.isHighlighted() || element.isBlinking() || element.isSelected()) {
        var alpha = me.getHighlightAlpha()*0.01;
        if ((element.isSelected() || element.isBlinking()) && !element.isHighlighted())
            alpha *= sV.getHighlightAlphaMultiplier(); //0.5*(1.0+Math.cos(me.sceneVisualization.getHighlightBlinkPhase()));

        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.getHighlightColor(), alpha);
        ctx.lineWidth = hW*rSize;
        var offset = bW + ctx.lineWidth, offset2 = offset/2;
        switch(element.getSubType()) {
            case "ellipse":
                miscFunctions.drawEllipse(ctx, (pTL.x + w/2)|0, (pTL.y + h/2)|0, (w/2 + offset2)|0, (h/2 + offset2)|0, false);
                break;
            case "rectangle":
                ctx.strokeRect((pTL.x - offset2)|0, (pTL.y - offset2)|0, (w + offset)|0, (h + offset)|0);
                break;
            default:
        }
    }

    ctx.fillStyle = element.isEmptyFill() ?
        "rgba(255, 255, 255, 0.0)" :
        miscFunctions.intAlpha2RGBAFunction(element.getFillColor(), element.getFillAlpha()*0.01);
    ctx.strokeStyle = element.isEmptyBorder() ?
        "rgba(255, 255, 255, 0.0)" :
        miscFunctions.intAlpha2RGBAFunction(element.getLineColor(), element.getLineAlpha()*0.01);
    ctx.lineWidth = bW*rSize;

    switch(element.getSubType()){
        case "ellipse":
            miscFunctions.drawEllipse(ctx, (pTL.x+w/2)|0, (pTL.y+h/2)|0, (w/2)|0, (h/2)|0, true);
            break;
        case "rectangle":
            ctx.fillRect(pTL.x|0, pTL.y|0, w|0, h|0);
            ctx.strokeRect(pTL.x|0, pTL.y|0, w|0, h|0);
            break;
        default:
    }
};
/**
 * @constructor
 */
function PolylineModelElementVisualization(_element,_sceneVisualization){
    PolylineModelElementVisualization.superclass.constructor.call(this, _element,_sceneVisualization);
    var me = this;
    me.sceneVisualization.addEventListener(
        ModelEvent.prototype.CANVAS_TO_SCENE_RATIO_CHANGED, {listener:me.ratioChangeListener, target:me}, false
    );

    PolylineModelElementVisualization.prototype.rSize = me.sceneVisualization.getCanvasToSceneSizeRatio();
    me.cnvPLO = document.createElement("CANVAS");
    me.ctxPLO = me.cnvPLO.getContext("2d");
    if(me.crossCanvas == null)
        me.init();
}
extend(PolylineModelElementVisualization, ModelElementVisualization);

PolylineModelElementVisualization.prototype.rSize = -1;
PolylineModelElementVisualization.prototype.crossCanvas = null;

PolylineModelElementVisualization.prototype.DEFAULT_HANDLER_OVER_COLOR = 0xFFFF80;
PolylineModelElementVisualization.prototype.DEFAULT_HANDLER_OVER_ALPHA = 0.75;
PolylineModelElementVisualization.prototype.DEFAULT_HANDLER_DOWN_COLOR = 0xFF8080;
PolylineModelElementVisualization.prototype.DEFAULT_HANDLER_DOWN_ALPHA = 0.75;

PolylineModelElementVisualization.prototype.DEFAULT_HANDLER_BORDER_OVER_COLOR = 0x000000;
PolylineModelElementVisualization.prototype.DEFAULT_HANDLER_BORDER_OVER_ALPHA = 1.0;
PolylineModelElementVisualization.prototype.DEFAULT_HANDLER_BORDER_DOWN_COLOR = 0x000000;
PolylineModelElementVisualization.prototype.DEFAULT_HANDLER_BORDER_DOWN_ALPHA = 1.0;

//for user-created (isUserCreated() == true) polylines only
PolylineModelElementVisualization.prototype.DEFAULT_EDITABLE_SEGMENT_OVER_COLOR = 0xFFFF00;
PolylineModelElementVisualization.prototype.DEFAULT_EDITABLE_SEGMENT_OVER_ALPHA = 1.0;
PolylineModelElementVisualization.prototype.DEFAULT_EDITABLE_SEGMENT_POINTER_COLOR = 0x00FF99;
PolylineModelElementVisualization.prototype.DEFAULT_EDITABLE_SEGMENT_POINTER_ALPHA = 1.0;

PolylineModelElementVisualization.prototype.DEFAULT_CROSS_STRIPE_COLOR = 0xFF0000;
PolylineModelElementVisualization.prototype.DEFAULT_CROSS_BACKGROUND_COLOR = 0x000000;

//
PolylineModelElementVisualization.prototype.ratioChangeListener = function(e){
    if(PolylineModelElementVisualization.prototype.rSize != e.data.value)
        this.init();
};

//
PolylineModelElementVisualization.prototype.init = function(){
    var me = this;
    var element = me.element;
    var sV = me.sceneVisualization;

    var rSize = sV.getCanvasToSceneSizeRatioZoom();
    PolylineModelElementVisualization.prototype.rSize = rSize;

    var wCross = element.CROSS_SIZE*rSize;
    var wStripe = (element.CROSS_STRIPE_WIDTH+1)*rSize;
    var offset = wStripe*0.7107;

    PolylineModelElementVisualization.prototype.crossCanvas = document.createElement("CANVAS");
    var cnv = PolylineModelElementVisualization.prototype.crossCanvas;
    var ctx = cnv.getContext("2d");
    var w = Math.ceil(wCross+wStripe*1.4215);
    if(cnv.width != w || cnv.height != w)
        cnv.width = cnv.height = w;
    else
        ctx.clearRect(0, 0, w, w);

    ctx.lineJoin = "miter";
    ctx.lineWidth = wStripe;
    ctx.strokeStyle =
        miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_CROSS_BACKGROUND_COLOR, 1.0);
    ctx.beginPath();
    ctx.moveTo(offset, offset);
    ctx.lineTo(wCross+offset, wCross+offset);
    ctx.moveTo(offset, wCross+offset);
    ctx.lineTo(wCross+offset, offset);
    ctx.stroke();

    ctx.lineWidth = wStripe-1*rSize;
    ctx.strokeStyle =
        miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_CROSS_STRIPE_COLOR, 1.0);
    ctx.beginPath();
    ctx.moveTo(offset, offset);
    ctx.lineTo(wCross+offset, wCross+offset);
    ctx.moveTo(offset, wCross+offset);
    ctx.lineTo(wCross+offset, offset);
    ctx.stroke();
};

//override
PolylineModelElementVisualization.prototype.update = function(){
    PolylineModelElementVisualization.superclass.update.call(this);
    var me = this;
    var sV = me.sceneVisualization;
    var ctx = sV.getCanvasContext();
    var element = this.element;

    var x = element.getX();
    var y = element.getY();
    var aPoint = element.getPoints();
    var bW = element.getBorderWidth();
    var hW = element.getHighlightWidth();
    var flagClosed = element.isCloseLine();

    var i, n;

    var hitDetails = element.getHitDetails();
    var flagStructureEditable = element.isStructureEditable();
    var flagVerticesMovable = element.isVerticesMovable();
    var handlerNumber, flagOverHandler;
    var segmentNumber, flagOverSegment, pointOnSegment;
    if(hitDetails!=null) {
        handlerNumber = hitDetails.handlerNumber;
        segmentNumber = hitDetails.segmentNumber;
        pointOnSegment = hitDetails.pointOnSegment;
    }else {
        handlerNumber = element.getPressedHandlerNo();
        segmentNumber = -1;
        pointOnSegment = null;
    }
    //flagOverXXX is calculated with account for flagStructureEditable
    flagOverHandler = (handlerNumber != -1) && (flagStructureEditable || flagVerticesMovable);
    flagOverSegment = (segmentNumber != -1) && flagStructureEditable;

    var rSize = PolylineModelElementVisualization.prototype.rSize;

    var aPointCanvas = [], pC, p;
    var xTL = Number.MAX_VALUE;
    var yTL = Number.MAX_VALUE;
    var xBR = -Number.MAX_VALUE;
    var yBR = -Number.MAX_VALUE;
    n = aPoint.length;

    for(i=0; i<n; i++){
        p = aPoint[i];
        pC = sV.elementCoordsToCanvasCoords(p.x+x, p.y+y);
        if(pC.x<xTL)
            xTL = pC.x;
        if(pC.x>xBR)
            xBR = pC.x;
        if(pC.y<yTL)
            yTL = pC.y;
        if(pC.y>yBR)
            yBR = pC.y;
        aPointCanvas.push(pC);
    }

    var bOffset = bW*rSize;
    var hlOffset = hW*rSize;
    var offset = bOffset;
    var dWidth2;
    var dHeight2;
    var wCross;

    if(element.isHighlighted() || element.isSelected() || element.isBlinking())
        offset += hlOffset;

    dWidth2 = dHeight2 = Math.ceil(offset);
    var w = (Math.round(xBR-xTL)|0)+2*dWidth2;
    var h = (Math.round(yBR-yTL)|0)+2*dHeight2;

    for(i=0; i<n; i++)
        aPointCanvas[i].offset(-(xTL-dWidth2), -(yTL-dHeight2));

    var cnvPLO = me.cnvPLO;
    var ctxPLO = me.ctxPLO;

    if(cnvPLO.width != w || cnvPLO.height != h){
        cnvPLO.width = w;
        cnvPLO.height = h;
    }else
        ctxPLO.clearRect(0, 0, w, h);

    ctxPLO.lineJoin = "round";
    ctxPLO.lineCap = "round";

    n = aPointCanvas.length;
    if(n>0){
        if(
            (element.isHighlighted() && !flagOverHandler && !flagOverSegment) ||
            element.isBlinking() || element.isSelected()
        ) {
            var alpha = me.getHighlightAlpha() * 0.01;
            if (
                (element.isSelected() || element.isBlinking()) &&
                (!element.isHighlighted() || flagOverHandler || flagOverSegment)
            )
                alpha *= sV.getHighlightAlphaMultiplier();
            ctxPLO.beginPath();
            ctxPLO.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.getHighlightColor(), alpha);
            ctxPLO.lineWidth = (2*hW+bW) * rSize;

            pC = aPointCanvas[0];
            ctxPLO.moveTo(pC.x, pC.y);
            for(i=1; i<n; i++){
                pC = aPointCanvas[i];
                ctxPLO.lineTo(pC.x, pC.y);
            }
            if(flagClosed){
                pC = aPointCanvas[0];
                ctxPLO.lineTo(pC.x, pC.y);
            }
            ctxPLO.stroke();

        }

        ctxPLO.beginPath();
        ctxPLO.strokeStyle = element.isEmptyBorder() ?
                    "rgba(255, 255, 255, 0.0)" :
                    miscFunctions.intAlpha2RGBAFunction(element.getLineColor(), element.getLineAlpha()*0.01);
        if(flagClosed)
            ctxPLO.fillStyle = element.isEmptyFill() ?
                    "rgba(255, 255, 255, 0.0)" :
                    miscFunctions.intAlpha2RGBAFunction(element.getFillColor(), element.getFillAlpha() * 0.01);

        ctxPLO.lineWidth = bW*rSize;
        pC = aPointCanvas[0];
        ctxPLO.moveTo(pC.x, pC.y);
        for(i=1; i<n; i++){
            pC = aPointCanvas[i];
            ctxPLO.lineTo(pC.x, pC.y);
        }
        if(flagClosed){
            pC = aPointCanvas[0];
            ctxPLO.lineTo(pC.x, pC.y);
        }
        if(flagClosed)
            ctxPLO.fill();
        ctxPLO.stroke();

        //the next clause is just to make the very initial part of a PLO visible on a mobile device
        if(n==2){
            var dx = aPointCanvas[1].x-aPointCanvas[0].x;
            var dy = aPointCanvas[1].y-aPointCanvas[0].y;
            if(dx == 0 || dy == 0){
                ctxPLO.beginPath();
                ctxPLO.fillStyle = ctxPLO.strokeStyle;
                ctxPLO.arc(aPointCanvas[1].x, aPointCanvas[1].y, ctxPLO.lineWidth, 0, miscFunctions.TWO_PI, false);
                ctxPLO.fill();
            }
        }

        //yellow segment if editable
        if(flagOverSegment && element.isStructureEditable()){
            ctxPLO.lineWidth = bW*rSize+2;
            ctxPLO.strokeStyle = miscFunctions.intAlpha2RGBAFunction(0x000000, 1.0);

            ctxPLO.beginPath();
            pC = aPointCanvas[segmentNumber];
            ctxPLO.moveTo(pC.x, pC.y);
            pC = aPointCanvas[((segmentNumber+1) == n) ? 0 : (segmentNumber+1)];
            ctxPLO.lineTo(pC.x, pC.y);
            ctxPLO.stroke();

            ctxPLO.lineWidth = bW*rSize;
            ctxPLO.strokeStyle =
                miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_EDITABLE_SEGMENT_OVER_COLOR, me.DEFAULT_EDITABLE_SEGMENT_OVER_ALPHA
                );
            ctxPLO.beginPath();
            pC = aPointCanvas[segmentNumber];
            ctxPLO.moveTo(pC.x, pC.y);
            pC = aPointCanvas[((segmentNumber+1) == n) ? 0 : (segmentNumber+1)];
            ctxPLO.lineTo(pC.x, pC.y);
            ctxPLO.stroke();
        }
    }

    ctx.drawImage(cnvPLO, xTL-dWidth2, yTL-dHeight2);

    //green point over the editable segment
    if(flagOverSegment && element.isStructureEditable()){
        var pOnSegment = sV.elementCoordsToCanvasCoords(pointOnSegment.x, pointOnSegment.y);
        if(!sV.isCtrl()) {
            var pOnSegmentR = element.HANDLER_DIAMETER * rSize / 2;
            ctx.lineWidth = 1 * rSize;
            ctx.fillStyle =
                miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_EDITABLE_SEGMENT_POINTER_COLOR, me.DEFAULT_EDITABLE_SEGMENT_POINTER_ALPHA
                );
            ctx.strokeStyle =
                miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_HANDLER_BORDER_OVER_COLOR, me.DEFAULT_HANDLER_BORDER_OVER_ALPHA
                );
            ctx.beginPath();
            ctx.arc(pOnSegment.x, pOnSegment.y, pOnSegmentR, 0, miscFunctions.TWO_PI, true);
            ctx.fill();
            ctx.stroke();
        }

        //Cross?
        if(element.isCloseLine() && sV.isCtrl()){
            wCross = me.crossCanvas.width;
            ctx.drawImage(me.crossCanvas, pOnSegment.x-wCross/2, pOnSegment.y-wCross/2);
        }
    }

    if(flagOverHandler){
        var pressedHandlerNumber = element.getPressedHandlerNo();
        var hP = sV.elementCoordsToCanvasCoords(aPoint[handlerNumber].x+x, aPoint[handlerNumber].y+y);
        var hR = element.HANDLER_DIAMETER*rSize/2;

        ctx.lineWidth = 1*rSize;
        if(handlerNumber == pressedHandlerNumber){
            ctx.fillStyle =
                miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_HANDLER_DOWN_COLOR, me.DEFAULT_HANDLER_DOWN_ALPHA
                );
            ctx.strokeStyle =
                miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_HANDLER_BORDER_DOWN_COLOR, me.DEFAULT_HANDLER_BORDER_DOWN_ALPHA
                );
        }else {
            ctx.fillStyle =
                miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_HANDLER_OVER_COLOR, me.DEFAULT_HANDLER_OVER_ALPHA
                );
            ctx.strokeStyle =
                miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_HANDLER_BORDER_OVER_COLOR, me.DEFAULT_HANDLER_BORDER_OVER_ALPHA
                );
        }
        ctx.beginPath();
        ctx.arc(hP.x, hP.y, hR, 0, miscFunctions.TWO_PI, true);
        ctx.fill();
        ctx.stroke();

        //Cross?
        if(sV.isCtrl() && element.isStructureEditable() && (element.getPointCount()>element.getMinimalPointCount())){
            wCross = me.crossCanvas.width;
            ctx.drawImage(me.crossCanvas, hP.x-wCross/2, hP.y-wCross/2);
        }
    }

    if(!element.isComplete()){
        if(element.isCloseEnds()) {
            var pHead = aPoint[0];
            var pTail = aPoint[element.getPointCount()-1];
            var pHeadCanvas = sV.elementCoordsToCanvasCoords(pHead.x+x, pHead.y+y);
            var pTailCanvas = sV.elementCoordsToCanvasCoords(pTail.x+x, pTail.y+y);
            var rLink = element.HANDLER_DIAMETER*rSize/2;
            var phi0 = Math.atan2(pTailCanvas.x-pHeadCanvas.x, -(pTailCanvas.y-pHeadCanvas.y));
            var cPhi0 = Math.cos(phi0);
            var sPhi0 = Math.sin(phi0);
            ctx.beginPath();
            ctx.fillStyle =
                miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_HANDLER_OVER_COLOR, me.DEFAULT_HANDLER_OVER_ALPHA
                );
            ctx.strokeStyle =
                miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_HANDLER_BORDER_OVER_COLOR, me.DEFAULT_HANDLER_BORDER_OVER_ALPHA
                );
            ctx.moveTo(pHeadCanvas.x+rLink*cPhi0, pHeadCanvas.y+rLink*sPhi0);
            ctx.lineTo(pTailCanvas.x+rLink*cPhi0, pTailCanvas.y+rLink*sPhi0);
            ctx.arc(pTailCanvas.x, pTailCanvas.y, rLink, phi0, phi0+Math.PI, true);
            ctx.moveTo(pTailCanvas.x-rLink*cPhi0, pTailCanvas.y-rLink*sPhi0);
            ctx.lineTo(pHeadCanvas.x-rLink*cPhi0, pHeadCanvas.y-rLink*sPhi0);
            ctx.arc(pHeadCanvas.x, pHeadCanvas.y, rLink, phi0+Math.PI, phi0, true);
            ctx.fill();
            ctx.stroke();
        }
    }
};

//
PolylineModelElementVisualization.prototype.deactivate = function(){
    PolylineModelElementVisualization.superclass.deactivate.call(this);
    var me = this;
    PolylineModelElementVisualization.superclass.deactivate.call(me);
    me.sceneVisualization.removeEventListener(
        ModelEvent.prototype.CANVAS_TO_SCENE_RATIO_CHANGED, {listener:me.ratioChangeListener, target:me}, false
    );
};
/**
 * @constructor
 */
function TextModelElementVisualization(_element,_sceneVisualization){
    TextModelElementVisualization.superclass.constructor.call(this, _element, _sceneVisualization);
    var me = this;
    me.cnvTO = document.createElement("CANVAS");
    me.ctxTO = me.cnvTO.getContext("2d");
    me.prevDrawParams = {
        w:0, h:0, dWidth2:0, dHeight2:0,
        borderColor:0, borderAlpha:0,
        backgroundColor:0, backgroundAlpha:0,
        sText:null, fontSize:0, fontFace:null, fontColor:0, fontAlpha:0, horizontalAlign:null
    };
    me.prevEditingMode = me.element.isEditingMode();
    me.onInput = function(e){
        //this event is listened to by the controller
        var evt = new ModelEvent(ModelEvent.prototype.INPUT, true, true);
        if(evt != null){
            me.element.invalidate();
            evt.data = {value:e.target.value};
            me.dispatchEvent(evt);
        }
    };
    me.propagationStopper = function(e){
        e.stopPropagation();
    };
    me.onMouseOver = function(e){
        var evt = new ModelEvent(ModelEvent.prototype.OVER_SELECTABLE_TEXT, true, true);
        if(evt != null)
            me.dispatchEvent(evt);
    };
    me.onMouseOut = function(e){
        var evt = new ModelEvent(ModelEvent.prototype.OUT_SELECTABLE_TEXT, true, true);
        if(evt != null)
            me.dispatchEvent(evt);
    };
    me.onBlur = function(e){
        var evt = new ModelEvent(ModelEvent.prototype.BLUR, true, true);
        if(evt != null)
            me.dispatchEvent(evt);
    };
    //textArea if the element is editable
    me.inputElement = null;
}
extend(TextModelElementVisualization, ModelElementVisualization);

TextModelElementVisualization.prototype.SELECT_COLOR_OVER = 0xFF20202;
TextModelElementVisualization.prototype.INPUT_ELEMENT_BACKGROUND_COLOR = 0xFFFFFF;
TextModelElementVisualization.prototype.INPUT_ELEMENT_BACKGROUND_ALPHA = 1.0;
TextModelElementVisualization.prototype.FONT_SIZE_NORM = 18;

//override
TextModelElementVisualization.prototype.deactivate = function(){
    TextModelElementVisualization.superclass.deactivate.call(this);
    var me = this;
    if(me.inputElement != null){
        me.inputElement.removeEventListener("input", me.onInput, false);
        me.inputElement.removeEventListener("mousedown", me.propagationStopper, true);
        me.inputElement.removeEventListener("keydown", me.propagationStopper, true);
        me.inputElement.removeEventListener("mousemove", me.propagationStopper, true);
        me.inputElement.removeEventListener("mouseover", me.onMouseOver, true);
        me.inputElement.removeEventListener("mouseout", me.onMouseOut, true);
        me.inputElement.removeEventListener("blur", me.onBlur, true);
        if(me.inputElement.parentNode != null)
            me.inputElement.parentNode.removeChild(me.inputElement);
        me.inputElement = null;
    }
};

//override
TextModelElementVisualization.prototype.update = function() {
    TextModelElementVisualization.superclass.update.call(this);
    var me = this;
    var ctx = me.ctx;
    var element = me.element;
    var sV = me.sceneVisualization;

    var flagEditingMode = element.isEditingMode();

    var prevDrawParams = me.prevDrawParams;

    var rSize = sV.getCanvasToSceneSizeRatioZoom();

    var rr = (element.getFontSize()*rSize<me.FONT_SIZE_NORM) ? me.FONT_SIZE_NORM/(element.getFontSize()*rSize) : 1.0;

    var objTextSplitParameters = element.getTextSplitParameters(rr*rSize);
    var aText = objTextSplitParameters.aText;
    var aWidth = objTextSplitParameters.aWidth;
    var tW = element.getWidth();

    var bW = element.getBorderWidth();
    var hW = element.getHighlightWidth();

    var xTL = element.getX();
    var yTL = element.getY();
    var xBR = xTL+tW;
    //pBR.y is calculated on the basis of tH
    var yBR = yTL+0;

    var bOffset = bW*rSize;
    var hlOffset = hW*rSize;
    var offset = bOffset+hlOffset;
    var dWidth2;
    var dHeight2;

    dWidth2 = dHeight2 = Math.ceil(offset);

    var pTL = sV.elementCoordsToCanvasCoords(xTL, yTL);
    var pBR = sV.elementCoordsToCanvasCoords(xBR, yBR);
    pBR.y = pTL.y+objTextSplitParameters.height/rr;

    var w = (Math.round(pBR.x-pTL.x)|0)+2*dWidth2;
    var h = (Math.round(pBR.y-pTL.y)|0)+2*dHeight2;

    var cnvTO = me.cnvTO;
    var ctxTO = me.ctxTO;

    var sText = element.getText();
    var bC = element.getBorderColor();
    var bA = element.getBorderAlpha();

    var bkgC = element.getBackgroundColor();
    var bkgA = element.getBackgroundAlpha();
    var fontFace = element.getFontFace();
    var fontSize = element.getFontSize()*rSize*rr;
    var fontColor = element.getFontColor();
    var fontAlpha = element.getFontAlpha();
    var horizontalAlign = element.getHorizontalAlign();

    var flagRedrawImg =
        (prevDrawParams.w != w) || (prevDrawParams.h != h) ||
        (prevDrawParams.dWidth2 != dWidth2) || (prevDrawParams.dHeight2 != dHeight2) ||
        (prevDrawParams.borderColor != bC) || (prevDrawParams.borderAlpha != bA) ||
        (prevDrawParams.backgroundColor != bkgC) || (prevDrawParams.backgroundAlpha != bkgA) ||
        (prevDrawParams.sText != sText) || (prevDrawParams.fontSize != fontSize) ||
        (prevDrawParams.fontFace != fontFace) || (prevDrawParams.fontColor != fontColor) ||
        (prevDrawParams.fontAlpha != fontAlpha) || (prevDrawParams.horizontalAlign != horizontalAlign) ||
        (prevDrawParams.flagEditingMode != flagEditingMode);

    if(flagRedrawImg) {
        if (cnvTO.width != w || cnvTO.height != h) {
            cnvTO.width = w;
            cnvTO.height = h;
        } else
            ctxTO.clearRect(0, 0, w, h);

        //background
        if(!(isNaN(bkgC) || isNaN(bkgA))){
            ctxTO.beginPath();
            ctxTO.fillStyle = miscFunctions.intAlpha2RGBAFunction(bkgC, bkgA * 0.01);
            ctxTO.fillRect((dWidth2|0)+0.5, (dHeight2|0)+0.5, (pBR.x - pTL.x)|0, (pBR.y - pTL.y)|0);
        }

        //frame for empty or verifiable text
        if(!flagEditingMode || (MOUSE_OR_TOUCH == 1)) {
            //border
            if (!(isNaN(bC) || isNaN(bA) || (bA==0))) {
                ctxTO.beginPath();
                ctxTO.lineWidth = Math.ceil(bOffset);
                ctxTO.strokeStyle = miscFunctions.intAlpha2RGBAFunction(bC, bA * 0.01);
                ctxTO.strokeRect((dWidth2 | 0) + 0.5, (dHeight2 | 0) + 0.5, (pBR.x - pTL.x) | 0, (pBR.y - pTL.y) | 0);
            }else
                if (sText.trim().length == 0 || element.isVerifiable()) {
                    ctxTO.beginPath();
                    ctxTO.lineWidth = 1;
                    ctxTO.strokeStyle = "#000000";
                    ctxTO.strokeRect((dWidth2 | 0) + 0.5, (dHeight2 | 0) + 0.5, (pBR.x - pTL.x) | 0, (pBR.y - pTL.y) | 0);
                }
        }

        //text itself
        ///////////////////////////////////////////////
        //scaling the text drawing place
        /*var r = TextFunctions.prototype.getXFactorByFontSize(fontSize, objTextSplitParameters.bold, objTextSplitParameters.italic);*/
        ctxTO.beginPath();
        ctxTO.setTransform(
            1/rr, 0, 0, 1/rr,
            dWidth2, dHeight2
        );

        var yText = TextFunctions.prototype.getVerticalShiftByFontSize(fontSize);
        var dHeightLine = TextFunctions.prototype.getLineHeightByFontSize(fontSize);

        //font face, style and weight
        ctxTO.font =
            (objTextSplitParameters.italic ? "italic" : "") + " " +
            (objTextSplitParameters.bold ? "bold" : "") + " " + fontSize + "px " + objTextSplitParameters.face;
        //font color
        ctxTO.fillStyle = miscFunctions.intAlpha2RGBAFunction(fontColor, fontAlpha * 0.01);

        //horizontal aligning
        var qAlign;
        switch (horizontalAlign) {
            case "left":
                qAlign = -1;
                break;
            case "center":
                qAlign = 0;
                break;
            case "right":
                qAlign = 1;
                break;
            default:
                qAlign = -1;
        }

        var s, wText, i;
        for (i = 0; i < aText.length; i++) {
            s = aText[i];
            wText = aWidth[i];
            switch (qAlign) {
                case -1:
                    ctxTO.fillText(s, (rr*rSize*TextModelElement.prototype.PADDING_LEFT)|0, yText|0);
                    break;
                case 0:
                    ctxTO.fillText(s, ((rr*rSize*tW - wText) / 2)|0, yText|0);
                    break;
                case 1:
                    ctxTO.fillText(s, (rr*rSize*(tW - TextModelElement.prototype.PADDING_RIGHT) - wText)|0, yText|0);
                    break;
            }
            yText += dHeightLine;
        }

        ctxTO.setTransform(1, 0, 0, 1, 0, 0);

        prevDrawParams.w = w;
        prevDrawParams.h = h;
        prevDrawParams.dWidth2 = dWidth2;
        prevDrawParams.dHeight2 = dHeight2;
        prevDrawParams.borderColor = bC;
        prevDrawParams.borderAlpha = bA;
        prevDrawParams.backgroundColor = bC;
        prevDrawParams.backgroundAlpha = bA;
        prevDrawParams.sText = sText;
        prevDrawParams.fontSize = fontSize;
        prevDrawParams.fontFace = fontFace;
        prevDrawParams.fontColor = fontColor;
        prevDrawParams.fontAlpha = fontAlpha;
        prevDrawParams.horizontalAlign = horizontalAlign;
        prevDrawParams.flagEditingMode = flagEditingMode;
    }

    ctx.drawImage(cnvTO, (pTL.x-dWidth2)|0, (pTL.y-dHeight2)|0);

    //highlight frame
    if(!flagEditingMode && (element.isHighlighted() || element.isBlinking() || element.isSelected())) {
        var alpha = me.getHighlightAlpha()*0.01;
        if ((element.isSelected() || element.isBlinking()) && !element.isHighlighted())
            alpha *= sV.getHighlightAlphaMultiplier(); //0.5*(1.0+Math.cos(me.sceneVisualization.getHighlightBlinkPhase()));
        ctx.beginPath();
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.getHighlightColor(), alpha);
        ctx.lineWidth = hlOffset;
        var offset2 = offset/2;
        //width and height of the hl-frame are "former" canvas width plus double line offset (=line width/2)
        ctx.strokeRect(
            ((pTL.x-offset2)|0)+0.5, ((pTL.y-offset2)|0)+0.5,
            ((pBR.x - pTL.x)+2*offset2)|0, ((pBR.y - pTL.y)+2*offset2)|0
        );

        var frameColor = me.SELECT_COLOR;
        if(element.isHighlighted()){
            var hitDetails = element.getHitDetails();
            if(hitDetails != null && hitDetails.hitCorner)
                frameColor = me.SELECT_COLOR_OVER;
        }

        if(
            element.isSelected() &&
            (
                (!element.isVerifiable() && (element.isColorModifiable() || element.isTextModifiable()))||
                (element.isVerifiable() && (element.isCheckFont() || element.isCheckColor() || element.isCheckSize()))
            )
        ){
            //a thin frame
            ctx.beginPath();
            ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(frameColor, 1.0);
            ctx.lineWidth = element.getSelectWidth() * rSize;
            ctx.strokeRect(pTL.x, pTL.y, pBR.x - pTL.x, pBR.y - pTL.y);
            ctx.beginPath();
            ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(frameColor, 1.0);
            ctx.moveTo(pBR.x-element.FRAME_CORNER_SIZE*rSize, pBR.y);
            ctx.lineTo(pBR.x, pBR.y);
            ctx.lineTo(pBR.x, pBR.y-element.FRAME_CORNER_SIZE*rSize);
            ctx.lineTo(pBR.x-element.FRAME_CORNER_SIZE*rSize, pBR.y);
            ctx.fill();
        }
    }else if(flagEditingMode && MOUSE_OR_TOUCH == 1){
        //red frame for screen keyboard-edited text field
        ctx.beginPath();
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.SELECT_COLOR_OVER, 1.0);
        var lineWidth2 = Math.ceil(1.5*rSize);
        ctx.lineWidth = 2*lineWidth2;
        ctx.strokeRect(((pTL.x-lineWidth2)|0)+0.5, ((pTL.y-lineWidth2)|0)+0.5, (pBR.x - pTL.x+2*lineWidth2)|0, (pBR.y - pTL.y+2*lineWidth2)|0);
    }

    if(me.prevEditingMode != flagEditingMode || flagRedrawImg) {
        if(MOUSE_OR_TOUCH == 1) {
            me.sceneVisualization.getModel().toggleScreenKeyboard({element:element, editingMode:flagEditingMode});
        }else {
            if (flagEditingMode) {
                var dialogDIV = sV.getDialogDIV();
                var inputDIV = sV.getInputDIV();
                var iEStyle;
                var pOrigin = new Point(sV.getLeftPadding() + sV.getOriginX(), sV.getTopPadding() + sV.getOriginY());
                if (me.inputElement == null)
                    me.initInputElement();
                var inputElement = me.inputElement;
                iEStyle = inputElement.style;
                iEStyle.fontFamily = miscFunctions.matchWebSafeFontFamily(objTextSplitParameters.face);
                if (objTextSplitParameters.bold)
                    iEStyle.fontWeight = "bold";
                if (objTextSplitParameters.italic)
                    iEStyle.fontStyle = "italic";
                var backgroundColor = me.INPUT_ELEMENT_BACKGROUND_COLOR;
                var backgroundAlpha = me.INPUT_ELEMENT_BACKGROUND_ALPHA;
                var fontColorHSL = miscFunctions.RGB2HSL(fontColor, null);
                var backgroundColorHSL = miscFunctions.RGB2HSL(backgroundColor, null);
                var delta = miscFunctions.colorSimilarityHSL_CIE94(fontColorHSL, backgroundColorHSL);
                var fontColorCorr;
                if (delta > 0.8) {
                    fontColorHSL.h = (backgroundColorHSL.h + 180) % 360;
                    fontColorHSL.s = 100;
                    fontColorHSL.l = 100 - backgroundColorHSL.l;
                    fontColorCorr = miscFunctions.HSL2RGB(fontColorHSL);
                } else
                    fontColorCorr = fontColor;
                iEStyle.color = miscFunctions.intAlpha2RGBAFunction(fontColorCorr, fontAlpha * 0.01);
                iEStyle.background = miscFunctions.intAlpha2RGBAFunction(backgroundColor, backgroundAlpha);

                var padding = TextFunctions.prototype.getVerticalPaddingByFontSize(fontSize);

                iEStyle.top = (pTL.y + pOrigin.y - 1 + parseInt(dialogDIV.style.top) - parseInt(inputDIV.style.top)) + "px";
                iEStyle.left = (pTL.x + pOrigin.x - 1 + parseInt(dialogDIV.style.left) - parseInt(inputDIV.style.left)) + "px";
                iEStyle.height = (pBR.y - pTL.y - padding) + "px";
                iEStyle.paddingTop = padding + "px";
                iEStyle.paddingLeft = padding + "px";
                //browser ignores fraction part; lines at the bottom of textArea may "lag" behind those in visualization
                var lH = ((pBR.y - pTL.y - padding) / aText.length);
                iEStyle.lineHeight = lH + "px";
                iEStyle.width = (pBR.x - pTL.x - padding) + "px";
                iEStyle.fontSize = (element.getFontSize()*rSize) + "px";

                if (me.inputElement.parentNode == null) {
                    inputDIV.appendChild(inputElement);
                    inputElement.focus();
                }

                inputElement.value = sText.replace(new RegExp("<br />", "g"), "\n");
            } else {
                if (me.inputElement != null && me.inputElement.parentNode != null) {
                    me.inputElement.parentNode.removeChild(me.inputElement);
                    me.inputElement.parentNode = null;
                }
            }
        }
        me.prevEditingMode = flagEditingMode;
    }
};

//
TextModelElementVisualization.prototype.initInputElement = function(){
    var me = this;
    if(me.inputElement != null)
        return;
    var inputElement = document.createElement("TEXTAREA");

    var iEStyle = inputElement.style;
    iEStyle.display = "block";
    iEStyle.overflow = "hidden";
    iEStyle.position = "absolute";//"fixed";
    iEStyle.resize = "none";
    iEStyle.padding = 0+"px";
    iEStyle.border = "1px solid red";
    iEStyle.outline = "none";
    iEStyle.textAlign = me.element.getHorizontalAlign();

    inputElement.addEventListener("input", me.onInput, false);
    inputElement.addEventListener("mousedown", me.propagationStopper, true);
    inputElement.addEventListener("keydown", me.propagationStopper, true);
    inputElement.addEventListener("mousemove", me.propagationStopper, true);
    inputElement.addEventListener("mouseover", me.onMouseOver, true);
    inputElement.addEventListener("mouseout", me.onMouseOut, true);
    inputElement.addEventListener("blur", me.onBlur, true);

    me.inputElement = inputElement;
};
/**
 * @constructor
 */
function BtextModelElementVisualization(_element,_sceneVisualization){
    BtextModelElementVisualization.superclass.constructor.call(this, _element, _sceneVisualization);
    var me = this;
    me.textFieldVisualization = new TextModelElementVisualization(_element.getTextField(), _sceneVisualization);
    me.leaderLineVisualization = new PartLeaderLineVisualization(_element.getLeaderLine(), _sceneVisualization);
}
extend(BtextModelElementVisualization, TextModelElementVisualization);

BtextModelElementVisualization.prototype.getTextFieldVisualization = function(){
    return this.textFieldVisualization;
};

//override
BtextModelElementVisualization.prototype.deactivate = function(){
    BtextModelElementVisualization.superclass.deactivate.call(this);
    this.textFieldVisualization.deactivate();
    this.leaderLineVisualization.deactivate();
};

//override
BtextModelElementVisualization.prototype.update = function() {
    this.textFieldVisualization.update();
    this.leaderLineVisualization.update();
};
/**
 * @constructor
 */
function PartLeaderLineVisualization(_element, _sceneVisualization){
    PartLeaderLineVisualization.superclass.constructor.call(this, _element, _sceneVisualization);
}
extend(PartLeaderLineVisualization, ModelElementVisualization);

PartLeaderLineVisualization.prototype.DEFAULT_LEADERLINE_BACKGROUND_COLOR = 0xFFFFFF;
PartLeaderLineVisualization.prototype.DEFAULT_LEADERLINE_COLOR = 0x000000;

//override
PartLeaderLineVisualization.prototype.update = function() {
    PartLeaderLineVisualization.superclass.update.call(this);
    var me = this;
    var ctx = this.ctx;
    var element = this.element;
    var parentElement  = element.getParentElement();
    var sV = me.sceneVisualization;

    var rSize = sV.getCanvasToSceneSizeRatioZoom();

    var tW = parentElement.getWidthTextField();
    var tH = parentElement.getHeightTextField();

    var xTL = parentElement.getXTextField();
    var yTL = parentElement.getYTextField();
    var xBR = xTL+tW;
    var yBR = yTL+tH;

    var xPointer = parentElement.getXPointer();
    var yPointer = parentElement.getYPointer();
    var hitDetails = element.getHitDetails();

    //which corner of the textField should the leader-line come to?
    var pLLTF = sV.elementCoordsToCanvasCoords((Math.abs(xTL-xPointer) < Math.abs(xBR-xPointer)) ? xTL : xBR, yBR);

    var pBL = sV.elementCoordsToCanvasCoords(xTL, yBR);
    var pBR = sV.elementCoordsToCanvasCoords(xBR, yBR);
    var pPointer = sV.elementCoordsToCanvasCoords(xPointer, yPointer);

    var leaderLineWidth = PartLeaderLine.prototype.DEFAULT_LEADERLINE_WIDTH*rSize;
    var leaderLineBackgroundWidth = PartLeaderLine.prototype.DEFAULT_LEADERLINE_HIT_WIDTH*rSize;
    var pointerDiameter = PartLeaderLine.prototype.DEFAULT_POINTER_DIAMETER*rSize;
    var pointerBackgroundDiameter = PartLeaderLine.prototype.DEFAULT_POINTER_HIT_DIAMETER*rSize;

    var flagSel = element.isSelected();
    var flagHl = element.isHighlighted();
    var flagBlink = element.isBlinking();

    //highlight if selected
    if(flagSel || flagBlink){
        var alpha = me.getHighlightAlpha()*0.01;
        alpha *= sV.getHighlightAlphaMultiplier(); //0.5*(1.0+Math.cos(me.sceneVisualization.getHighlightBlinkPhase()));
        ctx.beginPath();
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.getHighlightColor(), alpha);
        ctx.lineWidth = element.getHighlightWidth()*rSize;
        var rHL = pointerBackgroundDiameter/2 + element.getHighlightWidth()*rSize;
        miscFunctions.drawEllipse(ctx, pPointer.x, pPointer.y, rHL, rHL, false);
    }

    if((flagSel || flagHl) && hitDetails != null && hitDetails.flagLeaderLine){
        //highlight leader line with a thin border
        var leaderLineHighlightWidth = leaderLineBackgroundWidth+2*PartLeaderLine.prototype.DEFAULT_BORDER_WIDTH*rSize;
        ctx.beginPath();
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.SELECT_COLOR, 1.0);
        ctx.lineWidth = leaderLineHighlightWidth;
        ctx.moveTo(pPointer.x, pPointer.y);
        ctx.lineTo(pLLTF.x, pLLTF.y);
        ctx.stroke();
    }

    ctx.beginPath();
    ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_LEADERLINE_BACKGROUND_COLOR, 1.0);
    ctx.lineWidth = leaderLineBackgroundWidth;
    ctx.moveTo(pPointer.x, pPointer.y);
    ctx.lineTo(pLLTF.x, pLLTF.y);
    ctx.stroke();
    //ctx.globalCompositeOperation = "source-over";
    var rB;
    var rP;
    var lW = leaderLineWidth*rSize;
    if(lW<1)
        lW = 1;
    if((flagSel || flagHl) && hitDetails != null && hitDetails.flagPointer){
        ctx.beginPath();
        ctx.lineWidth = lW;
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_LEADERLINE_COLOR, 1.0);
        ctx.moveTo(pPointer.x, pPointer.y);
        ctx.lineTo(pLLTF.x, pLLTF.y);
        ctx.moveTo(pBL.x, pBL.y);
        ctx.lineTo(pBR.x, pBR.y);
        ctx.stroke();
        ctx.beginPath();
        ctx.strokeStyle = "rgba(255, 255, 255, 0)";
        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_LEADERLINE_COLOR, 1.0);
        rB = pointerBackgroundDiameter/2 + PartLeaderLine.prototype.DEFAULT_BORDER_WIDTH*rSize;
        miscFunctions.drawEllipse(ctx, pPointer.x, pPointer.y, rB, rB, true);
        ctx.beginPath();
        ctx.strokeStyle = "rgba(255, 255, 255, 0)";
        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_LEADERLINE_BACKGROUND_COLOR, 1.0);
        rP = pointerBackgroundDiameter/2;
        miscFunctions.drawEllipse(ctx, pPointer.x, pPointer.y, rP, rP, true);
    }else{
        ctx.beginPath();
        ctx.strokeStyle = "rgba(255, 255, 255, 0)";
        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_LEADERLINE_BACKGROUND_COLOR, 1.0);
        rB = pointerBackgroundDiameter/2;
        miscFunctions.drawEllipse(ctx, pPointer.x, pPointer.y, rB, rB, true);
        ctx.beginPath();
        ctx.beginPath();
        ctx.lineWidth = lW;
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_LEADERLINE_COLOR, 1.0);
        ctx.moveTo(pPointer.x, pPointer.y);
        ctx.lineTo(pLLTF.x, pLLTF.y);
        ctx.moveTo(pBL.x, pBL.y);
        ctx.lineTo(pBR.x, pBR.y);
        ctx.stroke();
        ctx.strokeStyle = "rgba(255, 255, 255, 0)";
        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_LEADERLINE_COLOR, 1.0);
        rP = pointerDiameter/2;
        miscFunctions.drawEllipse(ctx, pPointer.x, pPointer.y, rP, rP, true);
    }
};


/**
 * @constructor
 */
function DiagramModelElementVisualization(_element,_sceneVisualization) {
    DiagramModelElementVisualization.superclass.constructor.call(this, _element, _sceneVisualization);
    var me = this;
    me.aSectorVisualization = [];
    me.cnvDO = document.createElement("CANVAS");
    me.ctxDO = me.cnvDO.getContext("2d");
    me.cnvDOSelect = document.createElement("CANVAS");
    me.ctxDOSelect = me.cnvDOSelect.getContext("2d");
    me.cnvHandler = document.createElement("CANVAS");
    me.ctxHandler = me.cnvHandler.getContext("2d");
    me.prevDrawParams = {
        sAngles:null, sColors:null, sAlphas:null, iSelected:-1
    };
}
extend(DiagramModelElementVisualization, ModelElementVisualization);

DiagramModelElementVisualization.prototype.DEFAULT_SECTOR_BORDER_COLOR = 0x3F3F3F;
DiagramModelElementVisualization.prototype.DEFAULT_SECTOR_BORDER_SELECTED_COLOR = 0xFF0000;
DiagramModelElementVisualization.prototype.DEFAULT_SECTOR_BORDER_BACKGROUND_SELECTED_COLOR = 0xFFFFFF;

DiagramModelElementVisualization.prototype.DEFAULT_HANDLER_BORDER_UP_COLOR = 0x000000;
DiagramModelElementVisualization.prototype.DEFAULT_HANDLER_UP_COLOR = 0xFFFFFF;
DiagramModelElementVisualization.prototype.DEFAULT_HANDLER_BORDER_OVER_COLOR = 0x740074;
DiagramModelElementVisualization.prototype.DEFAULT_HANDLER_OVER_COLOR = 0xFFCC00;
DiagramModelElementVisualization.prototype.DEFAULT_HANDLER_BORDER_DOWN_COLOR = 0x740074;
DiagramModelElementVisualization.prototype.DEFAULT_HANDLER_DOWN_COLOR = 0xFF0000;

DiagramModelElementVisualization.prototype.DEFAULT_PERCENTAGE_SECTOR_BACKROUND_COLOR = 0xD7DfC6;
DiagramModelElementVisualization.prototype.DEFAULT_PERCENTAGE_SECTOR_BACKROUND_ALPHA = 0.75;
DiagramModelElementVisualization.prototype.DEFAULT_PERCENTAGE_SECTOR_BORDER_COLOR = 0xFFFFFF;
DiagramModelElementVisualization.prototype.DEFAULT_PERCENTAGE_SECTOR_BORDER_ALPHA = 0.75;
DiagramModelElementVisualization.prototype.DEFAULT_PERCENTAGE_SECTOR_FONT = "Arial";
DiagramModelElementVisualization.prototype.DEFAULT_PERCENTAGE_SECTOR_FONT_COLOR = 0x000000;
DiagramModelElementVisualization.prototype.DEFAULT_PERCENTAGE_SECTOR_FONT_SIZE = 14;

//override
DiagramModelElementVisualization.prototype.update = function(){
    RectangleModelElementVisualization.superclass.update.call(this);
    var me = this;
    var sV = me.sceneVisualization;
    var ctx = this.ctx;
    var cnvDO = this.cnvDO;
    var ctxDO = this.ctxDO;
    var cnvDOSelect = this.cnvDOSelect;
    var ctxDOSelect = this.ctxDOSelect;
    var element = this.element;

    var rSize = sV.getCanvasToSceneSizeRatioZoom();

    var diameter = element.getDiameter();
    var centralDiameter = element.getCentralDiameter();
    var handlerDiameter = element.getHandlerDiameter();
    var highlightWidth = element.getHighlightWidth();
    var selectWidth = element.getSelectWidth();
    var sectorCount = element.getSectorCount();

    var flagNeedRedraw = false;

    var marginWidth = 2*(highlightWidth+selectWidth);
    var canvasWidth = cnvDO.width;
    var canvasWidthRequired = Math.ceil(
        rSize*(diameter + ((marginWidth>handlerDiameter) ? marginWidth : handlerDiameter))
    );

    if(canvasWidthRequired != canvasWidth){
        flagNeedRedraw = true;
        cnvDO.width = cnvDO.height = cnvDOSelect.width = cnvDOSelect.height = canvasWidth = canvasWidthRequired;
    }

    var sAngles = element.getSAngles();
    var sColors = element.getSColors();
    var sAlphas = element.getSAlphas();
    var iSelected = element.getSelectedSectorNumber();

    if(!flagNeedRedraw)
        flagNeedRedraw = (
            me.prevDrawParams.sAngles != sAngles || me.prevDrawParams.sColors != sColors ||
            me.prevDrawParams.sAlphas != sAlphas || me.prevDrawParams.iSelected != iSelected
        );

    var i;

    var pC = sV.elementCoordsToCanvasCoords(element.getX(), element.getY());
    var rDiagram = rSize*diameter/2;
    var startAngle, endAngle, cosStart, sinStart, cosEnd, sinEnd;
    var pCenter = new Point(canvasWidth/2, canvasWidth/2);
    var rCenter = rSize*centralDiameter/2;

    function fDrawSector(_ctx){
        _ctx.moveTo(pCenter.x+rDiagram*cosStart, pCenter.y+rDiagram*sinStart);
        _ctx.arc(pCenter.x, pCenter.y, rDiagram, startAngle, endAngle, false);
        _ctx.lineTo(pCenter.x+rCenter*cosEnd, pCenter.y+rCenter*sinEnd);
        _ctx.arc(pCenter.x, pCenter.y, rCenter, endAngle, startAngle, true);
        _ctx.lineTo(pCenter.x+rDiagram*cosStart, pCenter.y+rDiagram*sinStart);
    }

    var selectedSectorNumber = element.getSelectedSectorNumber();

    if(flagNeedRedraw){
        ctxDO.clearRect(0, 0, canvasWidth, canvasWidth);
        ctxDOSelect.clearRect(0, 0, canvasWidth, canvasWidth);

        for(i=0; i<sectorCount; i++){
            startAngle = -miscFunctions.PI_OVER_2 + element.getSectorStartAngle(i);
            endAngle = -miscFunctions.PI_OVER_2 + element.getSectorEndAngle(i);
            cosStart = Math.cos(startAngle);
            sinStart = Math.sin(startAngle);
            cosEnd = Math.cos(endAngle);
            sinEnd = Math.sin(endAngle);
            ctxDO.beginPath();
            ctxDO.lineJoin = "miter";
            ctxDO.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_SECTOR_BORDER_COLOR, 1.0);

            var cSector = element.getSectorColor(i);

            ctxDO.fillStyle = (cSector == undefined) ?
                "rgba(255, 255, 255, 0.0)" :
                    miscFunctions.intAlpha2RGBAFunction(element.getSectorColor(i), element.getSectorAlpha(i)*0.01);
            ctxDO.lineWidth = element.getSectorBorderThickness()*rSize;
            fDrawSector(ctxDO);
            ctxDO.fill();
            ctxDO.stroke();
            if(i == selectedSectorNumber){
                ctxDOSelect.beginPath();
                ctxDOSelect.lineJoin = "miter";
                ctxDOSelect.strokeStyle = miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_SECTOR_BORDER_BACKGROUND_SELECTED_COLOR, 1.0
                );
                ctxDOSelect.lineWidth = element.getSectorSelectedBorderBackgroundThickness()*rSize;
                fDrawSector(ctxDOSelect);
                ctxDOSelect.stroke();
                ctxDOSelect.beginPath();
                ctxDOSelect.lineJoin = "miter";
                ctxDOSelect.strokeStyle = miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_SECTOR_BORDER_SELECTED_COLOR, 1.0
                );
                ctxDOSelect.lineWidth = element.getSectorSelectedBorderThickness()*rSize;
                fDrawSector(ctxDOSelect);
                ctxDOSelect.stroke();
            }
        }

        me.prevDrawParams.sAngles = sAngles;
        me.prevDrawParams.sColors = sColors;
        me.prevDrawParams.sAlphas = sAlphas;
        me.prevDrawParams.iSelected = iSelected;
    }

    ctx.drawImage(cnvDO, pC.x-canvasWidth/2, pC.y-canvasWidth/2);

    if(element.isHighlighted() || element.isBlinking() || element.isSelected()) {
        var alpha = me.getHighlightAlpha()*0.01;
        if ((element.isSelected() || element.isBlinking()) && !element.isHighlighted())
            alpha *= sV.getHighlightAlphaMultiplier(); //0.5*(1.0+Math.cos(sV.getHighlightBlinkPhase()));
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.getHighlightColor(), alpha);
        ctx.lineWidth = highlightWidth*rSize;
        ctx.beginPath();
        ctx.arc(pC.x, pC.y, rDiagram+highlightWidth*rSize/2, 0, miscFunctions.TWO_PI, true);
        ctx.stroke();
    }

    if(selectedSectorNumber != -1)
        ctx.drawImage(cnvDOSelect, pC.x-canvasWidth/2, pC.y-canvasWidth/2);


    //highlight/common selection/handlers – in cnv
    //sectors/sectors selection – in cnvDO

    var hitDetails = element.getHitDetails();
    var handlerNumber;
    if(hitDetails!=null && hitDetails.handlerNumber != -1)
        handlerNumber = hitDetails.handlerNumber;
    else
        handlerNumber = element.getPressedHandlerNo();

    var handlerCanvasSize = (element.handlerDiameter+2)*rSize;
    var handlerRadius = (element.getHandlerDiameter()-1)*rSize/2;
    var cnvHandler = me.cnvHandler;
    var ctxHandler = me.ctxHandler;
    if(cnvHandler.width != handlerCanvasSize)
        cnvHandler.width = cnvHandler.height = handlerCanvasSize;
    else
        ctxHandler.clearRect(0, 0, handlerCanvasSize, handlerCanvasSize);

    if(element.isAnglesModifiable()) {
        ctxHandler.beginPath();
        ctxHandler.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_HANDLER_BORDER_UP_COLOR, 1.0);
        ctxHandler.lineWidth = element.getSectorBorderThickness() * rSize;
        ctxHandler.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_HANDLER_UP_COLOR, 1.0);
        ctxHandler.arc(handlerCanvasSize / 2, handlerCanvasSize / 2, handlerRadius, 0, miscFunctions.TWO_PI, false);
        ctxHandler.fill();
        ctxHandler.stroke();

        for (i = 1; i < sectorCount; i++) {
            if (i != handlerNumber) {
                startAngle = -miscFunctions.PI_OVER_2 + element.getSectorStartAngle(i);
                cosStart = Math.cos(startAngle);
                sinStart = Math.sin(startAngle);
                ctx.drawImage(
                    cnvHandler,
                    pC.x - handlerCanvasSize / 2 + rDiagram * cosStart, pC.y - handlerCanvasSize / 2 + rDiagram * sinStart
                );
            }
        }

        var pressedHandlerNumber = element.getPressedHandlerNo();
        if (handlerNumber != -1) {
            startAngle = -miscFunctions.PI_OVER_2 + element.getSectorStartAngle(handlerNumber);
            ctx.beginPath();
            if (pressedHandlerNumber == handlerNumber) {
                ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_HANDLER_BORDER_DOWN_COLOR, 1.0);
                ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_HANDLER_DOWN_COLOR, 1.0);
            } else {
                ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_HANDLER_BORDER_OVER_COLOR, 1.0);
                ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_HANDLER_OVER_COLOR, 1.0);
            }
            ctx.lineWidth = element.getSectorBorderThickness() * rSize;
            ctx.arc(
                pC.x + rDiagram * Math.cos(startAngle), pC.y + rDiagram * Math.sin(startAngle),
                handlerRadius + 1, 0, miscFunctions.TWO_PI, false
            );
            ctx.fill();
            ctx.stroke();

            //
            if (pressedHandlerNumber != -1) {
                //sectors with percentages
                var rInner = rDiagram + element.getPercentageSectorInnerRadiusOffset() * rSize;
                var rOuter = rDiagram + element.getPercentageSectorOuterRadiusOffset() * rSize;

                var sector1StartAngle = startAngle - element.getPercentageSectorEndAngleOffset() * miscFunctions.PI_OVER_180;
                var sector1EndAngle = startAngle - element.getPercentageSectorStartAngleOffset() * miscFunctions.PI_OVER_180;
                if (sector1StartAngle > sector1EndAngle)
                    sector1EndAngle += miscFunctions.TWO_PI;
                ctx.lineWidth = 1 * rSize;
                ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_PERCENTAGE_SECTOR_BORDER_COLOR, me.DEFAULT_PERCENTAGE_SECTOR_BORDER_ALPHA
                );
                ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(
                    me.DEFAULT_PERCENTAGE_SECTOR_BACKROUND_COLOR, me.DEFAULT_PERCENTAGE_SECTOR_BACKROUND_ALPHA
                );
                ctx.beginPath();
                ctx.moveTo(pC.x + rInner * Math.cos(sector1StartAngle), pC.y + rInner * Math.sin(sector1StartAngle));
                ctx.arc(pC.x, pC.y, rInner, sector1StartAngle, sector1EndAngle, false);
                ctx.lineTo(pC.x + rOuter * Math.cos(sector1EndAngle), pC.y + rOuter * Math.sin(sector1EndAngle));
                ctx.arc(pC.x, pC.y, rOuter, sector1EndAngle, sector1StartAngle, true);
                ctx.lineTo(pC.x + rInner * Math.cos(sector1StartAngle), pC.y + rInner * Math.sin(sector1StartAngle));
                ctx.fill();
                ctx.stroke();

                var sector2StartAngle = startAngle + element.getPercentageSectorStartAngleOffset() * miscFunctions.PI_OVER_180;
                var sector2EndAngle = startAngle + element.getPercentageSectorEndAngleOffset() * miscFunctions.PI_OVER_180;
                if (sector2StartAngle > sector2EndAngle)
                    sector2EndAngle += miscFunctions.TWO_PI;
                ctx.beginPath();
                ctx.moveTo(pC.x + rInner * Math.cos(sector2StartAngle), pC.y + rInner * Math.sin(sector2StartAngle));
                ctx.arc(pC.x, pC.y, rInner, sector2StartAngle, sector2EndAngle, false);
                ctx.lineTo(pC.x + rOuter * Math.cos(sector2EndAngle), pC.y + rOuter * Math.sin(sector2EndAngle));
                ctx.arc(pC.x, pC.y, rOuter, sector2EndAngle, sector2StartAngle, true);
                ctx.lineTo(pC.x + rInner * Math.cos(sector2StartAngle), pC.y + rInner * Math.sin(sector2StartAngle));
                ctx.fill();
                ctx.stroke();

                var rText = (rInner + rOuter) / 2;
                var angleText = (sector1StartAngle + sector1EndAngle) / 2;
                var paddingText =
                    TextFunctions.prototype.getVerticalPaddingByFontSize(me.DEFAULT_PERCENTAGE_SECTOR_FONT_SIZE);

                ctx.font = Math.floor(me.DEFAULT_PERCENTAGE_SECTOR_FONT_SIZE * rSize) + "px " + me.DEFAULT_PERCENTAGE_SECTOR_FONT;
                ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_PERCENTAGE_SECTOR_FONT_COLOR, 1.0);
                var percentageSectorText = Math.round(element.getSectorWeight(pressedHandlerNumber - 1) * 100 / sectorCount) + "%";
                var percentageSectorTextHeight =
                    TextFunctions.prototype.getLineHeightByFontSize(me.DEFAULT_PERCENTAGE_SECTOR_FONT_SIZE);
                var percentageSectorTextWidth = ctx.measureText(percentageSectorText).width;
                ctx.fillText(
                    percentageSectorText,
                    pC.x + rText * Math.cos(angleText) - percentageSectorTextWidth / 2,
                    pC.y + rText * Math.sin(angleText) + rSize*percentageSectorTextHeight / 2
                );

                angleText = (sector2StartAngle + sector2EndAngle) / 2;
                percentageSectorText = Math.round(element.getSectorWeight(pressedHandlerNumber) * 100 / sectorCount) + "%";
                percentageSectorTextWidth = ctx.measureText(percentageSectorText).width;
                ctx.fillText(
                    percentageSectorText,
                    pC.x + rText * Math.cos(angleText) - percentageSectorTextWidth / 2,
                    pC.y + rText * Math.sin(angleText) + percentageSectorTextHeight / 2 - paddingText
                );

            }
        }
    }

};
/**
 * @constructor
 */
function ArrowModelElementVisualization(_element,_sceneVisualization){
    ArrowModelElementVisualization.superclass.constructor.call(this, _element,_sceneVisualization);
    var me = this;
    me.cnvAO = document.createElement("CANVAS");
    me.ctxAO = me.cnvAO.getContext("2d");
    //a dedicated canvas for selection/highlight outline in case the arrow is dashed
    //because thence in that case two conflicting globalCompositeOperations would be required
    me.cnvSel = document.createElement("CANVAS");
    me.ctxSel = me.cnvSel.getContext("2d");
    //
    me.prevDrawParams = {symbolId:null, xFrom:0, yFrom:0, xTo:0, yTo:0, flagOutline:false, wCanvas:0, hCanvas:0};
}
extend(ArrowModelElementVisualization, ModelElementVisualization);

ArrowModelElementVisualization.prototype.DEFAULT_HANDLER_COLOR = 0xFFFFFF;
ArrowModelElementVisualization.prototype.DEFAULT_HANDLER_ALPHA = 1.0;
ArrowModelElementVisualization.prototype.DEFAULT_HANDLER_BORDER_COLOR = 0x000000;
ArrowModelElementVisualization.prototype.DEFAULT_HANDLER_BORDER_ALPHA = 1.0;

//override
ArrowModelElementVisualization.prototype.update = function() {
    ArrowModelElementVisualization.superclass.update.call(this);
    var me = this;
    var ctx = this.ctx;
    var sV = me.sceneVisualization;
    var element = this.element;

    var prevDrawParams = me.prevDrawParams;

    var cnvAO = me.cnvAO;
    var ctxAO = me.ctxAO;

    var cnvSel = me.cnvSel;
    var ctxSel = me.ctxSel;

    var xFrom = element.getXFrom();
    var yFrom = element.getYFrom();
    var xTo = element.getXTo();
    var yTo = element.getYTo();

    var dX = xTo-xFrom;
    var dY = yTo-yFrom;

    var lArrow = Math.sqrt(dX*dX+dY*dY);
    var cArrow;
    var sArrow;
    if(lArrow<1E-10){
        cArrow = 1;
        sArrow = 0;
    }else{
        cArrow = dY/lArrow;
        sArrow = dX/lArrow;
    }

    var objSize = ArrowModelLibrary.prototype.getSizeById(element.getSymbolId());
    var scaleLength = (typeof objSize.scaleLength == TYPE_NUMBER) ? objSize.scaleLength : 0;
    var scaleFactor = (lArrow>scaleLength) ? 1 : (lArrow/scaleLength);

    //circles are visible if the arrow is movable
    var  flagMovable = element.isMovable();
    var flagModifiable = element.isShapeModifiable();

    var headWidth = objSize.headWidth*scaleFactor;
    var headLength = objSize.headLength*scaleFactor;

    var flagWedgelike = typeof objSize.headWedge == TYPE_NUMBER;
    var tailWedge = (flagWedgelike ? objSize.tailWedge : objSize.tailWidth)*scaleFactor;
    var flagCleaved = typeof objSize.tailCleavage == TYPE_NUMBER;
    var tailCleavage = flagCleaved ? (objSize.tailCleavage*scaleFactor) : 0;

    var circleDiameter = element.getCircleDiameter();
    //circles are to be put directly on cnv over cnvAO

    //determine the arrow size as it were with rSize==1
    //suppose it is vertical pointing downward, origin being xFrom, yFrom
    var wTotal = (headWidth < tailWedge) ? tailWedge : headWidth;
    var hTotal = lArrow+tailCleavage;

    //now calculate points to be joined for the arrow outline
    //in unrotated and unscaled coords

    var aPoint = ArrowModelLibrary.prototype.getArrowOutlineArray(element);

    var hW = element.getHighlightWidth();
    var selectW = element.getSelectWidth();

    //+4 for outlines with angles
    var dSize2 = hW+selectW+4;

    var originShift = new Point(wTotal/2, tailCleavage);
    var originShiftScene = new Point(
        originShift.x*cArrow+originShift.y*sArrow, -originShift.x*sArrow+originShift.y*cArrow
    );
    if(dX>0 && dY>0)
        originShiftScene.y += wTotal*sArrow;
    else if(dX<=0 && dY>0)
        originShiftScene.x += -hTotal*sArrow;
    else if(dX<=0 && dY<=0){
        originShiftScene.x += -hTotal*sArrow-wTotal*cArrow;
        originShiftScene.y += -hTotal*cArrow;
    }else if(dX>0 && dY<=0){
        originShiftScene.x += -wTotal*cArrow;
        originShiftScene.y += -hTotal*cArrow+wTotal*sArrow;
    }

    originShiftScene.x += dSize2;
    originShiftScene.y += dSize2;

    var rSize = sV.getCanvasToSceneSizeRatioZoom();

    var hitDetails = element.getHitDetails();

    var i, n = aPoint.length, p, pCanvas;

    for(i=0; i<n; i++){
        p = aPoint[i];
        aPoint[i] = new Point(((p.x+originShiftScene.x)*rSize)|0, ((p.y+originShiftScene.y)*rSize)|0);
    }

    var wScene = hTotal*Math.abs(sArrow)+wTotal*Math.abs(cArrow);
    var hScene = hTotal*Math.abs(cArrow)+wTotal*Math.abs(sArrow);

    var wCanvas = Math.round((wScene+2*dSize2)*rSize)|0;
    var hCanvas = Math.round((hScene+2*dSize2)*rSize)|0;

    var fillColor = element.getFillColor();
    var fillAlpha = element.getFillAlpha();

    var flagDashed = (typeof objSize.dashPeriod == TYPE_NUMBER);
    var flagOutline = element.isSelected() && (element.isShapeModifiable() || element.isColorModifiable());
    //flagOutline only affects flagRedrawImg if the arrow is not dashed, otherwise the dedicated canvas is used
    var flagRedrawImg =
        (prevDrawParams.symbolId != element.getSymbolId()) ||
        (prevDrawParams.xFrom != xFrom) || (prevDrawParams.yFrom != yFrom) ||
        (prevDrawParams.xTo != xTo) || (prevDrawParams.yTo != yTo) ||
        (prevDrawParams.wCanvas != wCanvas) || (prevDrawParams.hCanvas != hCanvas) ||
        (!flagDashed && (prevDrawParams.flagOutline != flagOutline)) ||
        (prevDrawParams.fillColor != fillColor) || (prevDrawParams.fillAlpha != fillAlpha);

    if(flagRedrawImg) {
        cnvAO.width = wCanvas;
        cnvAO.height = hCanvas;
    }

    pCanvas = sV.elementCoordsToCanvasCoords(xFrom-originShiftScene.x, yFrom-originShiftScene.y);

    if(element.isHighlighted() || element.isBlinking() || element.isSelected()) {
        //the selected arrow outline drawn in case the arrow is selectable or its color can be changed
        var ctxCurr;
        var flagRedrawOutline =
            flagRedrawImg ||
            (flagDashed && (prevDrawParams.flagOutline != flagOutline));

        if(flagRedrawOutline) {
            if (flagDashed) {
                //in case of a dashed arrow use the dedicated canvas
                ctxCurr = ctxSel;
                if(cnvSel.width != wCanvas || cnvSel.height != hCanvas) {
                    cnvSel.width = wCanvas;
                    cnvSel.height = hCanvas;
                }else
                    ctxSel.clearRect(0,0, wCanvas, hCanvas);

            } else {
                //use the arrow's canvas otherwise
                ctxCurr = ctxAO;
            }

            if (flagOutline) {
                ctxCurr.beginPath();
                ctxCurr.lineJoin = "miter";
                ctxCurr.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.SELECT_COLOR, 1.0);
                ctxCurr.lineWidth = 2 * (hW + selectW) * rSize;
                ctxCurr.moveTo(aPoint[0].x, aPoint[0].y);
                for (i = 1; i <= n; i++) {
                    p = aPoint[i % n];
                    ctxCurr.lineTo(p.x, p.y);
                }
                ctxCurr.stroke();
                //
                ctxCurr.globalCompositeOperation = "destination-out";
                ctxCurr.beginPath();
                ctxCurr.lineJoin = "miter";
                ctxCurr.fillStyle = miscFunctions.intAlpha2RGBAFunction(0xFFFFFF, 1.0);
                ctxCurr.strokeStyle = miscFunctions.intAlpha2RGBAFunction(0xFFFFFF, 1.0);
                ctxCurr.lineWidth = 2 * hW * rSize;
                ctxCurr.moveTo(aPoint[0].x, aPoint[0].y);
                for (i = 1; i <= n; i++) {
                    p = aPoint[i % n];
                    ctxCurr.lineTo(p.x, p.y);
                }
                ctxCurr.stroke();
                ctxCurr.fill();
                ctxCurr.globalCompositeOperation = "source-over";
            }
        }

        if(flagOutline && flagDashed)
            ctx.drawImage(cnvSel, pCanvas.x, pCanvas.y);

        //highlight is drawn in the common ctx
        var alpha = me.getHighlightAlpha() * 0.01;
        if ((element.isSelected() || element.isBlinking()) && !element.isHighlighted())
            alpha *= sV.getHighlightAlphaMultiplier(); //0.5 * (1.0 + Math.cos(sV.getHighlightBlinkPhase()));
        ctx.beginPath();
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.getHighlightColor(), alpha);
        ctx.lineWidth = 2*hW*rSize;
        ctx.moveTo(aPoint[0].x+pCanvas.x, aPoint[0].y+pCanvas.y);
        for(i=1; i<=n; i++){
            p = aPoint[i%n];
            ctx.lineTo(p.x+pCanvas.x, p.y+pCanvas.y);
        }
        ctx.stroke();
    }

    prevDrawParams.flagOutline = flagOutline;

    //is the arrow dashed?
    if(flagDashed && flagRedrawImg){
        //for dashed arrows, stripes must be "drawn" first, so that further drawing
        //with globalCompositeOperation=="source-out"
        var dashPeriod = objSize.dashPeriod;
        var dashLength = objSize.dashLength;
        var dashFrom = dashLength, dashTo, lDashedPart = lArrow-headLength;
        var pDashTL, pDashBL, pDashBR, pDashTR, wTotal2 = wTotal/2;

        ctxAO.fillStyle = "#FFFFFF";
        while(dashFrom<=lDashedPart){
            dashTo = dashFrom+(dashPeriod-dashLength);
            if(dashTo>lDashedPart)
                dashTo = lDashedPart;
            pDashTL = new Point(
                (-wTotal2*cArrow+dashFrom*sArrow+originShiftScene.x)*rSize,  (wTotal2*sArrow+ dashFrom*cArrow+originShiftScene.y)*rSize
            );
            pDashBL = new Point(
                (-wTotal2*cArrow+dashTo*sArrow+originShiftScene.x)*rSize,  (wTotal2*sArrow+ dashTo*cArrow+originShiftScene.y)*rSize
            );
            pDashBR = new Point(
                (wTotal2*cArrow+dashTo*sArrow+originShiftScene.x)*rSize,  (-wTotal2*sArrow+ dashTo*cArrow+originShiftScene.y)*rSize
            );
            pDashTR = new Point(
                (wTotal2*cArrow+dashFrom*sArrow+originShiftScene.x)*rSize,  (-wTotal2*sArrow+ dashFrom*cArrow+originShiftScene.y)*rSize
            );
            dashFrom += dashPeriod;
            ctxAO.beginPath();
            ctxAO.moveTo(pDashTL.x, pDashTL.y);
            ctxAO.lineTo(pDashBL.x, pDashBL.y);
            ctxAO.lineTo(pDashBR.x, pDashBR.y);
            ctxAO.lineTo(pDashTR.x, pDashTR.y);
            ctxAO.lineTo(pDashTL.x, pDashTL.y);
            ctxAO.fill();
        }
        ctxAO.globalCompositeOperation = "source-out";
    }

    if(flagRedrawImg) {
        //the arrow itself
        ctxAO.beginPath();
        ctxAO.fillStyle = miscFunctions.intAlpha2RGBAFunction(fillColor, fillAlpha * 0.01);
        ctxAO.moveTo(aPoint[0].x, aPoint[0].y);
        for (i = 1; i <= n; i++) {
            p = aPoint[i % n];
            ctxAO.lineTo(p.x, p.y);
        }
        ctxAO.fill();
        ctxAO.globalCompositeOperation = "source-over";

        prevDrawParams.symbolId = element.getSymbolId();
        prevDrawParams.xFrom = xFrom;
        prevDrawParams.yFrom = yFrom;
        prevDrawParams.xTo = xTo;
        prevDrawParams.yTo = yTo;
        prevDrawParams.wCanvas = wCanvas;
        prevDrawParams.hCanvas = hCanvas;
        prevDrawParams.fillColor = fillColor;
        prevDrawParams.fillAlpha = fillAlpha;
    }

    ctx.drawImage(cnvAO, pCanvas.x, pCanvas.y);

    if(flagMovable){
        //start and end handlers
        var hR, hBW;
        //start
        var pFromCanvas = sV.elementCoordsToCanvasCoords(xFrom, yFrom);
        ctx.beginPath();
        if(hitDetails != null && typeof hitDetails.hitStart == TYPE_BOOLEAN && hitDetails.hitStart) {
            hR = element.HANDLER_OVER_DIAMETER*rSize/2;
            hBW = element.HANDLER_OVER_BORDER_WIDTH*rSize;
        }else {
            hR = element.HANDLER_DIAMETER * rSize / 2;
            hBW = element.HANDLER_BORDER_WIDTH * rSize;
        }
        ctx.lineWidth = hBW;
        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_HANDLER_COLOR, me.DEFAULT_HANDLER_ALPHA);
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(
            me.DEFAULT_HANDLER_BORDER_COLOR, me.DEFAULT_HANDLER_BORDER_ALPHA
        );
        ctx.arc(pFromCanvas.x, pFromCanvas.y, hR, 0, 2*Math.PI, true);
        ctx.fill();
        ctx.stroke();
        //end
        var pToCanvas = sV.elementCoordsToCanvasCoords(xTo, yTo);
        ctx.beginPath();
        if(hitDetails != null && typeof hitDetails.hitEnd == TYPE_BOOLEAN && hitDetails.hitEnd) {
            hR = element.HANDLER_OVER_DIAMETER*rSize/2;
            hBW = element.HANDLER_OVER_BORDER_WIDTH*rSize;
        }else {
            hR = element.HANDLER_DIAMETER * rSize / 2;
            hBW = element.HANDLER_BORDER_WIDTH * rSize;
        }
        ctx.lineWidth = hBW;
        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.DEFAULT_HANDLER_COLOR, me.DEFAULT_HANDLER_ALPHA);
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(
            me.DEFAULT_HANDLER_BORDER_COLOR, me.DEFAULT_HANDLER_BORDER_ALPHA
        );
        ctx.arc(pToCanvas.x, pToCanvas.y, hR, 0, 2*Math.PI, true);
        ctx.fill();
        ctx.stroke();
    }
};

/**
 * @constructor
 */
function CheckboxSetModelElementVisualization(_element,_sceneVisualization){
    CheckboxSetModelElementVisualization.superclass.constructor.call(this, _element, _sceneVisualization);
    var me = this;
    //buttons and captions are drawn separately
    //the canvas with buttons only gets repainted if there were changes
    me.cnvButtons = document.createElement("CANVAS");
    me.ctxButtons = me.cnvButtons.getContext("2d");
    me.cnvCaptions = document.createElement("CANVAS");
    me.ctxCaptions = me.cnvCaptions.getContext("2d");
    me.aCaptionVisualization  = [];
    var captionVisualization;
    var i, n = me.element.getCheckboxCount();
    for(i=0; i<n; i++){
        captionVisualization =
            new TextModelElementVisualization(me.element.getCheckboxByOrderNo(i).getCaption(), me.sceneVisualization);
        captionVisualization.setCanvasContext(me.ctxCaptions);
        me.aCaptionVisualization.push(captionVisualization);
    }
    me.prevDrawParams = {
        sChecked:"",
        flagRadiobuttons:me.element.isRadiobuttons()
    };
}
extend(CheckboxSetModelElementVisualization, ModelElementVisualization);

//override
CheckboxSetModelElementVisualization.prototype.deactivate = function() {
    CheckboxSetModelElementVisualization.superclass.deactivate.call(this);
    var me = this;
};

//override
CheckboxSetModelElementVisualization.prototype.update = function() {
    CheckboxSetModelElementVisualization.superclass.update.call(this);
    var me = this;
    var ctx = this.ctx;
    var element = this.element;
    var sV = me.sceneVisualization;

    var cnvButtons = me.cnvButtons;
    var ctxButtons = me.ctxButtons;
    var cnvCaptions = me.cnvCaptions;
    var ctxCaptions = me.ctxCaptions;

    var prevDrawParams = me.prevDrawParams;

    var rSize = sV.getCanvasToSceneSizeRatioZoom();

    var flagSizeChanged = false;
    var flagRadiobuttons = element.isRadiobuttons();
    var sChecked = element.getSChecked();

    var xTL = element.getX();
    var yTL = element.getY();
    var pTL = sV.elementCoordsToCanvasCoords(xTL, yTL);

    var checkbox, i, n = element.getCheckboxCount();
    //calculate necessary canvas size
    var captionShift = PartCheckbox.prototype.captionXShift*rSize;
    var w, wMax = 0, h = 0;
    for(i=0; i<n; i++){
        checkbox = element.getCheckboxByOrderNo(i);
        w = checkbox.getWidth();
        if(w>wMax)
            wMax = w;
        h += checkbox.getHeight();
    }
    w = wMax;
    h += (n-1)*element.verticalGap + element.paddingTop + element.paddingBottom;
    w += element.paddingLeft + element.paddingRight;
    h *= rSize;
    w *= rSize;
    w -= captionShift;
    h = Math.ceil(h);
    w = Math.ceil(w);
    if(w != cnvCaptions.width || h != cnvCaptions.height) {
        flagSizeChanged = true;
        cnvCaptions.width = w;
        cnvCaptions.height = h;
        cnvButtons.width = captionShift;
        cnvButtons.height = h;
    }
    ctxCaptions.clearRect(0, 0, cnvCaptions.width, cnvCaptions.height);

    var hW = element.getHighlightWidth();
    if(element.isHighlighted() || element.isBlinking() || element.isSelected()) {
        var alpha = me.getHighlightAlpha()*0.01;
        if ((element.isSelected() || element.isBlinking()) && !element.isHighlighted())
            alpha *= sV.getHighlightAlphaMultiplier(); //0.5*(1.0+Math.cos(me.sceneVisualization.getHighlightBlinkPhase()));

        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.getHighlightColor(), alpha);
        ctx.lineWidth = hW*rSize;
        var offset = ctx.lineWidth, offset2 = offset/2;
        ctx.strokeRect((pTL.x - offset2)|0, (pTL.y - offset2)|0, (w + offset)|0, (h + offset)|0);
    }

    var fontSize, lineHeight;
    ctxCaptions.setTransform(1, 0, 0, 1, (-captionShift)|0, 0);
    for(i=0; i<n; i++)
        me.aCaptionVisualization[i].update();
    ctxCaptions.setTransform(1, 0, 0, 1, 0, 0);
    ctx.drawImage(cnvCaptions, (pTL.x+captionShift)|0, pTL.y|0);

    if(flagSizeChanged || prevDrawParams.flagRadiobuttons != flagRadiobuttons || prevDrawParams.sChecked != sChecked) {
        ctxButtons.clearRect(0, 0, cnvButtons.width, cnvButtons.height);
        for (i = 0; i < n; i++) {
            checkbox = element.getCheckboxByOrderNo(i);
            fontSize = checkbox.getCaption().getFontSize();
            lineHeight =
                TextFunctions.prototype.getLineHeightByFontSize(fontSize) +
                2 * TextFunctions.prototype.getVerticalPaddingByFontSize(fontSize);
            var dButton = checkbox.getButtonSize();

            var flagChecked = checkbox.isChecked();
            var imgId;
            if (flagRadiobuttons) {
                imgId = flagChecked ? "radiobutton.selected" : "radiobutton.unselected";
            } else {
                imgId = flagChecked ? "checkbox.selected" : "checkbox.unselected";
            }
            var objSize = CheckboxSetModelLibrary.prototype.getSizeById(imgId);
            var img = CheckboxSetModelLibrary.prototype.getCachedSVG(imgId);
            var pButton = new Point(checkbox.getX() + dButton / 2 + objSize.xShift, checkbox.getY() + lineHeight / 2 + objSize.yShift);
            ctxButtons.setTransform(rSize, 0, 0, rSize, 0, 0);
            ctxButtons.drawImage(img, pButton.x | 0, pButton.y | 0);
            ctxButtons.setTransform(1.0, 0, 0, 1.0, 0, 0);
        }
    }
    ctx.drawImage(cnvButtons, pTL.x | 0, pTL.y | 0);

    prevDrawParams.flagRadiobuttons = flagRadiobuttons;
    prevDrawParams.sChecked = sChecked;
};

/**
 * @constructor
 */
function TextModelElementController(element, visualization, mapSceneController){
    TextModelElementController.superclass.constructor.call(this, element, visualization, mapSceneController);
    var me = this;
    me.ownToolId = ModelTool.prototype.TOOL_TEXT;
    me.flagDownOnCorner = false;
    me.sizeChangeStartPoint = null;
    me.sizeChangeStartWidth = 0;

    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_DOWN, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_UP, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_DRAG, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.DOUBLE_CLICK, {listener:me.mouseOnElementListener, target:me}, false);
    //
    if(me.visualization != null) {
        me.visualization.addEventListener(
            ModelEvent.prototype.INPUT, {listener: me.inputListener, target: me}, false);
        me.visualization.addEventListener(
            ModelEvent.prototype.OVER_SELECTABLE_TEXT, {listener: me.overInputListener, target: me}, false);
        me.visualization.addEventListener(
            ModelEvent.prototype.OUT_SELECTABLE_TEXT, {listener: me.outInputListener, target: me}, false);
        me.visualization.addEventListener(
            ModelEvent.prototype.BLUR, {listener: me.blurInputListener, target: me}, false);
    }
}
extend(TextModelElementController, ModelElementController);

TextModelElementController.prototype.mouseOnElementListener = function(e){
    var me = this;
    var element = me.element;
    var visualization = me.visualization;
    switch(e.type){
        case ModelEvent.prototype.MOUSE_DOWN:
            if(
                typeof e.data.currentToolId == TYPE_UNDEFINED ||
                (e.data.currentToolId != ModelTool.prototype.TOOL_SELECT && e.data.currentToolId != me.ownToolId)
            )
                break;
            var flagOnElement = e.data.id != null && e.data.id == element.getId();
            me.flagDownOnCorner = flagOnElement && element.getHitDetails().hitCorner && !element.isVerifiable();
            if(me.flagDownOnCorner){
                me.sizeChangeStartPoint = new Point(e.data.x, e.data.y);
                me.sizeChangeStartWidth = element.getWidth();
            }
            if((!flagOnElement || me.flagDownOnCorner) && element.isEditingMode()) {
                element.setEditingMode(false);
            }else if(
                flagOnElement && !me.flagDownOnCorner && !element.isEditingMode() &&
                (
                    element.isVerifiable() || (element.isTextModifiable() && !element.isMovable())
                )
            ){
                //for editor-created verifiable text fields, a single click invokes edit mode
                element.setSelected(true);
                element.setHitPoint(new Point(e.data.x, e.data.y));
                element.setEditingMode(true);
                e.stopPropagation();
            }

            break;

        case ModelEvent.prototype.MOUSE_UP:
            me.flagDownOnCorner = false;
            break;

        case ModelEvent.prototype.MOUSE_DRAG:
            if(me.flagDownOnCorner && element.isTextModifiable()){
                var p = new Point(e.data.x, e.data.y);
                var w = p.x-me.sizeChangeStartPoint.x+me.sizeChangeStartWidth;
                var wMin = 2*TextModelElement.prototype.DEFAULT_BORDER_WIDTH+
                    TextModelElement.prototype.FRAME_CORNER_SIZE;
                if(w<wMin)
                    w = wMin;
                if(element.getWidth() != w)
                    element.setWidth(w);
            }
            break;

        case ModelEvent.prototype.DOUBLE_CLICK:
            if(e.data.id == element.getId() && element.isTextModifiable() && element.isMovable() && !element.isEditingMode()
            ){
                //editing mode is invoked by double click for user-created text fields only
                element.setHitPoint(new Point(e.data.x, e.data.y));
                element.setEditingMode(true);
                e.stopPropagation();
            }
            break;
    }
};

//
TextModelElementController.prototype.inputListener = function(e){
    var me = this;
    me.element.setText(e.data.value.replace(new RegExp("\n", "g"),"<br />"));
    me.visualization.update();
};

//
TextModelElementController.prototype.overInputListener = function(e){
    this.mapSceneController.notify(ModelEvent.prototype.OVER_SELECTABLE_TEXT, null);
};

//
TextModelElementController.prototype.outInputListener = function(e){
    this.mapSceneController.notify(ModelEvent.prototype.OUT_SELECTABLE_TEXT, null);
};

//
TextModelElementController.prototype.blurInputListener = function(e){
    this.element.setEditingMode(false);
    this.mapSceneController.notify(ModelEvent.prototype.BLUR, null);
};

//override
TextModelElementController.prototype.deactivate = function(){
    TextModelElementController.superclass.deactivate.call(this);
    var me = this;
    var mapSceneController = me.mapSceneController;
    mapSceneController.removeEventListener(
        ModelEvent.prototype.MOUSE_DOWN, {listener:me.mouseOnElementListener, target:me}, false);
    mapSceneController.removeEventListener(
        ModelEvent.prototype.MOUSE_UP, {listener:me.mouseOnElementListener, target:me}, false);
    mapSceneController.removeEventListener(
        ModelEvent.prototype.MOUSE_DRAG, {listener:me.mouseOnElementListener, target:me}, false);
    mapSceneController.removeEventListener(
        ModelEvent.prototype.DOUBLE_CLICK, {listener:me.mouseOnElementListener, target:me}, false);
    if(me.visualization != null) {
        me.visualization.removeEventListener(
            ModelEvent.prototype.INPUT, {listener: me.inputListener, target: me}, false);
        me.visualization.removeEventListener(
            ModelEvent.prototype.OVER_SELECTABLE_TEXT, {listener: me.overInputListener, target: me}, false);
        me.visualization.removeEventListener(
            ModelEvent.prototype.OUT_SELECTABLE_TEXT, {listener: me.outInputListener, target: me}, false);
        me.visualization.removeEventListener(
            ModelEvent.prototype.BLUR, {listener: me.blurInputListener, target: me}, false);
    }
};
/**
 * @constructor
 */
function BtextModelElementController(element, visualization, mapSceneController){
    BtextModelElementController.superclass.constructor.call(this, element, visualization, mapSceneController);
    var me = this;
    me.ownToolId = ModelTool.prototype.TOOL_BOUND_TEXT;
    me.textFieldController = new TextModelElementController(
        element.getTextField(), visualization.getTextFieldVisualization(), mapSceneController
    );
    me.flagDownOnPointer = false;
    me.flagDownOnText = false;
    me.mouseDownPointShift = null;
}
extend(BtextModelElementController, ModelElementController);

//override
BtextModelElementController.prototype.mouseOnElementListener = function(e) {
    var me = this;
    if(me.isToolStarted())
        return;
    var element = me.element;
    var elementTF = element.getTextField();
    var hitDetails = element.getHitDetails();
    var hitDetailsTF = elementTF.getHitDetails();
    switch (e.type) {
        case ModelEvent.prototype.MOUSE_DOWN:
            var flagOnElement = e.data.id != null && e.data.id == element.getId();
            me.flagDownOnPointer = element.isMovable() && flagOnElement && (hitDetails != null && hitDetails.flagPointer);
            me.flagDownOnText = flagOnElement &&
                (
                hitDetailsTF != null && hitDetailsTF.flagInside && !hitDetailsTF.hitCorner
                );
            if(flagOnElement) {
                if (me.flagDownOnPointer) {
                    if (me.isSelectToolOn())
                        me.mouseDownPointShift = new Point(
                            element.getXPointer() - e.data.x, element.getYPointer() - e.data.y
                        );
                    else
                        me.flagDownOnPointer = false;
                } else if (me.flagDownOnText) {
                    if (me.isSelectToolOn())
                        me.mouseDownPointShift = new Point(elementTF.getX() - e.data.x, elementTF.getY() - e.data.y);
                    else
                        me.flagDownOnText = false;
                }
            }
            break;

        case ModelEvent.prototype.MOUSE_UP:
            me.mouseDownPointShift = null;
            break;

        case ModelEvent.prototype.MOUSE_MOVE:
            break;

        case ModelEvent.prototype.MOUSE_DRAG:
            if(me.flagDownOnPointer){
                element.setXPointer(e.data.x + me.mouseDownPointShift.x);
                element.setYPointer(e.data.y + me.mouseDownPointShift.y);
            }else if(me.flagDownOnText){
                var aEl = me.mapSceneController.model.getOtherElementsInGroup(element);
                if((aEl.length === 0) || ModelElement.prototype.checkIfAllAre(aEl, "isMovableByMouse")) {
                    elementTF.setX(e.data.x + me.mouseDownPointShift.x);
                    elementTF.setY(e.data.y + me.mouseDownPointShift.y);
                }
            }
            break;
    }
};

//override
BtextModelElementController.prototype.deactivate = function() {
    BtextModelElementController.superclass.deactivate.call(this);
    var me = this;
    me.textFieldController.deactivate();
};
/**
 * @constructor
 */
function DiagramModelElementController(element, visualization, mapSceneController) {
    DiagramModelElementController.superclass.constructor.call(this, element, visualization, mapSceneController);
    var me = this;
    me.ownToolId = ModelTool.prototype.TOOL_DIAGRAM;
    me.flagDownOnHandler = false;
    me.flagDownOnSector = false;
    me.currentCenter = null;
    me.alphaStart = 0;
    me.alphaOffset = 0;
    me.alphaPrev = 0;
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_DOWN, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_UP, {listener:me.mouseOnElementListener, target:me}, false);
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_DRAG, {listener:me.mouseOnElementListener, target:me}, false);
//

}
extend(DiagramModelElementController, ModelElementController);

DiagramModelElementController.prototype.mouseOnElementListener = function(e) {
    var me = this;
    var element = me.element;
    var flagStructureEditable = element.isStructureEditable();
    var flagHandlersMovable = element.isHandlersMovable();
    var i, n, dx, dy;
    switch (e.type) {
        case ModelEvent.prototype.MOUSE_DOWN:
            var flagOnElement = e.data.id != null && e.data.id == element.getId();
            me.flagDownOnHandler = flagHandlersMovable && flagOnElement && (element.getHitDetails().handlerNumber != -1);

            if(me.flagDownOnHandler){
                //change start
                var dAlpha = Math.atan(element.getHandlerDiameter()/element.getDiameter());
                element.setPressedHandlerNo(element.getHitDetails().handlerNumber);
                me.currentCenter = new Point(element.getX(), element.getY());
                //we need the angle range from -PI to +PI
                var angleStart = Math.atan2(me.currentCenter.x-e.data.x, e.data.y-me.currentCenter.y);
                me.alphaStart = me.alphaPrev = element.getSectorStartAngle(element.getHitDetails().handlerNumber);
                //not to get stuck at zero
                if(me.alphaStart<=dAlpha && angleStart>-Math.PI+2*dAlpha)
                    angleStart -= miscFunctions.TWO_PI;
                else if(me.alphaStart>=miscFunctions.TWO_PI-dAlpha && angleStart<Math.PI-2*dAlpha)
                    angleStart += miscFunctions.TWO_PI;
                me.alphaOffset = me.alphaStart-angleStart;
            }else{
                if(typeof e.data.currentToolId == TYPE_UNDEFINED || e.data.currentToolId != ModelTool.prototype.TOOL_SELECT)
                    break;
                me.flagDownOnSector = flagOnElement && (element.getHitDetails().sectorNumber != -1);

                if(me.flagDownOnSector && element.isColorModifiable())
                    element.setSelectedSectorNumber(element.getHitDetails().sectorNumber);
            }

            break;

        case ModelEvent.prototype.MOUSE_UP:
            me.flagDownOnHandler = false;
            me.flagDownOnSector = false;
            element.setPressedHandlerNo(-1);
            element.invalidate();
            break;

        case ModelEvent.prototype.MOUSE_DRAG:
            if(!me.flagDownOnHandler)
                break;
            try {
                dx = me.currentCenter.x - e.data.x;
                dy = e.data.y-me.currentCenter.y;
            }catch(err){
            }
            var alpha = Math.atan2(dx, dy)+me.alphaOffset;

            //not to skip over zero
            if(alpha<0)
                alpha = 0;
            else if (alpha>miscFunctions.TWO_PI)
                alpha = miscFunctions.TWO_PI;
            if((alpha-me.alphaPrev)>Math.PI && alpha>miscFunctions.THREE_PI_OVER_2 && me.alphaPrev<miscFunctions.PI_OVER_2)
                alpha = 0;
            else if((me.alphaPrev-alpha)>Math.PI && alpha<miscFunctions.PI_OVER_2 && me.alphaPrev>miscFunctions.THREE_PI_OVER_2)
                alpha = miscFunctions.TWO_PI;

            var no = element.getPressedHandlerNo();
            var angleStep = element.getAngleStep()*Math.PI/180;
            alpha = Math.floor(alpha/angleStep + 0.5)*angleStep;
            var alphaPrev = element.getSectorStartAngle(no);
            if(alpha != alphaPrev) {
                n = element.getSectorCount();
                for (i = 1; i < no; i++)
                    if (element.getSectorStartAngle(i) > alpha)
                        element.setSectorStartAngle(i, alpha);
                element.setSectorStartAngle(no, alpha);
                for (i = no+1; i < n; i++)
                    if (element.getSectorStartAngle(i) < alpha)
                        element.setSectorStartAngle(i, alpha);

                for (i = 1; i < no-1; i++)
                    if (element.getSectorEndAngle(i) > alpha)
                        element.setSectorEndAngle(i, alpha);
                element.setSectorEndAngle(no-1, alpha);
                for (i = no; i < n; i++)
                    if (element.getSectorEndAngle(i) < alpha)
                        element.setSectorEndAngle(i, alpha);
                element.invalidate();
            }
            me.alphaPrev = alpha;
            break;
    }
};

//
DiagramModelElementController.prototype.fSetToolListener = function(e){
    DiagramModelElementController.superclass.fSetToolListener.call(this, e);
    var element = this.element;
    element.setHandlersMovable(
        (e.data.id == ModelTool.prototype.TOOL_SELECT || e.data.id == this.ownToolId) &&
        (element.isUserCreated() || element.isAnglesModifiable())
    );
};
/**
 * @constructor
 */
function PolylineModelElementController(element, visualization, mapSceneController){
    DiagramModelElementController.superclass.constructor.call(this, element, visualization, mapSceneController);
    var me = this;
    me.ownToolId = ModelTool.prototype.TOOL_POLYLINE;
    me.mouseDownPointShift = null;
    me.flagDownOnHandler = false;
}
extend(PolylineModelElementController, ModelElementController);

//override
PolylineModelElementController.prototype.mouseOnElementListener = function(e) {
    var me = this;
    if(me.isToolStarted())
        return;
    var element = me.element;
    var flagStructureEditable = element.isStructureEditable();
    var flagVerticesMovable = element.isVerticesMovable();
    var no;
    var ctrlKey = e.data.ctrlKey;
    var x = element.getX(), y = element.getY();
    var aPoint  = element.getPoints();
    var hitDetails = element.getHitDetails();
    switch (e.type) {
        case ModelEvent.prototype.MOUSE_DOWN:
            var flagOnElement = e.data.id != null && e.data.id == element.getId();
            me.flagDownOnHandler = flagOnElement && (hitDetails.handlerNumber != -1);
            if(flagOnElement) {
                if (me.flagDownOnHandler) {
                    no = hitDetails.handlerNumber;
                    if (flagStructureEditable && me.isOwnToolOn()) {
                        if (ctrlKey) {
                            //remove vertex
                            element.removePoint(no, true);
                            element.setHitDetails(null);
                            me.flagDownOnHandler = false;
                        }else if(flagVerticesMovable){
                            //change start
                            me.mouseDownPointShift = new Point(x + aPoint[no].x - e.data.x, y + aPoint[no].y - e.data.y);
                            element.setPressedHandlerNo(no);
                        }
                    }else if (flagVerticesMovable && me.isSelectToolOn()) {
                        //change start
                        me.mouseDownPointShift = new Point(x + aPoint[no].x - e.data.x, y + aPoint[no].y - e.data.y);
                        element.setPressedHandlerNo(no);
                    }else
                        me.flagDownOnHandler = false;
                } else if (flagStructureEditable && hitDetails.segmentNumber != -1) {
                    if (flagStructureEditable && me.isOwnToolOn()) {
                        if(ctrlKey){
                            element.removeSegment(hitDetails.segmentNumber);
                            element.setHitDetails(null);
                        }else {
                            element.addPoint(
                                hitDetails.pointOnSegment.subtract(new Point(x, y)),
                                hitDetails.segmentNumber+1
                            );
                            element.setHitDetails(null);
                        }
                    }
                }
            }
            break;

        case ModelEvent.prototype.MOUSE_UP:
            me.mouseDownPointShift = null;
            element.setPressedHandlerNo(-1);
            if(MOUSE_OR_TOUCH>0) {
                //there's no mouseOver for touch-screen
                element.setHitDetails(null);
            }
            break;

        case ModelEvent.prototype.MOUSE_MOVE:
            break;

        case ModelEvent.prototype.MOUSE_DRAG:
            if(!me.flagDownOnHandler)
                return;
            no = element.getPressedHandlerNo();
            aPoint[no].x = e.data.x + me.mouseDownPointShift.x - x;
            aPoint[no].y = e.data.y + me.mouseDownPointShift.y - y;
            element.invalidate();
            break;

    }
};

//
PolylineModelElementController.prototype.fSetToolListener = function(e){
    PolylineModelElementController.superclass.fSetToolListener.call(this, e);
    var element = this.element;
    element.setVerticesMovable(
        (e.data.id == ModelTool.prototype.TOOL_SELECT || e.data.id == this.ownToolId) &&
        (element.isUserCreated() || element.isShapeModifiable())
    );
};

/**
 * @constructor
 */
function ArrowModelElementController(element, visualization, mapSceneController){
    ArrowModelElementController.superclass.constructor.call(this, element, visualization, mapSceneController);
    var me = this;
    me.ownToolId = ModelTool.prototype.TOOL_ARROW;
    me.mouseDownPointShift = null;
    me.flagDownOnStartHandler = false;
    me.flagDownOnEndHandler = false;
}
extend(ArrowModelElementController, ModelElementController);

ArrowModelElementController.prototype.mouseOnElementListener = function(e) {
    var me = this;
    if (me.isToolStarted())
        return;
    var element = me.element;
    var xFrom = element.getXFrom();
    var yFrom = element.getYFrom();
    var xTo = element.getXTo();
    var yTo = element.getYTo();
    var hitDetails = element.getHitDetails();
    switch (e.type) {
        case ModelEvent.prototype.MOUSE_DOWN:
            var flagOnElement = e.data.id != null && e.data.id == element.getId();
            me.flagDownOnStartHandler = flagOnElement && hitDetails.hitStart;
            me.flagDownOnEndHandler = flagOnElement && hitDetails.hitEnd;
            if(flagOnElement && me.isSelectToolOn()) {
                if(me.flagDownOnStartHandler)
                    me.mouseDownPointShift = new Point(xFrom - e.data.x, yFrom - e.data.y);
                else if(me.flagDownOnEndHandler)
                    me.mouseDownPointShift = new Point(xTo - e.data.x, yTo - e.data.y);
            }else{
                me.flagDownOnStartHandler = false;
                me.flagDownOnEndHandler = false;
            }
            break;
        case ModelEvent.prototype.MOUSE_UP:
            me.mouseDownPointShift = null;
            me.flagDownOnStartHandler = false;
            me.flagDownOnEndHandler = false;
            break;
        case ModelEvent.prototype.MOUSE_MOVE:
            break;
        case ModelEvent.prototype.MOUSE_DRAG:
            if (!(me.flagDownOnStartHandler || me.flagDownOnEndHandler))
                return;
            if(me.flagDownOnStartHandler){
                var aEl = me.mapSceneController.model.getOtherElementsInGroup(element);
                if((aEl.length === 0) || ModelElement.prototype.checkIfAllAre(aEl, "isMovableByMouse")) {
                    element.setXFrom(e.data.x + me.mouseDownPointShift.x);
                    element.setYFrom(e.data.y + me.mouseDownPointShift.y);
                }
            }else if(me.flagDownOnEndHandler){
                element.setXTo(e.data.x + me.mouseDownPointShift.x);
                element.setYTo(e.data.y + me.mouseDownPointShift.y);
            }
            break;
    }
};
/**
 * @constructor
 */
function RectangleModelElementController(element, visualization, mapSceneController){
    RectangleModelElementController.superclass.constructor.call(this, element, visualization, mapSceneController);
    var me = this;
    me.ownToolId = ModelTool.prototype.TOOL_RECTANGLE;
}
extend(RectangleModelElementController, ModelElementController);
/**
 * @constructor
 */
function CheckboxSetModelElementController(element, visualization, mapSceneController){
    CheckboxSetModelElementController.superclass.constructor.call(this, element, visualization, mapSceneController);
    var me = this;
    me.ownToolId = ModelTool.prototype.TOOL_CHECKBOX_SET;
    me.mapSceneController.addEventListener(
        ModelEvent.prototype.MOUSE_DOWN, {listener:me.mouseOnElementListener, target:me}, false);
}
extend(CheckboxSetModelElementController, ModelElementController);

CheckboxSetModelElementController.prototype.mouseOnElementListener = function(e) {
    var me = this;
    var element = me.element;
    var hitDetails = element.getHitDetails();
    var checkbox;
    switch (e.type) {
        case ModelEvent.prototype.MOUSE_DOWN:
            if(typeof e.data.currentToolId == TYPE_UNDEFINED || e.data.currentToolId != ModelTool.prototype.TOOL_SELECT)
                break;
            var flagOnElement = e.data.id != null && e.data.id == element.getId();
            if(flagOnElement && hitDetails != null && typeof hitDetails.orderNo == TYPE_NUMBER){
                var flagToggled = element.toggleByOrderNo(hitDetails.orderNo);
                if(flagToggled) {
                    checkbox = element.getCheckboxByOrderNo(hitDetails.orderNo);
                    var evt  = new ModelEvent(ModelEvent.prototype.CHECKBOX_TOGGLED, true, true);
                    if(evt != null){
                        evt.data = {id: e.data.id, orderNo: hitDetails.orderNo, checked: checkbox.isChecked()};
                        me.dispatchEvent(evt);
                    }
                }
            }
            break;
        default:
    }
};

//override
CheckboxSetModelElementController.prototype.deactivate = function() {
    CheckboxSetModelElementController.superclass.deactivate.call(this);
    var me = this;
    me.mapSceneController.removeEventListener(
        ModelEvent.prototype.MOUSE_DOWN, {listener:me.mouseOnElementListener, target:me}, false);
};