/**
 * @constructor
 */
function ToolbarSymbolSetButtonModel(toolbar, objProps) {
    ToolbarSymbolSetButtonModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;
    var cls =
        (typeof window[me.DROPLIST_MODEL_CLASS] == TYPE_FUNCTION) ?
            window[me.DROPLIST_MODEL_CLASS] : null;
    me.droplist = (cls != null) ? new cls(me) : null;
    //whether the droplist is open
    me.flagOpen = false;
    //open the list even if the main part of the button was clicked
    me.flagOpenListAlways = false;
    //
    me.droplistAlign = me.DROPLIST_ALIGN_BUTTON_EDGE;
}
extend(ToolbarSymbolSetButtonModel, ToolbarButtonModel);

ToolbarSymbolSetButtonModel.prototype.ICON_SCALE_FACTOR = 1.25;
ToolbarSymbolSetButtonModel.prototype.DEFAULT_DROPLIST_BUTTON_WIDTH = 12;
ToolbarSymbolSetButtonModel.prototype.DROPLIST_ALIGN_LEFT = -2;
ToolbarSymbolSetButtonModel.prototype.DROPLIST_ALIGN_BUTTON_EDGE = 0;
ToolbarSymbolSetButtonModel.prototype.DROPLIST_ALIGN_RIGHT = 2;

ToolbarSymbolSetButtonModel.prototype.DROPLIST_MODEL_CLASS = "ToolbarSymbolSetDroplistModel";

//override
ToolbarSymbolSetButtonModel.prototype.setConfiguration = function(obj){
    var me = this;
    me.objConfiguration = {
        aSymbol: obj.aSymbol.slice(0),
        flagSeparateButtons: obj.flagSeparateButtons,
        iconAlpha: obj.iconAlpha,
        iconColor: obj.iconColor,
        symbolId: obj.symbolId
    };
};

//
ToolbarSymbolSetButtonModel.prototype.getPropertyForSymbolNo = function(s, q){
    var objConfiguration = this.getConfiguration();
    switch(s){
        case "iconColor":
        case "lineColor":
            return (typeof objConfiguration[s] == TYPE_UNDEFINED) ? undefined : objConfiguration[s];
            break;
        case "iconAlpha":
            return (typeof objConfiguration.iconColor == TYPE_UNDEFINED) ? 0 : objConfiguration.iconAlpha;
            break;
        case "lineAlpha":
            return (typeof objConfiguration.lineColor == TYPE_UNDEFINED) ? 0 : objConfiguration.lineAlpha;
            break;
        default:
            return null;
    }
};

//
ToolbarSymbolSetButtonModel.prototype.getSymbolId = function(){
    var me = this;
    var obj = me.objConfiguration;
    return (obj != null && typeof obj.symbolId != TYPE_UNDEFINED) ? obj.symbolId : null;
};

//
ToolbarSymbolSetButtonModel.prototype.setSymbolId = function(sId){
    var me = this;
    var obj = me.objConfiguration;
    if(obj == null || !Array.isArray(obj.aSymbol) || obj.aSymbol.indexOf(sId) == -1)
        return;
    obj.symbolId = sId;
    var evt = new ModelEvent(ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, true, true);
    if(evt!=null) {
        evt.data = {subType:ModelEvent.prototype.SYMBOL_SELECTED, buttonId:me.getId(), symbolId:obj.symbolId, buttonNumber:obj.aSymbol.indexOf(sId)};
        me.dispatchEvent(evt);
    }
};

//
ToolbarSymbolSetButtonModel.prototype.getSymbolIds = function(){
    var me = this;
    var obj = me.objConfiguration;
    return (obj != null && Array.isArray(obj.aSymbol)) ? obj.aSymbol : [];
};

//
ToolbarSymbolSetButtonModel.prototype.getIconColor = function(){
    var me = this;
    var obj = me.objConfiguration;
    return (obj != null && typeof obj.iconColor != TYPE_UNDEFINED) ? obj.iconColor : undefined;
};

//
ToolbarSymbolSetButtonModel.prototype.getIconAlpha = function(){
    var me = this;
    var obj = me.objConfiguration;
    return (obj != null && typeof obj.iconAlpha != TYPE_UNDEFINED) ? obj.iconAlpha : undefined;
};

ToolbarSymbolSetButtonModel.prototype.getLineColor = function(){
    var me = this;
    var obj = me.objConfiguration;
    return (obj != null && typeof obj.lineColor != TYPE_UNDEFINED) ? obj.lineColor : undefined;
};

//
ToolbarSymbolSetButtonModel.prototype.getLineAlpha = function(){
    var me = this;
    var obj = me.objConfiguration;
    return (obj != null && typeof obj.lineAlpha != TYPE_UNDEFINED) ? obj.lineAlpha : undefined;
};

//
ToolbarSymbolSetButtonModel.prototype.isWithDropList = function(){
    var me = this;
    var objConfiguration = me.objConfiguration;
    if(objConfiguration == null || (typeof objConfiguration.flagSeparateButtons != TYPE_BOOLEAN))
        me.flagWithDroplist = false;
    else
        me.flagWithDroplist =
            (
                !objConfiguration.flagSeparateButtons &&
                Array.isArray(objConfiguration.aSymbol) && objConfiguration.aSymbol.length>1
            );
    return ToolbarSymbolSetButtonModel.superclass.isWithDropList.call(me);
};

//override
ToolbarSymbolSetButtonModel.prototype.setSelected = function(flag, objDetails){
    //ToolbarSymbolSetButtonModel.superclass.setSelected.call(this, flag, objDetails);
    var me = this;
    if(typeof objDetails != TYPE_OBJECT || objDetails == null){
        ToolbarSymbolSetButtonModel.superclass.setSelected.call(this, flag, objDetails);
        me.setDroplistOpen(false);
        return;
    }
    me.objSelectionDetails = objDetails;

    if(me.flagSelected != flag) {
        me.flagSelected = flag;
    }
    me.setDroplistOpen(false);
    if(
        me.objConfiguration != null &&
        Array.isArray(me.objConfiguration.aSymbol) &&
        objDetails != null &&
        typeof objDetails.buttonNumber == TYPE_NUMBER &&
        objDetails.buttonNumber>=0 && objDetails.buttonNumber < me.objConfiguration.aSymbol.length
    )
        me.objConfiguration.symbolId = me.objConfiguration.aSymbol[objDetails.buttonNumber];

};

//
ToolbarSymbolSetButtonModel.prototype.getTotalWidth = function(){
    var me = this;
    var w = me.getWidth();
    var objConfiguration = me.getConfiguration();
    if(
        objConfiguration == null ||
        (typeof objConfiguration.flagSeparateButtons != TYPE_BOOLEAN) ||
        !Array.isArray(objConfiguration.aSymbol)
    )
        return w;
    var nSymbol = objConfiguration.aSymbol.length;
    if(objConfiguration.flagSeparateButtons)
        return w*nSymbol+me.getToolbar().getButtonSpacing()*(nSymbol-1);
    else
        return (nSymbol==1) ? w : (w+me.DEFAULT_DROPLIST_BUTTON_WIDTH);
};

//
ToolbarSymbolSetButtonModel.prototype.isDroplistOpen = function(){
    return this.flagOpen;
};

//
ToolbarSymbolSetButtonModel.prototype.setDroplistOpen = function(flag){
    var me = this;
    if(me.flagOpen != flag){
        me.flagOpen = flag;
        me.toolbar.invalidate();
    }
};

//
ToolbarSymbolSetButtonModel.prototype.isOpenListAlways = function(){
    return this.flagOpenListAlways;
};

//
ToolbarSymbolSetButtonModel.prototype.getDroplist = function(){
    return this.droplist;
};

//
ToolbarSymbolSetButtonModel.prototype.getDroplistAlign = function(){
    return this.droplistAlign;
};

//
ToolbarSymbolSetButtonModel.prototype.setDroplistAlign = function(v){
    var me = this;
    if(me.droplistAlign != v){
        me.droplistAlign = v;
        me.toolbar.invalidate();
    }
};

//override
ToolbarSymbolSetButtonModel.prototype.getHitTestInfoForPoint = function(p0){
    var me = this;
    var objConfiguration = me.objConfiguration;
    var flagSeparateButtons = objConfiguration.flagSeparateButtons;
    var i, n = flagSeparateButtons ? objConfiguration.aSymbol.length : 1;
    var buttonSpacing = me.getToolbar().getButtonSpacing(), wOne = me.getWidth(), xLeft, xRight;
    var flagHit = false;
    var flagHitOpenButton = false;
    var buttonNumber = -1;
    var iSelected = objConfiguration.aSymbol.indexOf(objConfiguration.symbolId);

    var wTotal = me.getTotalWidth();
    if((p0.x >= me.x) && (p0.x <= me.x+wTotal) && (p0.y >= me.y) && (p0.y <= me.y+me.height)) {
        xRight = me.x-buttonSpacing;
        for (i = 0; i < n; i++) {
            xLeft = xRight+buttonSpacing;
            xRight += wOne+buttonSpacing;
            if((p0.x >= xLeft) && (p0.x <= xRight)){
                flagHit = true;
                buttonNumber = flagSeparateButtons ? i : iSelected;
                break;
            }
        }
        if(!flagHit && me.isWithDropList()){
            xLeft = xRight;
            xRight = xLeft+me.DEFAULT_DROPLIST_BUTTON_WIDTH;
            flagHitOpenButton = (p0.x >= xLeft) && (p0.x <= xRight);
            flagHit = flagHitOpenButton;
        }
    }
    var flagInside = flagHit && !flagHitOpenButton;
    //flagHitListOpener – if click on the main button should open or close its list
    return new HitTestInfo(
        me, flagHit, flagInside,
        {
            flagHit:flagHit,
            flagHitOpenButton:flagHitOpenButton,
            flagHitListOpener:flagInside && me.isOpenListAlways(),
            buttonNumber:buttonNumber
        }
    );
};

//override
ToolbarSymbolSetButtonModel.prototype.deactivate = function(){
    ToolbarSymbolSetButtonModel.superclass.deactivate.call(this);
    var me = this;
    if(me.droplist != null)
        me.droplist.deactivate();
};
/**
 * @constructor
 */
function ToolbarSymbolSetButtonVisualization(model, toolbarVisualization){
    ToolbarSymbolSetButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
    var me = this;
    me.aIconData = [];
    me.sASymbol = "";
    //
    var cls =
        (typeof window[me.DROPLIST_VISIALIZATION_CLASS] == TYPE_FUNCTION) ?
            window[me.DROPLIST_VISIALIZATION_CLASS] : null;
    me.droplistVisualization =
        (cls != null && typeof model[me.GET_DROPLIST_METHOD] == TYPE_FUNCTION) ?
        new cls(model[me.GET_DROPLIST_METHOD](), toolbarVisualization) : null;
    //
    this.toolbarVisualization.addEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:this, listener:this.toolbarVisualizationListener}, false
    );
}
extend(ToolbarSymbolSetButtonVisualization, ToolbarButtonVisualization);

ToolbarSymbolSetButtonVisualization.prototype.DROPLIST_ICON_ID = "bn.droplist";
ToolbarSymbolSetButtonVisualization.prototype.DROPLIST_VISIALIZATION_CLASS = "ToolbarSymbolSetDroplistVisualization";
ToolbarSymbolSetButtonVisualization.prototype.GET_DROPLIST_METHOD = "getDroplist";

//
ToolbarSymbolSetButtonVisualization.prototype.getSelectedNumber = function(){
    var model = this.model;
    var objConfiguration = model.getConfiguration();
    return model.isSelected() ? objConfiguration.aSymbol.indexOf(objConfiguration.symbolId) : -1;

};

//
ToolbarSymbolSetButtonVisualization.prototype.getSelectedVisualizationNumber = function(){
    var model = this.model;
    var objConfiguration = model.getConfiguration();
    if(model.isSelected())
        return ((objConfiguration.flagSeparateButtons) ? objConfiguration.aSymbol : [objConfiguration.symbolId]).indexOf(objConfiguration.symbolId);
    else
        return -1;
};

//override
ToolbarSymbolSetButtonVisualization.prototype.deactivate = function(){
    ToolbarSymbolSetButtonVisualization.superclass.deactivate.call(this);
    this.droplistVisualization.deactivate();
    this.toolbarVisualization.removeEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:this, listener:this.toolbarVisualizationListener}, false
    )
};

//override
ToolbarSymbolSetButtonVisualization.prototype.update = function(toolbarLeft, toolbarTop){
    var me = this;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
    var cnvToolbar = me.toolbarVisualization.getCanvas();
    var ctxToolbar = cnvToolbar.getContext("2d");
    var cnv = me.cnv;
    var cnvIcon;
    var ctx = cnv.getContext("2d");
    var model = me.model;
    var objConfiguration = model.getConfiguration();

    var i, n;
    var w;
    var wOne = model.getWidth()*rSize;
    var h = Math.ceil(model.getHeight()*rSize);
    //visible set of symbols
    var aSymbolVis, nSymbolVis;
    var iconColor;
    var iconAlpha;
    var lineColor;
    var lineAlpha;
    if(objConfiguration.flagSeparateButtons) {
        aSymbolVis = objConfiguration.aSymbol;
        nSymbolVis = aSymbolVis.length;
    }else{
        aSymbolVis = [objConfiguration.symbolId];
        nSymbolVis = 1;
        iconColor = objConfiguration.iconColor;
        iconAlpha = (iconColor == undefined) ? 0 : objConfiguration.iconAlpha;
        lineColor = objConfiguration.lineColor;
        lineAlpha = (lineColor == undefined) ? 0 : objConfiguration.lineAlpha;
    }

    var wTotal = model.getTotalWidth()*rSize;
    w = Math.ceil(wTotal+1*rSize);

    var s = aSymbolVis.join(",");
    if(me.sASymbol != s){
        me.aIconData.splice(0);
        me.sASymbol = s;
        n = aSymbolVis.length;
        for(i=0; i<n; i++)
            me.aIconData.push(PointModelLibrary.prototype.getDataById(aSymbolVis[i]));
    }

    if(cnv.width != w || cnv.height != h){
        cnv.width = w;
        cnv.height = h;
    }
    ctx.clearRect(0,0,w,h);
    var c1;
    var c2;
    var objHitDetails = model.getHitDetails();
    //iSelected – among the button's symbol set
    var iSelected = me.getSelectedNumber();
    //iSelectedVis – among visible set (either button's set or one symbol in case of !separateButtons)
    var iSelectedVis = me.getSelectedVisualizationNumber();
    var xCurr = 0.5*rSize;
    var xIcon, yIcon;
    var flag;
    var objData;

    var xStart = xCurr;
    var dX = (nSymbolVis>1) ? (wTotal-wOne)/(nSymbolVis-1) : 0;

    for(i=0; i<nSymbolVis; i++){
        xCurr = xStart+i*dX;
        objData = me.aIconData[i];
        flag = (i == iSelectedVis);
        if(flag) {
            ctx.beginPath();
            ctx.fillStyle = "rgba(255, 255, 255, 0.5)";
            ctx.fillRect((xCurr|0)+0.5, 0.5, (wOne - 1)|0, (h - 1)|0);
        }
        if(objConfiguration.flagSeparateButtons) {
            iconColor = model.getPropertyForSymbolNo("iconColor", i);
            iconAlpha = model.getPropertyForSymbolNo("iconAlpha", i);
            lineColor = model.getPropertyForSymbolNo("lineColor", i);
            lineAlpha = model.getPropertyForSymbolNo("lineAlpha", i);
        }
        //shadow
        if(objData.colorTargetIndex != -1) {
            cnvIcon = PointModelLibrary.prototype.makeCanvasByData(
                objData, rSize / ToolbarSymbolSetButtonModel.prototype.ICON_SCALE_FACTOR,
                !isNaN(iconColor) ? 0x000000 : 0xFFFFFF, 0.01 * 15,
                0x000000, 0.01 * 15,
                0xFFFFFF, 0.0,
                false, true
            );
            xIcon = xCurr + (wOne - cnvIcon.width - 1) / 2 - objData.xShift*rSize;
            yIcon = (h - cnvIcon.height - 1) / 2 - objData.yShift*rSize;
            ctx.drawImage(cnvIcon, (xIcon + 1 * rSize)|0, (yIcon + 1 * rSize)|0);
        }
        cnvIcon = PointModelLibrary.prototype.makeCanvasByData(
            objData, rSize/ToolbarSymbolSetButtonModel.prototype.ICON_SCALE_FACTOR,
            iconColor, 0.01*iconAlpha,
            !isNaN(lineColor) ? lineColor : 0xFFFFFF, 0.01*lineAlpha,
            0xFFFFFF, 0.0,
            false, false
        );
        xIcon = xCurr + (wOne - cnvIcon.width - 1) / 2 - objData.xShift*rSize;
        yIcon = (h - cnvIcon.height - 1) / 2 - objData.yShift*rSize;
        ctx.drawImage(cnvIcon, xIcon|0, yIcon|0);
        if(flag){
            c1 = me.BORDER_COLOR_DARK;
            c2 = me.BORDER_COLOR_LIGHT;

            ctx.beginPath();
            ctx.lineWidth = Math.ceil(model.DEFAULT_LINE_WIDTH*rSize);
            ctx.strokeStyle = c1;
            ctx.moveTo(((xCurr+wOne-1)|0)-0.5, 0.5);
            ctx.lineTo((xCurr|0)+0.5, 0.5);
            ctx.lineTo((xCurr|0)+0.5, ((h-1)|0)-0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0)+0.5, ((h-1)|0)-0.5);
            ctx.lineTo(((xCurr+wOne)|0)-1-0.5, ((h-1)|0)-0.5);
            ctx.lineTo(((xCurr+wOne-1)|0)-0.5, 0.5);
            ctx.stroke();
        }
        //xCurr += (wOne+buttonSpacing);
    }

    //droplist button
    var wDLB = model.DEFAULT_DROPLIST_BUTTON_WIDTH*rSize;
    if(model.isWithDropList()) {
        xCurr = 0.5 * rSize + wOne;
        if(model.isDroplistOpen()) {
            ctx.beginPath();
            ctx.fillStyle = "rgba(255, 255, 255, 0.5)";
            ctx.fillRect((xCurr|0)+0.5, 0.5, (wDLB|0) - 1, (h|0) - 1);

            c1 = me.BORDER_COLOR_DARK;
            c2 = me.BORDER_COLOR_LIGHT;

            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.stroke();
        }
        var img = ToolbarButtonLibrary.prototype.getCachedSVG(me.DROPLIST_ICON_ID);
        var objSize = ToolbarButtonLibrary.prototype.getSizeById(me.DROPLIST_ICON_ID);
        ctx.setTransform(
            rSize, 0, 0, rSize,
            wOne+1*rSize+(rSize*(model.DEFAULT_DROPLIST_BUTTON_WIDTH-objSize.width)-1)/2, (h-rSize*objSize.height-1)/2
        );

        ctx.drawImage(img, 0, 0);
        ctx.setTransform(1, 0, 0, 1, 0, 0);
        //
        if(model.isDroplistOpen()) {
            if(!me.droplistVisualization.isVisible())
                me.droplistVisualization.show();
            me.droplistVisualization.update();
            //update position of droplist
            if(typeof toolbarLeft == TYPE_NUMBER && typeof toolbarTop == TYPE_NUMBER)
                this.updateDroplistPosition(toolbarLeft, toolbarTop, rSize, xCurr, h);

        }else if(me.droplistVisualization.isVisible())
            me.droplistVisualization.hide();
    }

    if(
        objHitDetails != null && objHitDetails.flagHit
    ){
        c1 = me.BORDER_COLOR_LIGHT;
        c2 = me.BORDER_COLOR_DARK;
        ctx.lineWidth = Math.ceil(model.DEFAULT_LINE_WIDTH*rSize);

        if(model.isEnabled() && objHitDetails.buttonNumber != -1 && objHitDetails.buttonNumber != iSelected) {
            xCurr =
                0.5 * rSize +
                (objConfiguration.flagSeparateButtons ? dX * objHitDetails.buttonNumber : 0);

            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(((xCurr + wOne)|0) - 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wOne)|0) - 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wOne)|0) - 0.5, 0.5);
            ctx.stroke();
        }else if(objHitDetails.flagHitOpenButton && !model.isDroplistOpen()){
            xCurr = 0.5 * rSize + wOne;

            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.stroke();
        }
    }

    ctxToolbar.drawImage(cnv, 0, 0);
};

//
ToolbarSymbolSetButtonVisualization.prototype.updateDroplistPosition = function(toolbarLeft, toolbarTop, rSize, xCurr, yCurr){
    if(typeof toolbarLeft == TYPE_NUMBER && typeof toolbarTop == TYPE_NUMBER) {
        var me = this;
        var model = this.model;
        var dlCanvas = me.droplistVisualization.getCanvas();
        dlCanvas.style.left = "0px";
        dlCanvas.style.top = "0px";
        var dlBounds = miscFunctions.getElementBounds(dlCanvas, true);

        var xDL;
        var yDL = toolbarTop + model.getY()*rSize + yCurr - 0.5 - dlBounds.top;

        switch(model.getDroplistAlign()){
            case ToolbarSymbolSetButtonModel.prototype.DROPLIST_ALIGN_LEFT:
                xDL = toolbarLeft + model.getX()*rSize + 0.5 - dlBounds.left;
                break;
            case ToolbarSymbolSetButtonModel.prototype.DROPLIST_ALIGN_RIGHT:
                xDL = toolbarLeft + model.getX()*rSize + xCurr + 0.5
                    - dlBounds.left -dlBounds.width+rSize*model.DEFAULT_DROPLIST_BUTTON_WIDTH+2.0*rSize;
                break;
            case ToolbarSymbolSetButtonModel.prototype.DROPLIST_ALIGN_BUTTON_EDGE:
            default:
                xDL = toolbarLeft + model.getX()*rSize + xCurr + 0.5 - dlBounds.left;
        }

        //take screen boundaries into account!
        //dlCanvas is shifted with respect to the browser window
        // hence non-zero dlBounds.left even though dlCanvas.style.left = 0
        //var xDLMax = toolbarLeft+cnvToolbar.width-dlBounds.width-dlBounds.left;
        var xDLMax = me.toolbarVisualization.getInterfaceVisualizationCanvasWidth()-dlBounds.width;
        if(xDL>xDLMax)
            xDL = xDLMax;
        else if(xDL<0)
            xDL = 0;

        dlCanvas.style.left = (xDL|0)+"px";
        dlCanvas.style.top = (yDL|0)+"px";
    }
};

//
ToolbarSymbolSetButtonVisualization.prototype.toolbarVisualizationListener = function(e){
    switch(e.type) {
        case ModelEvent.prototype.RESET_SCENE:
            if(this.model.isDroplistOpen())
                this.model.setDroplistOpen(false);
            break;
        default:
    }
};
/**
 * @constructor
 */
function ToolbarSymbolSetDroplistModel(parentModel){
    ToolbarSymbolSetDroplistModel.superclass.constructor.call(this, parentModel);
    var me = this;
    me.sASymbol = "";
    me.aShowcaseId = [];
    me.showcaseWidth = 0;
    me.showcaseHeight = 0;
    me.showcaseWidthMax = 0;
    me.showcaseHeightMax = 0;
    me.fUpdateSymbolShowcase();
}
extend(ToolbarSymbolSetDroplistModel, ToolbarDroplistModel);

//
ToolbarSymbolSetDroplistModel.prototype.DEFAULT_SAMPLE_WIDTH = 44;
ToolbarSymbolSetDroplistModel.prototype.DEFAULT_SAMPLE_HEIGHT = 44;
ToolbarSymbolSetDroplistModel.prototype.DEFAULT_PADDING = 10;
ToolbarSymbolSetDroplistModel.prototype.DEFAULT_SPACING = 0;

//
ToolbarSymbolSetDroplistModel.prototype.getSymbolId = function(){
    return this.getParentModel().getSymbolId();
};

//
ToolbarSymbolSetDroplistModel.prototype.setSymbolId = function(sId){
    return this.getParentModel().setSymbolId(sId);
};

//
ToolbarSymbolSetDroplistModel.prototype.getSymbolIds = function(){
    return this.getParentModel().getSymbolIds();
};

//
ToolbarSymbolSetDroplistModel.prototype.fUpdateSymbolShowcase = function() {
    var me = this;
    var aRow;
    var i, j, ii, n;

    var aSymbolId = me.getSymbolIds();
    me.sASymbol = aSymbolId.join(",");

    var aShowcaseId = me.aShowcaseId;

    n = aShowcaseId.length;
    for (i = 0; i < n; i++) {
        aRow = aShowcaseId[i];
        aRow.splice();
    }
    aShowcaseId.splice(0);

    n = aSymbolId.length;

    var p, q;

    if(me.showcaseWidthMax <= 0 && me.showcaseHeightMax <= 0) {
        //find minimal nf >= n so that nf = p*q and q<=p<=q+1, q – number of rows, p – number of columns
        q = Math.floor(Math.sqrt(n + 0.01));
        p = (q * q == n) ? q : (q + 1);
        if (p * q < n)
            q++;
    }else if(me.showcaseHeightMax>0){
        q = (me.showcaseHeightMax>n) ? n : me.showcaseHeightMax;
        p = Math.floor(n/q);
        if(p * q < n)
            p++;
    }else if(me.showcaseWidthMax>0){
        p = (me.showcaseWidthMax>n) ? n : me.showcaseWidthMax;
        q = Math.floor(n/p);
        if (p * q < n)
            p++;
    }

    for (i = 0, ii = 0; (i < q) && (ii < n); i++) {
        aRow = [];
        for (j = 0; (j < p) && (ii < n); j++)
            aRow.push(aSymbolId[ii++]);
        aShowcaseId.push(aRow);
    }

    me.showcaseHeight = q;
    me.showcaseWidth = p;
};

//
ToolbarSymbolSetDroplistModel.prototype.getShowcaseWidth = function(){
    var me = this;
    if(me.isShowcaseChanged())
        me.fUpdateSymbolShowcase();
    return me.showcaseWidth;
};

//
ToolbarSymbolSetDroplistModel.prototype.getShowcaseHeight = function(){
    var me = this;
    if(me.isShowcaseChanged())
        me.fUpdateSymbolShowcase();
    return me.showcaseHeight;
};

//
ToolbarSymbolSetDroplistModel.prototype.isShowcaseChanged = function(){
    var me = this;
    var s = me.getSymbolIds().join(",");
    return (s != me.sASymbol);
};

//
ToolbarSymbolSetDroplistModel.prototype.getSymbolShowcase = function() {
    var me = this;
    var aSymbolId = me.getSymbolIds();
    var s = aSymbolId.join(",");
    if (s != me.sASymbol) {
        me.sASymbol = s;
        me.fUpdateSymbolShowcase();
    }
    return me.aShowcaseId;
};

//
ToolbarSymbolSetDroplistModel.prototype.getSymbolIdByRowColumn = function(r, c){
    var a = this.aShowcaseId;
    if(r < 0 || r >= a.length)
        return null;
    a = a[r];
    if(c < 0 || c >= a.length)
        return null;
    return a[c];
};

//
ToolbarSymbolSetDroplistModel.prototype.getIconColor = function(){
    return this.getParentModel().getIconColor();
};

//
ToolbarSymbolSetDroplistModel.prototype.getIconAlpha = function(){
    return this.getParentModel().getIconAlpha();
};

//
ToolbarSymbolSetDroplistModel.prototype.getLineColor = function(){
    return this.getParentModel().getLineColor();
};

//
ToolbarSymbolSetDroplistModel.prototype.getLineAlpha = function(){
    return this.getParentModel().getLineAlpha();
};

/**
 * @constructor
 */
function ToolbarSymbolSetDroplistVisualization(_model, _mainInterfaceVisualization){
    ToolbarSymbolSetDroplistVisualization.superclass.constructor.call(this, _model, _mainInterfaceVisualization);
    var me = this;
    if(ToolbarSymbolSetDroplistVisualization.prototype.symbolSubstrate == null){
        ToolbarSymbolSetDroplistVisualization.prototype.symbolSubstrate =
            ToolbarButtonLibrary.prototype.getCachedSVG(me.SYMBOL_SUBSTRATE_ID);
        ToolbarSymbolSetDroplistVisualization.prototype.symbolSubstrateSize =
            ToolbarButtonLibrary.prototype.getSizeById(me.SYMBOL_SUBSTRATE_ID);
    }

    me.aSymbolId = [];
    me.aIconData = [];

    me.cnvIcons = document.createElement("CANVAS");
    me.cnvSubstrate = document.createElement("CANVAS");
    me.cnvHighlight = document.createElement("CANVAS");

    me.rowHit = -1;
    me.columnHit = -1;
    me.prevRowHit = -1;
    me.prevColumnHit = -1;

    me.cnvIcons.width = me.cnvSubstrate.width = me.cnvHighlight.width = 1;
    me.cnvIcons.height = me.cnvSubstrate.height = me.cnvHighlight.height = 1;
}
//this panel should fire sample-related events
extend(ToolbarSymbolSetDroplistVisualization, ToolbarDroplistVisualization);

ToolbarSymbolSetDroplistVisualization.prototype.SYMBOL_SUBSTRATE_ID = "gr.symbol-substrate";
ToolbarSymbolSetDroplistVisualization.prototype.HIGHLIGHT_COLOR = 0x333333;
ToolbarSymbolSetDroplistVisualization.prototype.HIGHLIGHT_ALPHA = 0.25;
ToolbarSymbolSetDroplistVisualization.prototype.symbolSubstrate = null;
ToolbarSymbolSetDroplistVisualization.prototype.symbolSubstrateSize = null;

//
ToolbarSymbolSetDroplistVisualization.prototype.fMouseOver = function(e){
};

//
ToolbarSymbolSetDroplistVisualization.prototype.fMouseOut = function(e){
    var me = this;
    if(me.rowHit != -1 || me.columnHit != -1)
        me.model.getParentModel().getToolbar().invalidate(ToolbarModel.prototype.INVALID_BUTTON_STATES);
    me.rowHit = -1;
    me.columnHit = -1;
};

//override
ToolbarSymbolSetDroplistVisualization.prototype.fMouseDown = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this;
    if(e.target == me.cnv){
        me.fUpdateRowColumnHit(me.getPointFromEvent(e));
        if(me.rowHit != -1 && me.columnHit != -1){
            var model = me.model;
            var p = model.getShowcaseWidth();
            var sSymbolId = me.aSymbolId[p*me.rowHit+me.columnHit];
            me.rowHit = -1;
            me.columnHit = -1;
            model.hide();
            if(model.getSymbolId() != sSymbolId) {
                model.setSymbolId(sSymbolId);
                model.getParentModel().getToolbar().invalidate(ToolbarModel.prototype.INVALID_BUTTON_STATES);
            }
        }
    }
};

//override
ToolbarSymbolSetDroplistVisualization.prototype.fMouseMove = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    if(e.target == this.getCanvas())
        this.fUpdateRowColumnHit(this.getPointFromEvent(e));
};

//override
ToolbarSymbolSetDroplistVisualization.prototype.fKeyupListener = function(e){
    ToolbarSymbolSetDroplistVisualization.superclass.fKeyupListener.call(this, e);
    if(MapSceneController.prototype.isKeyInEvent("CTRL", e))
        this.fNotifyOfRollOverState();
};

//override
ToolbarSymbolSetDroplistVisualization.prototype.fKeydownListener = function(e){
    ToolbarSymbolSetDroplistVisualization.superclass.fKeydownListener.call(this, e);
    if(MapSceneController.prototype.isKeyInEvent("CTRL", e))
        this.fNotifyOfRollOverState();
};

//
ToolbarSymbolSetDroplistVisualization.prototype.fUpdateRowColumnHit = function(pHit){
    var me = this;
    var model = me.model;
    var pSample = model.DEFAULT_PADDING;
    var sSample = model.DEFAULT_SPACING;
    var wSample = model.DEFAULT_SAMPLE_WIDTH;
    var hSample = model.DEFAULT_SAMPLE_HEIGHT;
    var x = pHit.x - pSample;
    var y = pHit.y - pSample;

    var p = model.getShowcaseWidth();
    var q = model.getShowcaseHeight();
    var rH = me.rowHit;
    var cH = me.columnHit;
    if(x<0 || y<0 || x>wSample*p+sSample*(p-1) || y>hSample*q+sSample*(q-1)){
        me.rowHit = -1;
        me.columnHit = -1;
    }else {
        var c = Math.floor(x / (wSample + sSample));
        var cr = x % (wSample + sSample);
        var r = Math.floor(y / (hSample + sSample));
        var rr = y % (hSample + sSample);
        if (cr > wSample || rr > hSample || (r*p+c >= me.aSymbolId.length)){
            me.rowHit = -1;
            me.columnHit = -1;
        }else{
            me.rowHit = r;
            me.columnHit = c;
        }
    }

    if(me.rowHit != rH || me.columnHit != cH) {
        model.getParentModel().getToolbar().invalidate(ToolbarModel.prototype.INVALID_BUTTON_STATES);
        if(me.isCtrl())
            me.fNotifyOfRollOverState();
    }
};

//
ToolbarSymbolSetDroplistVisualization.prototype.fNotifyOfRollOverState = function(){
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.SYMBOL_HIGHLIGHTED, true, true);
    if (evt != null) {
        evt.data = {ctrlKey:me.isCtrl(), symbolId:me.model.getSymbolIdByRowColumn(me.rowHit, me.columnHit)};
        me.cnv.dispatchEvent(evt);
    }
};

//override
ToolbarSymbolSetDroplistVisualization.prototype.update = function(){
    var me = this;

    var i, j, ii, n;

    var model = me.model;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();

    var flagChanged = model.isShowcaseChanged();
    if(flagChanged){
        me.aIconData.splice(0);
        me.aSymbolId = model.getSymbolIds();
        var aSymbolId = me.aSymbolId;
        n = aSymbolId.length;
        for(i=0; i<n; i++)
            me.aIconData.push(PointModelLibrary.prototype.getDataById(aSymbolId[i]));
    }
    var iconColor = model.getIconColor();
    var iconAlpha = model.getIconAlpha();
    var lineColor = model.getLineColor();
    var lineAlpha = model.getLineAlpha();

    var p = model.getShowcaseWidth();
    var q = model.getShowcaseHeight();

    var pSample = model.DEFAULT_PADDING;
    var sSample = model.DEFAULT_SPACING;
    var wSample = model.DEFAULT_SAMPLE_WIDTH;
    var hSample = model.DEFAULT_SAMPLE_HEIGHT;

    var wIcons = Math.ceil((p*wSample+(p-1)*sSample)*rSize);
    var hIcons = Math.ceil((q*hSample+(q-1)*sSample)*rSize);

    if(wIcons != me.cnvIcons.width || hIcons != me.cnvIcons.height){
        me.cnvIcons.width = me.cnvHighlight.width = me.cnvSubstrate.width = wIcons;
        me.cnvIcons.height = me.cnvHighlight.height = me.cnvSubstrate.height = hIcons;
        flagChanged = true;
    }

    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");

    if(flagChanged || !me.flagValid) {

        me.cnvIcons.getContext("2d").clearRect(0, 0, wIcons, hIcons);
        me.cnvSubstrate.getContext("2d").clearRect(0, 0, wIcons, hIcons);

        var ctxIcons = me.cnvIcons.getContext("2d");
        var cnvIcon;
        var xIcon, yIcon;
        var aIconData = me.aIconData;
        n = aIconData.length;
        var xCurr = 0;
        var yCurr = 0;
        var objData;
        for (j = 0, ii = 0; (j < q) && (ii < n); j++) {
            for (i = 0; (i < p) && (ii < n); i++, ii++) {
                objData = aIconData[ii];
                //shadow
                if(objData.colorTargetIndex != -1) {
                    cnvIcon = PointModelLibrary.prototype.makeCanvasByData(
                        objData, rSize / ToolbarSymbolSetButtonModel.prototype.ICON_SCALE_FACTOR,
                        0x000000, 0.01 * 15,
                        0xFFFFFF, 0.0,
                        0xFFFFFF, 0.0,
                        false, true
                    );
                    xIcon = xCurr + (wSample * rSize - cnvIcon.width - 1) / 2 - objData.xShift*rSize;
                    yIcon = yCurr + (hSample * rSize - cnvIcon.height - 1) / 2 - objData.yShift*rSize;
                    ctxIcons.drawImage(cnvIcon, xIcon + 1 * rSize, yIcon + 1 * rSize);
                }
                cnvIcon = PointModelLibrary.prototype.makeCanvasByData(
                    objData, rSize/ToolbarSymbolSetButtonModel.prototype.ICON_SCALE_FACTOR,
                    iconColor, 0.01*iconAlpha,
                    lineColor, 0.01*lineAlpha,
                    0xFFFFFF, 0.0,
                    false, false
                );
                xIcon = xCurr + (wSample * rSize - cnvIcon.width - 1) / 2 - objData.xShift*rSize;
                yIcon = yCurr + (hSample * rSize - cnvIcon.height - 1) / 2 - objData.yShift*rSize;
                ctxIcons.drawImage(cnvIcon, xIcon, yIcon);
                xCurr += (wSample+sSample) * rSize;
            }
            xCurr = 0;
            yCurr += (hSample+sSample) * rSize;
        }

        var ctxSubstrate = me.cnvSubstrate.getContext("2d");
        var symbolSubstrate = me.symbolSubstrate;
        var symbolSubstrateSize = me.symbolSubstrateSize;
        ctxSubstrate.setTransform(rSize, 0, 0, rSize, 0, 0);
        xCurr = 0;
        yCurr = 0;
        for (j = 0, ii = 0; (j < q) && (ii < n); j++) {
            for (i = 0; (i < p) && (ii < n); i++, ii++) {
                ctxSubstrate.drawImage(
                    symbolSubstrate,
                    xCurr+(wSample-symbolSubstrateSize.width)/2, yCurr+(hSample-symbolSubstrateSize.height)/2
                );
                xCurr += wSample+sSample;
            }
            xCurr = 0;
            yCurr += hSample+sSample;
        }
        ctxSubstrate.setTransform(1, 0, 0, 1, 0, 0);
    }

    if(me.rowHit != me.prevRowHit || me.columnHit != me.prevColumnHit){
        flagChanged = true;
        me.prevColumnHit = me.columnHit;
        me.prevRowHit = me.rowHit;

        var ctxHighlight = me.cnvHighlight.getContext("2d");
        ctxHighlight.clearRect(0, 0, wIcons, hIcons);
        if(me.rowHit!=-1 && me.columnHit!=-1) {
            ctxHighlight.setTransform(rSize, 0, 0, rSize, 0, 0);
            ctxHighlight.beginPath();
            ctxHighlight.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.HIGHLIGHT_COLOR, me.HIGHLIGHT_ALPHA);
            ctxHighlight.fillRect(me.columnHit*(wSample+sSample), me.rowHit*(hSample+sSample), wSample, hSample);
            ctxHighlight.setTransform(1, 0, 0, 1, 0, 0);
        }
    }

    if(flagChanged){
        //background and shadow
        var wTotal = Math.ceil((2 * pSample + p * wSample) * rSize);
        var hTotal = Math.ceil((2 * pSample + q * hSample) * rSize);

        //+2*rSize for shadow
        cnv.width = Math.ceil(wTotal + 2 * rSize);
        cnv.height = Math.ceil(hTotal + 2 * rSize);

        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.SHADOW_COLOR, 0.50);
        ctx.beginPath();
        ctx.fillRect(2 * rSize, 2 * rSize, wTotal, hTotal);

        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.BACKGROUND_COLOR, 1.0);
        ctx.lineWidth = 1;
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.BORDER_COLOR, 1.0);
        ctx.beginPath();
        ctx.fillRect(0, 0, wTotal, hTotal);
        ctx.strokeRect(0, 0, wTotal, hTotal);
        //
        ctx.drawImage(me.cnvSubstrate, pSample*rSize, pSample*rSize);
        ctx.drawImage(me.cnvHighlight, pSample*rSize, pSample*rSize);
        ctx.drawImage(me.cnvIcons, pSample*rSize, pSample*rSize);
        //
        me.flagValid = true;
    }

};

/**
 * @constructor
 */
function ToolbarRectangleButtonModel(toolbar, objProps) {
    ToolbarRectangleButtonModel.superclass.constructor.call(this, toolbar, objProps);
}
extend(ToolbarRectangleButtonModel, ToolbarSymbolSetButtonModel);

//override
ToolbarRectangleButtonModel.prototype.setConfiguration = function(obj){
    var me = this;
    var symbolId =
        (me.objConfiguration == null || typeof me.objConfiguration.symbolId != TYPE_STRING) ?
            "mcSymbolRN_Size0" : me.objConfiguration.symbolId;
    me.objConfiguration = {
        aSymbol: ["mcSymbolRN_Size0", "mcSymbolEN_Size0"],
        flagSeparateButtons: false,
        iconAlpha: obj.fillAlpha,
        iconColor: obj.fillColor,
        lineAlpha: obj.lineAlpha,
        lineColor:obj.lineColor,
        symbolId: symbolId
    };
};

/**
 * @constructor
 */
function ToolbarRectangleButtonVisualization(_model, _mainInterfaceVisualization) {
    ToolbarRectangleButtonVisualization.superclass.constructor.call(this, _model, _mainInterfaceVisualization);
}
extend(ToolbarRectangleButtonVisualization, ToolbarSymbolSetButtonVisualization);

/**
 * @constructor
 */
function ToolbarColorPaletteModel(parentModel){
    ToolbarColorPaletteModel.superclass.constructor.call(this, parentModel);
    var me = this;
    me.width = 324;
    me.height = 184;

    me.hPadding = 8;
    me.vPadding = 8;

    me.sampleWidth = 42;
    me.sampleHeight = 42;

    me.noColorWidth = 82;
    me.noColorHeight = 42;

    me.currentColorWidth = 82;
    me.currentColorHeight = 86;

    me.sampleHSpacing = 2;
    me.sampleVSpacing = 2;

    me.nRows = 3;
    me.nColumns = 5;
    //
    me.aSampleX = [];
    me.aSampleY = [];
    //
    me.alphaSlider = new ToolbarSliderModel(
        parentModel.getToolbar(),
        {
            id:"colorPaletteAlpha",
            value:100, minimalValue:0, maximalValue:100, dragLength:100
        }
    );
    me.alphaSlider.setX(me.width-me.hPadding-me.alphaSlider.getWidth());
    me.alphaSlider.setY(me.height-me.vPadding-me.alphaSlider.getHeight());
    //
    me.alphaSlider.addEventListener(ModelEvent.prototype.VALUE_CHANGED, {target:me, listener:me.fAlphaSliderListener}, false);
    //
    me.flagInitialized = false;
}
extend(ToolbarColorPaletteModel, ToolbarDroplistModel);

//
ToolbarColorPaletteModel.prototype.init = function(){
    var me = this;
    if(me.flagInitialized)
        return;
    me.flagInitialized = true;
    var i;
    var xCurr, yCurr;
    for(i=0, xCurr = me.hPadding; i<me.nColumns; i++){
        me.aSampleX.push(xCurr);
        xCurr += (me.sampleWidth + me.sampleHSpacing);
    }
    for(i=0, yCurr = me.vPadding; i<me.nRows; i++){
        me.aSampleY.push(yCurr);
        yCurr += (me.sampleHeight + me.sampleVSpacing);
    }
    //
    me.noColorX = me.width-me.noColorWidth-me.hPadding;
    me.noColorY = me.vPadding;
    //
    me.currentColorX = me.noColorX;
    me.currentColorY = me.aSampleY[me.nRows-1]+me.sampleHeight-me.currentColorHeight;
    //
};

//
ToolbarColorPaletteModel.prototype.fAlphaSliderListener = function(e){
    var me = this;
    me.setColorAndAlpha(me.getColor(), e.data.value);
};

//
ToolbarColorPaletteModel.prototype.getSampleColorByOrderNumber = function (q) {
    return this.getParentModel().getSampleColors()[q];
};

//
ToolbarColorPaletteModel.prototype.getSampleColorsCount = function(){
    return this.getParentModel().getSampleColors().length;
};

//
ToolbarColorPaletteModel.prototype.getColorSampleXByOrderNumber = function(q){
    return this.aSampleX[q%this.nColumns];
};

//
ToolbarColorPaletteModel.prototype.getColorSampleYByOrderNumber = function(q){
    return this.aSampleY[Math.floor(q/this.nColumns)];
};

//
ToolbarColorPaletteModel.prototype.getNoColorX = function(){
    return this.noColorX;
};

//
ToolbarColorPaletteModel.prototype.getNoColorY = function(){
    return this.noColorY;
};

//
ToolbarColorPaletteModel.prototype.getCurrentColorX = function(){
    return this.currentColorX;
};

//
ToolbarColorPaletteModel.prototype.getCurrentColorY = function(){
    return this.currentColorY;
};

//
ToolbarColorPaletteModel.prototype.getAlpha = function(){
    return this.getParentModel().getIconAlpha();
};

//
ToolbarColorPaletteModel.prototype.getColor = function () {
    return this.getParentModel().getIconColor();
};

//
ToolbarColorPaletteModel.prototype.getInitialColor = function () {
    return this.getParentModel().getInitialColor();
};

//
ToolbarColorPaletteModel.prototype.setColorAndAlpha = function(color, alpha){
    this.getParentModel().setColorAndAlpha(color, alpha);
};

//
ToolbarColorPaletteModel.prototype.resetColor = function(){
    this.getParentModel().resetColor();
};

//
ToolbarColorPaletteModel.prototype.isEmptyFill = function(){
    //stub, needs to be finished
    return false;
};

//
ToolbarColorPaletteModel.prototype.getSampleColors = function(){
    return this.getParentModel().getSampleColors();
};

//
ToolbarColorPaletteModel.prototype.getAlphaSlider = function(){
    return this.alphaSlider;
};

//override
ToolbarColorPaletteModel.prototype.deactivate = function(){
    ToolbarColorPaletteModel.superclass.deactivate.call(this);
    var me = this;
    me.alphaSlider.removeEventListener(ModelEvent.prototype.VALUE_CHANGED, {target:me, listener:me.fAlphaSliderListener}, false);
};

/**
 * @constructor
 */
function ToolbarColorPaletteVisualization(_model, _toolbarVisualization){
    ToolbarColorPaletteVisualization.superclass.constructor.call(this, _model, _toolbarVisualization);
    var me = this;
    me.sPrevParams = "";
    me.prevSampleHit = -1;
    //
    me.alphaSliderVisualization = new ToolbarSliderVisualization(me.model.getAlphaSlider());
    me.xStartDragAlpha = undefined;
    me.valueStartDragAlpha = undefined;
}
extend(ToolbarColorPaletteVisualization, ToolbarDroplistVisualization);


//override
ToolbarColorPaletteVisualization.prototype.fMouseMove = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    if(e.target == this.getCanvas()){
        var me = this;
        var model = me.model;
        var q = me.prevSampleHit;

        var  p = me.getPointFromEvent(e);

        var alphaSlider = model.getAlphaSlider();
        if(!me.flagMouseDown) {
            var hitDetails = miscFunctions.shallowCloneObject(alphaSlider.getHitTestInfoForPoint(p).getDetails());
            alphaSlider.setHitDetails(hitDetails);

            me.fUpdateColorSampleHit(p);
            if(me.prevSampleHit != q){
                if(me.prevSampleHit>=0 && me.prevSampleHit<model.getSampleColorsCount())
                    model.setColorAndAlpha(model.getSampleColorByOrderNumber(me.prevSampleHit), model.getAlpha());
                else if(me.prevSampleHit == model.getSampleColorsCount())
                    model.setColorAndAlpha(undefined, model.getAlpha());
                else if(me.prevSampleHit == -1)
                    model.resetColor();
            }
        }else if(me.xStartDragAlpha != undefined){
            var dx = p.x-me.xStartDragAlpha;
            var v = me.valueStartDragAlpha+dx*(alphaSlider.maxValue-alphaSlider.minValue)/alphaSlider.dragLength;
            if(v<alphaSlider.minValue)
                v = alphaSlider.minValue;
            else if(v>alphaSlider.maxValue)
                v = alphaSlider.maxValue;
            alphaSlider.setValue(v);
        }
        me.invalidate();
    }
};

//override
ToolbarColorPaletteVisualization.prototype.fMouseDown = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    if(e.target == this.getCanvas()){
        var me = this;
        var model = me.model;

        me.flagMouseDown = true;

        var  p = me.getPointFromEvent(e);

        var alphaSlider = model.getAlphaSlider();
        var hitDetails = miscFunctions.shallowCloneObject(alphaSlider.getHitTestInfoForPoint(p).getDetails());
        if(hitDetails.flagHitDragger) {
            me.xStartDragAlpha = p.x;
            me.valueStartDragAlpha = alphaSlider.getValue();
            hitDetails.flagPressDragger = true;
        }else if(hitDetails.flagHitRange)
            alphaSlider.setValue(hitDetails.hitValue);
        else{
            me.fUpdateColorSampleHit(p);
            var n = model.getSampleColorsCount();
            if(me.prevSampleHit>=0 && me.prevSampleHit<=n){
                var color = (me.prevSampleHit == n) ? undefined : model.getSampleColorByOrderNumber(me.prevSampleHit);
                model.setColorAndAlpha(color, model.getAlpha());
                model.hide();
            }else if(me.prevSampleHit == n+1){
                model.setColorAndAlpha(model.getColor(), model.getAlpha());
                model.hide();
            }
        }
        alphaSlider.setHitDetails(hitDetails);
        me.invalidate();

    }
};

//override
ToolbarColorPaletteVisualization.prototype.fMouseUp = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this;
    var model = me.model;
    var alphaSlider = model.getAlphaSlider();
    me.flagMouseDown = false;
    me.xStartDragAlpha = undefined;
    me.valueStartDragAlpha = undefined;
    var  p = me.getPointFromEvent(e);
    var hitDetails = alphaSlider.getHitTestInfoForPoint(p).getDetails();
    hitDetails.flagPressDragger = false;
    alphaSlider.setHitDetails(hitDetails);
    me.invalidate();
};

//
ToolbarColorPaletteVisualization.prototype.fUpdateColorSampleHit = function(p){
    var me = this;

    var model = me.model;
    var x = p.x;
    var y = p.y;

    var wSample = model.sampleWidth;
    var hSample = model.sampleHeight;
    var i, n = model.getSampleColorsCount();
    var xCurr, yCurr;
    var sampleHit = -1;
    //
    for(i=0; i<n; i++){
        xCurr = model.getColorSampleXByOrderNumber(i);
        yCurr = model.getColorSampleYByOrderNumber(i);
        if(x>=xCurr && x<=xCurr+wSample && y>=yCurr && y<=yCurr+hSample) {
            sampleHit = i;
            break;
        }
    }
    if(sampleHit == -1){
        xCurr = model.getNoColorX();
        yCurr = model.getNoColorY();
        wSample = model.noColorWidth;
        hSample = model.noColorHeight;
        if(x>=xCurr && x<=xCurr+wSample && y>=yCurr && y<=yCurr+hSample)
            sampleHit = n;
    }
    if(sampleHit == -1){
        xCurr = model.getCurrentColorX();
        yCurr = model.getCurrentColorY();
        wSample = model.currentColorWidth;
        hSample = model.currentColorHeight;
        if(x>=xCurr && x<=xCurr+wSample && y>=yCurr && y<=yCurr+hSample)
            sampleHit = n+1;
    }
    me.prevSampleHit = sampleHit;
};

//override
ToolbarColorPaletteVisualization.prototype.update = function(){
    var me = this;

    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");

    var model = me.model;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();

    var aRGB = model.getSampleColors();
    var alpha = model.getAlpha();
    var color = model.getColor();
    var flagEmptyFill = (color == undefined) || (alpha == undefined);

    var wTotal = Math.ceil(model.width * rSize);
    var hTotal = Math.ceil(model.height * rSize);

    //+2*rSize for shadow
    var w = Math.ceil(wTotal + 2 * rSize);
    var h = Math.ceil(hTotal + 2 * rSize);

    var flagChanged = (cnv.width != w) || (cnv.height != h);
    if(flagChanged){
        cnv.width = w;
        cnv.height = h;
    }

    var sCurrParams = aRGB.join(",")+","+alpha+","+color;

    if(sCurrParams != me.sPrevParams){
        flagChanged = true;
        me.sPrevParams = sCurrParams;
    }

    if(flagChanged || !me.flagValid){
        //background and shadow
        ctx.clearRect(0, 0, w, h);

        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.SHADOW_COLOR, 0.50);
        ctx.beginPath();
        ctx.fillRect(2 * rSize, 2 * rSize, wTotal, hTotal);

        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.BACKGROUND_COLOR, 1.0);
        ctx.lineWidth = 1;
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.BORDER_COLOR, 1.0);
        ctx.beginPath();
        ctx.fillRect(0, 0, wTotal, hTotal);
        ctx.strokeRect(0, 0, wTotal, hTotal);
        //
        var i;
        var nColors = model.getSampleColorsCount();
        var xCurr;
        var yCurr;
        var wSample = Math.ceil(model.sampleWidth*rSize);
        var hSample = Math.ceil(model.sampleHeight*rSize);
        //
        for(i=0; i<nColors; i++){
            xCurr = model.getColorSampleXByOrderNumber(i)*rSize;
            yCurr = model.getColorSampleYByOrderNumber(i)*rSize;
            ctx.beginPath();
            ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(model.getSampleColorByOrderNumber(i), 1.0);
            ctx.fillRect(xCurr, yCurr, wSample, hSample);
        }
        //
        var objSize = ColorPaletteLibrary.prototype.getSizeById("gr.empty-color");
        var img = ColorPaletteLibrary.prototype.getCachedSVG("gr.empty-color");
        ctx.setTransform(rSize, 0, 0, rSize, 0, 0);
        ctx.drawImage(
            img,
            model.getNoColorX()+(model.noColorWidth/2+objSize.xShift),
            model.getNoColorY()+(model.noColorHeight/2+objSize.yShift)
        );
        //
        objSize = ColorPaletteLibrary.prototype.getSizeById("gr.color-background");
        img = ColorPaletteLibrary.prototype.getCachedSVG("gr.color-background");
        ctx.drawImage(
            img,
            model.getCurrentColorX()+(model.currentColorWidth/2+objSize.xShift),
            model.getCurrentColorY()+(model.currentColorHeight/2+objSize.yShift)
        );
        //
        if(flagEmptyFill){
            objSize = ColorPaletteLibrary.prototype.getSizeById("gr.empty-color-current");
            img = ColorPaletteLibrary.prototype.getCachedSVG("gr.empty-color-current");
            ctx.drawImage(
                img,
                model.getCurrentColorX()+(model.currentColorWidth/2+objSize.xShift),
                model.getCurrentColorY()+(model.currentColorHeight/2+objSize.yShift)
            );
        }else {
            ctx.beginPath();
            ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(color, alpha * 0.01);
            ctx.fillRect(
                model.getCurrentColorX(), model.getCurrentColorY(),
                model.currentColorWidth, model.currentColorHeight
            );
        }

        //alpha slider
        var alphaVis = me.alphaSliderVisualization;
        var alphaMod = alphaVis.getModel();
        alphaMod.setValue(alpha);
        alphaVis.update();
        var alphaCnv = alphaVis.getCanvas();
        ctx.drawImage(alphaCnv, alphaMod.getX(), alphaMod.getY());

        ctx.setTransform(1, 0, 0, 1, 0, 0);
        //
        me.flagValid = true;
    }
};

/**
 * @constructor
 */
function ToolbarListedCommandButtonModel(toolbar, objProps) {
    ToolbarListedCommandButtonModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;
    //
    me.flagDroplistAlwaysEnabled = true;
    me.flagOpenListAlways = true;
    //
    //me.setDeselectOnRelease(true);
}
extend(ToolbarListedCommandButtonModel, ToolbarSymbolSetButtonModel);

ToolbarListedCommandButtonModel.prototype.PLATE_WIDTH = 30;
ToolbarListedCommandButtonModel.prototype.PLATE_HEIGHT = 26;
//this class is "abstract" DROPLIST_MODEL_CLASS is defined in the subclasses
ToolbarListedCommandButtonModel.prototype.DROPLIST_MODEL_CLASS = null;

//
ToolbarListedCommandButtonModel.prototype.getElementsCount = function(){
    var me = this;
    var objC = me.objConfiguration;
    return (typeof objC.aElement != TYPE_UNDEFINED && Array.isArray(objC.aElement)) ? objC.aElement.length : 0;
};

//
ToolbarListedCommandButtonModel.prototype.getElementNo = function(q){
    var me = this;
    var objC = me.objConfiguration;
    return (
        typeof objC.aElement != TYPE_UNDEFINED &&
        Array.isArray(objC.aElement) && (q>=0) && (q<objC.aElement.length)
    ) ? objC.aElement[q] : null;
};

//override
ToolbarListedCommandButtonModel.prototype.setConfiguration = function(obj){
    var me = this;
    me.objConfiguration = miscFunctions.shallowCloneObject(obj);
    me.objConfiguration.flagSeparateButtons = false;
};

//override
ToolbarListedCommandButtonModel.prototype.isWithDropList = function(){
    return true;
};

//override
ToolbarListedCommandButtonModel.prototype.getTotalWidth = function(){
    return this.getWidth()+this.DEFAULT_DROPLIST_BUTTON_WIDTH;
};

//override
ToolbarListedCommandButtonModel.prototype.setDroplistOpen = function(flag){
    ToolbarColorButtonModel.superclass.setDroplistOpen.call(this, flag);
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, true, true);
    if(evt!=null) {
        evt.data = {subType:ModelEvent.prototype.TOGGLE_DROPLIST, buttonId:me.getId(), open:flag};
        me.dispatchEvent(evt);
    }
};


/**
 * @constructor
 */
function ToolbarListedCommandButtonVisualization(model, toolbarVisualization) {
    ToolbarListedCommandButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
}
extend(ToolbarListedCommandButtonVisualization, ToolbarSymbolSetButtonVisualization);

ToolbarListedCommandButtonVisualization.prototype.DROPLIST_VISIALIZATION_CLASS =
    "ToolbarListedCommandDroplistVisualization";

ToolbarListedCommandButtonVisualization.prototype.PLATE_COLOR = "#FFFFFF";
ToolbarListedCommandButtonVisualization.prototype.PLATE_BORDER_COLOR_DARK = "#666666";
ToolbarListedCommandButtonVisualization.prototype.PLATE_BORDER_COLOR_GRAY = "#CCCCCC";
ToolbarListedCommandButtonVisualization.prototype.PLATE_BORDER_COLOR_LIGHT = "rgba(204, 204, 204, 0.25)";

//override
ToolbarListedCommandButtonVisualization.prototype.update = function(toolbarLeft, toolbarTop){
    var me = this;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
    var cnvToolbar = me.toolbarVisualization.getCanvas();
    var ctxToolbar = cnvToolbar.getContext("2d");
    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");
    var model = me.model;
    var objConfiguration = model.getConfiguration();

    var objHitDetails = model.getHitDetails();

    var i;
    var w = Math.ceil(model.getWidth()*rSize);
    //+1 due to lineWidth of droplist button
    var wTotal = Math.ceil((model.getTotalWidth()+1)*rSize);
    var h = Math.ceil(model.getHeight()*rSize);
    var wPlate = Math.ceil(model.PLATE_WIDTH*rSize);
    var hPlate = Math.ceil(model.PLATE_HEIGHT*rSize);
    var dx = Math.ceil((w-wPlate)/2);
    var dy = Math.ceil((h-hPlate)/2);

    if(cnv.width != wTotal || cnv.height != h){
        cnv.width = wTotal;
        cnv.height = h;
    }

    ctx.clearRect(0, 0, wTotal, h);

    ctx.beginPath();
    ctx.fillStyle = me.PLATE_COLOR;
    ctx.fillRect(dx, dy, wPlate, hPlate);

    ctx.lineWidth = Math.ceil(model.DEFAULT_LINE_WIDTH*rSize);

    ctx.beginPath();
    ctx.strokeStyle = me.PLATE_BORDER_COLOR_GRAY;
    ctx.moveTo(dx+wPlate-ctx.lineWidth, dy+ctx.lineWidth);
    ctx.lineTo(dx+ctx.lineWidth, dy+ctx.lineWidth);
    ctx.lineTo(dx+ctx.lineWidth, dy+hPlate);
    ctx.stroke();

    ctx.beginPath();
    ctx.strokeStyle = me.PLATE_BORDER_COLOR_LIGHT;
    ctx.moveTo(dx+wPlate, dy+ctx.lineWidth);
    ctx.lineTo(dx+wPlate, dy+hPlate);
    ctx.lineTo(dx+ctx.lineWidth, dy+hPlate);
    ctx.stroke();

    ctx.beginPath();
    ctx.strokeStyle = me.PLATE_BORDER_COLOR_DARK;
    ctx.moveTo(dx+wPlate, dy);
    ctx.lineTo(dx, dy);
    ctx.lineTo(dx, dy+hPlate);
    ctx.stroke();
    i = objConfiguration.aSymbol.indexOf(objConfiguration.symbolId);
    var objElement = (i == -1) ? objConfiguration.defaultElement : objConfiguration.aElement[i];
    if(objElement != null) {
        var fFS;
        if(typeof objConfiguration.fontForSelected != TYPE_UNDEFINED && objConfiguration.fontForSelected != null)
            fFS = objConfiguration.fontForSelected;
        else
            fFS = objElement;

        var fontSize = Math.ceil(fFS.size * rSize);
        ctx.beginPath();
        ctx.font =
            (fFS.italic ? "italic " : "") + (fFS.bold ? "bold " : "") +
            fontSize + "px " + fFS.face;
        ctx.fillStyle = (typeof fFS.color == TYPE_UNDEFINED) ? "#000000" : fFS.color;
        var wSample = ctx.measureText(objElement.sample).width;
        var vShift = TextFunctions.prototype.getVerticalShiftByFontSize(fontSize);
        var lHeight = TextFunctions.prototype.getLineHeightByFontSize(fontSize);
        ctx.fillText(objElement.sample, (dx + (wPlate - wSample) / 2) | 0, (dy + vShift + (hPlate - lHeight) / 2) | 0);
    }
    var wDLB = model.DEFAULT_DROPLIST_BUTTON_WIDTH*rSize;
    var c1 = me.BORDER_COLOR_LIGHT;
    var c2 = me.BORDER_COLOR_DARK;
    var xCurr = (-1.0) * rSize + w;

    ctx.lineWidth = Math.ceil(model.DEFAULT_LINE_WIDTH*rSize);

    //draw droplist depressed button outline
    if(model.isDroplistOpen()) {
        ctx.beginPath();
        ctx.fillStyle = "rgba(255, 255, 255, 0.5)";
        ctx.fillRect(xCurr, 0.5, wDLB - 1, h - 1);

        c1 = me.BORDER_COLOR_DARK;
        c2 = me.BORDER_COLOR_LIGHT;

        ctx.strokeStyle = c1;
        ctx.beginPath();
        ctx.moveTo(xCurr + wDLB - 0.5, 0.5);
        ctx.lineTo(xCurr + 0.5, 0.5);
        ctx.lineTo(xCurr + 0.5, h - 0.5);
        ctx.stroke();

        ctx.strokeStyle = c2;
        ctx.beginPath();
        ctx.moveTo(xCurr + 0.5, h - 0.5);
        ctx.lineTo(xCurr + wDLB - 0.5, h - 0.5);
        ctx.lineTo(xCurr + wDLB - 0.5, 0.5);
        ctx.stroke();
    }

    //draw droplist button symbol
    ctx.beginPath();
    var img = ToolbarButtonLibrary.prototype.getCachedSVG(me.DROPLIST_ICON_ID);
    var objSize = ToolbarButtonLibrary.prototype.getSizeById(me.DROPLIST_ICON_ID);
    ctx.setTransform(
        rSize, 0, 0, rSize,
        xCurr+(rSize*(model.DEFAULT_DROPLIST_BUTTON_WIDTH-objSize.width)-1)/2, (h-rSize*objSize.height-1)/2
    );
    ctx.drawImage(img, 0, 0);
    ctx.setTransform(1, 0, 0, 1, 0, 0);
    //
    if(model.isDroplistOpen()) {
        if(!me.droplistVisualization.isVisible())
            me.droplistVisualization.show();
        me.droplistVisualization.update();
        //update position of droplist
        if(typeof toolbarLeft == TYPE_NUMBER && typeof toolbarTop == TYPE_NUMBER)
            this.updateDroplistPosition(toolbarLeft, toolbarTop, rSize, xCurr, h);
    }else if(me.droplistVisualization.isVisible())
        me.droplistVisualization.hide();

    //draw droplist button rollover outline
    if(
        objHitDetails != null && objHitDetails.flagHit
    ) {
        if(objHitDetails.flagHitOpenButton && !model.isDroplistOpen()){
            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(xCurr + wDLB - 0.5, 0.5);
            ctx.lineTo(xCurr + 0.5, 0.5);
            ctx.lineTo(xCurr + 0.5, h - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo(xCurr + 0.5, h - 0.5);
            ctx.lineTo(xCurr + wDLB - 0.5, h - 0.5);
            ctx.lineTo(xCurr + wDLB - 0.5, 0.5);
            ctx.stroke();
        }
    }

    ctxToolbar.drawImage(cnv, 0, 0);
};

/**
 * @constructor
 */
function ToolbarListedCommandDroplistModel(parentModel){
    ToolbarListedCommandDroplistModel.superclass.constructor.call(this, parentModel);
    var me = this;
    me.flagHideOnSelect = true;
    //width not limited
    me.widthLowerLimit = 0;
}
extend(ToolbarListedCommandDroplistModel, ToolbarDroplistModel);

ToolbarListedCommandDroplistModel.prototype.PADDING_LEFT = 2;
ToolbarListedCommandDroplistModel.prototype.PADDING_RIGHT = 2;
ToolbarListedCommandDroplistModel.prototype.PADDING_TOP = 2;
ToolbarListedCommandDroplistModel.prototype.PADDING_BOTTOM = 3;
ToolbarListedCommandDroplistModel.prototype.HORIZONTAL_ALIGN = "left";
//
ToolbarListedCommandDroplistModel.prototype.getSymbolId = function(){
    return this.getParentModel().getSymbolId();
};

//
ToolbarListedCommandDroplistModel.prototype.setSymbolId = function(sId){
    return this.getParentModel().setSymbolId(sId);
};

//
ToolbarListedCommandDroplistModel.prototype.getElementsCount = function(){
    return this.getParentModel().getElementsCount();
};

//
ToolbarListedCommandDroplistModel.prototype.getElementNo = function(q){
    return this.getParentModel().getElementNo(q);
};

//
ToolbarListedCommandDroplistModel.prototype.isHideOnSelect = function(){
    return this.flagHideOnSelect;
};

//
ToolbarListedCommandDroplistModel.prototype.setHideOnSelect = function(flag){
    this.flagHideOnSelect = flag;
};

//
ToolbarListedCommandDroplistModel.prototype.getWidthLowerLimit = function(){
    return this.widthLowerLimit;
};

//
ToolbarListedCommandDroplistModel.prototype.setWidthLowerLimit = function(v){
    this.widthLowerLimit = v;
};
/**
 * @constructor
 */
function ToolbarListedCommandDroplistVisualization(_model, _toolbarVisualization){
    ToolbarListedCommandDroplistVisualization.superclass.constructor.call(this, _model, _toolbarVisualization);
    var me = this;
    //
    me.sPrevParams = "";
    //
    me.wSinglePlate = 0;
    me.hSinglePlate = 0;
    me.prevSymbolIdHit = -1;
}
//this panel should fire sample-related events
extend(ToolbarListedCommandDroplistVisualization, ToolbarDroplistVisualization);

ToolbarListedCommandDroplistVisualization.prototype.ELEMENT_PLATE_COLOR = "#FFFFF3";
ToolbarListedCommandDroplistVisualization.prototype.ELEMENT_PLATE_COLOR_OVER = "#EEF7FF";
ToolbarListedCommandDroplistVisualization.prototype.ELEMENT_PLATE_COLOR_SELECTED = "#DDE6EE";

//override
ToolbarListedCommandDroplistVisualization.prototype.show = function(){
    ToolbarListedCommandDroplistVisualization.superclass.show.call(this);
    this.prevSymbolIdHit = -1;
};

//override
ToolbarListedCommandDroplistVisualization.prototype.fMouseMove = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    var me = this;
    if(e.target == this.getCanvas()){
        var model = me.model;
        var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
        var q = me.prevSymbolIdHit;
        var n = model.getElementsCount();

        var  p = me.getPointFromEvent(e);

        if(!me.flagMouseDown) {
            if(p.x>=0 && p.x*rSize<me.wSinglePlate && p.y>=0 && p.y*rSize<me.hSinglePlate*n)
                me.prevSymbolIdHit = Math.floor(p.y*rSize/me.hSinglePlate);
            else
                me.prevSymbolIdHit = -1;
        }
        if(q != me.prevSymbolIdHit)
            me.invalidate();
    }else if(me.prevSymbolIdHit != -1){
        me.prevSymbolIdHit = -1;
        me.invalidate();
    }
};

//override
ToolbarListedCommandDroplistVisualization.prototype.fMouseDown = function(e){
    if (!ModelEvent.prototype.isProperEvent(e))
        return;
    if(e.target == this.getCanvas()){
        var me = this;
        var model = me.model;
        var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
        var q = me.prevSymbolIdHit;
        var n = model.getElementsCount();

        me.flagMouseDown = true;

        var  p = me.getPointFromEvent(e);

        if(p.x>=0 && p.x*rSize<me.wSinglePlate && p.y>=0 && p.y*rSize<me.hSinglePlate*n)
            me.prevSymbolIdHit = Math.floor(p.y*rSize/me.hSinglePlate);
        else
            me.prevSymbolIdHit = -1;

        if(me.prevSymbolIdHit>=0 && me.prevSymbolIdHit<n){
            model.setSymbolId(""+me.prevSymbolIdHit);
            if(model.isHideOnSelect())
                model.hide();
            else
                me.invalidate();
        }

        if(q != me.prevSymbolIdHit)
            me.invalidate();
    }
};

//
ToolbarListedCommandDroplistVisualization.prototype.getMaximalElementSize = function(){
    var me = this;
    var model = me.model;
    var ctx = me.cnv.getContext("2d");
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
    var obj;
    var sFont;
    var wMax = 0, wCurr;
    var hMax = 0, hCurr;
    var fontSize;
    var i, n = model.getElementsCount();

    for(i=0; i<n; i++){
        obj = model.getElementNo(i);
        fontSize = Math.ceil(obj.size*rSize);
        sFont = (obj.italic ? "italic" : "") + " " + (obj.bold ? "bold" : "") + " " + fontSize + "px " + obj.face;
        ctx.font = sFont;
        wCurr = ctx.measureText(obj.panelSample).width;
        if(wCurr>wMax)
            wMax = wCurr;
        hCurr = TextFunctions.prototype.getLineHeightByFontSize(fontSize);
        if(hCurr>hMax)
            hMax = hCurr;
    }
    return {width:wMax, height:hMax};
};

//
ToolbarListedCommandDroplistVisualization.prototype.drawElement = function(obj, sFillStyle, x, y, w, h){
    var me = this;
    var model = me.model;
    var ctx = me.cnv.getContext("2d");
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
    var xText;

    var paddingTop = model.PADDING_TOP*rSize;
    var paddingBottom = model.PADDING_BOTTOM*rSize;
    var paddingLeft = model.PADDING_LEFT*rSize;
    var paddingRight = model.PADDING_RIGHT*rSize;
    //
    var wSinglePlate = me.wSinglePlate = paddingLeft + w + paddingRight;
    var hSinglePlate = me.hSinglePlate = paddingTop + h + paddingBottom;

    var panelSample = obj.panelSample;
    var fontSize = Math.ceil(obj.size*rSize);
    var verticalShift = TextFunctions.prototype.getVerticalShiftByFontSize(fontSize);
    var sFont = (obj.italic ? "italic" : "") + " " + (obj.bold ? "bold" : "") + " " + fontSize + "px " + obj.face;
    var sColor = (typeof obj.color == TYPE_UNDEFINED) ? "#000000" : obj.color;

    ctx.beginPath();
    ctx.fillStyle = sFillStyle;
    ctx.fillRect(x|0, y|0, wSinglePlate|0, hSinglePlate|0);
    ctx.beginPath();
    ctx.font = sFont;
    ctx.fillStyle = sColor;

    switch(model.HORIZONTAL_ALIGN){
        case "left":
            xText = paddingLeft;
            break;
        case "right":
            xText = wSinglePlate-paddingLeft-ctx.measureText(panelSample).width;
            break;
        case "center":
            xText = (wSinglePlate-ctx.measureText(panelSample).width)/2;
            break;
        default :
    }
    ctx.fillText(panelSample, (x+xText)|0, (y+paddingTop+verticalShift)|0);
};

//override
ToolbarListedCommandDroplistVisualization.prototype.update = function(){
    var me = this;

    var i, n;

    var model = me.model;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();

    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");

    var symbolId = parseInt(model.getSymbolId());
    var sParams = ""+me.prevSymbolIdHit+","+rSize+","+symbolId;

    var flagChanged = (sParams != me.sPrevParams);

    if(flagChanged || !me.flagValid){
        //
        var paddingTop = model.PADDING_TOP*rSize;
        var paddingBottom = model.PADDING_BOTTOM*rSize;
        var paddingLeft = model.PADDING_LEFT*rSize;
        var paddingRight = model.PADDING_RIGHT*rSize;
        //
        var widthLowerLimit = model.getWidthLowerLimit()*rSize-(paddingLeft+paddingRight+ 6 * rSize);
        //
        var obj = me.getMaximalElementSize();
        var wMax = (widthLowerLimit>obj.width) ? widthLowerLimit : obj.width;
        var hMax = obj.height;
        n = model.getElementsCount();
        //
        var wSinglePlate = me.wSinglePlate = paddingLeft + wMax + paddingRight;
        var hSinglePlate = me.hSinglePlate = paddingTop + hMax + paddingBottom;
        //
        var wTotal = wSinglePlate;
        var hTotal = n*hSinglePlate;
        //+4*rSize for shadow +2*rSize for border
        var w = Math.ceil(wTotal + 6 * rSize);
        var h = Math.ceil(hTotal + 6 * rSize);

        if(cnv.width != w || cnv.height != h){
            cnv.width = w;
            cnv.height = h;
        }
        //
        ctx.clearRect(0, 0, w, h);
        //background and shadow
        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.SHADOW_COLOR, 0.50);
        ctx.beginPath();
        ctx.fillRect(2 * rSize, 2 * rSize, wTotal+2*rSize, hTotal+2*rSize);
        //
        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.BACKGROUND_COLOR, 1.0);
        ctx.lineWidth = 1;
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.BORDER_COLOR, 1.0);
        ctx.beginPath();
        ctx.fillRect(0, 0, wTotal, hTotal);
        ctx.strokeRect(0, 0, wTotal, hTotal);
        //
        ctx.setTransform(1, 0, 0, 1, Math.ceil(1*rSize), Math.ceil(1*rSize));
        //no-gaps fill
        ctx.fillStyle = me.ELEMENT_PLATE_COLOR;
        ctx.beginPath();
        ctx.fillRect(0, 0, wSinglePlate, (hSinglePlate*n));

        //draw list elements
        for(i=0; i<n; i++){
            obj = model.getElementNo(i);

            me.drawElement(
                obj,
                (i == me.prevSymbolIdHit) ? me.ELEMENT_PLATE_COLOR_OVER :
                    (
                        (i==symbolId) ? me.ELEMENT_PLATE_COLOR_SELECTED : me.ELEMENT_PLATE_COLOR
                    ),
                0, hSinglePlate*i, wMax, hMax
            );
        }
        ctx.setTransform(1, 0, 0, 1, 0, 0);
        //
        me.sPrevParams = sParams;
        me.flagValid = true;
    }
};
/**
 * @constructor
 */
function ToolbarFontFaceButtonModel(toolbar, objProps) {
    ToolbarFontFaceButtonModel.superclass.constructor.call(this, toolbar, objProps);
}
extend(ToolbarFontFaceButtonModel, ToolbarListedCommandButtonModel);

ToolbarFontFaceButtonModel.prototype.DROPLIST_MODEL_CLASS = "ToolbarFontFaceDroplistModel";
/**
 * @constructor
 */
function ToolbarFontFaceButtonVisualization(model, toolbarVisualization) {
    ToolbarFontFaceButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
}
extend(ToolbarFontFaceButtonVisualization, ToolbarListedCommandButtonVisualization);
/**
 * @constructor
 */
function ToolbarFontFaceDroplistModel(parentModel){
    ToolbarFontFaceDroplistModel.superclass.constructor.call(this, parentModel);
}
extend(ToolbarFontFaceDroplistModel, ToolbarListedCommandDroplistModel);

ToolbarFontFaceDroplistModel.prototype.PADDING_LEFT = 15;
ToolbarFontFaceDroplistModel.prototype.PADDING_RIGHT = 15;
ToolbarFontFaceDroplistModel.prototype.PADDING_TOP = 2;
ToolbarFontFaceDroplistModel.prototype.PADDING_BOTTOM = 3;
/**
 * @constructor
 */
function ToolbarFontSizeButtonModel(toolbar, objProps) {
    ToolbarFontSizeButtonModel.superclass.constructor.call(this, toolbar, objProps);
}
extend(ToolbarFontSizeButtonModel, ToolbarListedCommandButtonModel);

ToolbarFontSizeButtonModel.prototype.DEFAULT_WIDTH = 48;

ToolbarFontSizeButtonModel.prototype.PLATE_WIDTH = 42;
ToolbarFontSizeButtonModel.prototype.PLATE_HEIGHT = 26;

ToolbarFontSizeButtonModel.prototype.DROPLIST_MODEL_CLASS = "ToolbarFontSizeDroplistModel";

/**
 * @constructor
 */
function ToolbarFontSizeButtonVisualization(model, toolbarVisualization) {
    ToolbarFontSizeButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
}
extend(ToolbarFontSizeButtonVisualization, ToolbarListedCommandButtonVisualization);

/**
 * @constructor
 */
function ToolbarFontSizeDroplistModel(parentModel){
    ToolbarFontSizeDroplistModel.superclass.constructor.call(this, parentModel);
}
extend(ToolbarFontSizeDroplistModel, ToolbarListedCommandDroplistModel);

ToolbarFontSizeDroplistModel.prototype.PADDING_LEFT = 2;
ToolbarFontSizeDroplistModel.prototype.PADDING_RIGHT = 2;
ToolbarFontSizeDroplistModel.prototype.PADDING_TOP = 2;
ToolbarFontSizeDroplistModel.prototype.PADDING_BOTTOM = 3;
ToolbarFontSizeDroplistModel.prototype.HORIZONTAL_ALIGN = "center";
/**
 * @constructor
 */
function ToolbarTextAlignButtonModel(toolbar, objProps) {
    ToolbarTextAlignButtonModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;
    me.flagOpenListAlways = false;
    me.setDeselectOnRelease(true);
}
extend(ToolbarTextAlignButtonModel, ToolbarListedCommandButtonModel);

ToolbarTextAlignButtonModel.prototype.DROPLIST_MODEL_CLASS = "ToolbarTextAlignDroplistModel";

//
ToolbarTextAlignButtonModel.prototype.getIconIds = function(){
    var me = this;
    var obj = me.objConfiguration;
    return (obj != null && Array.isArray(obj.aIcon)) ? obj.aIcon : [];
};
/**
 * @constructor
 */
function ToolbarTextAlignButtonVisualization(model, toolbarVisualization) {
    ToolbarTextAlignButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
    var me = this;
    me.iconScale = 0.8;
    me.icon = null;
    me.objSize = null;
    me.sSymbol = "";
}
extend(ToolbarTextAlignButtonVisualization, ToolbarListedCommandButtonVisualization);

ToolbarTextAlignButtonVisualization.prototype.DROPLIST_VISIALIZATION_CLASS = "ToolbarTextAlignDroplistVisualization";

//override
ToolbarTextAlignButtonVisualization.prototype.update = function(toolbarLeft, toolbarTop){
    var me = this;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
    var cnvToolbar = me.toolbarVisualization.getCanvas();
    var ctxToolbar = cnvToolbar.getContext("2d");
    var cnv = me.cnv;
    var cnvIcon;
    var ctx = cnv.getContext("2d");
    var model = me.model;
    var objConfiguration = model.getConfiguration();

    var i, n;
    var wOne = model.getWidth()*rSize;
    var h = Math.ceil(model.getHeight()*rSize);
    var wTotal = model.getTotalWidth()*rSize;
    var w = Math.ceil(wTotal+1*rSize);
    var s;

    if(objConfiguration.symbolId != me.sSymbol){
        s = objConfiguration.aIcon[parseInt(objConfiguration.symbolId)];
        me.icon = ToolbarButtonLibrary.prototype.getCachedSVG(s);
        me.objSize = ToolbarButtonLibrary.prototype.getSizeById(s);
        me.sSymbol = objConfiguration.symbolId;
    }

    if(cnv.width != w || cnv.height != h){
        cnv.width = w;
        cnv.height = h;
    }
    ctx.clearRect(0,0,w,h);

    var xCurr = 0.5*rSize;
    var xIcon, yIcon;
    var flag;
    var objData;

    var xStart = xCurr;

    var c1;
    var c2;
    var sc = me.iconScale;
    var objHitDetails = model.getHitDetails();
    var flagSelected = model.isSelected();
    var iSelected =flagSelected ? (parseInt(objConfiguration.symbolId) != -1) : -1;

    if(flagSelected) {
        ctx.beginPath();
        ctx.fillStyle = "rgba(255, 255, 255, 0.5)";
        ctx.fillRect((xCurr|0)+0.5, 0.5, (wOne - 1)|0, (h - 1)|0);
    }

    ctx.setTransform(
        rSize*sc, 0, 0, rSize*sc,
        (rSize*(model.getWidth()-me.objSize.width*sc)/2)|0 , (rSize*(model.getHeight()-me.objSize.height*sc)/2)|0
    );
    ctx.drawImage(me.icon, 0, 0);
    ctx.setTransform(1, 0, 0, 1, 0, 0);

    if(flagSelected){
        c1 = me.BORDER_COLOR_DARK;
        c2 = me.BORDER_COLOR_LIGHT;

        ctx.beginPath();
        ctx.lineWidth = Math.ceil(model.DEFAULT_LINE_WIDTH*rSize);
        ctx.strokeStyle = c1;
        ctx.moveTo(((xCurr+wOne-1)|0)-0.5, 0.5);
        ctx.lineTo((xCurr|0)+0.5, 0.5);
        ctx.lineTo((xCurr|0)+0.5, ((h-1)|0)-0.5);
        ctx.stroke();

        ctx.strokeStyle = c2;
        ctx.beginPath();
        ctx.moveTo((xCurr|0)+0.5, ((h-1)|0)-0.5);
        ctx.lineTo(((xCurr+wOne)|0)-1-0.5, ((h-1)|0)-0.5);
        ctx.lineTo(((xCurr+wOne-1)|0)-0.5, 0.5);
        ctx.stroke();
    }

    //droplist button
    var wDLB = model.DEFAULT_DROPLIST_BUTTON_WIDTH*rSize;
    if(model.isWithDropList()) {
        xCurr = 0.5 * rSize + wOne;
        if(model.isDroplistOpen()) {
            ctx.beginPath();
            ctx.fillStyle = "rgba(255, 255, 255, 0.5)";
            ctx.fillRect((xCurr|0)+0.5, 0.5, (wDLB|0) - 1, (h|0) - 1);

            c1 = me.BORDER_COLOR_DARK;
            c2 = me.BORDER_COLOR_LIGHT;

            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.stroke();
        }
        var img = ToolbarButtonLibrary.prototype.getCachedSVG(me.DROPLIST_ICON_ID);
        var objSize = ToolbarButtonLibrary.prototype.getSizeById(me.DROPLIST_ICON_ID);
        ctx.setTransform(
            rSize, 0, 0, rSize,
            wOne+1*rSize+(rSize*(model.DEFAULT_DROPLIST_BUTTON_WIDTH-objSize.width)-1)/2, (h-rSize*objSize.height-1)/2
        );

        ctx.drawImage(img, 0, 0);
        ctx.setTransform(1, 0, 0, 1, 0, 0);
        //
        if(model.isDroplistOpen()) {
            if(!me.droplistVisualization.isVisible())
                me.droplistVisualization.show();
            me.droplistVisualization.update();
            //update position of droplist
            if(typeof toolbarLeft == TYPE_NUMBER && typeof toolbarTop == TYPE_NUMBER) {
                var dlCanvas = me.droplistVisualization.getCanvas();
                dlCanvas.style.left = "0px";
                dlCanvas.style.top = "0px";
                var dlBounds = miscFunctions.getElementBounds(dlCanvas, true);
                var xDL = toolbarLeft + model.getX()*rSize + xCurr + 0.5 - dlBounds.left;
                var yDL = toolbarTop + model.getY()*rSize + h - 0.5 - dlBounds.top;
                dlCanvas.style.left = (xDL|0)+"px";
                dlCanvas.style.top = (yDL|0)+"px";
            }
        }else if(me.droplistVisualization.isVisible())
            me.droplistVisualization.hide();
    }

    if(
        objHitDetails != null && objHitDetails.flagHit
    ){
        c1 = me.BORDER_COLOR_LIGHT;
        c2 = me.BORDER_COLOR_DARK;
        ctx.lineWidth = Math.ceil(model.DEFAULT_LINE_WIDTH*rSize);

        if(model.isEnabled() && objHitDetails.buttonNumber != -1 && objHitDetails.buttonNumber != iSelected) {
            xCurr = 0.5 * rSize;

            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(((xCurr + wOne)|0) - 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wOne)|0) - 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wOne)|0) - 0.5, 0.5);
            ctx.stroke();
        }else if(objHitDetails.flagHitOpenButton && !model.isDroplistOpen()){
            xCurr = 0.5 * rSize + wOne;

            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.stroke();
        }
    }

    ctxToolbar.drawImage(cnv, 0, 0);
};
/**
 * @constructor
 */
function ToolbarTextAlignDroplistModel(parentModel){
    ToolbarTextAlignDroplistModel.superclass.constructor.call(this, parentModel);
    var me = this;
    me.showcaseWidthMax = 1;
    me.showcaseHeightMax = 0;
}
extend(ToolbarTextAlignDroplistModel, ToolbarSymbolSetDroplistModel);

ToolbarTextAlignDroplistModel.prototype.DEFAULT_PADDING = 2;

//
ToolbarTextAlignDroplistModel.prototype.getIconIds = function () {
    return this.getParentModel().getIconIds();
};
/**
 * @constructor
 */
function ToolbarTextAlignDroplistVisualization(_model, _mainInterfaceVisualization){
    ToolbarTextAlignDroplistVisualization.superclass.constructor.call(this, _model, _mainInterfaceVisualization);
    var me = this;
    me.iconScale = 0.8;
    me.aIcon = [];
    me.aIconSize = [];
}
extend(ToolbarTextAlignDroplistVisualization, ToolbarSymbolSetDroplistVisualization);

//override
ToolbarTextAlignDroplistVisualization.prototype.update = function(){
    var me = this;
    var i, j, ii, n;

    var model = me.model;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();

    me.aSymbolId = model.getSymbolIds();
    var flagChanged = model.isShowcaseChanged();
    if(flagChanged){
        me.aIcon.splice(0);
        me.aIconSize.splice(0);
        me.aSymbolId = model.getIconIds();
        var aSymbolId = me.aSymbolId;
        n = aSymbolId.length;
        for(i=0; i<n; i++) {
            me.aIcon.push(ToolbarButtonLibrary.prototype.getCachedSVG(aSymbolId[i]));
            me.aIconSize.push(ToolbarButtonLibrary.prototype.getSizeById(aSymbolId[i]));
        }
    }

    var p = model.getShowcaseWidth();
    var q = model.getShowcaseHeight();

    var pSample = model.DEFAULT_PADDING;
    var sSample = model.DEFAULT_SPACING;
    var wSample = model.DEFAULT_SAMPLE_WIDTH;
    var hSample = model.DEFAULT_SAMPLE_HEIGHT;

    var wIcons = Math.ceil((p*wSample+(p-1)*sSample)*rSize);
    var hIcons = Math.ceil((q*hSample+(q-1)*sSample)*rSize);

    if(wIcons != me.cnvIcons.width || hIcons != me.cnvIcons.height){
        me.cnvIcons.width = me.cnvHighlight.width = me.cnvSubstrate.width = wIcons;
        me.cnvIcons.height = me.cnvHighlight.height = me.cnvSubstrate.height = hIcons;
        flagChanged = true;
    }

    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");

    if(flagChanged || !me.flagValid) {
        me.cnvIcons.getContext("2d").clearRect(0, 0, wIcons, hIcons);
        me.cnvSubstrate.getContext("2d").clearRect(0, 0, wIcons, hIcons);

        var sc = me.iconScale;
        var ctxIcons = me.cnvIcons.getContext("2d");
        var aIcon = me.aIcon;
        var xCurr = 0;
        var yCurr = 0;
        var img, objSize;

        n = aIcon.length;

        for (j = 0, ii = 0; (j < q) && (ii < n); j++) {
            for (i = 0; (i < p) && (ii < n); i++, ii++) {
                img = me.aIcon[ii];
                objSize = me.aIconSize[ii];
                ctxIcons.setTransform(
                    rSize*sc, 0, 0, rSize*sc,
                    (xCurr + (rSize*(wSample-objSize.width*sc)/2))|0,
                    (yCurr + (rSize*(hSample-objSize.height*sc)/2))|0
                );
                ctxIcons.drawImage(img, 0, 0);
                xCurr += (wSample+sSample) * rSize;
            }
            xCurr = 0;
            yCurr += (hSample+sSample) * rSize;
        }
        ctxIcons.setTransform(1, 0, 0, 1, 0, 0);
    }

    if(me.rowHit != me.prevRowHit || me.columnHit != me.prevColumnHit){
        flagChanged = true;
        me.prevColumnHit = me.columnHit;
        me.prevRowHit = me.rowHit;

        var ctxHighlight = me.cnvHighlight.getContext("2d");
        ctxHighlight.clearRect(0, 0, wIcons, hIcons);
        if(me.rowHit!=-1 && me.columnHit!=-1) {
            ctxHighlight.setTransform(rSize, 0, 0, rSize, 0, 0);
            ctxHighlight.beginPath();
            ctxHighlight.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.HIGHLIGHT_COLOR, me.HIGHLIGHT_ALPHA);
            ctxHighlight.fillRect(me.columnHit*(wSample+sSample), me.rowHit*(hSample+sSample), wSample, hSample);
            ctxHighlight.setTransform(1, 0, 0, 1, 0, 0);
        }
    }

    if(flagChanged){
        //background and shadow
        var wTotal = Math.ceil((2 * pSample + p * wSample) * rSize);
        var hTotal = Math.ceil((2 * pSample + q * hSample) * rSize);

        //+2*rSize for shadow
        cnv.width = Math.ceil(wTotal + 2 * rSize);
        cnv.height = Math.ceil(hTotal + 2 * rSize);

        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.SHADOW_COLOR, 0.50);
        ctx.beginPath();
        ctx.fillRect(2 * rSize, 2 * rSize, wTotal, hTotal);

        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.BACKGROUND_COLOR, 1.0);
        ctx.lineWidth = 1;
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.BORDER_COLOR, 1.0);
        ctx.beginPath();
        ctx.fillRect(0, 0, wTotal, hTotal);
        ctx.strokeRect(0, 0, wTotal, hTotal);
        //
        ctx.drawImage(me.cnvSubstrate, pSample*rSize, pSample*rSize);
        ctx.drawImage(me.cnvHighlight, pSample*rSize, pSample*rSize);
        ctx.drawImage(me.cnvIcons, pSample*rSize, pSample*rSize);
        //
        me.flagValid = true;
    }
};
/**
 * @constructor
 */
function ToolbarShowHiddenButtonModel(toolbar, objProps) {
    ToolbarShowHiddenButtonModel.superclass.constructor.call(this, toolbar, objProps);
    this.droplistAlign = this.DROPLIST_ALIGN_RIGHT;
}
extend(ToolbarShowHiddenButtonModel, ToolbarListedCommandButtonModel);

ToolbarShowHiddenButtonModel.prototype.PLATE_WIDTH = 160;
ToolbarShowHiddenButtonModel.prototype.PLATE_HEIGHT = ToolbarButtonModel.prototype.DEFAULT_HEIGHT;//26;
ToolbarShowHiddenButtonModel.prototype.DROPLIST_MODEL_CLASS = "ToolbarShowHiddenDroplistModel";

//override
ToolbarShowHiddenButtonModel.prototype.getSymbolId = function(){
    return null;
};
/**
 * @constructor
 */
function ToolbarShowHiddenButtonVisualization(model, toolbarVisualization) {
    ToolbarShowHiddenButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
}
extend(ToolbarShowHiddenButtonVisualization, ToolbarListedCommandButtonVisualization);

ToolbarShowHiddenButtonVisualization.prototype.DROPLIST_VISIALIZATION_CLASS = "ToolbarShowHiddenDroplistVisualization";

/**
 * @constructor
 */
function ToolbarShowHiddenDroplistModel(parentModel){
    ToolbarShowHiddenDroplistModel.superclass.constructor.call(this, parentModel);
    var me = this;
    me.setHideOnSelect(document.localization.getParameter("droplists.show-hidden.hide-on-select", false));
    if(!document.localization.getParameter("droplists.show-hidden.shrinking-width", false))
        me.setWidthLowerLimit(me.getParentModel().getTotalWidth());
}
extend(ToolbarShowHiddenDroplistModel, ToolbarListedCommandDroplistModel);

ToolbarShowHiddenDroplistModel.prototype.PADDING_LEFT = 5;
ToolbarShowHiddenDroplistModel.prototype.PADDING_RIGHT = 5;
ToolbarShowHiddenDroplistModel.prototype.PADDING_TOP = 2;
ToolbarShowHiddenDroplistModel.prototype.PADDING_BOTTOM = 3;

ToolbarShowHiddenDroplistModel.prototype.CHECK_MARK_GAP = 2;
ToolbarShowHiddenDroplistModel.prototype.CHECK_MARK_WIDTH = 16;
/**
 * @constructor
 */
function ToolbarShowHiddenDroplistVisualization(_model, _toolbarVisualization){
    ToolbarShowHiddenDroplistVisualization.superclass.constructor.call(this, _model, _toolbarVisualization);
    var me = this;
    me.cnvCheck = document.createElement("CANVAS");
    me.cnvCheck.width = me.cnvCheck.height = 1;
    me.toolbarVisualization.addEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:me, listener:me.toolbarVisualizationListener}, false
    )
}
extend(ToolbarShowHiddenDroplistVisualization, ToolbarListedCommandDroplistVisualization);

ToolbarShowHiddenDroplistVisualization.prototype.CHECK_MARK_ID = "gr.check";
ToolbarShowHiddenDroplistVisualization.prototype.INTERLINE_HEIGHT_RATIO = 1.4;

//
ToolbarShowHiddenDroplistVisualization.prototype.getMaximalElementSize = function(){
    var me = this;
    var model = me.model;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();

    var objSize = ToolbarShowHiddenDroplistVisualization.superclass.getMaximalElementSize.call(me);
    objSize.height *= me.INTERLINE_HEIGHT_RATIO;

    //add check mark width
    if(me.hasVisible())
        objSize.width += (model.CHECK_MARK_GAP+model.CHECK_MARK_WIDTH)*rSize;

    return objSize;
};

//
ToolbarShowHiddenDroplistVisualization.prototype.drawElement = function(obj, sFillStyle, x, y, w, h){
    var me = this;
    var model = me.model;
    var ctx = me.cnv.getContext("2d");
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
    var xText;

    var paddingTop = model.PADDING_TOP*rSize;
    var paddingBottom = model.PADDING_BOTTOM*rSize;
    var paddingLeft = model.PADDING_LEFT*rSize;
    var paddingRight = model.PADDING_RIGHT*rSize;
    //
    var wSinglePlate = me.wSinglePlate = paddingLeft + w + paddingRight;
    var hSinglePlate = me.hSinglePlate = paddingTop + h + paddingBottom;

    var panelSample = obj.panelSample;
    var fontSize = Math.ceil(obj.size*rSize);
    var verticalShift = TextFunctions.prototype.getVerticalShiftByFontSize(fontSize);
    var lineHeight = TextFunctions.prototype.getLineHeightByFontSize(fontSize);
    var sFont = (obj.italic ? "italic" : "") + " " + (obj.bold ? "bold" : "") + " " + fontSize + "px " + obj.face;

    var flagHasVisible = me.hasVisible();
    var xCheckMark;
    var checkMarkWidth;
    if(flagHasVisible){
        xCheckMark = paddingLeft;
        checkMarkWidth = (model.CHECK_MARK_GAP+model.CHECK_MARK_WIDTH)*rSize;
    }else{
        xCheckMark = 0;
        checkMarkWidth = 0;
    }

    ctx.beginPath();
    ctx.fillStyle = sFillStyle;
    ctx.fillRect(x|0, y|0, wSinglePlate|0, hSinglePlate|0);
    ctx.beginPath();
    ctx.font = sFont;
    ctx.fillStyle = "#000000";

    switch(model.HORIZONTAL_ALIGN){
        case "left":
            xText = paddingLeft;
            break;
        case "right":
            xText = wSinglePlate-paddingLeft-ctx.measureText(panelSample).width;
            break;
        case "center":
            xText = (wSinglePlate-ctx.measureText(panelSample).width)/2;
            break;
        default :
    }

    xText += checkMarkWidth;

    /*ctx.save();
    ctx.beginPath();
    ctx.rect((x+xText)|0, 0, w-checkMarkWidth, me.cnv.height);
    ctx.clip();*/
    ctx.fillText(panelSample, (x+xText)|0, (y+/*paddingTop*/((h-lineHeight)*0.5)+verticalShift)|0);
    /*ctx.restore();*/

    if(obj.visible){
        //add check
        var wC = Math.ceil(model.CHECK_MARK_WIDTH*rSize);
        var hC = Math.ceil(h);
        var cnvCheck = me.cnvCheck;
        if(cnvCheck.width != wC || cnvCheck.height != hC){
            cnvCheck.width = wC;
            cnvCheck.height = hC;
            var ctxCheck = cnvCheck.getContext("2d");
            ctxCheck.clearRect(0, 0, wC, hC);
            var img = ToolbarButtonLibrary.prototype.getCachedSVG(me.CHECK_MARK_ID);
            var objSize = ToolbarButtonLibrary.prototype.getSizeById(me.CHECK_MARK_ID);
            ctxCheck.setTransform(
                rSize, 0, 0, rSize, (wC-rSize*objSize.width-1)/2, (hC-rSize*objSize.height-1)/2
            );
            ctxCheck.drawImage(img, 0, 0);
            ctxCheck.setTransform(1, 0, 0, 1, 0, 0);
        }

        //ctx.drawImage(cnvCheck, (x+paddingLeft+w-cnvCheck.width)|0, y|0);
        ctx.drawImage(cnvCheck, xCheckMark|0, y|0);
    }
};

//
ToolbarShowHiddenDroplistVisualization.prototype.hasVisible = function(){
    var model = this.model;
    var i, n = model.getElementsCount();
    for(i=0; i<n; i++)
        if(model.getElementNo(i).visible)
            break;
    return (i<n);
};

//
ToolbarShowHiddenDroplistVisualization.prototype.deactivate = function(){
    ToolbarShowHiddenDroplistVisualization.superclass.deactivate.call(this);
    this.toolbarVisualization.removeEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:this, listener:this.toolbarVisualizationListener}, false
    )
};

//
ToolbarShowHiddenDroplistVisualization.prototype.toolbarVisualizationListener = function(e){
    //this invalidate() causes update of the droplist without its being closed
    //however, in ToolbarShowHiddenButtonVisualization, in toolbarVisualizationListener, the droplist is closed compulsorily
    switch(e.type){
        case ModelEvent.prototype.RESET_SCENE:
            this.invalidate();
            break;
        default:
    }
};
/**
 * @constructor
 */
function ToolbarZoomButtonModel(toolbar, objProps) {
    ToolbarZoomButtonModel.superclass.constructor.call(this, toolbar, objProps);
}
extend(ToolbarZoomButtonModel, ToolbarListedCommandButtonModel);

ToolbarZoomButtonModel.prototype.DEFAULT_WIDTH = 60;

ToolbarZoomButtonModel.prototype.PLATE_WIDTH = 56;
ToolbarZoomButtonModel.prototype.PLATE_HEIGHT = 26;

ToolbarZoomButtonModel.prototype.DROPLIST_MODEL_CLASS = "ToolbarZoomDroplistModel";


/**
 * @constructor
 */
function ToolbarZoomButtonVisualization(model, toolbarVisualization) {
    ToolbarZoomButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
}
extend(ToolbarZoomButtonVisualization, ToolbarListedCommandButtonVisualization);

/**
 * @constructor
 */
function ToolbarZoomDroplistModel(parentModel){
    ToolbarZoomDroplistModel.superclass.constructor.call(this, parentModel);
}
extend(ToolbarZoomDroplistModel, ToolbarListedCommandDroplistModel);

ToolbarZoomDroplistModel.prototype.PADDING_LEFT = 2;
ToolbarZoomDroplistModel.prototype.PADDING_RIGHT = 2;
ToolbarZoomDroplistModel.prototype.PADDING_TOP = 2;
ToolbarZoomDroplistModel.prototype.PADDING_BOTTOM = 3;
ToolbarZoomDroplistModel.prototype.HORIZONTAL_ALIGN = "right";

/**
 * @constructor
 */
function ToolbarTextAlignButtonModel(toolbar, objProps) {
    ToolbarTextAlignButtonModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;
    me.flagOpenListAlways = false;
    me.setDeselectOnRelease(true);
}
extend(ToolbarTextAlignButtonModel, ToolbarListedCommandButtonModel);

ToolbarTextAlignButtonModel.prototype.DROPLIST_MODEL_CLASS = "ToolbarTextAlignDroplistModel";

//
ToolbarTextAlignButtonModel.prototype.getIconIds = function(){
    var me = this;
    var obj = me.objConfiguration;
    return (obj != null && Array.isArray(obj.aIcon)) ? obj.aIcon : [];
};
/**
 * @constructor
 */
function ToolbarTextAlignButtonVisualization(model, toolbarVisualization) {
    ToolbarTextAlignButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
    var me = this;
    me.iconScale = 0.8;
    me.icon = null;
    me.objSize = null;
    me.sSymbol = "";
}
extend(ToolbarTextAlignButtonVisualization, ToolbarListedCommandButtonVisualization);

ToolbarTextAlignButtonVisualization.prototype.DROPLIST_VISIALIZATION_CLASS = "ToolbarTextAlignDroplistVisualization";

//override
ToolbarTextAlignButtonVisualization.prototype.update = function(toolbarLeft, toolbarTop){
    var me = this;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
    var cnvToolbar = me.toolbarVisualization.getCanvas();
    var ctxToolbar = cnvToolbar.getContext("2d");
    var cnv = me.cnv;
    var cnvIcon;
    var ctx = cnv.getContext("2d");
    var model = me.model;
    var objConfiguration = model.getConfiguration();

    var i, n;
    var wOne = model.getWidth()*rSize;
    var h = Math.ceil(model.getHeight()*rSize);
    var wTotal = model.getTotalWidth()*rSize;
    var w = Math.ceil(wTotal+1*rSize);
    var s;

    if(objConfiguration.symbolId != me.sSymbol){
        s = objConfiguration.aIcon[parseInt(objConfiguration.symbolId)];
        me.icon = ToolbarButtonLibrary.prototype.getCachedSVG(s);
        me.objSize = ToolbarButtonLibrary.prototype.getSizeById(s);
        me.sSymbol = objConfiguration.symbolId;
    }

    if(cnv.width != w || cnv.height != h){
        cnv.width = w;
        cnv.height = h;
    }
    ctx.clearRect(0,0,w,h);

    var xCurr = 0.5*rSize;
    var xIcon, yIcon;
    var flag;
    var objData;

    var xStart = xCurr;

    var c1;
    var c2;
    var sc = me.iconScale;
    var objHitDetails = model.getHitDetails();
    var flagSelected = model.isSelected();
    var iSelected =flagSelected ? (parseInt(objConfiguration.symbolId) != -1) : -1;

    if(flagSelected) {
        ctx.beginPath();
        ctx.fillStyle = "rgba(255, 255, 255, 0.5)";
        ctx.fillRect((xCurr|0)+0.5, 0.5, (wOne - 1)|0, (h - 1)|0);
    }

    ctx.setTransform(
        rSize*sc, 0, 0, rSize*sc,
        (rSize*(model.getWidth()-me.objSize.width*sc)/2)|0 , (rSize*(model.getHeight()-me.objSize.height*sc)/2)|0
    );
    ctx.drawImage(me.icon, 0, 0);
    ctx.setTransform(1, 0, 0, 1, 0, 0);

    if(flagSelected){
        c1 = me.BORDER_COLOR_DARK;
        c2 = me.BORDER_COLOR_LIGHT;

        ctx.beginPath();
        ctx.lineWidth = Math.ceil(model.DEFAULT_LINE_WIDTH*rSize);
        ctx.strokeStyle = c1;
        ctx.moveTo(((xCurr+wOne-1)|0)-0.5, 0.5);
        ctx.lineTo((xCurr|0)+0.5, 0.5);
        ctx.lineTo((xCurr|0)+0.5, ((h-1)|0)-0.5);
        ctx.stroke();

        ctx.strokeStyle = c2;
        ctx.beginPath();
        ctx.moveTo((xCurr|0)+0.5, ((h-1)|0)-0.5);
        ctx.lineTo(((xCurr+wOne)|0)-1-0.5, ((h-1)|0)-0.5);
        ctx.lineTo(((xCurr+wOne-1)|0)-0.5, 0.5);
        ctx.stroke();
    }

    //droplist button
    var wDLB = model.DEFAULT_DROPLIST_BUTTON_WIDTH*rSize;
    if(model.isWithDropList()) {
        xCurr = 0.5 * rSize + wOne;
        if(model.isDroplistOpen()) {
            ctx.beginPath();
            ctx.fillStyle = "rgba(255, 255, 255, 0.5)";
            ctx.fillRect((xCurr|0)+0.5, 0.5, (wDLB|0) - 1, (h|0) - 1);

            c1 = me.BORDER_COLOR_DARK;
            c2 = me.BORDER_COLOR_LIGHT;

            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.stroke();
        }
        var img = ToolbarButtonLibrary.prototype.getCachedSVG(me.DROPLIST_ICON_ID);
        var objSize = ToolbarButtonLibrary.prototype.getSizeById(me.DROPLIST_ICON_ID);
        ctx.setTransform(
            rSize, 0, 0, rSize,
            wOne+1*rSize+(rSize*(model.DEFAULT_DROPLIST_BUTTON_WIDTH-objSize.width)-1)/2, (h-rSize*objSize.height-1)/2
        );

        ctx.drawImage(img, 0, 0);
        ctx.setTransform(1, 0, 0, 1, 0, 0);
        //
        if(model.isDroplistOpen()) {
            if(!me.droplistVisualization.isVisible())
                me.droplistVisualization.show();
            me.droplistVisualization.update();
            //update position of droplist
            if(typeof toolbarLeft == TYPE_NUMBER && typeof toolbarTop == TYPE_NUMBER) {
                var dlCanvas = me.droplistVisualization.getCanvas();
                dlCanvas.style.left = "0px";
                dlCanvas.style.top = "0px";
                var dlBounds = miscFunctions.getElementBounds(dlCanvas, true);
                var xDL = toolbarLeft + model.getX()*rSize + xCurr + 0.5 - dlBounds.left;
                var yDL = toolbarTop + model.getY()*rSize + h - 0.5 - dlBounds.top;
                dlCanvas.style.left = (xDL|0)+"px";
                dlCanvas.style.top = (yDL|0)+"px";
            }
        }else if(me.droplistVisualization.isVisible())
            me.droplistVisualization.hide();
    }

    if(
        objHitDetails != null && objHitDetails.flagHit
    ){
        c1 = me.BORDER_COLOR_LIGHT;
        c2 = me.BORDER_COLOR_DARK;
        ctx.lineWidth = Math.ceil(model.DEFAULT_LINE_WIDTH*rSize);

        if(model.isEnabled() && objHitDetails.buttonNumber != -1 && objHitDetails.buttonNumber != iSelected) {
            xCurr = 0.5 * rSize;

            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(((xCurr + wOne)|0) - 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wOne)|0) - 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wOne)|0) - 0.5, 0.5);
            ctx.stroke();
        }else if(objHitDetails.flagHitOpenButton && !model.isDroplistOpen()){
            xCurr = 0.5 * rSize + wOne;

            ctx.strokeStyle = c1;
            ctx.beginPath();
            ctx.moveTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, 0.5);
            ctx.lineTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.stroke();

            ctx.strokeStyle = c2;
            ctx.beginPath();
            ctx.moveTo((xCurr|0) + 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, (h|0) - 0.5);
            ctx.lineTo(((xCurr + wDLB)|0) - 0.5, 0.5);
            ctx.stroke();
        }
    }

    ctxToolbar.drawImage(cnv, 0, 0);
};
/**
 * @constructor
 */
function ToolbarTextAlignDroplistModel(parentModel){
    ToolbarTextAlignDroplistModel.superclass.constructor.call(this, parentModel);
    var me = this;
    me.showcaseWidthMax = 1;
    me.showcaseHeightMax = 0;
}
extend(ToolbarTextAlignDroplistModel, ToolbarSymbolSetDroplistModel);

ToolbarTextAlignDroplistModel.prototype.DEFAULT_PADDING = 2;

//
ToolbarTextAlignDroplistModel.prototype.getIconIds = function () {
    return this.getParentModel().getIconIds();
};
/**
 * @constructor
 */
function ToolbarTextAlignDroplistVisualization(_model, _mainInterfaceVisualization){
    ToolbarTextAlignDroplistVisualization.superclass.constructor.call(this, _model, _mainInterfaceVisualization);
    var me = this;
    me.iconScale = 0.8;
    me.aIcon = [];
    me.aIconSize = [];
}
extend(ToolbarTextAlignDroplistVisualization, ToolbarSymbolSetDroplistVisualization);

//override
ToolbarTextAlignDroplistVisualization.prototype.update = function(){
    var me = this;
    var i, j, ii, n;

    var model = me.model;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();

    me.aSymbolId = model.getSymbolIds();
    var flagChanged = model.isShowcaseChanged();
    if(flagChanged){
        me.aIcon.splice(0);
        me.aIconSize.splice(0);
        me.aSymbolId = model.getIconIds();
        var aSymbolId = me.aSymbolId;
        n = aSymbolId.length;
        for(i=0; i<n; i++) {
            me.aIcon.push(ToolbarButtonLibrary.prototype.getCachedSVG(aSymbolId[i]));
            me.aIconSize.push(ToolbarButtonLibrary.prototype.getSizeById(aSymbolId[i]));
        }
    }

    var p = model.getShowcaseWidth();
    var q = model.getShowcaseHeight();

    var pSample = model.DEFAULT_PADDING;
    var sSample = model.DEFAULT_SPACING;
    var wSample = model.DEFAULT_SAMPLE_WIDTH;
    var hSample = model.DEFAULT_SAMPLE_HEIGHT;

    var wIcons = Math.ceil((p*wSample+(p-1)*sSample)*rSize);
    var hIcons = Math.ceil((q*hSample+(q-1)*sSample)*rSize);

    if(wIcons != me.cnvIcons.width || hIcons != me.cnvIcons.height){
        me.cnvIcons.width = me.cnvHighlight.width = me.cnvSubstrate.width = wIcons;
        me.cnvIcons.height = me.cnvHighlight.height = me.cnvSubstrate.height = hIcons;
        flagChanged = true;
    }

    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");

    if(flagChanged || !me.flagValid) {
        me.cnvIcons.getContext("2d").clearRect(0, 0, wIcons, hIcons);
        me.cnvSubstrate.getContext("2d").clearRect(0, 0, wIcons, hIcons);

        var sc = me.iconScale;
        var ctxIcons = me.cnvIcons.getContext("2d");
        var aIcon = me.aIcon;
        var xCurr = 0;
        var yCurr = 0;
        var img, objSize;

        n = aIcon.length;

        for (j = 0, ii = 0; (j < q) && (ii < n); j++) {
            for (i = 0; (i < p) && (ii < n); i++, ii++) {
                img = me.aIcon[ii];
                objSize = me.aIconSize[ii];
                ctxIcons.setTransform(
                    rSize*sc, 0, 0, rSize*sc,
                    (xCurr + (rSize*(wSample-objSize.width*sc)/2))|0,
                    (yCurr + (rSize*(hSample-objSize.height*sc)/2))|0
                );
                ctxIcons.drawImage(img, 0, 0);
                xCurr += (wSample+sSample) * rSize;
            }
            xCurr = 0;
            yCurr += (hSample+sSample) * rSize;
        }
        ctxIcons.setTransform(1, 0, 0, 1, 0, 0);
    }

    if(me.rowHit != me.prevRowHit || me.columnHit != me.prevColumnHit){
        flagChanged = true;
        me.prevColumnHit = me.columnHit;
        me.prevRowHit = me.rowHit;

        var ctxHighlight = me.cnvHighlight.getContext("2d");
        ctxHighlight.clearRect(0, 0, wIcons, hIcons);
        if(me.rowHit!=-1 && me.columnHit!=-1) {
            ctxHighlight.setTransform(rSize, 0, 0, rSize, 0, 0);
            ctxHighlight.beginPath();
            ctxHighlight.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.HIGHLIGHT_COLOR, me.HIGHLIGHT_ALPHA);
            ctxHighlight.fillRect(me.columnHit*(wSample+sSample), me.rowHit*(hSample+sSample), wSample, hSample);
            ctxHighlight.setTransform(1, 0, 0, 1, 0, 0);
        }
    }

    if(flagChanged){
        //background and shadow
        var wTotal = Math.ceil((2 * pSample + p * wSample) * rSize);
        var hTotal = Math.ceil((2 * pSample + q * hSample) * rSize);

        //+2*rSize for shadow
        cnv.width = Math.ceil(wTotal + 2 * rSize);
        cnv.height = Math.ceil(hTotal + 2 * rSize);

        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.SHADOW_COLOR, 0.50);
        ctx.beginPath();
        ctx.fillRect(2 * rSize, 2 * rSize, wTotal, hTotal);

        ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.BACKGROUND_COLOR, 1.0);
        ctx.lineWidth = 1;
        ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.BORDER_COLOR, 1.0);
        ctx.beginPath();
        ctx.fillRect(0, 0, wTotal, hTotal);
        ctx.strokeRect(0, 0, wTotal, hTotal);
        //
        ctx.drawImage(me.cnvSubstrate, pSample*rSize, pSample*rSize);
        ctx.drawImage(me.cnvHighlight, pSample*rSize, pSample*rSize);
        ctx.drawImage(me.cnvIcons, pSample*rSize, pSample*rSize);
        //
        me.flagValid = true;
    }
};
/**
 * @constructor
 */
function ToolbarColorButtonModel(toolbar, objProps) {
    ToolbarColorButtonModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;
    //
    me.symbolId = null;
    me.colorProperty = null;
    me.alphaProperty = null;
    me.flagDroplistAlwaysEnabled = true;
    me.palette = new ToolbarColorPaletteModel(me);
    me.colorOnOpen = -1;
    //
    me.setDeselectOnRelease(true);
}
extend(ToolbarColorButtonModel, ToolbarSymbolSetButtonModel);

//no droplist in this class; palette instead
ToolbarColorButtonModel.prototype.DROPLIST_MODEL_CLASS = null;

//override
ToolbarColorButtonModel.prototype.getPropertyForSymbolNo = function(s, q){
    if(s != "iconColor")
        return ToolbarColorButtonModel.superclass.getPropertyForSymbolNo.call(this, s, q);
    else{
        var objConfiguration = this.getConfiguration();
        if(typeof objConfiguration.aRGB != TYPE_UNDEFINED && Array.isArray(objConfiguration.aRGB)){
            var aRGB =  objConfiguration.aRGB;
            return (q>=0 && q<aRGB.length) ? aRGB[q] : null;
        }else
            return null;
    }
};

//
ToolbarColorButtonModel.prototype.getPalette = function(){
    return this.palette;
};

//override
ToolbarColorButtonModel.prototype.setConfiguration = function(obj){
    var me = this;
    me.objConfiguration = {
        aSymbol: obj.aSymbol.slice(0),
        flagSeparateButtons: obj.flagSeparateButtons,
        iconAlpha: (typeof obj.alpha == TYPE_NUMBER) ? obj.alpha :100,
        iconColor: obj.color,
        symbolId: me.symbolId,
        aRGB: obj.aRGB.slice(0)
    };
    me.palette.init();
};

//
ToolbarColorButtonModel.prototype.setColorAndAlpha = function(color, alpha){
    var me = this;
    var objConfiguration = me.objConfiguration;
    objConfiguration.iconAlpha = (typeof alpha == TYPE_NUMBER) ? alpha :100;
    objConfiguration.iconColor = color;
    me.toolbar.invalidate();
    var evt = new ModelEvent(ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, true, true);
    if(evt!=null) {
        evt.data = {
            subType:ModelEvent.prototype.PALETTE_COLOR_SET, buttonId:me.getId(),
            propertyColor:me.colorProperty, propertyAlpha:me.alphaProperty,
            color:color, alpha:alpha
        };
        me.dispatchEvent(evt);
    }
};

//
ToolbarColorButtonModel.prototype.resetColor = function(){
    this.setColorAndAlpha(this.colorOnOpen, this.getIconAlpha());
};

//
ToolbarColorButtonModel.prototype.getSampleColors = function(obj){
    return this.objConfiguration.aRGB;
};

//
ToolbarColorButtonModel.prototype.getInitialColor = function(){
    return this.colorOnOpen;
};

//override
ToolbarColorButtonModel.prototype.setDroplistOpen = function(flag){
    ToolbarColorButtonModel.superclass.setDroplistOpen.call(this, flag);
    var me = this;
    var evt = new ModelEvent(ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, true, true);
    if(evt!=null) {
        evt.data = {subType:ModelEvent.prototype.TOGGLE_DROPLIST, buttonId:me.getId(), open:flag};
        me.dispatchEvent(evt);
    }
    if(flag){
        me.colorOnOpen = me.getIconColor();
    }
};

//override
ToolbarColorButtonModel.prototype.deactivate = function(){
    ToolbarColorButtonModel.superclass.deactivate.call(this);
    this.palette.deactivate();
};
/**
 * @constructor
 */
function ToolbarColorButtonVisualization(model, toolbarVisualization) {
    ToolbarColorButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
}
extend(ToolbarColorButtonVisualization, ToolbarSymbolSetButtonVisualization);

ToolbarColorButtonVisualization.prototype.DROPLIST_VISIALIZATION_CLASS = "ToolbarColorPaletteVisualization";
ToolbarColorButtonVisualization.prototype.GET_DROPLIST_METHOD = "getPalette";

//override
ToolbarColorButtonVisualization.prototype.getSelectedNumber = function(){
    return this.getSelectedVisualizationNumber();
};

//override
ToolbarColorButtonVisualization.prototype.getSelectedVisualizationNumber = function(){
    var model = this.model;
    var objHitDetails = model.getHitDetails();
    return (model.isSelected() && objHitDetails != null && typeof objHitDetails.buttonNumber == TYPE_NUMBER) ? objHitDetails.buttonNumber : -1;
};
/**
 * @constructor
 */
function ToolbarTextModel(toolbar, objProps){
    ToolbarTextModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;

}
extend(ToolbarTextModel, ToolbarControlModel);

/**
 * @constructor
 */
function ToolbarTextVisualization(model, toolbarVisualization){
    var me = this;
    me.flagValid = true;
    me.model = model;
    me.toolbarVisualization = toolbarVisualization;
    //
    me.cnv = document.createElement("CANVAS");
}
extend(ToolbarTextVisualization, Object);

//
ToolbarTextVisualization.prototype.update = function(){
    var me = this;
    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");

}
/**
 * @constructor
 */
function ToolbarTextButtonModel(toolbar, objProps){
    ToolbarTextButtonModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;
    me.objProps.hAlign = "center";
    me.objProps.vAlign = "middle";
    me.width = objProps.width;
    me.height = objProps.height;

}
extend(ToolbarTextButtonModel, ToolbarButtonModel);

ToolbarTextButtonModel.prototype.DEFAULT_BORDER_WIDTH = 2;

//
ToolbarTextButtonModel.prototype.getCaption = function(){
    return this.objProps.caption;
};

//
ToolbarTextButtonModel.prototype.getFontFace = function(){
    return this.objProps.fontFace;
};

//
ToolbarTextButtonModel.prototype.getFontSize = function(){
    return this.objProps.fontSize;
};

//
ToolbarTextButtonModel.prototype.getFontColor = function(){
    return this.objProps.fontColor;
};

//
ToolbarTextButtonModel.prototype.getFontAlpha = function(){
    return this.objProps.fontAlpha;
};

/**
 * @constructor
 */
function ToolbarTextButtonVisualization(model, toolbarVisualization){
    ToolbarTextButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
    var me = this;
    //me.captionVisualization = new TextModelElementVisualization(me.model.getCaption(), toolbarVisualization);
}
extend(ToolbarTextButtonVisualization, ToolbarButtonVisualization);

ToolbarTextButtonVisualization.prototype.TEXT_COLOR_DISABLED = "#777777";
ToolbarTextButtonVisualization.prototype.COLOR_UP = "#CCCCCC";
ToolbarTextButtonVisualization.prototype.COLOR_OVER = "#DDDDDD";
ToolbarTextButtonVisualization.prototype.BORDER_COLOR_DARK = "#666666";
ToolbarTextButtonVisualization.prototype.BORDER_COLOR_LIGHT = "#FFFFFF";

//override
ToolbarTextButtonVisualization.prototype.update = function(toolbarLeft, toolbarTop){
    var me = this;
    var rSize = me.toolbarVisualization.getCanvasToSceneSizeRatio();
    var cnvToolbar = me.toolbarVisualization.getCanvas();
    var ctxToolbar = cnvToolbar.getContext("2d");
    var model = me.model;
    var flagEnabled = model.isEnabled();
    var w = Math.ceil(model.getWidth()*rSize);
    var h = Math.ceil(model.getHeight()*rSize);
    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");
    //
    if(cnv.width != w || cnv.height != h){
        cnv.width = w;
        cnv.height = h;
    }
    ctx.clearRect(0,0,w,h);

    var objProps = model.getProperties();

    var htDetails = model.getHitDetails();
    var flagOver = (htDetails != null) ? htDetails.flagHit : false;
    var flagSelected = model.isSelected();

    //button itself
    var padding = model.DEFAULT_BORDER_WIDTH*rSize/2;
    ctx.lineWidth = padding*2;
    ctx.beginPath();
    ctx.fillStyle = flagOver ? me.COLOR_OVER : me.COLOR_UP;
    ctx.fillRect(padding, padding, w-2*padding, h-2*padding);
    ctx.beginPath();
    ctx.strokeStyle = flagSelected ? me.BORDER_COLOR_DARK : me.BORDER_COLOR_LIGHT;
    ctx.moveTo(w-padding, padding);
    ctx.lineTo(padding, padding);
    ctx.lineTo(padding, h-padding);
    ctx.stroke();
    ctx.beginPath();
    ctx.strokeStyle = flagSelected ? me.BORDER_COLOR_LIGHT : me.BORDER_COLOR_DARK;
    ctx.moveTo(padding, h-padding);
    ctx.lineTo(w-padding, h-padding);
    ctx.lineTo(w-padding, padding);
    ctx.stroke();

    //caption
    var aFontPart = objProps.fontFace.split("_");
    var i = aFontPart.indexOf("bold");
    var flagBold = (i!=-1);
    i = aFontPart.indexOf("italic");
    var flagItalic = (i!=-1);
    var fontSizeVis = Math.floor(objProps.fontSize * rSize);
    ctx.font = (flagItalic ? "italic " : "") + (flagBold ? "bold " : "") +
        fontSizeVis + "px " + aFontPart[0];
    ctx.fillStyle =
        flagEnabled ?
            miscFunctions.intAlpha2RGBAFunction(objProps.fontColor, objProps.fontAlpha) : me.TEXT_COLOR_DISABLED;

    var textHShift = flagSelected ? (1*rSize) : 0;
    var textVShift = TextFunctions.prototype.getVerticalShiftByFontSize(fontSizeVis)+(flagSelected ? 0 : (-1*rSize));
    var textHeight = TextFunctions.prototype.getLineHeightByFontSize(fontSizeVis);
    var textWidth = ctx.measureText(objProps.caption).width;

    ctx.fillText(objProps.caption, ((textHShift+(w-textWidth)/2)|0)-0.5, ((textVShift+(h-textHeight)/2)|0)-0.5 );

    ctxToolbar.drawImage(cnv, 0, 0);
};
/**
 * @constructor
 */
function ToolbarFillColorButtonModel(toolbar, objProps) {
    ToolbarFillColorButtonModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;
    me.symbolId = "mcFillColor";
    me.colorProperty = "fillColor";
    me.alphaProperty = "fillAlpha";
}
extend(ToolbarFillColorButtonModel, ToolbarColorButtonModel);
/**
 * @constructor
 */
function ToolbarFillColorButtonVisualization(model, toolbarVisualization) {
    ToolbarFillColorButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
}
extend(ToolbarFillColorButtonVisualization, ToolbarColorButtonVisualization);

/**
 * @constructor
 */
function ToolbarLineColorButtonModel(toolbar, objProps) {
    ToolbarLineColorButtonModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;
    me.symbolId = "mcLineColor";
    me.colorProperty = "lineColor";
    me.alphaProperty = "lineAlpha";
}
extend(ToolbarLineColorButtonModel, ToolbarColorButtonModel);
/**
 * @constructor
 */
function ToolbarLineColorButtonVisualization(model, toolbarVisualization) {
    ToolbarLineColorButtonVisualization.superclass.constructor.call(this, model, toolbarVisualization);
}
extend(ToolbarLineColorButtonVisualization, ToolbarColorButtonVisualization);

/**
 * @constructor
 */
function ToolbarSliderModel(toolbar, objProps) {
    ToolbarSliderModel.superclass.constructor.call(this, toolbar, objProps);
    var me = this;
    me.hitDetails = {flagHitDragger:false, flagHitRange:false, flagPressDragger:false};
    //
    me.dragLength = (typeof objProps.dragLength == TYPE_NUMBER) ? objProps.dragLength : me.DEFAULT_RANGE_LENGTH;
    me.value = (typeof objProps.value == TYPE_NUMBER) ? objProps.value : me.DEFAULT_VALUE;
    me.minValue = (typeof objProps.minimalValue == TYPE_NUMBER) ? objProps.minimalValue : me.DEFAULT_MINIMAL_VALUE;
    me.maxValue = (typeof objProps.maximalValue == TYPE_NUMBER) ? objProps.maximalValue : me.DEFAULT_MAXIMAL_VALUE;
}
extend(ToolbarSliderModel, ToolbarControlModel);

ToolbarSliderModel.prototype.DEFAULT_RANGE_LENGTH = 100;
ToolbarSliderModel.prototype.DEFAULT_RANGE_HEIGHT = 4;
ToolbarSliderModel.prototype.DRAGGER_WIDTH = 12;
ToolbarSliderModel.prototype.DRAGGER_HEIGHT = 24;
ToolbarSliderModel.prototype.DEFAULT_VALUE = 0;
ToolbarSliderModel.prototype.DEFAULT_MINIMAL_VALUE = 0;
ToolbarSliderModel.prototype.DEFAULT_MAXIMAL_VALUE = 100;

//
ToolbarSliderModel.prototype.getValue = function(){
    var me = this;
    return (me.value<0) ? 0 : ((me.value>me.dragLength) ? me.dragLength : me.value);
};

//
ToolbarSliderModel.prototype.setValue = function(v){
    var me = this;
    me.value = v;
    var evt = new ModelEvent(ModelEvent.prototype.VALUE_CHANGED, true, true);
    if(evt != null){
        evt.data = {value:me.value, id:me.id};
        me.dispatchEvent(evt);
    }
};

//
ToolbarSliderModel.prototype.getWidth = function(){
    var me = this;
    return ((2*me.DRAGGER_WIDTH)>me.dragLength) ? me.DRAGGER_WIDTH : (me.DRAGGER_WIDTH+me.dragLength);
};

//
ToolbarSliderModel.prototype.getHeight = function(){
    var me = this;
    return (me.DRAGGER_HEIGHT>me.DEFAULT_RANGE_HEIGHT) ? me.DRAGGER_HEIGHT : me.DEFAULT_RANGE_HEIGHT;
};

//
ToolbarSliderModel.prototype.getHitTestInfoForPoint = function(p0){
    var me = this;
    var dw = me.DRAGGER_WIDTH;
    var dh = me.DRAGGER_HEIGHT;
    var rh = me.DEFAULT_RANGE_HEIGHT;
    //
    var rTouch = me.TOUCH_SIZE_FACTOR/ModelElement.prototype.ZOOM_RATIO;
    if(MOUSE_OR_TOUCH>0){
        dw *= rTouch;
        dh *= rTouch;
        rh *= rTouch;
    }
    //
    var dw2 = dw/2;
    var dh2 = dh/2;
    var rh2 = rh/2;
    //
    var w = me.getWidth();
    var h = me.getHeight();

    var dx = (me.value-me.minValue)*me.dragLength/(me.maxValue-me.minValue);

    var x = p0.x-me.x-(w-me.dragLength)/2;
    var y = p0.y-me.y-h/2;
    var flagHitDragger = (y>=-dh2) && (y<=dh2) && (x>=dx-dw2) && (x<=dx+dw2);
    var flagHitRange;
    var hitValue = undefined;
    if(!flagHitDragger){
        flagHitRange = (y>=-rh2) && (y<=rh2) && (x>=0) && (x<=me.dragLength);
        if(flagHitRange)
            hitValue = me.minValue+(me.maxValue-me.minValue)*x/me.dragLength;
    }else
        flagHitRange = false;
    //
    var flagHit = flagHitDragger || flagHitRange;
    var flagInside = flagHit;
    return new HitTestInfo(
        me, flagHit, flagInside,
        {
            flagHitDragger:flagHitDragger, flagHitRange:flagHitRange, hitValue:hitValue
        }
    );
};
/**
 * @constructor
 */
function ToolbarSliderVisualization(model){
    ToolbarSliderVisualization.superclass.constructor.call(this);
    var me = this;
    me.model = model;
    me.cnv = document.createElement("CANVAS");
    me.cnv.width = 1;
    me.cnv.height = 1;
}
extend(ToolbarSliderVisualization, NonDOMEventDispatcher);

//
ToolbarSliderVisualization.prototype.getModel = function(){
    return this.model;
};

//
ToolbarSliderVisualization.prototype.getCanvas = function(){
    return this.cnv;
};

//
ToolbarSliderVisualization.prototype.update = function(){
    var me = this;
    var model = me.model;
    var htDetails = model.getHitDetails();
    var w = Math.ceil(model.getWidth());
    var h = Math.ceil(model.getHeight());
    var cnv = me.cnv;
    var xLeft = (w-model.dragLength)/2;
    if(cnv.width != w || cnv.height != h){
        cnv.width = w;
        cnv.height = h;
    }
    var ctx = cnv.getContext("2d");
    ctx.clearRect(0, 0, w, h);

    var hitDetails = model.getHitDetails();

    var objSize = ColorPaletteLibrary.prototype.getSizeById("gr.slider-range");
    var img = ColorPaletteLibrary.prototype.getCachedSVG("gr.slider-range");
    ctx.drawImage(img, xLeft+objSize.width/2+objSize.xShift, h/2+objSize.yShift);

    if(hitDetails.flagPressDragger){
        objSize = ColorPaletteLibrary.prototype.getSizeById("gr.slider-dragger.down");
        img = ColorPaletteLibrary.prototype.getCachedSVG("gr.slider-dragger.down");
    }else if(hitDetails.flagHitDragger){
        objSize = ColorPaletteLibrary.prototype.getSizeById("gr.slider-dragger.over");
        img = ColorPaletteLibrary.prototype.getCachedSVG("gr.slider-dragger.over");
    }else{
        objSize = ColorPaletteLibrary.prototype.getSizeById("gr.slider-dragger.up");
        img = ColorPaletteLibrary.prototype.getCachedSVG("gr.slider-dragger.up");
    }
    var dx = (model.value-model.minValue)*model.dragLength/(model.maxValue-model.minValue);
    ctx.drawImage(img, w/2-model.dragLength/2+objSize.xShift+dx, h/2+objSize.yShift);
};
/**
 * @constructor
 */
function TopToolbarModel(modelResource, interfaceModel){
    TopToolbarModel.superclass.constructor.call(this, interfaceModel);
    var me = this;
    me.position = ToolbarModel.prototype.POSITION_TOP;
    //
    var aMatch = me.playerToolAttributesMatch;
    var i, j, n = aMatch.length;
    var s;
    //
    me.aToolNo = [0];
    me.aToolData = [null];
    me.aToolName = [me.fGetMatchingPropertyByToolClassName("SelectModelTool", "name")];
    me.aButtonId = [me.fGetMatchingPropertyByToolClassName("SelectModelTool", "id")];
    me.aToolClassName = ["SelectModelTool"];
    me.aButtonClassName = [me.fGetMatchingPropertyByToolClassName("SelectModelTool", "buttonClassName")];
    var aToolNo = me.aToolNo;
    var aToolData = me.aToolData;
    var aToolName = me.aToolName;
    var aToolClassName = me.aToolClassName;
    var aButtonClassName = me.aButtonClassName;
    var aButtonId = me.aButtonId;
    //
    var objCustomizeAttributes = modelResource.customizePlayerAttributes;
    var flagInducedRemoveTool = document.localization.getParameter("induced-remove-tool", false);
    var flagCreating = false;
    //
    for(i=0; i<n; i++){
        s = aMatch[i].name;
        if(typeof objCustomizeAttributes[s] != TYPE_UNDEFINED && (""+objCustomizeAttributes[s]).toLowerCase() == "true") {
            aToolNo.push(i);
            if (flagInducedRemoveTool && !flagCreating && me.getButtonPropertyById(aMatch[i].id, "creating"))
                flagCreating = true;
        }
    }
    //in case there are tools that create model elements, the remove tool button is added
    if(flagInducedRemoveTool && flagCreating) {
        i = me.fGetMatchingPropertyByToolClassName("RemoveModelTool", "indexOf");
        if(aToolNo.indexOf(i) == -1)
            aToolNo.push(i);
    }
    //
    var objCustomizeData = modelResource.customizePlayerData;
    var objToolData;
    n = aToolNo.length;
    //from i==1 since i==0 is always the SelectModelTool
    for(i=1; i<n; i++){
        j = aToolNo[i];
        s = aMatch[j].name;
        if(typeof objCustomizeData[s] == TYPE_OBJECT)
            objToolData = objCustomizeData[s];
        else
            objToolData = null;
        aToolData.push(objToolData);
        aToolName.push(s);
        aToolClassName.push(aMatch[j].className);
        aButtonClassName.push(aMatch[j].buttonClassName);
        aButtonId.push(aMatch[j].id);
    }
    //
    n = aToolName.length;
    var button, cls;
    for(i=0; i<n; i++){
        s = aButtonId[i];
        cls = window[aButtonClassName[i]];
        button  = new cls(me, {id:s, tool:null});
        button.setSelected(i==0);
        me.addButton(button);
        button.addEventListener(
            ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, {target:me, listener:me.fButtonListener}, false
        );
    }
}
extend(TopToolbarModel, ToolbarModel);

//"name" – in xml-file, "className" – tool class name, "id" – button id, "buttonClassName" – class name of the button
TopToolbarModel.prototype.playerToolAttributesMatch = [
    {name:"Select",             className:"SelectModelTool",        id:"bnSelect",      buttonClassName:"ToolbarSelectButtonModel"},
    {name:"Delete",             className:"RemoveModelTool",        id:"bnRemove",      buttonClassName:"ToolbarButtonModel"},
    {name:"Pan",                className:"PanModelTool",           id:"bnPan",         buttonClassName:"ToolbarButtonModel"},
    {name:"Help",               className:"",                       id:"bnHelp",        buttonClassName:"ToolbarButtonModel"},
    {name:"CreateLine",         className:"PolylineModelTool",      id:"bnPolyline",    buttonClassName:"ToolbarButtonModel"},
    {name:"CreateText",         className:"TextModelTool",          id:"bnText",        buttonClassName:"ToolbarButtonModel"},
    {name:"CreateBoundText",    className:"BtextModelTool",         id:"bnBtext",       buttonClassName:"ToolbarButtonModel"},
    {name:"CreateArrow",        className:"ArrowModelTool",         id:"bnArrow",       buttonClassName:"ToolbarSymbolSetButtonModel"},
    {name:"CreateDiagram",      className:"DiagramModelTool",       id:"bnDiagram",     buttonClassName:"ToolbarSymbolSetButtonModel"},
    {name:"CreatePoint",        className:"PointModelTool",         id:"bnSymbol",      buttonClassName:"ToolbarSymbolSetButtonModel"},
    {name:"CreateRectangle",    className:"RectangleModelTool",     id:"bnRectangle",   buttonClassName:"ToolbarRectangleButtonModel"},
    {name:"Duplicate",          className:""},
    {name:"FillCol",            className:"FillColorModelTool",     id:"bnFillColor",   buttonClassName:"ToolbarFillColorButtonModel"},
    {name:"LineColBtn",         className:"LineColorModelTool",     id:"bnLineColor",   buttonClassName:"ToolbarLineColorButtonModel"},
    {name:"TextSizePanel",      className:"FontSizeModelTool",      id:"bnFontSize",    buttonClassName:"ToolbarFontSizeButtonModel"},
    {name:"FontFacePanel",      className:"FontFaceModelTool",      id:"bnFontFace",    buttonClassName:"ToolbarFontFaceButtonModel"},
    {name:"TextAlign",          className:"TextAlignModelTool",     id:"bnTextAlign",   buttonClassName:"ToolbarTextAlignButtonModel"},
    {name:"LayerDn",            className:""},
    {name:"LayerUp",            className:""},
    {name:"ZoomList",           className:"ZoomModelTool",          id:"bnZoom",        buttonClassName:"ToolbarZoomButtonModel"}
];

TopToolbarModel.prototype.buttonProps = [
    {id:"bnSelect",     orderNo:0,      radioGroup:0,   selectable:true,    creating:false},
    {id:"bnPan",        orderNo:1,      radioGroup:0,   selectable:true,    creating:false},
    {id:"bnSymbol",     orderNo:3,      radioGroup:0,   selectable:true,    creating:true},
    {id:"bnArrow",      orderNo:4,      radioGroup:0,   selectable:true,    creating:true},
    {id:"bnPolyline",   orderNo:5,      radioGroup:0,   selectable:true,    creating:true},
    {id:"bnRectangle",  orderNo:6,      radioGroup:0,   selectable:true,    creating:true},
    {id:"bnDiagram",    orderNo:7,      radioGroup:0,   selectable:true,    creating:true},
    {id:"bnText",       orderNo:8,      radioGroup:0,   selectable:true,    creating:true},
    {id:"bnBtext",      orderNo:9,      radioGroup:0,   selectable:true,    creating:true},
    {id:"bnRemove",     orderNo:10,     radioGroup:-1,  selectable:false,   creating:false},
    {id:"bnLineColor",  orderNo:11,     radioGroup:-1,  selectable:false,   creating:false},
    {id:"bnFillColor",  orderNo:12,     radioGroup:-1,  selectable:false,   creating:false},
    {id:"bnFontSize",   orderNo:13,     radioGroup:-1,  selectable:false,   creating:false},
    {id:"bnFontFace",   orderNo:14,     radioGroup:-1,  selectable:false,   creating:false},
    {id:"bnTextAlign",  orderNo:15,     radioGroup:-1,  selectable:false,   creating:false},
    {id:"bnZoom",       orderNo:16,     radioGroup:-1,  selectable:false,   creating:false}
];

TopToolbarModel.prototype.fGetMatchingPropertyByToolClassName = function(sClassName, sProp){
    var aMatch = TopToolbarModel.prototype.playerToolAttributesMatch, i, nn = aMatch.length;
    for(i=0; i<nn; i++)
        if(aMatch[i].className == sClassName)
            break;
    if(i<nn){
        if(typeof aMatch[i][sProp] != TYPE_UNDEFINED)
            return aMatch[i][sProp];
        else if(sProp == "indexOf")
            return i;
    }
    return null;
};

//override
TopToolbarModel.prototype.getButtonIdByToolClassName = function(sClassName){
    return TopToolbarModel.prototype.fGetMatchingPropertyByToolClassName(sClassName, "id");
};

//
TopToolbarModel.prototype.getToolDataByClassName = function(sClassName){
    var me = this;
    var i = me.aToolClassName.indexOf(sClassName);
    return (i != -1) ? me.aToolData[i] : null;
};

//
TopToolbarModel.prototype.isOnlySelectionTool = function(){
    var me = this;
    return (me.getButtonCount()==1) && (me.getButtonNo(0).getId() == me.getButtonIdByToolClassName("SelectModelTool"));
};

//
TopToolbarModel.prototype.fButtonListener = function(e){
    this.conveyEvent(e);
    /*var evt = new ModelEvent(e.type);
    evt.data = miscFunctions.shallowCloneObject(e.data);
    this.dispatchEvent(evt);*/
};

//override
TopToolbarModel.prototype.deactivate = function(){
    TopToolbarModel.superclass.deactivate.call(this);
    var me = this;
    var aButton = me.aButton, button;
    var i, n = aButton.length;
    for(i=0; i<n; i++){
        button = aButton[i];
        button.removeEventListener(
            ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, {target:me, listener:me.fButtonListener}, false
        );
        button.deactivate();
    }
    aButton.splice(0);
};
/**
 * @constructor
 */
function RightToolbarModel(modelResource, interfaceModel){
    RightToolbarModel.superclass.constructor.call(this, interfaceModel);
    var me = this;
    //
    me.flagShowHint = modelResource.objectsAttributes["ShowHint"].toLowerCase() == "true";
    me.showHintAfter = (
        typeof modelResource.objectsAttributes["HintAlways"] != TYPE_UNDEFINED &&
        modelResource.objectsAttributes["HintAlways"].toLowerCase() == "true"
    ) ?
        0 :
        parseInt(modelResource.objectsAttributes["ShowHintAfter"]);
    //
    me.flagShowHiddenHint = modelResource.objectsAttributes["ShowHiddenHint"].toLowerCase() == "true";
    //
    me.flagShowSolution = modelResource.objectsAttributes["ShowSolution"].toLowerCase() == "true";
    me.flagSelfTest = typeof modelResource.objectsAttributes["SelfTest"] == TYPE_UNDEFINED ||
        modelResource.objectsAttributes["SelfTest"].toLowerCase() == "true";

    me.showSolutionAfter = (
        typeof modelResource.objectsAttributes["SolutionAlways"] != TYPE_UNDEFINED &&
        modelResource.objectsAttributes["SolutionAlways"].toLowerCase() == "true"
    ) ?
        0 :
        parseInt(modelResource.objectsAttributes["ShowSolutionAfter"]);
    //
    me.aCommentNode = TextFunctions.prototype.getTextBlocksFromNode(modelResource.nodeComment);
    //
    me.position = ToolbarModel.prototype.POSITION_RIGHT;
    //
    me.flagWithVisibilityList = false;
    me.flagVerifiableTask = false;
    //
    me.interfaceModel.addEventListener(
        ModelEvent.prototype.VERIFY_ELEMENTS, {target:me, listener:me.fVerifyElementsListener}, false
    );
    //
    me.buttonVisibility = new ToolbarShowHiddenButtonModel(
        me,
        {
            id: me.BUTTON_SHOW_HIDDEN_ID,
            y: 12,
            width: 166,
            tool: null
        }
    );
    me.buttonVisibility.setVisible(me.flagWithVisibilityList && (!me.flagShowHiddenHint || me.showHintAfter==0));
    me.buttonVisibility.addEventListener(
        ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, {target:me, listener:me.fButtonListener}, false
    );
    me.aButton.push(me.buttonVisibility);
    //
    me.buttonCorrectElements = new ToolbarTextButtonModel(
        me,
        {
            id: me.BUTTON_CORRECT_ELEMENTS_ID,
            caption:document.localization.getString("interface.toolbars.button-captions.show-correct", "Правильные"),
            fontSize:14,
            fontFace:"Arial",
            fontAlpha:1.0,
            fontColor:0x000000,
            width:me.BUTTON_FULL_WIDTH,
            height:me.BUTTON_FULL_HEIGHT,
            deselectOnRelease:true,
            applyOnRelease:true
        }
    );
    me.buttonCorrectElements.setVisible(me.flagShowSolution);
    me.buttonCorrectElements.setEnabled(me.flagShowSolution && me.showSolutionAfter==0);
    me.aButton.push(me.buttonCorrectElements);
    //
    me.buttonHint = new ToolbarTextButtonModel(
        me,
        {
            id: me.BUTTON_HINT_ID,
            caption:document.localization.getString("interface.toolbars.button-captions.show-hint", "Подсказка"),
            fontSize:14,
            fontFace:"Arial",
            fontAlpha:1.0,
            fontColor:0x000000,
            width:me.BUTTON_FULL_WIDTH,
            height:me.BUTTON_FULL_HEIGHT,
            deselectOnRelease:true,
            applyOnRelease:true
        }
    );
    me.buttonHint.setVisible(me.flagShowHint);
    me.buttonHint.setEnabled(me.flagShowHint && me.showHintAfter==0);
    me.aButton.push(me.buttonHint);
    //
    me.buttonReset = new ToolbarTextButtonModel(
        me,
        {
            id: me.BUTTON_RESET_ID,
            caption:document.localization.getString("interface.toolbars.button-captions.reset", "Сбросить"),
            fontSize:14,
            fontFace:"Arial",
            fontAlpha:1.0,
            fontColor:0x000000,
            width:me.BUTTON_FULL_WIDTH,
            height:me.BUTTON_FULL_HEIGHT,
            deselectOnRelease:true,
            applyOnRelease:true
        }
    );
    me.buttonReset.setVisible(true);
    me.aButton.push(me.buttonReset);
    //
    me.buttonConfirm = new ToolbarTextButtonModel(
        me,
        {
            id: me.BUTTON_CONFIRM_ID,
            caption:document.localization.getString("interface.toolbars.button-captions.confirm", "Подтвердить ответ"),
            fontSize:14,
            fontFace:"Arial_bold",
            fontAlpha:1.0,
            fontColor:0x000000,
            width:me.BUTTON_FULL_WIDTH,
            height:me.BUTTON_FULL_HEIGHT,
            deselectOnRelease:true,
            applyOnRelease:true
        }
    );
    me.buttonConfirm.setVisible(false);
    me.aButton.push(me.buttonConfirm);
    //
    me.aBottomButton = [[me.buttonCorrectElements, me.buttonHint], [me.buttonReset], [me.buttonConfirm]];
    //
    me.buttonGap = me.DEFAULT_BUTTON_GAP;
    //
    var aToolData = me.aToolData;
    var aToolName = me.aToolName;
    var aToolClassName = me.aToolClassName;
    var aButtonClassName = me.aButtonClassName;
    var aButtonId = me.aButtonId;
    //
    //buttonVisibility
    aToolData.push({});
    aToolName.push(null);
    aToolClassName.push("ShowHiddenModelTool");
    aButtonClassName.push("ToolbarShowHiddenButtonModel");
    aButtonId.push("bnShowHidden");

    //buttonCorrectElements
    aToolData.push({selfTest:me.flagSelfTest.toString()});
    aToolName.push(null);
    aToolClassName.push("ShowCorrectModelTool");
    aButtonClassName.push("ToolbarTextButtonModel");
    aButtonId.push("bnCorrectElements");
    //
    me.invalidate();
}
extend(RightToolbarModel, ToolbarModel);

RightToolbarModel.prototype.DEFAULT_BUTTON_GAP = 6;
RightToolbarModel.prototype.BUTTON_CORRECT_ELEMENTS_ID = "bnCorrectElements";
RightToolbarModel.prototype.BUTTON_HINT_ID = "bnHint";
RightToolbarModel.prototype.BUTTON_RESET_ID = "bnReset";
RightToolbarModel.prototype.BUTTON_CONFIRM_ID = "bnConfirm";

RightToolbarModel.prototype.BUTTON_SHOW_HIDDEN_ID = "bnShowHidden";

RightToolbarModel.prototype.COMMENT_WIDTH = 180;
RightToolbarModel.prototype.COMMENT_PADDING_LEFT = 17;
RightToolbarModel.prototype.COMMENT_PADDING_TOP = 0;

RightToolbarModel.prototype.BUTTON_FULL_WIDTH = 186;
RightToolbarModel.prototype.BUTTON_FULL_HEIGHT = 24;

RightToolbarModel.prototype.playerToolAttributesMatch = [
    {name:null, className:"ShowHiddenModelTool",    id:"bnShowHidden",  buttonClassName:"ToolbarShowHiddenButtonModel"}
];

//override
RightToolbarModel.prototype.getButtonIdByToolClassName = function(sClassName){
    var aMatch = RightToolbarModel.prototype.playerToolAttributesMatch, i, nn = aMatch.length;
    for(i=0; i<nn; i++)
        if(aMatch[i].className == sClassName)
            break;
    return (i<nn) ? aMatch[i].id : null;
};

//
RightToolbarModel.prototype.isWithVisibilityList = function(){
    return this.flagWithVisibilityList;
};

RightToolbarModel.prototype.setWithVisibilityList = function(flag){
    var me = this;
    if(me.flagWithVisibilityList != flag){
        me.flagWithVisibilityList = flag;
        var flagAllowed = (!me.flagShowHiddenHint && me.showHintAfter<=me.interfaceModel.getAttemptsCount());
        me.buttonVisibility.setVisible(me.flagWithVisibilityList && flagAllowed);
        me.invalidate();
    }
};

//
RightToolbarModel.prototype.isVerifiableTask = function(){
    return this.flagVerifiableTask;
};

RightToolbarModel.prototype.setVerifiableTask = function(flag){
    var me = this;
    if(me.flagVerifiableTask != flag){
        me.flagVerifiableTask = flag;
        me.buttonConfirm.setVisible(me.flagVerifiableTask);
        me.invalidate();
    }
};

//
RightToolbarModel.prototype.configureVisibilityList = function(aData){
    var me = this;
    var ii = me.aButtonId.indexOf(RightToolbarModel.prototype.BUTTON_SHOW_HIDDEN_ID);
    if(ii==-1)
        return;
    me.aToolData[ii].items = aData;
};

//
RightToolbarModel.prototype.isWithButton = function(sId){
    var me = this;
    var button = me.getButtonById(sId);
    return (button != null) ? button.isVisible() : false;
};

//
RightToolbarModel.prototype.setWithButton = function(sId, flag){
    var me = this;
    var button = me.getButtonById(sId);
    if(button != null && button.isVisible() != flag) {
        button.setVisible(flag);
        me.invalidate();
    }
};

//
RightToolbarModel.prototype.getButtonById = function(sId){
    var me = this;
    var a = me.aButton;
    var i, n = a.length;
    var button;
    for(i=0; i<n; i++){
        button = a[i];
        if(button.getId() == sId)
            return button;
    }
    return null;
};

//
RightToolbarModel.prototype.fButtonListener = function(e){
    this.conveyEvent(e);
    /*var evt = new ModelEvent(e.type);
    evt.data = miscFunctions.shallowCloneObject(e.data);
    this.dispatchEvent(evt);*/
};

//override
//
RightToolbarModel.prototype.fRearrangeButtons = function(){
    //update coordinates according to actual spacing and me.aButton
    var me = this;
    if(me.getAreas() == null)
        return;
    var wTB = me.getWidth();
    var hTB = me.getHeight();
    var hTopTB = me.getAreas().topHeight;
    var button;
    var wB, hB;
    var i, n = me.aBottomButton.length;
    var a, j, l, c, w;
    var yCurr = hTB;
    //
    for(i=n-1; i>=0; i--){
        a = me.aBottomButton[i];
        l = a.length;
        c = 0;
        for(j=0; j<l; j++)
            if(a[j].isVisible())
                c++;
        if(c==0)
            continue;
        yCurr -= me.BUTTON_FULL_HEIGHT;//hB;
        w = me.BUTTON_FULL_WIDTH/c-me.buttonGap*(c-1)/c;
        for(j=0; j<l; j++){
            button = a[j];
            button.setWidth(w);
            hB = button.getHeight();
            button.setX((wTB-me.BUTTON_FULL_WIDTH)/2+j*(w+me.buttonGap));
            button.setY(yCurr);
        }
        yCurr -= me.buttonGap;
    }
    //
    if(me.buttonVisibility.isVisible()) {
        wB = me.buttonVisibility.getTotalWidth();
        me.buttonVisibility.setX((wTB - wB) / 2);
    }
};

//
RightToolbarModel.prototype.getCommentTextBlocks = function(rSize){
    return TextFunctions.prototype.getTextBlocksForWidthAndScaleFactor(this.aCommentNode, this.COMMENT_WIDTH, rSize);
};

//
RightToolbarModel.prototype.fVerifyElementsListener = function(e){
    var me = this;
    var data = e.data;
    if(me.flagWithVisibilityList && me.flagShowHiddenHint && me.showHintAfter<=data.attemptNo) {
        var prev = me.buttonVisibility.isVisible();
        me.buttonVisibility.setVisible(true);
        if(!prev)
            me.invalidate();
    }
    if(me.flagShowHint && me.showHintAfter<=data.attemptNo)
        me.buttonHint.setEnabled(true);
    if(me.flagShowSolution && me.showSolutionAfter<=data.attemptNo)
        me.buttonCorrectElements.setEnabled(true);
    me.validate();
};

//override
RightToolbarModel.prototype.reset = function(){
    RightToolbarModel.superclass.reset.call(this);
    var me = this;
    me.buttonCorrectElements.setEnabled(me.flagShowSolution && me.showSolutionAfter==0);
    if(me.buttonVisibility.isVisible())
        me.buttonVisibility.setVisible(me.flagWithVisibilityList && (!me.flagShowHiddenHint || me.showHintAfter==0));
};

//
//override
RightToolbarModel.prototype.deactivate = function(){
    RightToolbarModel.superclass.deactivate.call(this);
    var me = this;
    me.buttonVisibility.removeEventListener(
        ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, {target:me, listener:me.fButtonListener}, false
    );
    me.interfaceModel.removeEventListener(
        ModelEvent.prototype.VERIFY_ELEMENTS, {target:me, listener:me.fVerifyElementsListener}, false
    );
    me.buttonVisibility.deactivate();
    var aBottomButton = me.aBottomButton, button;
    var i, n = aBottomButton.length, a, j, l;
    for(i=0; i<n; i++) {
        a = aBottomButton[i];
        l = a.length;
        for(j=0; j<l; j++)
            a[j].deactivate();
    }
};
/**
 * @constructor
 */
function MainInterfaceModel(modelResource, mapSceneModel) {
    MainInterfaceModel.superclass.constructor.call(this, modelResource);
    var me = this;
    me.flagValid = true;
    //
    me.flagCtrl = false;
    //
    me.mapSceneModel = mapSceneModel;
    //
    me.leftWidth = -1;
    me.rightWidth = -1;
    me.topHeight = -1;
    me.bottomHeight = -1;
    //
    me.topToolbar = new TopToolbarModel(modelResource, me);
    //
    me.rightToolbar = new RightToolbarModel(modelResource, me);
    //
    me.screenKeyboard = new ScreenKeyboardModel(me, me.mapSceneModel);
    //
    me.aToolbar = [me.topToolbar, me.rightToolbar];
    //
    var i, n = me.aToolbar.length;
    for(i=0; i<n; i++)
        me.aToolbar[i].addEventListener(
            ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, {target:me, listener:me.fToolbarListener}, false
        );
    //
    var flagVerifiableTask = mapSceneModel.isWithVerifiableElements();
    me.rightToolbar.setVerifiableTask(flagVerifiableTask);
    //
    var flagVisibilityList = mapSceneModel.isWithSwitchableVisibilityElements();
    me.rightToolbar.setWithVisibilityList(flagVisibilityList);
    //
    if(flagVisibilityList)
        me.rightToolbar.configureVisibilityList(mapSceneModel.getSwitchableVisibilityArray());

    //
    me.setInterfaceAreas(
        MainInterfaceModel.prototype.leftWidth,
        MainInterfaceModel.prototype.rightWidth,
        (me.topToolbar.isOnlySelectionTool() && !flagVisibilityList)?
            MainInterfaceModel.prototype.topHeightOnlySelection : MainInterfaceModel.prototype.topHeight,
        MainInterfaceModel.prototype.bottomHeight
    );
    //
    me.aDialog = [];
    //
    me.mapSceneModel.addEventListener(
        ModelEvent.prototype.VERIFY_ELEMENTS, {target:me, listener:me.fVerifyElementsListener}, false
    );
    //
    me.mapSceneModel.addEventListener(
        ModelEvent.prototype.SHOW_HINT, {target:me, listener:me.fShowHintListener}, false
    );
    //
    me.mapSceneModel.addEventListener(
        ModelEvent.prototype.SHOW_RESPONSE, {target:me, listener:me.fShowResponseListener}, false
    );
    //
    me.mapSceneModel.addEventListener(
        ModelEvent.prototype.TOGGLE_SCREEN_KEYBOARD, {target:me, listener:me.fToggleScreenKeyboardListener}, false
    );
    //
    me.mapSceneModel.addEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:me, listener:me.fResetSceneListener}, false
    );

}
extend(MainInterfaceModel, NonDOMEventDispatcher);

//these parameters are to be set on MapSceneVisualization via event
MainInterfaceModel.prototype.leftWidth = 18;
MainInterfaceModel.prototype.rightWidth = 208;
MainInterfaceModel.prototype.topHeight = 54;
MainInterfaceModel.prototype.topHeightOnlySelection = 18;
MainInterfaceModel.prototype.bottomHeight = 10;

//
MainInterfaceModel.prototype.isCtrl = function(){
    return this.flagCtrl;
};

MainInterfaceModel.prototype.setCtrl = function(flag){
    if(this.flagCtrl != flag)
        this.flagCtrl = flag;
};

//
MainInterfaceModel.prototype.invalidate = function(){
    if(this.flagValid)
        this.flagValid = false;
};

//
MainInterfaceModel.prototype.validate = function(){
    var me = this;
    var flagToolbarsValid;
    var flagScreenKeyboardValid = me.screenKeyboard.isValid();
    var i, n = me.aToolbar.length;
    for(i=0; i<n; i++)
        if(!me.aToolbar[i].isValid())
            break;
    flagToolbarsValid = (i==n);
    if(!(me.flagValid && flagToolbarsValid && flagScreenKeyboardValid)){
        var evt = new ModelEvent(ModelEvent.prototype.VALIDATE_INTERFACE, false, true);
        if(evt == null)
            return;
        evt.data = {time:new Date().getTime()};
        me.dispatchEvent(evt);
        me.flagValid = true;
    }
};

//
MainInterfaceModel.prototype.getInterfaceAreas = function(){
    var me = this;
    return {
        sceneWidth:me.mapSceneModel.getSceneWidth(), sceneHeight:me.mapSceneModel.getSceneHeight(),
        leftWidth:me.leftWidth, rightWidth:me.rightWidth, topHeight:me.topHeight, bottomHeight:me.bottomHeight
    };
};

//
MainInterfaceModel.prototype.setInterfaceAreas = function(leftWidth, rightWidth, topHeight, bottomHeight){
    var me = this;
    var flagChange =
        (leftWidth != me.leftWidth) || (rightWidth != me.rightWidth) ||
        (topHeight != me.topHeight) || (bottomHeight != me.bottomHeight);
    if(flagChange){
        me.flagValid = false;
        me.leftWidth = leftWidth;
        me.rightWidth = rightWidth;
        me.topHeight = topHeight;
        me.bottomHeight = bottomHeight;
        var evt = new ModelEvent(ModelEvent.prototype.INTERFACE_AREAS_SET, true, true);
        if(evt != null){
            evt.data = {
                sceneWidth:me.mapSceneModel.getSceneWidth(), sceneHeight:me.mapSceneModel.getSceneHeight(),
                leftWidth:me.leftWidth, rightWidth:me.rightWidth, topHeight:me.topHeight, bottomHeight:me.bottomHeight
            };
            me.dispatchEvent(evt);
        }
    }
};

//
MainInterfaceModel.prototype.fToolbarListener = function(e){
    this.conveyEvent(e, {ctrlKey:this.isCtrl()});
    /*var evt = new ModelEvent(e.type);
    evt.data = miscFunctions.shallowCloneObject(e.data);
    evt.data.ctrlKey = this.isCtrl();
    this.dispatchEvent(evt);*/
};

//
MainInterfaceModel.prototype.getTopToolbar = function(){
    return this.topToolbar;
};

//
MainInterfaceModel.prototype.getRightToolbar = function(){
    return this.rightToolbar;
};

//
MainInterfaceModel.prototype.getScreenKeyboard = function(){
    return this.screenKeyboard;
};

//
MainInterfaceModel.prototype.getToolbars = function(){
    return this.aToolbar;
};

//
MainInterfaceModel.prototype.isPointWithinInterface = function(p){
    var me = this;
    var xScene = p.x-me.leftWidth;
    var yScene = p.y-me.topHeight;
    return (xScene<0) || (xScene>me.mapSceneModel.getSceneWidth()) ||
        (yScene<0) || (yScene>me.mapSceneModel.getSceneHeight());
};

//
MainInterfaceModel.prototype.getHitTestInfosForPoint = function(p0){
    var me = this;
    var objResult = {};
    var objAreas = me.getInterfaceAreas();
    var i, n = me.aToolbar.length;
    var toolbar, wTB, hTB;
    var pTB, sProp;
    for(i=0; i<n; i++){
        toolbar = me.aToolbar[i];
        wTB = toolbar.getWidth();
        hTB = toolbar.getHeight();
        switch(toolbar.getPosition()){
            case ToolbarModel.prototype.POSITION_TOP:
                pTB = new Point(p0.x-objAreas.leftWidth, p0.y-objAreas.topHeight+hTB);
                sProp = "topToolbar";
                break;
            case ToolbarModel.prototype.POSITION_RIGHT:
                pTB = new Point(p0.x-objAreas.leftWidth-objAreas.sceneWidth, p0.y);
                sProp = "rightToolbar";
                break;
            case ToolbarModel.prototype.POSITION_LEFT:
                pTB = new Point(p0.x, p0.y-objAreas.topHeight+hTB);
                sProp = "leftToolbar";
                break;
            case ToolbarModel.prototype.POSITION_BOTTOM:
                pTB = new Point(p0.x-objAreas.leftWidth, p0.y-objAreas.topHeight-objAreas.sceneHeight+hTB);
                sProp = "bottomToolbar";
                break;
            default:
                pTB = null;
        }
        if(pTB != null && (pTB.x>0 && pTB.x<wTB && pTB.y>0 && pTB.y<hTB))
            objResult[sProp] = toolbar.getHitTestInfosForPoint(pTB);
    }

    return objResult;
};

//
MainInterfaceModel.prototype.resetHitTestInfos = function(){
    var me = this;
    var i, n = me.aToolbar.length;
    for(i=0; i<n; i++)
        me.aToolbar[i].resetHitDetails();
};

//

//
MainInterfaceModel.prototype.fVerifyElementsListener = function(e){
    this.conveyEvent(e);
    /*var evt = new ModelEvent(e.type);
    evt.data = miscFunctions.shallowCloneObject(e.data);
    this.dispatchEvent(evt);*/
};

//
MainInterfaceModel.prototype.fShowHintListener = function(e){
    this.addDialog(e.data.dialogProperties);
};

//
MainInterfaceModel.prototype.fShowResponseListener = function(e){
    this.addDialog(e.data.dialogProperties);
};

//
MainInterfaceModel.prototype.fResetSceneListener = function(e){
    this.conveyEvent(e);
    /*var evt = new ModelEvent(e.type);
    evt.data = miscFunctions.shallowCloneObject(e.data);
    this.dispatchEvent(evt);*/
};

//
MainInterfaceModel.prototype.fToggleScreenKeyboardListener = function(e){
    var me = this;
    var data = e.data;
    var flag = data.editingMode;
    var element = data.element;
    if(flag) {
        if(me.screenKeyboard.getElement() != element){
            me.screenKeyboard.setElement(element);
            me.flagValid = false;
        }
        if(!me.screenKeyboard.isVisible()) {
            me.screenKeyboard.show();
            me.flagValid = false;
        }
    }else{
        //element == null – hide unconditionally
        if(me.screenKeyboard.getElement() == element || element == null) {
            me.screenKeyboard.hide();
            me.flagValid = false;
        }
    }
};

//
MainInterfaceModel.prototype.addDialog = function(objProps){
    var me = this;
    me.removeDialog(objProps.id);
    var dialog = new ModalDialogModel(me, objProps);
    me.aDialog.push(dialog);
    var evt = new ModelEvent(ModelEvent.prototype.ADD_DIALOG);
    if(evt != null) {
        evt.data = {id: dialog.getId()};
        me.dispatchEvent(evt);
    }
    return dialog.getId();
};

//
MainInterfaceModel.prototype.removeDialog = function(id){
    var me = this;
    var i, n = me.aDialog.length;
    var dialog;
    for(i=0; i<n; i++){
        dialog = me.aDialog[i];
        if(dialog.getId() == id){
            var evt = new ModelEvent(ModelEvent.prototype.REMOVE_DIALOG);
            if(evt != null) {
                evt.data = {id: id};
                me.dispatchEvent(evt);
            }
            me.aDialog.splice(i, 1);
            dialog.deactivate();
            return true;
        }
    }
    return false;
};

//
MainInterfaceModel.prototype.getDialogs = function(){
    return this.aDialog;
};

//
MainInterfaceModel.prototype.getAttemptsCount = function(){
    return this.mapSceneModel.getAttemptsCount();
};

//
MainInterfaceModel.prototype.deactivate = function(){
    MainInterfaceModel.superclass.deactivate.call(this);
    var me = this;
    var i, n = me.aToolbar.length;
    var toolbar;
    for(i=0; i<n; i++) {
        toolbar = me.aToolbar[i];
        toolbar.addEventListener(
            ModelEvent.prototype.TOOLBAR_BUTTON_EVENT, {target: me, listener: me.fToolbarListener}, false
        );
        toolbar.deactivate();
    }
    //
    me.screenKeyboard.deactivate();
    //
    me.mapSceneModel.removeEventListener(
        ModelEvent.prototype.VERIFY_ELEMENTS, {target:me, listener:me.fVerifyElementsListener}, false
    );
    //
    me.mapSceneModel.removeEventListener(
        ModelEvent.prototype.SHOW_HINT, {target:me, listener:me.fShowHintListener}, false
    );
    //
    me.mapSceneModel.removeEventListener(
        ModelEvent.prototype.SHOW_RESPONSE, {target:me, listener:me.fShowResponseListener}, false
    );
    //
    me.mapSceneModel.removeEventListener(
        ModelEvent.prototype.TOGGLE_SCREEN_KEYBOARD, {target:me, listener:me.fToggleScreenKeyboardListener}, false
    );
    //
    me.mapSceneModel.removeEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:me, listener:me.fResetSceneListener}, false
    );
};

/**
 * @constructor
 */
function TopToolbarVisualization(_model, _mainInterfaceVisualization){
    TopToolbarVisualization.superclass.constructor.call(this, _model, _mainInterfaceVisualization);
    var me = this;
    me.aButtonVisualization = [];
}
extend(TopToolbarVisualization, ToolbarVisualization);

//override
TopToolbarVisualization.prototype.update = function(left, top){
    var me = this;

    var aButtonVisualization = me.aButtonVisualization;
    var model = me.getModel();
    var n = model.getButtonCount();
    var i, sId, ii;
    var l = aButtonVisualization.length;
    var sClassName, cls;
    var button, buttonVisualization;

    if(!me.getModel().isOnlySelectionTool()) {
        for (i = 0; i < n; i++) {
            button = model.getButtonNo(i);
            sId = button.getId();
            sClassName = miscFunctions.getClassName(button);
            ii = sClassName.indexOf(model.MODEL_SUFFIX);
            cls = window[((ii != -1) ? sClassName.slice(0, ii) : sClassName) + me.VISUALIZATION_SUFFIX];
            if (
                typeof cls == TYPE_FUNCTION &&
                (aButtonVisualization.length <= i || aButtonVisualization[i].getButton().getId() != sId)
            ) {
                buttonVisualization = new cls(button, me);
                aButtonVisualization.splice(i, 0, buttonVisualization);
            }
        }
        if (aButtonVisualization.length != l)
            me.invalidate();
    }

    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");
    var cnvToolbar = me.interfaceVisualization.getCanvas();
    var ctxToolbar = cnvToolbar.getContext("2d");
    var rSize = me.getCanvasToSceneSizeRatio();
    var w = Math.ceil(me.model.getWidth()*rSize);
    var h = Math.ceil(me.model.getHeight()*rSize);
    if(cnv.width != w || cnv.height != h){
        cnv.width = w;
        cnv.height = h;
        me.invalidate();
    }

    me.checkCanvasBoundsChange();

    if(!(me.model.isValid() && me.isValid())) {
        me.model.validate();
        me.flagValid = true;
        ctx.clearRect(0, 0, w, h);

        //background for debugging
        /*ctx.fillStyle = "rgba(0, 0, 128, 0.05)";
        ctx.lineWidth = 1;
        ctx.beginPath();
        ctx.moveTo(0.5, 0.5);
        ctx.lineTo(0.5, h - 0.5);
        ctx.lineTo(w - 0.5, h - 0.5);
        ctx.lineTo(w - 0.5, 0.5);
        ctx.lineTo(0.5, 0.5);
        ctx.fill();*/

        n = aButtonVisualization.length;
        for(i=0; i<n; i++){
            buttonVisualization = aButtonVisualization[i];
            button = buttonVisualization.getButton();
            ctx.setTransform(1, 0, 0, 1, (button.getX()*rSize)|0, (button.getY()*rSize)|0);
            buttonVisualization.update(left, top);
        }
        ctx.setTransform(1, 0, 0, 1, 0, 0);
    }
    //
    ctxToolbar.drawImage(cnv, 0, 0);
};
/**
 * @constructor
 */
function RightToolbarVisualization(_model, _mainInterfaceVisualization){
    RightToolbarVisualization.superclass.constructor.call(this, _model, _mainInterfaceVisualization);
    var me = this;
    var model = me.model;

    me.commentCanvas = document.createElement("CANVAS");
    me.commentCanvas.height = 1;

    me.bnVisibilityVisualization =
        new ToolbarShowHiddenButtonVisualization(model.getButtonById(model.BUTTON_SHOW_HIDDEN_ID), me);

    me.bnCorrectElementsVisualization = new ToolbarTextButtonVisualization(model.getButtonById(model.BUTTON_CORRECT_ELEMENTS_ID), me);
    me.bnHintVisualization = new ToolbarTextButtonVisualization(model.getButtonById(model.BUTTON_HINT_ID), me);
    me.bnResetVisualization = new ToolbarTextButtonVisualization(model.getButtonById(model.BUTTON_RESET_ID), me);
    me.bnConfirmVisualization = new ToolbarTextButtonVisualization(model.getButtonById(model.BUTTON_CONFIRM_ID), me);

    me.aBnVisualization = [
        me.bnVisibilityVisualization,
        me.bnCorrectElementsVisualization, me.bnHintVisualization, me.bnResetVisualization, me.bnConfirmVisualization
    ];
    me.aBottomBnVisualization = [
        me.bnCorrectElementsVisualization, me.bnHintVisualization, me.bnResetVisualization, me.bnConfirmVisualization
    ];

}
extend(RightToolbarVisualization, ToolbarVisualization);

//override
RightToolbarVisualization.prototype.deactivate = function(){
    RightToolbarVisualization.superclass.deactivate.call(this);
    var me = this;
    var i, n = me.aBnVisualization.length;
    for(i=0; i<n; i++)
        me.aBnVisualization[i].deactivate();
};

//override
RightToolbarVisualization.prototype.update = function(left, top){
    var me = this;

    var cnv = me.cnv;
    var ctx = cnv.getContext("2d");
    //
    var i, n, obj;
    //
    var model = me.getModel();
    var rSize = me.getCanvasToSceneSizeRatio();

    var w = Math.ceil(me.model.getWidth()*rSize);
    var h = Math.ceil(me.model.getHeight()*rSize);
    if(cnv.width != w || cnv.height != h){
        cnv.width = w;
        cnv.height = h;
        me.invalidate();
    }

    me.checkCanvasBoundsChange();

    if(!(me.model.isValid() && me.isValid())) {
        me.model.validate();
        me.flagValid = true;
        ctx.clearRect(0, 0, w, h);

        //background for debugging
        /*ctx.fillStyle = "rgba(0, 64, 64, 0.05)";
        ctx.lineWidth = 1;
        ctx.beginPath();
        ctx.moveTo(0.5, 0.5);
        ctx.lineTo(0.5, h - 0.5);
        ctx.lineTo(w - 0.5, h - 0.5);
        ctx.lineTo(w - 0.5, 0.5);
        ctx.lineTo(0.5, 0.5);
        ctx.fill();*/

        var wTB = me.model.getWidth();
        var hTB = me.model.getHeight();
        var hTopTB = me.model.getAreas().topHeight;

        var aBnVisualization = me.aBnVisualization;
        var aBottomBnVisualization =  me.aBottomBnVisualization;
        var button, bnVisualization, droplistVisualization;
        var xButton, yButton;
        var yButtonMin = hTB;

        n = aBnVisualization.length;

        for(i=0; i<n; i++){
            bnVisualization = aBnVisualization[i];
            droplistVisualization = (bnVisualization instanceof ToolbarSymbolSetButtonVisualization) ? bnVisualization.droplistVisualization : null;
            button = bnVisualization.getButton();
            if(button.isVisible()) {
                xButton = button.getX();
                yButton = button.getY();
                if(yButton<yButtonMin && aBottomBnVisualization.indexOf(bnVisualization) != -1)
                    yButtonMin = yButton;
                ctx.setTransform(1, 0, 0, 1, (rSize * xButton)|0, (rSize * yButton)|0);
                bnVisualization.update(left, top);
            }else if(droplistVisualization !== null && droplistVisualization.isVisible())
                droplistVisualization.hide();
        }

        ctx.setTransform(1, 0, 0, 1, 0, 0);

        //comment text is redrawn whenever rSize is changed
        var hCommentCanvas = Math.ceil(rSize*(yButtonMin-hTopTB-model.COMMENT_PADDING_TOP));
        var ctxComment, aCommentBlock;
        if(me.commentCanvas.height != hCommentCanvas){
            me.commentCanvas.width = Math.ceil(rSize*model.COMMENT_WIDTH);
            me.commentCanvas.height = hCommentCanvas;
            ctxComment = me.commentCanvas.getContext("2d");
            aCommentBlock = model.getCommentTextBlocks(rSize);
            n = aCommentBlock.length;
            if(n>0) {
                for (i = 0; i < n; i++) {
                    obj = aCommentBlock[i];
                    ctxComment.font = obj.font;
                    ctxComment.fillStyle = obj.color;
                    ctxComment.fillText(obj.text, obj.x|0, obj.y|0);
                }
            }
        }
        ctx.drawImage(me.commentCanvas, (rSize*model.COMMENT_PADDING_LEFT)|0, (rSize*(model.COMMENT_PADDING_TOP+hTopTB))|0);
    }

    var cnvToolbar = me.interfaceVisualization.getCanvas();
    var ctxToolbar = cnvToolbar.getContext("2d");
    ctxToolbar.drawImage(cnv, 0, 0);
};

/**
 * @constructor
 */
function MainInterfaceVisualization(_model, _mapSceneVisualization){
    MainInterfaceVisualization.superclass.constructor.call(this);
    var me = this;
    me.model = _model;
    me.mapSceneVisualization = _mapSceneVisualization;

    me.canvas = null;

    me.flagScaleInterface = ModelMainInstance.prototype.flagScaleInterface;

    me.dialogDIV = null;

    me.topToolbarVisualization = new TopToolbarVisualization(me.model.getTopToolbar(), me);

    me.rightToolbarVisualization = new RightToolbarVisualization(me.model.getRightToolbar(), me);

    me.screenKeyboardVisualization = new ScreenKeyboardVisualization(
        me.model.getScreenKeyboard(), me, me.mapSceneVisualization
    );

    me.aDialogVisualization = [];
    me.aDialogController = [];

    me.model.addEventListener(
        ModelEvent.prototype.VALIDATE_INTERFACE, {target:me, listener:me.modelListener}, false
    );
    //
    me.model.addEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:me, listener:me.modelListener}, false
    );
    //
    var s;
    s = document.localization.getString("interface.toolbars.background.color");
    if(s!=null)
        MainInterfaceVisualization.prototype.BACKGROUND_COLOR = miscFunctions.RGBString2Int(s);
    s = document.localization.getString("interface.toolbars.background.alpha");
    if(s!=null && !isNaN(s))
        MainInterfaceVisualization.prototype.BACKGROUND_ALPHA = parseFloat(s);
}
extend(MainInterfaceVisualization, NonDOMEventDispatcher);

MainInterfaceVisualization.prototype.BACKGROUND_COLOR = 0xF5F4D6;
MainInterfaceVisualization.prototype.BACKGROUND_ALPHA = 1.0;
MainInterfaceVisualization.prototype.SCENE_FRAME_COLOR_LEFT = 0x000066;
MainInterfaceVisualization.prototype.SCENE_FRAME_COLOR_RIGHT = 0x999999;
MainInterfaceVisualization.prototype.SCENE_FRAME_COLOR_TOP = 0x000066;
MainInterfaceVisualization.prototype.SCENE_FRAME_COLOR_BOTTOM = 0x999999;
MainInterfaceVisualization.prototype.SCENE_FRAME_ALPHA = 1.0;

//
MainInterfaceVisualization.prototype.fOnValidate = function(e){
    var me = this;
    me.time = e.data.time;
    me.update();
};

//
MainInterfaceVisualization.prototype.getHitDialog = function(target){
    var me = this;
    var aDVis = me.aDialogVisualization;
    var i, n = aDVis.length;
    var vis;
    for(i=0; i<n; i++){
        vis = aDVis[i];
        if(vis.getCanvas().contains(target))
            return vis.getModel();
    }
    return null;
};

//
MainInterfaceVisualization.prototype.isModalDialogPresent = function(){
    var me = this;
    var aDVis = me.aDialogVisualization;
    var i, n = aDVis.length;
    for(i=0; i<n; i++)
        if(aDVis[i].getModel().isModal())
            return true;
    return false;
};

//
MainInterfaceVisualization.prototype.addDialogVisualization = function(sId) {
    var me = this;
    var aDialog = me.model.getDialogs();
    var i, n = aDialog.length;
    var dialog, vis = null, dCtrl;
    for(i=0; i<n; i++){
        dialog = aDialog[i];
        if(dialog.getId() == sId){
            vis = new ModalDialogVisualization(dialog, me);
            dCtrl = new ModalDialogController(dialog, vis);
            me.aDialogVisualization.push(vis);
            me.aDialogController.push(dCtrl);
            vis.show();
            break;
        }
    }
    return vis;
};

//
MainInterfaceVisualization.prototype.removeDialogVisualization = function(sId){
    var me = this;
    var i, n = me.aDialogVisualization.length;
    var vis, dCtrl;
    for(i=0; i<n; i++){
        vis = me.aDialogVisualization[i];
        if(vis.getModel().getId() == sId){
            dCtrl = me.aDialogController[i];
            dCtrl.deactivate();
            me.aDialogController.splice(i, 1);
            vis.hide();
            vis.deactivate();
            me.aDialogVisualization.splice(i, 1);
            break;
        }
    }
    return i<n;
};

//
MainInterfaceVisualization.prototype.modelListener = function(e){
    var me = this;
    switch(e.type){
        case ModelEvent.prototype.VALIDATE_INTERFACE:
            me.update();
            break;
        case ModelEvent.prototype.RESET_SCENE:
            me.conveyEvent(e);
            break;
        default:
    }
};

//
MainInterfaceVisualization.prototype.getCanvas = function(){
    return this.canvas;
};

//
MainInterfaceVisualization.prototype.setCanvas = function(canvas){
    this.canvas = canvas;
};

MainInterfaceVisualization.prototype.getDialogDIV = function(){
    return this.dialogDIV;
};

MainInterfaceVisualization.prototype.setDialogDIV = function(div){
    this.dialogDIV = div;
};

MainInterfaceVisualization.prototype.getCanvasToSceneSizeRatio = function(){
    return this.flagScaleInterface ? this.mapSceneVisualization.getCanvasToSceneSizeRatio() : 1;
};

MainInterfaceVisualization.prototype.canvasCoordsToInterfaceCoords = function(x0, y0){
    var me = this;
    var leftPadding = me.mapSceneVisualization.getLeftPadding();
    var topPadding = me.mapSceneVisualization.getTopPadding();
    var rSize = me.getCanvasToSceneSizeRatio();
    var objAreas = me.model.getInterfaceAreas();
    return new Point((x0-leftPadding)/rSize+objAreas.leftWidth, (y0-topPadding)/rSize+objAreas.topHeight);
};

MainInterfaceVisualization.prototype.update = function() {
    var me = this;

    var prevLP = me.mapSceneVisualization.getLeftPadding100();
    var prevTP = me.mapSceneVisualization.getTopPadding100();
    var prevRP = me.mapSceneVisualization.getRightPadding100();
    var prevBP = me.mapSceneVisualization.getBottomPadding100();
    var objAreas = me.model.getInterfaceAreas();

    var flagAreasChanged =
        (prevLP != objAreas.leftWidth) || (prevRP != objAreas.rightWidth) ||
        (prevTP != objAreas.topHeight) || (prevBP != objAreas.bottomHeight);

    //resize model scene if necessary
    if(flagAreasChanged){
        me.mapSceneVisualization.setLeftPadding100(objAreas.leftWidth);
        me.mapSceneVisualization.setTopPadding100(objAreas.topHeight);
        me.mapSceneVisualization.setRightPadding100(objAreas.rightWidth);
        me.mapSceneVisualization.setBottomPadding100(objAreas.bottomHeight);
        me.mapSceneVisualization.update();
    }

    //
    if (me.canvas == null)
        return;
    var canvas = me.canvas;
    var ctx = canvas.getContext("2d");
    var wCanvas = canvas.width;
    var hCanvas = canvas.height;
    var wSV = me.mapSceneVisualization.getWidth();
    var hSV = me.mapSceneVisualization.getHeight();
    var rSize = me.getCanvasToSceneSizeRatio();

    //background fill
    var leftPadding = me.mapSceneVisualization.getLeftPadding();
    var topPadding = me.mapSceneVisualization.getTopPadding();

    //clear interface areas
    ctx.clearRect(0, 0, leftPadding, hCanvas);
    ctx.clearRect(leftPadding, 0, wSV, topPadding);
    ctx.clearRect(leftPadding, topPadding+hSV, wSV, hCanvas-hSV-topPadding);
    ctx.clearRect(leftPadding+wSV, 0, wCanvas-wSV-leftPadding, hCanvas);

    //canvas background fill
    ctx.beginPath();
    ctx.fillStyle = miscFunctions.intAlpha2RGBAFunction(me.BACKGROUND_COLOR, me.BACKGROUND_ALPHA);
    ctx.moveTo(0, 0);
    ctx.lineTo(0, hCanvas);
    ctx.lineTo(wCanvas, hCanvas);
    ctx.lineTo(wCanvas, 0);
    ctx.lineTo(0, 0);
    ctx.moveTo(leftPadding, topPadding);
    ctx.lineTo(leftPadding+wSV, topPadding);
    ctx.lineTo(leftPadding+wSV, topPadding+hSV);
    ctx.lineTo(leftPadding, topPadding+hSV);
    ctx.lineTo(leftPadding, topPadding);
    ctx.fill();

    var wF = 1*rSize;
    ctx.lineWidth = wF;
    ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.SCENE_FRAME_COLOR_TOP, me.SCENE_FRAME_ALPHA);
    ctx.beginPath();
    ctx.moveTo((leftPadding+wSV+wF)|0, (topPadding-wF)|0);
    ctx.lineTo((leftPadding-wF)|0, (topPadding-wF)|0);
    ctx.stroke();
    ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.SCENE_FRAME_COLOR_LEFT, me.SCENE_FRAME_ALPHA);
    ctx.beginPath();
    ctx.moveTo((leftPadding-wF)|0, (topPadding-wF)|0);
    ctx.lineTo((leftPadding-wF)|0, (topPadding+hSV+wF)|0);
    ctx.stroke();
    ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.SCENE_FRAME_COLOR_BOTTOM, me.SCENE_FRAME_ALPHA);
    ctx.beginPath();
    ctx.moveTo((leftPadding-wF)|0, (topPadding+hSV+wF)|0);
    ctx.lineTo((leftPadding+wSV+wF)|0, (topPadding+hSV+wF)|0);
    ctx.stroke();
    ctx.strokeStyle = miscFunctions.intAlpha2RGBAFunction(me.SCENE_FRAME_COLOR_RIGHT, me.SCENE_FRAME_ALPHA);
    ctx.beginPath();
    ctx.moveTo((leftPadding+wSV+wF)|0, (topPadding+hSV+wF)|0);
    ctx.lineTo((leftPadding+wSV+wF)|0, (topPadding-wF)|0);
    ctx.stroke();

    me.topToolbarVisualization.invalidate();
    me.rightToolbarVisualization.invalidate();
    var left = leftPadding;
    var top = (topPadding-rSize*me.mapSceneVisualization.getTopPadding100());
    ctx.setTransform(1, 0, 0, 1, left|0, top|0);

    //top toolbar
    //left and top are specified as arguments in order to update droplist positions
    var canvasBounds = miscFunctions.getElementBounds(canvas);
    me.topToolbarVisualization.update(canvasBounds.left+left, canvasBounds.top+top);
    ctx.setTransform(1, 0, 0, 1, 0, 0);

    //right toolbar
    ctx.setTransform(1, 0, 0, 1, (left+wSV)|0, top|0);
    me.rightToolbarVisualization.update(canvasBounds.left+left+wSV, canvasBounds.top+top);
    ctx.setTransform(1, 0, 0, 1, 0, 0);

    //dialogs
    var i, n = me.aDialogVisualization.length;
    for(i=0; i<n; i++) {
        var vis = me.aDialogVisualization[i];
        var objStyle = vis.getCanvas().style;
        objStyle.left = (((canvasBounds.width-rSize*vis.getModel().getWidth())/2)|0)+"px";
        objStyle.top = (((canvasBounds.height-rSize*vis.getModel().getHeight())/2)|0)+"px";
        vis.update(0, 0);
    }

    //screenKeyboard
    var screenKeyboard = me.model.getScreenKeyboard();
    if(screenKeyboard.isVisible() && !me.screenKeyboardVisualization.isVisible())
        me.screenKeyboardVisualization.show();
    else if(!screenKeyboard.isVisible() && me.screenKeyboardVisualization.isVisible())
        me.screenKeyboardVisualization.hide();
    if(me.screenKeyboardVisualization.isVisible())
        me.screenKeyboardVisualization.update();
    if(!screenKeyboard.isValid())
        screenKeyboard.validate();
};

MainInterfaceVisualization.prototype.deactivate = function(){
    MainInterfaceVisualization.superclass.deactivate.call(this);
    var me = this;
    me.model.removeEventListener(
        ModelEvent.prototype.VALIDATE_INTERFACE, {target:me, listener:me.modelListener}, false
    );
    me.model.removeEventListener(
        ModelEvent.prototype.RESET_SCENE, {target:me, listener:me.modelListener}, false
    );
    me.rightToolbarVisualization.deactivate();
    me.topToolbarVisualization.deactivate();
    me.screenKeyboardVisualization.deactivate();
};